// SoftPixel Engine - Getting started

#include <SoftPixelEngine.hpp>

#pragma comment(lib, "SoftPixelEngine.lib")

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    // The softpixel-device opens the screen (windowed screen / fullscreen are possible)
    SoftPixelDevice* spDevice = createDevice(ChooseRenderer(), dim::size2di(640, 480), 32, "Getting started");
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver(); // Video manager for the video buffer
    scene::SceneManager* spSmngr = spDevice->getSceneManager(); // Now we can render 3d scenes
    UserControl* spControl = spDevice->getUserControl(); // Now we can check the keyboard- and the mouse input events
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    scene::Camera* cam = spSmngr->createCamera(); // Creates a camera to see our scene
    scene::Light* light = spSmngr->createLight(); // Creates a light (default: Diffuse light)
    spSmngr->setLighting(); // Activates the lighting subsystem
    
    scene::Entity* object = spSmngr->createModel(scene::ENTITY_TEAPOT, 20); // Creates a primitive object
    object->setPosition(dim::vector3df(0, 0, 2.5)); // Sets the objects position (x, y, z)
    
    #if 1
    // Sphere map
    object->addTexture(spDriver->loadTexture("../../../media/SphereMap4.jpg"));
    object->setMappingGen(0, video::MAPGEN_SPHERE_MAP);
    object->setNormalsType(scene::NORMALS_SMOOTH);
    object->updateNormals();
    #endif
    
    // The main loop will update our device
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers(); // Clears the video buffer
        
        spSmngr->renderScene(); // Renders the scene
        
        object->doTurn(dim::vector3df(
            -(f32)spControl->getCursorSpeed().Y, -(f32)spControl->getCursorSpeed().X, 0
        ));
        
        spDriver->flipBuffers(); // Swaps the video buffer, so we can see the current frame
    }
    
    deleteDevice(); // Deletes the device context and closes the screen
    
    return 0;
}
// END-OF-LINE
