//
// SoftPixel Engine - Tutorial1 (2DDrawing) - (12.1.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    
    const s32 ScrWidth = 640, ScrHeight = 480;
    
    // The softpixel-device will create the screen (windowed screen / fullscreen are possible)
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(ScrWidth, ScrHeight), 32, "Tutorial1 (2DDrawing)", 0, 1
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver(); // Main manager for drawing operations
    UserControl* spControl = spDevice->getUserControl(); // Now we can check the keyboard- and the mouse events
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    video::Texture* tex1 = spDriver->loadTexture("../../../media/DemoImage1.png"); // Loads the specified texture
    tex1->setTransFilter(video::color(255, 0, 255, 0)); // Sets the color-key, color 255,0,255 with an alpha-channel of 0 (zero)
    
    f32 Rotation = 0.0f;
    
    video::Font* font = spDriver->loadFont("arial", dim::size2di(15, 35));
    
    // The main loop will update our device
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers(); // Clears the drawing buffer
        
        spDriver->beginDrawing2D();
        
        // Draws a multicolored rectangle
        spDriver->draw2DRectangle(
            dim::rect2di(0, 0, ScrWidth, ScrHeight),
            video::color(255, 255, 0),
            video::color(255, 0, 0),
            video::color(0, 255, 0),
            video::color(0, 0, 255)
        );
        
        ++Rotation;
        
        // Draws a rotatable 2d image
        spDriver->draw2DImage(
            tex1,
            dim::rect2di(100, 50, 400, 200),
            dim::rect2df(0, 2, 2, 0),
            true,
            Rotation,
            dim::point2df(0.5f, 1.0f),
            video::color(0, 0, 255, 0),
            video::color(255, 255, 255, 255),
            video::color(255, 255, 255, 255),
            video::color(255, 255, 255, 0)
        );
        
        // Draws a rotatable ellipse
        spDriver->draw2DEllipse(
            dim::point2di(ScrWidth/2, ScrHeight/2),
            dim::size2di(150, 75),
            -Rotation,
            video::color(255, 255, 0, 128),
            true
        );
        spDriver->draw2DEllipse(
            dim::point2di(ScrWidth/2, ScrHeight/2),
            dim::size2di(150, 75),
            Rotation*2,
            video::color(255, 255, 0, 128),
            false
        );
        
        // Draws a bezier curve
        spDriver->draw2DBezier(
            dim::point2di(50, 50),
            dim::point2di(50, ScrHeight-50),
            dim::point2di(150, 150),
            dim::point2di(-50, ScrHeight-150)
        );
        
        // Draws a dashed line
        spDriver->draw2DLine(dim::point2di(50, 50), dim::point2di(ScrWidth-50, ScrHeight-50), video::color(0, 0, 255), 5);
        
        // Draws a 2d font text
        spDriver->draw2DText(font, dim::point2di(25, 75), "See the power of the 'SoftPixel Engine'!", video::color(0, 0, 255, 128));
        
        spDriver->endDrawing2D();
        
        spDriver->flipBuffers(); // The buffer will swap
    }
    
    deleteDevice(); // The device context will delete
    
    return 0;
}




// ======================================================
