//
// SoftPixel Engine - Tutorial10(StencilManager) - (1.3.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    
    // The folowing lines should not surpriseful
    
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(640, 480), 32, "Tutorial10 (StencilManager)"
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver();
    scene::SceneManager* spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    scene::StencilManager* spStencil = spSmngr->getStencilManager();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    video::Texture* tex1 = spDriver->loadTexture("../../../media/DemoImage6.JPG");
    
    scene::Camera* cam = spSmngr->createCamera();
    
    scene::Light* light = spSmngr->createLight();
    //smngr->setLighting();
    
    math::SeedRandom();
    
    // Now we don't want to create allways the same objects, so I will show you a very nice standard model.
    // Have you ever heart anything about a 'supershape'? It is a complex model which can look like a sphere,
    // cube, cylinder or a crystal. But, see for your self:
    // At first we need a 'ModelCreate' class temporaryly. We also need 12 float values for a super shape but
    // don't worry, the 'SoftPixel Engine' has a function to create over 10 standard supershape models.
    scene::ModelCreator modelgen; f32 val[12];
    modelgen.getSuperShapeModel(scene::SUPERSHAPE_RANDOM, val);
    scene::Entity* object = spSmngr->createSuperShape(val, 90);
    
    object->setScale(dim::vector3df(0.75f));
    object->setPosition(dim::vector3df(0, 0, 4));
    object->addTexture(tex1);
    object->doTextureTransform(0, dim::size2df(0.5));
    
    // Before creating the stencil manager we will add the mirror. That will be just a normal small plane.
    // A mirror should flat structured and not like a highmap
    scene::Entity* plane = spSmngr->createModel(scene::ENTITY_PLANE, 1);
    plane->setScale(dim::vector3df(4));
    plane->setColor(video::color(255, 255, 255, 128));
    
    spDriver->setClearColor(video::color(0, 0, 64));
    
    cam->lookAt(object->getPosition());
    
    // Adds a mirror plane to the stencil manager. You can maximal add six (6) mirrors! Sorry but also OpenGL has a limit.
    // The 'MIRROR_Y' constant is important for the plane-equation for 'OpenGL' to render the stencil-mirror correctly.
    // If you have a mirror which is normal, unrotated vertical, you can use the constants 'MIRROR_X' or 'MIRROR_Z'.
    // Try something out to find the correct value.
    spStencil->addMirrorPlane(plane, scene::MIRROR_Y);
    // Adds our object which should be mirrored to the stencil manager
    spStencil->addMirrorObject(object); // Adds the 
    // Now we can switch the stenil effects on. That is possible with the scene manager. If you want to switch it off
    // you can get this function the value 'false', 'true' is the default value.
    spSmngr->setStencilEffects();
    
    f32 Degree1 = 0.0f, Degree2;
    dim::point2di MouseSpeed;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        spSmngr->renderScene();
        
        // Let's swing our mirror
        ++Degree1;
        Degree2 = 180+SIN(Degree1)*90;
        plane->setPosition(dim::vector3df(SIN(Degree2)*1.25f, COS(Degree2)*1.25f, 4));
        plane->setRotation(dim::vector3df(0, 0, 180-Degree2));
        
        // When we rotate a mirror plane we have to update the plane-equation with the following function.
        // You can let this function away but it is possible that you will sometimes see a crazy result
        spStencil->updateMirrorPlane(plane);
        
        MouseSpeed = spControl->getCursorSpeed();
        object->doTurn(dim::vector3df(1));
        object->doTranslate(dim::vector3df(f32(MouseSpeed.X)/100, -f32(MouseSpeed.Y)/100, 0.0f));
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}


// Sprite functions




// ======================================================
