//
// SoftPixel Engine - Tutorial12(Picking) - (15.9.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp;

#include "../common.hpp"

int main()
{
    
    // The softpixel-device will create the screen (windowed screen / fullscreen are possible)
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(1280, 1024), 32, "Tutorial12 (Picking)", true
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver(); // Main manager for drawing operations
    scene::SceneManager* spSmngr = spDevice->getSceneManager(); // Now we can render 3d worlds
    UserControl* spControl = spDevice->getUserControl(); // Now we can check the keyboard- and the mouse events
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    spControl->hideCursor(false);
    
    scene::CollisionDetector* spCmngr = spDevice->getCollisionDetector();
    
    scene::Camera* cam = spSmngr->createCamera(); // Without a camera we would see nothing
    
    scene::Light* light = spSmngr->createLight(); // Creates a light (default: Diffuse light)
    light->setPosition(dim::vector3df(0, 5, -5));
    spSmngr->setLighting(); // Activates the lighting engine
    
    #define MAX_OBJ 10
    
    scene::Entity* obj[MAX_OBJ];
    scene::Entity* lastobj = 0;
    scene::Entity* point;
    f32 ColorDegree = 0.0f;
    f32 ColorRange = 0.0f;
    bool isPoint = true;
    
    math::SeedRandom();
    
    point = spSmngr->createModel(scene::ENTITY_SPHERE);
    point->setVisible(false);
    point->setColor(video::color(255, 0, 0), 0, true);
    point->setScale(0.1);
    
    for (s32 i = 0; i < MAX_OBJ; ++i)
    {
        // Create a random model
        obj[i] = spSmngr->createModel(math::Rnd(scene::ENTITY_CUBE, scene::ENTITY_TEAPOT));
        
        // Smooth normals
        obj[i]->setNormalsType(scene::NORMALS_SMOOTH);
        obj[i]->updateNormals();
        
        // No color material (we only need one color for each object)
        obj[i]->setColorMaterial(false);
        
        // Random location
        obj[i]->setPosition(dim::vector3df((f32)math::Random(-150, 150)/100, (f32)math::Random(-150, 150)/100, (f32)math::Random(250, 1500)/100));
        obj[i]->setRotation(dim::vector3df((f32)math::Rnd(3600)/10, (f32)math::Rnd(3600)/10, (f32)math::Rnd(3600)/10));
        
        // Oct tree for optimization
        obj[i]->createOctTree(1);
        
        // Add to the picking list
        spCmngr->addPickingEntity(obj[i], scene::PICKMODE_POLYGON);
    }
    
    // Our picking list
    std::list<scene::SPickingObject> PickingList;
    
    // The main loop will update our device
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers(); // Clears the drawing buffer
        
        PickingList = spCmngr->pickLineIntersection(cam->getPickingLine(spControl->getCursorPosition()));
        
        if (PickingList.size())
        {
            scene::SPickingObject* Picked = &(*PickingList.begin());
            
            if (lastobj && lastobj != Picked->Object)
                lastobj->getThisMaterial().ColorDiffuse = 255;
            
            if ( ( ColorDegree += 10 ) > 360.0f )
                ColorDegree -= 360.0f;
            ColorRange = 128 + SIN(ColorDegree)*125;
            
            // Re-color the last object
            if (lastobj)
            {
                if (spDriver->getDriverType() == video::DRIVER_SOFTWARE)
                    lastobj->setColor(255, 0, true);
                else
                    lastobj->getThisMaterial().ColorDiffuse = 255;
            }
            
            // Get the first picked object
            if (spDriver->getDriverType() == video::DRIVER_SOFTWARE)
                Picked->Object->setColor(video::color((u8)ColorRange, (u8)ColorRange, 0), 0, true);
            else
                Picked->Object->getThisMaterial().ColorDiffuse = video::color((u8)ColorRange, (u8)ColorRange, 0);
            
            lastobj = Picked->Object;
            
            point->setPosition(Picked->Intersection);
            point->setVisible(true);
        }
        else if (lastobj)
        {
            point->setVisible(false);
            
            // Re-color the last object
            if (spDriver->getDriverType() == video::DRIVER_SOFTWARE)
                lastobj->setColor(255, 0, true);
            else
                lastobj->getThisMaterial().ColorDiffuse = 255;
        }
        
        // Hide/ Show the intersection/ picking point
        if (spControl->isKeyHit(KEY_RETURN))
            isPoint = !isPoint;
        if (!isPoint)
            point->setVisible(false);
        
        // Locate the object new
        if (spControl->isKeyHit(KEY_SPACE))
        {
            for (s32 i = 0; i < MAX_OBJ; ++i)
            {
                obj[i]->setPosition(dim::vector3df((f32)math::Random(-150, 150)/100, (f32)math::Random(-150, 150)/100, (f32)math::Random(250, 1500)/100));
                obj[i]->setRotation(dim::vector3df((f32)math::Rnd(3600)/10, (f32)math::Rnd(3600)/10, (f32)math::Rnd(3600)/10));
            }
        }
        
        spSmngr->renderScene(); // Renders the world
        
        spDriver->flipBuffers(); // The buffer will swap
    }
    
    deleteDevice(); // The device context will delete
    
    return 0;
}



// ======================================================
