//
// SoftPixel Engine - Tutorial13 (MiniMatrix) - (29.1.2008)
//

#include "main.h"

#include "../common.hpp"


int main(void)
{
    
    InitGame();
    LoadGame();
    
    // Main loop
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        RenderEffects();
        
        spCmngr->updateScene();
        //UpdateScene();
        
        spSmngr->renderScene();
        spAnim8or->updateAnimations();
        
        UpdateGame();
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    // Delete all physics obejcts
    for (std::vector<PhysicObject*>::iterator it = PhysicObjectList.begin(); it != PhysicObjectList.end(); ++it)
        spDeletePointer(*it);
    // Delete all dustcloud objects
    for (std::vector<DustCloud*>::iterator it = DustCloudList.begin(); it != DustCloudList.end(); ++it)
        spDeletePointer(*it);
    
    return EXIT_SUCCESS;
}


// ALL FUNCTIONS


void InitGame()
{
    
    spDevice    = createDevice(
        ChooseRenderer(), size2di(ScrWidth, ScrHeight), 32, "Tutorial13 (MiniMatrix)", isFullScreen, SDeviceFlags(0, 1, 1, 8)
    );
    spListener  = createListener(0.0, 75.0, 1.0, 0.0);
    
    spDriver    = spDevice->getVideoDriver();
    spSmngr     = spDevice->getSceneManager();
    spCmngr     = spDevice->getCollisionDetector();
    spControl   = spDevice->getUserControl();
    
    spAnim8or   = spSmngr->getAnimator();
    spEffects   = spSmngr->getSpecialEffectsActor();
    spSclmngr   = spSmngr->getStencilManager();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    spDriver->setTextureGenFlags(video::TEXGEN_FILTER, video::TEXTURE_ANISOTROPIC);
    spDriver->setTextureGenFlags(video::TEXGEN_ANISOTROPY, 16);
    
    rayEnable(RAY_COLOR_SMOOTH);
    
}

void LoadGame()
{
    
    /* ====== First creations ======= */
    
    player = spSmngr->createPivot();
    player->setPosition(vector3df(25, 2, 25));
    cam = spSmngr->createCamera();
    cam->setParent(player);
    cam->setPosition(vector3df(0, CAM_HEIGHT, 0));
    cam->setPerspective(rect2di(0, 0, ScrWidth, ScrHeight), 0.75f, 1000.0f);
    
    spSmngr->setLighting();
    
    light1 = spSmngr->createLight();
    //light1->setPosition(vector3df(-50, 200, -75));
    light1->setRotation(vector3df(70, 25, 0));
    light1->setLightingColor(255, 255, 0);
    
    lightStick = spSmngr->createModel(ENTITY_CYLINDER, 4);
    lightStick->fitMesh(vector3df(-0.5, 0.0, -0.5), vector3df(1.0, 1.0, 1.0));
    lightStick->flipMesh(true, true, false);
    lightStick->setScale(vector3df(0.25, 5.0, 0.25));
    lightStick->setPosition(vector3df(130, 24, 91));
    
    lamp = spSmngr->createModel(ENTITY_SPHERE, 5, 0.35);
    lamp->setColor(video::color(255, 255, 0));
    
    light2 = spSmngr->createLight(LIGHT_SPOT);
    light2->setPosition(vector3df(0, -1.0, 0));
    light2->setRotation(vector3df(90, 0, 0));
    light2->setParent(lightStick);
    
    light3 = spSmngr->createLight(LIGHT_SPOT);
    light3->setPosition(vector3df(150, 0, 50));
    light3->setRotation(vector3df(-90, 0, 0));
    
    lense = spSmngr->createSprite(spDriver->loadTexture("data/Sprite.bmp"));
    lense->setLighting(false);
    lense->setScale(4);
    lense->setVisible(false);
    
    spControl->hideCursor();
    
    player->setPosition(vector3df(195, 90, 30));
    
    /* ======= Loading ======= */
    
    // Reset arrays
    
    memset(tex, 0, sizeof(tex));
    memset(snd, 0, sizeof(snd));
    
    // Loading textures
    
    tex[0] = spDriver->loadTexture("data/Ladder1.PNG");
    for (s32 i = 1; i <= 4; ++i)
        tex[i] = spDriver->loadTexture(io::stringc("data/Stone")+io::stringc(i)+io::stringc(".JPG"));
    
    tex[5] = spDriver->loadTexture("data/Crate.PNG");
    tex[6] = spDriver->loadTexture("data/DustCloud.JPG");
    tex[7] = spDriver->loadTexture("data/SwitchOn.JPG");
    tex[8] = spDriver->loadTexture("data/SwitchOff.JPG");
    
    // Loading sounds
    
    for (s32 i = 0; i <= 3; ++i)
    {
        snd[i] = spListener->loadSound(stringc("data/Walk")+stringc(i+1)+stringc(".wav"));
        spListener->setSoundVolume(snd[i], 0.2f);
    }
    for (s32 i = 0; i <= 3; ++i)
    {
        snd[i+4] = spListener->loadSound(stringc("data/Ladder")+stringc(i+1)+stringc(".wav"));
        spListener->setSoundVolume(snd[i+4], 0.25f);
    }
    
    snd[8] = spListener->loadSound("data/translate.wav");
    spListener->setSoundVolume(snd[8], 0.5f);
    
    snd[9] = spListener->loadSound("data/nature.wav");
    spListener->setSoundVolume(snd[9], 0.25f);
    spListener->playSound(snd[9], true);
    
    snd[10] = spListener->loadSound("data/switcher.wav");
    
    // Loading a shader
    
    #ifdef USE_SHADER
    
    shFlashLight = spDriver->loadShader("data/LightShaderVert.txt", "data/LightShaderFrag.txt");
    shFlashLight->setConstant("Texture1", 1);
    
    LightDepthTex = spDriver->createTexture(1024, video::TEXTURE_DEPTH);
    LightTex = spDriver->loadTexture("data/FlashLight.JPG");
    
    #endif
    
    // Loading 3d models
    
    world = spSmngr->loadModel("data/World.spm");
    world->setScale(50.0f);
    world->updateNormals();
    
    world->selectSurface(0); world->addTexture(tex[4]);
    world->selectSurface(1); world->addTexture(tex[3]);
    world->selectSurface(2); world->addTexture(tex[2]);
    world->selectSurface(3); world->addTexture(tex[1]);
    world->selectSurface(4); world->addTexture(tex[1]); world->addTexture(tex[0]);
    
    // Ladder
    
    ladder = spSmngr->createPivot();
    ladder->createSurface();
    {
        ladder->addVertex(vector3df(-1,  2, 0), point2df(0.0f, 0.0f));
        ladder->addVertex(vector3df( 1,  2, 0), point2df(1.0f, 0.0f));
        ladder->addVertex(vector3df( 1, -2, 0), point2df(1.0f, 2.0f));
        ladder->addVertex(vector3df(-1, -2, 0), point2df(0.0f, 2.0f));
        ladder->addTriangle(0, 1, 2);
        ladder->addTriangle(0, 2, 3);
    }
    ladder->updateModeling();
    ladder->setScale(5.0f);
    ladder->setPosition(vector3df(25, 10, 50-0.01f));
    
    /* ======= Collisions ======= */
    
    std::vector<u32> DestIDList;
    std::vector<s32> CollTypeList;
    CollTypeList.push_back(COLLISION_SPHERE_TO_POLYGON);
    CollTypeList.push_back(COLLISION_SPHERE_TO_POLYGON);
    CollTypeList.push_back(COLLISION_SPHERE_TO_POLYGON);
    CollTypeList.push_back(COLLISION_SPHERE_TO_POLYGON);
    
    Collision* collLadder = spCmngr->createCollision(1, CollTypeList, DestIDList);
    Collision* collWorld = spCmngr->createCollision(2, CollTypeList, DestIDList);
    
    DestIDList.push_back(2);
    
    collPhysics = spCmngr->createCollision(3, CollTypeList, DestIDList);
    
    DestIDList.push_back(1);
    
    Collision* collPlayer = spCmngr->createCollision(0, CollTypeList, DestIDList);
    
    spCmngr->addCollisionEntity(ladder, collLadder);
    spCmngr->addCollisionEntity(world, collWorld);
    spCmngr->addCollisionEntity(player, collPlayer, 2.5f);
    
    for (s32 z = 0; z < 1; ++z)
        for (s32 x = 0; x < 1; ++x)
            PhysicObjectList.push_back(new PhysicObject(vector3df(107 + x*7, 30, 7 + z*7)));
    
    ladder->clearTriangleList(0);
    
    /* ======= Characters ======= */
    
    Texture* faerieTex = spDriver->loadTexture("data/faerie.bmp");
    Texture* dragonTex = spDriver->loadTexture("data/dragon.bmp");
    
    Entity* faerie = spSmngr->loadModel("data/faerie.md2");
    faerie->addTexture(faerieTex);
    faerie->setScale(0.2f);
    faerie->setPosition(vector3df(195, 85, 10));
    faerie->setRotation(vector3df(0, -180, 0));
    faerie->setNormalsType(NORMALS_SMOOTH);
    faerie->updateNormals();
    
    Entity* dragon = spSmngr->loadModel("data/dragon.md2");
    dragon->addTexture(dragonTex);
    dragon->setScale(0.2f);
    dragon->setPosition(vector3df(170, 5, 50));
    dragon->setRotation(vector3df(0, 90, 0));
    dragon->setNormalsType(NORMALS_SMOOTH);
    dragon->updateNormals();
    
    Animation* faerieAnim = spAnim8or->getAnimation(faerie);
    Animation* dragonAnim = spAnim8or->getAnimation(dragon);
    
    faerieAnim->doAnimate(ANIM_LOOP);
    dragonAnim->doAnimate(ANIM_LOOP);
    
    /* ======= Special objects ====== */
    
    // Effect model
    
    object = spSmngr->createModel(ENTITY_TORUS);
    object->setNormalsType(NORMALS_SMOOTH);
    object->updateNormals();
    object->setPosition(vector3df(150, 75, 50));
    object->setScale(10.0f);
    object->setVisible(false);
    Shadow1 = spEffects->createCelShadow(8, color(255, 255, 0), color(255, 0, 0));
    spEffects->registerCamera(cam);
    
    // Truck
    
    Entity* truck = spSmngr->loadModel("data/watertruck.md2");
    truck->addTexture(spDriver->loadTexture("data/watertruck.bmp"));
    truck->doMeshTransform(0.2);
    truck->setPosition(vector3df(150, 0, 80));
    truck->setRotation(vector3df(0, -165, 0));
    truck->setNormalsType(NORMALS_SMOOTH);
    truck->updateNormals();
    
    spCmngr->addCollisionEntity(truck, collWorld);
    
    // Excavator
    
    Texture* excavatorTex = spDriver->loadTexture("data/excavator.bmp");
    
    Entity* excavatorBody = spSmngr->loadModel("data/excavator_b.md2");
    excavatorBody->addTexture(excavatorTex);
    excavatorBody->setNormalsType(NORMALS_SMOOTH);
    excavatorBody->updateNormals();
    
    Entity* excavatorBox = spSmngr->loadModel("data/excavator_t.md2");
    excavatorBox->addTexture(excavatorTex);
    excavatorBox->setNormalsType(NORMALS_SMOOTH);
    excavatorBox->updateNormals();
    
    std::vector<Entity*> excavatorList;
    excavatorList.push_back(excavatorBody);
    excavatorList.push_back(excavatorBox);
    
    Entity* excavator = spSmngr->createModelList(excavatorList);
    excavator->setScale(0.175);
    excavator->setPosition(vector3df(150, 0, 25));
    excavator->setRotation(vector3df(0, 70, 0));
    
    spCmngr->addCollisionEntity(excavator, collWorld);
    
    // Switcher
    
    switcher = spSmngr->createModel(ENTITY_CUBE);
    switcher->setScale(dim::vector3df(1.5f, 2.0f, 1.5f));
    switcher->setPosition(dim::vector3df(110, 7.5, 100.5));
    switcher->addTexture(tex[8]);
    
    spCmngr->addPickingEntity(switcher, scene::PICKMODE_POLYGON);
    
    // Set the shader
    
    #ifdef USE_SHADER
    
    /*world->setShader(shFlashLight);
    excavator->setShader(shFlashLight);
    truck->setShader(shFlashLight);
    switcher->setShader(shFlashLight);
    faerie->setShader(shFlashLight);
    dragon->setShader(shFlashLight);
    lightStick->setShader(shFlashLight);*/
    
    #endif
    
    /* ======= Optimization ======= */
    
    world->createOctTree(4);
    excavator->createOctTree(2);
    truck->createOctTree(2);
    
    /* ======= Rest creations ======= */
    
    snd[9] = spListener->loadSound("data/engine.wav");
    spListener->playSound(snd[9], true, true);
    snd[9]->setPosition(truck->getPosition());
    
    spSclmngr->addEntityCastCloud(switcher);
    spSclmngr->addEntityCastCloud(truck);
    spSclmngr->addLightSource(new scene::SShadowLightSource(light2));
    
    /* Fog effect, check something out =) */
    /*spSmngr->setFogMode(FOG_STATIC);
    spSmngr->setFogRange(0.075f);
    spSmngr->setFogColor(color(128));*/
    
}

void UpdateScene()
{
    
    dim::vector3df FirstPosition;
    
    do
    {
        FirstPosition = player->getPosition();
        spCmngr->updateScene();
    }
    while (player->getCollisionCount() >= 2 && FirstPosition != player->getPosition());
    
}

void RenderEffects()
{
    
    // Special spEffects
    
    object->doTurn(vector3df(1));
    
    spEffects->renderCelShading(object, Shadow1, light1);
    
}

void UpdateGame()
{
    
    static bool isWalking, isJumping;
    static u32 WalkTime;
    static f32 Falling, Jumping;
    static f32 LightStickDegree;
    
    // Movement
    
    if (spControl->isKeyPressed(KEY_W))
    {
        player->doMove(vector3df(0, 0, PLAYER_SPEED));
        isWalking = true;
    }
    if (spControl->isKeyPressed(KEY_S))
    {
        player->doMove(vector3df(0, 0, -PLAYER_SPEED));
        isWalking = true;
    }
    if (spControl->isKeyPressed(KEY_D))
    {
        player->doMove(vector3df(PLAYER_SPEED, 0, 0));
        isWalking = true;
    }
    if (spControl->isKeyPressed(KEY_A))
    {
        player->doMove(vector3df(-PLAYER_SPEED, 0, 0));
        isWalking = true;
    }
    
    if (!isJumping && spControl->isKeyHit(KEY_SPACE))
    {
        spListener->playSound( snd[ WalkSound*4+Random(3) ] );
        Jumping = JUMP_HEIGHT;
        isJumping = true;
    }
    
    // Look around
    
    MouseSpeed = spControl->getCursorSpeed();
    PlayerRotation.X += (f32)MouseSpeed.Y * MOUSE_SPEED;
    PlayerRotation.Y += (f32)MouseSpeed.X * MOUSE_SPEED;
    
    if (PlayerRotation.X > 90)
        PlayerRotation.X = 90;
    else if (PlayerRotation.X < -90)
        PlayerRotation.X = -90;
    
    player->setRotation(vector3df(0, PlayerRotation.Y, 0));
    cam->setRotation(vector3df(PlayerRotation.X, 0, 0));
    
    spControl->setCursorPosition(point2di(ScrWidth/2, ScrHeight/2));
    
    // Picking
    
    PickingList = spCmngr->pickLineIntersection(
        cam->getPickingLine( dim::point2di(ScrWidth/2, ScrHeight/2) )
    );
    
    if (PickingList.size() && spControl->isKeyHit(KEY_E))
    {
        spListener->playSound(snd[10]);
        isRealTimeShadow = !isRealTimeShadow;
        spSmngr->setStencilEffects(isRealTimeShadow);
        
        // Check the switch state
        if (isRealTimeShadow)
        {
            lamp->setLighting(false);
            lamp->setColor(video::color(255, 255, 0));
            switcher->setTexture(0, tex[7]);
            lense->setVisible(true);
        }
        else
        {
            lamp->setLighting(true);
            lamp->setColor(255);
            switcher->setTexture(0, tex[8]);
            lense->setVisible(false);
        }
    }
    
    // Flashlight
    
    #ifdef USE_SHADER
    
    spDriver->createScreenShot(LightDepthTex);
    
    spDriver->beginDrawing2D();
    {
        //shFlashLight->bind();
        
        //LightTex->bind(0);
        LightDepthTex->bind(1);
        
        spDriver->draw2DImage(LightDepthTex, dim::rect2di(0, 0, ScrWidth, ScrHeight));
        
        LightTex->unbind(0);
        LightDepthTex->unbind(1);
        
        //shFlashLight->unbind();
    }
    spDriver->endDrawing2D();
    
    if (spControl->isKeyHit(KEY_F))
    {
        static s32 isFlashLight;
        isFlashLight = 1 - isFlashLight;
        
        shFlashLight->setConstant("isFlashLight", (f32)isFlashLight);
        
        spListener->playSound(snd[10]);
    }
    
    #endif
    
    // Jumping & gravity
    
    if (isJumping)
    {
        
        // Jumping
        
        if (player->detectCollision(2) && Jumping < JUMP_HEIGHT)
            isJumping = false;
        
        Jumping += GRAVITY;
        player->doTranslate(vector3df(0, Jumping, 0));
        
        if (player->getPosition().Y < 3)
            Jumping = GRAVITY;
        
    }
    else
    {
        
        // Walking
        
        WalkSound = 0;
        if (isWalking && player->detectCollision(1)) // Ladder
        {
            player->doTranslate(vector3df(0, LADDER_MOVEUP, 0));
            WalkSound = 1;
        }
        else if (!player->detectCollision(2) || player->getCollisionNormal().Y < 0.5f) // Falling
        {
            Falling += GRAVITY;
            player->doTranslate(vector3df(0, Falling, 0));
        }
        else
            Falling = GRAVITY;
        
        if (isWalking)
        {
            if (spDevice->getMilliseconds() > WalkTime+WALK_TIME)
            {
                WalkTime = spDevice->getMilliseconds();
                if (player->detectCollision(2))
                    spListener->playSound( snd[ WalkSound*4+Random(3) ] );
                isWalking = false;
            }
        }
        
    }
    
    // Other calculations
    
    lense->setPosition(light2->getPosition(true));
    lamp->setPosition(light2->getPosition(true));
    
    if ( ( LightStickDegree += 2 ) > 360 )
        LightStickDegree -= 360;
    lightStick->setRotation(vector3df(SIN(LightStickDegree)*10, 0, COS(LightStickDegree)*8));
    
    for (std::vector<PhysicObject*>::iterator it = PhysicObjectList.begin(), itDest; it != PhysicObjectList.end(); ++it)
    {
        // Check the player position
        (*it)->Update(player->getPosition(), true);
        // Check the objects position
        /*for (itDest = PhysicObjectList.begin(); itDest != PhysicObjectList.end(); ++itDest)
        {
            if (it != itDest)
                (*it)->Update((*itDest)->getPosition());
        }*/
    }
    
    if (math::Rnd(10) == 1)
        DustCloudList.push_back(new DustCloud(dim::vector3df(134, 13, 21)));
    
    for (std::vector<DustCloud*>::iterator it = DustCloudList.begin(); it != DustCloudList.end();)
    {
        if ((*it)->Update())
        {
            spSmngr->deleteNode((*it)->getObject());
            spDeletePointer(*it);
            it = DustCloudList.erase(it);
        }
        else
            ++it;
    }
    
    // For debuging
    #if 0
    spDevice->setWindowTitle(
        str("X = ")+str(player->getPosition().X)+str(", Y = ")+str(player->getPosition().Y)+str(", Z = ")+str(player->getPosition().Z)
    );
    #else
    //spDevice->setWindowTitle(spDevice->getFPS());
    #endif
    
    // Update the sounds (3d sounds are automaticaly updated)
    spListener->updateSounds();
    
}


/*
 * ======= PhysicObject class =======
 */

PhysicObject::PhysicObject(vector3df Pos) : hObj_(0)
{
    std::vector<u32> DestIDList;
    
    hObj_ = spSmngr->createModel(ENTITY_CUBE);
    //hObj_->setShader(shFlashLight);
    
    spCmngr->addCollisionEntity(hObj_, collPhysics, 2.5f);
    
    hObj_->addTexture(tex[5]);
    hObj_->setScale(5);
    hObj_->setPosition(Pos);
    
    Falling_ = 0.0f;
}
PhysicObject::~PhysicObject()
{
}

void PhysicObject::Update(vector3df PlayerPos, bool isOnlyPlayer)
{
    if (isOnlyPlayer)
    {
        // Falling
        if (!hObj_->detectCollision(2))
        {
            Falling_ += GRAVITY;
            hObj_->doTranslate(vector3df(0, Falling_, 0));
        }
        else if (Falling_ < -0.5)
        {
            Falling_ = -Falling_ / 3;
            hObj_->doTranslate(vector3df(0, Falling_, 0));
        }
        else
            Falling_ = 0.0f;
    }
    
    // Movement
    if (math::getDistance(PlayerPos, hObj_->getPosition()) <= 5)
    {
        // Translate the object
        hObj_->doTranslate(vector3df(SIN(PlayerRotation.Y)*PLAYER_SPEED, 0, COS(PlayerRotation.Y)*PLAYER_SPEED));
        // Play a sound
        if (spListener->isSoundFinished(snd[8]) || spListener->getSoundSeek(snd[8]) == 0.0f)
            spListener->playSound(snd[8]);
    }
}


/*
 * ======= DustCloud class =======
 */

DustCloud::DustCloud(vector3df Pos) : hObj_(0)
{
    Alpha_  = 255;
    hObj_   = spSmngr->createSprite(tex[6]);
    hObj_->setPosition(Pos);
    hObj_->setLighting(false);
    Direction_.set((f32)(Random(-2, 2)) / 100, 0.25f, (f32)(Random(-2, 2)) / 100);
}
DustCloud::~DustCloud()
{
}

bool DustCloud::Update()
{
    hObj_->setColor(color(255, 255, 255, Alpha_));
    hObj_->doTranslate(Direction_);
    hObj_->doTransform(0.025f);
    return (Alpha_ -= 2) <= 2;
}










// ======================================================
