//
// SoftPixel Engine - Tutorial13 (MiniMatrix) - (29.1.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp;

using namespace io;
using namespace dim;
using namespace math;
using namespace video;
using namespace scene;
using namespace sound;
using namespace anim;
using namespace scene::effect;

// Definations

#define CAM_HEIGHT      6.0f
#define PLAYER_SPEED    0.5f
#define WALK_TIME       350
#define MOUSE_SPEED     0.25f
#define LADDER_MOVEUP   0.5f
#define JUMP_HEIGHT     0.75f
#define GRAVITY         -0.05f

#define MAX_TEXTURES    10
#define MAX_SOUNDS      15

/* Try this and hit the 'F' key to switch the flash-light on/off ;-) */
//#define USE_SHADER


// Classes declarations

class PhysicObject
{
    
    public:
        
        PhysicObject(vector3df Pos);
        ~PhysicObject();
        
        void Update(vector3df PlayerPos, bool isOnlyPlayer = false);
        
        inline vector3df getPosition()
        {
            return hObj_->getPosition();
        }
        
    private:
        
        // Members
        
        Entity* hObj_;
        
        vector3df Movement_;
        f32 Falling_;
        
};

extern std::vector<PhysicObject*> PhysicObjectList;

extern Collision* collPhysics;

class DustCloud
{
    
    public:
        
        DustCloud(vector3df Pos);
        ~DustCloud();
        
        bool Update();
        
        inline Sprite* getObject()
        {
            return hObj_;
        }
        
    private:
        
        Sprite* hObj_;
        vector3df Direction_;
        u8 Alpha_;
        
};

extern std::vector<DustCloud*> DustCloudList;

// Global variable declarations

extern SoftPixelDevice* spDevice;
extern SensitiveSoundDevice* spListener;

extern VideoDriver* spDriver;
extern SceneManager* spSmngr;
extern CollisionDetector* spCmngr;
extern UserControl* spControl;

extern Animator* spAnim8or;
extern SpecialEffectsActor* spEffects;
extern StencilManager* spSclmngr;

// Some objects declarations

extern Entity* player;
extern Camera* cam;

extern Entity* world;
extern Entity* ladder;
extern Entity* object;

extern Entity* lightStick;
extern Light* light1;
extern Light* light2;
extern Light* light3;
extern Sprite* lense;
extern Entity* switcher;
extern Entity* lamp;

extern Texture* tex[MAX_TEXTURES];
extern Sound* snd[MAX_SOUNDS];

extern CelShadow* Shadow1;

#ifdef USE_SHADER

extern Shader* shFlashLight;
extern video::Texture* LightDepthTex;
extern video::Texture* LightTex;

#endif

// Something else declarations

extern const s32 ScrWidth, ScrHeight;
extern const bool isFullScreen;

extern point2di MouseSpeed;
extern vector3df PlayerRotation;
extern s32 WalkSound;

extern std::list<scene::SPickingObject> PickingList;
extern bool isRealTimeShadow;

// Function declaration

void InitGame();
void LoadGame();
void UpdateScene();
void RenderEffects();
void UpdateGame();





// ======================================================
