//
// SoftPixel Engine - Tutorial10 (Win32App) - (3.2.2008)
//

#include <windows.h>

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

char ClassName[] = "Win32AppTutorialClassName";

HWND hMainWnd, hWndScreen, hButton;

#define BTN_OK  1

//The windows callback function like in a normal win32 application

LRESULT CALLBACK WindowCallback(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
    
    switch (Message)
    {
        case WM_COMMAND:
            switch (LOWORD(wParam))
            {
                case BTN_OK:
                    MessageBox(hWnd, TEXT("You pressed the OK button!"), TEXT("Information"), MB_OK | MB_ICONINFORMATION);
                    break;
            }
            return 0;
            
        case WM_DESTROY:
            PostQuitMessage(0);
            exit(0);
            return 0;
    }
    
    //if (hWnd == hWndScreen)
    //    return spWindowCallback(hWnd, Message, wParam, lParam);
    
    return DefWindowProc(hWnd, Message, wParam, lParam);
}

int main()
{
    
    video::EDriverTypes Driver = ChooseRenderer();
    
    HINSTANCE hInstance = 0;
    
    const s32 ScrWidth = 400, ScrHeight = 350;
    
    // At first we must create an own win32 class
    
    WNDCLASS wincl;
    wincl.style         = CS_HREDRAW | CS_VREDRAW;
    wincl.lpfnWndProc   = (WNDPROC)WindowCallback;
    wincl.hInstance     = hInstance;
    wincl.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wincl.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wincl.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
    wincl.lpszMenuName  = NULL;
    wincl.lpszClassName = ClassName;
    wincl.cbClsExtra    = 0;
    wincl.cbWndExtra    = 0;
    
    if (!RegisterClass(&wincl))
    {
        MessageBox(NULL, TEXT("Window class could not registrated!"), TEXT("Fatal error"), MB_OK | MB_ICONERROR);
        return 0;
    }
    
    // This is the main window
    hMainWnd = CreateWindow(
        ClassName,
        "Tutorial10 (Win32App)",
        WS_OVERLAPPEDWINDOW | WS_VISIBLE,
        100, 100,
        500, 500,
        HWND_DESKTOP,
        NULL,
        hInstance,
        NULL
    );
    
    // This is the window for the device-context (render-context)
    hWndScreen = CreateWindow(
        ClassName, // ! very important that the screen-window has the same class such as the main window !
        "Screen button",
        WS_CHILD | WS_VISIBLE | BS_OWNERDRAW,
        50, 50,
        ScrWidth, ScrHeight,
        hMainWnd,
        NULL,
        hInstance,
        NULL
    );
    
    // This is a normal button window
    hButton = CreateWindow(
        "BUTTON",
        "OK",
        WS_CHILD | WS_VISIBLE,
        200, 15,
        100, 25,
        hMainWnd,
        (HMENU)BTN_OK,
        hInstance,
        NULL
    );
    
    SetForegroundWindow(hMainWnd);
    
    // The normal 'SoftPixel Engine' code
    
    // In this tutorial we must call the 'createDeviceEx' (Extendet) function to set the window-render-context
    SoftPixelDevice* spDevice = createDeviceWnd(
        Driver, hWndScreen, dim::size2di(ScrWidth, ScrHeight), 32, "Screen title"
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver();
    scene::SceneManager* spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    rayEnable(RAY_TEXTURE_SMOOTH);
    
    video::Texture* tex = spDriver->loadTexture("../../../media/HLC.PNG");
    
    scene::Camera* cam = spSmngr->createCamera();
    
    scene::Light* light = spSmngr->createLight();
    spSmngr->setLighting();
    
    scene::Entity* cube = spSmngr->createModel(scene::ENTITY_CUBE);
    cube->setPosition(dim::vector3df(0, 0, 2));
    cube->setNormalsType(scene::NORMALS_SIMPLE);
    cube->updateModeling();
    cube->addTexture(tex);
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers(); // Clears the drawing buffer
        
        spSmngr->renderScene(); // Renders the world
        
        cube->doTurn(dim::vector3df(0, 1, 0));
        
        spDriver->flipBuffers(); // The buffer will swap
    }
    
    deleteDevice(); // The device context will delete
    
    return 0;
}



// ======================================================
