//
// SoftPixel Engine - Tutorial15(LowLevel) - (3.4.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(640, 480), 32, "Tutorial15 (LowLevel) - Normal rotation"
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver();
    scene::SceneManager* spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    // We do not need a camera for low level programing because
    // we are not calling the 'renderScene' function!
    
    scene::Entity* obj = spSmngr->createModel(scene::ENTITY_CUBE);
    scene::Light* light = spSmngr->createLight();
    spSmngr->setLighting();
    
    scene::Camera* cam = spSmngr->createCamera();
    
    spDriver->setClearColor(video::color(64));
    
    f32 Angle = 0.0f;
    bool NormalRot = true;
    
    obj->setAutoMatrix(false);
    
    dim::matrix4f Matrix;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        if (spControl->isKeyReleased(KEY_RETURN))
        {
            NormalRot = !NormalRot;
            if (NormalRot)
                spDevice->setWindowTitle("Tutorial15 (LowLevel) - Normal rotation");
            else
                spDevice->setWindowTitle("Tutorial15 (LowLevel) - Extendet rotation");
        }
        
        // Now low-level: translate, rotate and scale our object manual with the
        // SceneManager-static functions or the original OpenGL functions:
        
        // Update the viewport, model- and projection matrices
        cam->updateViewmode();
        cam->updatePerspective();
        
        // Enables the VBOs (VertexBufferObjects) to render 3d objects
        spDriver->enableVertexBuffer();
        
        // Loads the identity of the current matrix
        Matrix.matrixReset();
        
        // Translates the current matrix
        Matrix.matrixTranslate(dim::vector3df(0, 0, 2));
        
        // Rotates the current matrix by all axis
        ++Angle;
        if (NormalRot)
        {
            // Normal 'SoftPixel Engine' rotation (YXZ), a camera rotation is (ZXY)
            #if 0 // "Check this out ;-)"
            Matrix.matrixScale(dim::vector3df(1, 2, 0.5f));
            Matrix.matrixRotate(Angle, dim::vector3df(0.0f, 1.0f, 0.0f));
            Matrix.matrixScale(dim::vector3df(1.25f, 0.75f, 1.0f));
            Matrix.matrixRotate(Angle, dim::vector3df(1.0f, 0.0f, 0.0f));
            Matrix.matrixScale(dim::vector3df(2, 0.5f, 1));
            Matrix.matrixRotate(Angle, dim::vector3df(0.0f, 0.0f, 1.0f));
            Matrix.matrixScale(dim::vector3df(1.0f, 1.25f, 0.75f));
            #else
            Matrix.matrixRotateYXZ(dim::vector3df(Angle));
            #endif
        }
        else
        {
            // Low-level rotations are very useful for rotation with more detail-posibility
            Matrix.matrixRotate(Angle, dim::vector3df(1.0f, 1.0f, 1.0f));
        }
        
        obj->setMatrix(Matrix);
        
        // Renders the object normal. Includes also object position, rotation and scale but not camera location
        obj->render();
        
        // Disables the VBOs
        spDriver->disableVertexBuffer();
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}

// ======================================================
