//
// SoftPixel Engine - Tutorial18 (Shader) - (6.2.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp;

#include "../common.hpp"

scene::Camera* cam = 0;
scene::Entity* mesh = 0;

UserControl* spControl          = 0;
video::VideoDriver* spDriver    = 0;
scene::SceneManager* spSmngr    = 0;

void MyShaderCallback(video::Shader* shd)
{
    dim::matrix4f Mat;
    
    /*Mat = cam->getPerspectiveMatrix();
    Mat.matrixTranslate(dim::vector3df(0, 0, 50));
    Mat.matrixRotateYXZ(mesh->getRotation());*/
    
    Mat = spDriver->getProjectionMatrix();
    Mat *= spDriver->getViewMatrix();
    Mat *= spDriver->getWorldMatrix();
    
    shd->setConstant("WorldViewProj", Mat);
}

int main()
{
    
    // !!! WARNING: Direct3D9 (HLSL) Shader does not work, it causes big graphics driver problems !!!
    
    SoftPixelDevice* spDevice = createDevice(
        /*ChooseRenderer()*/video::DRIVER_OPENGL, dim::size2di(640, 480), 32, "Tutorial18 (Shader)"
    );
    
    spControl   = spDevice->getUserControl();
    spDriver    = spDevice->getVideoDriver();
    spSmngr     = spDevice->getSceneManager();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    cam = spSmngr->createCamera();
    
    mesh = spSmngr->loadModel("data/character.ms3d");
    
    mesh->setPosition(dim::vector3df(0, 0, 50));
    
    video::Shader* fxShader = 0;
    
    if (spDriver->getDriverType() == video::DRIVER_OPENGL)
    {
        fxShader = spDriver->loadShader(
            "data/ShaderOpenGL.vert", "data/ShaderOpenGL.frag"
        );
        
        mesh->setShader(fxShader);
    }
    else if (spDriver->getDriverType() == video::DRIVER_DIRECT3D9)
    {
        fxShader = spDriver->loadShader(
            "data/ShaderDirect3D9.hlsl", "data/ShaderDirect3D9.hlsl",
            "MainVertex", "MainPixel",
            video::HLSL_VERTEX_3_0, video::HLSL_PIXEL_3_0
        );
        
        fxShader->setShaderCallback(MyShaderCallback);
        
        mesh->setShader(fxShader);
    }
    
    spDriver->setClearColor(video::color(255));
    
    s32 Demo = 0;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        spSmngr->renderScene();
        
        mesh->doTurn(1);
        
        // Change the demonstration type
        if (spControl->isKeyHit(KEY_RETURN) && fxShader)
        {
            Demo = 1 - Demo;
            // Change the uniform variable in the shader
            fxShader->setConstant("Demo", Demo);
        }
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}

// ======================================================
