//
// SoftPixel Engine - Tutorial2 (Primitives) - (24.1.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    
    // The softpixel-device will create the screen (windowed screen / fullscreen are possible)
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(640, 480), 32, "Tutorial2 (Primitives)"
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver(); // Main manager for drawing operations
    scene::SceneManager* spSmngr = spDevice->getSceneManager(); // Now we can render 3d worlds
    UserControl* spControl = spDevice->getUserControl(); // Now we can check the keyboard- and the mouse events
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    scene::Camera* cam = spSmngr->createCamera(); // Without a camera we would see nothing
    cam->setPosition(dim::vector3df(0, 0, -6));
    
    // (With OpenGL you can maximal create 'GL_MAX_LIGHTS' = 3377 lights! 'That should reach ;-)')
    #if 1
    scene::Light* light1 = spSmngr->createLight(scene::LIGHT_POINT); // Creates a light (default: Diffuse light)
    light1->setPosition(dim::vector3df(-10, 0, 0));
    light1->setLightingColor(video::color(255, 255, 0), video::color(222), video::color(255)); // Yellow light color
    
    scene::Light* light2 = spSmngr->createLight(scene::LIGHT_POINT); // Creates a light (default: Diffuse light)
    light2->setPosition(dim::vector3df(10, 0, 0));
    light2->setLightingColor(video::color(0, 0, 255), video::color(222), video::color(128)); // Blue light color
    #else
    scene::Light* light = spSmngr->createLight();
    light->setLightingColor(255, 32, 255);
    #endif
    
    spSmngr->setLighting(); // Activates the lighting engine
    
    scene::Entity* obj[13];
    
    // Creates all the primitives (cube, cone, cylinger, sphere, torus, all polyedrons and the teapot)
    obj[0] = spSmngr->createModel(scene::ENTITY_CUBE); obj[0]->setPosition(dim::vector3df(-4, 2, 0));
    obj[1] = spSmngr->createModel(scene::ENTITY_CONE, 18); obj[1]->setPosition(dim::vector3df(-2, 2, 0));
    obj[2] = spSmngr->createModel(scene::ENTITY_CYLINDER, 18); obj[2]->setPosition(dim::vector3df(0, 2, 0));
    obj[3] = spSmngr->createModel(scene::ENTITY_SPHERE); obj[3]->setPosition(dim::vector3df(2, 2, 0));
    obj[4] = spSmngr->createModel(scene::ENTITY_TORUS); obj[4]->setPosition(dim::vector3df(4, 2, 0));
    obj[5] = spSmngr->createModel(scene::ENTITY_TETRAHEDRON); obj[5]->setPosition(dim::vector3df(-4, 0, 0));
    obj[6] = spSmngr->createModel(scene::ENTITY_CUBOCTAHEDRON); obj[6]->setPosition(dim::vector3df(-2, 0, 0));
    obj[7] = spSmngr->createModel(scene::ENTITY_DODECAHEDRON); obj[7]->setPosition(dim::vector3df(0, 0, 0));
    obj[8] = spSmngr->createModel(scene::ENTITY_OCTAHEDRON); obj[8]->setPosition(dim::vector3df(2, 0, 0));
    obj[9] = spSmngr->createModel(scene::ENTITY_ICOSAHEDRON); obj[9]->setPosition(dim::vector3df(4, 0, 0));
    obj[10] = spSmngr->createModel(scene::ENTITY_SPIRAL); obj[10]->setPosition(dim::vector3df(-4, -2, 0));
    obj[11] = spSmngr->createModel(scene::ENTITY_PIPE, 18); obj[11]->setPosition(dim::vector3df(-2, -2, 0));
    obj[12] = spSmngr->createModel(scene::ENTITY_TEAPOT); obj[12]->setPosition(dim::vector3df(0, -2, 0));
    
    for (s32 i = 0; i <= 12; ++i)
    {
        obj[i]->setNormalsType(scene::NORMALS_SMOOTH);
        obj[i]->updateModeling();
        obj[i]->setAutoMatrix(false);
    }
    
    u8 NormalsType = 1;
    
    obj[0]->setColor(video::color(255, 255, 0));
    obj[4]->setColor(video::color(0, 255, 0));
    obj[9]->setColor(video::color(255, 0, 0));
    obj[10]->setColor(video::color(0, 255, 255));
    obj[12]->setColor(video::color(255, 128, 64));
    
    f32 deg = 0.0f;
    
    // The main loop will update our device
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers(); // Clears the drawing buffer
        
        spSmngr->renderScene(); // Renders the world
        
        if (spControl->isKeyReleased(KEY_TAB))
        {
            NormalsType = 1 - NormalsType;
            for (s32 i = 0; i <= 12; ++i)
            {
                obj[i]->setNormalsType(NormalsType);
                obj[i]->updateModeling();
            }
        }
        
        dim::matrix4f Mat;
        deg += 2.0f;
        
        for (u32 i = 0; i <= 12; ++i)
        {
            Mat.matrixReset();
            Mat.matrixTranslate(obj[i]->getPosition());
            Mat.matrixRotate(-spControl->getCursorPosition().Y*2, dim::vector3df(1.0f));
            
            obj[i]->setMatrix(Mat);
        //    obj[i]->doTurn(dim::vector3df(1, 1, 1));
        }
        
        spDriver->flipBuffers(); // The buffer will swap
    }
    
    deleteDevice(); // The device context will delete
    
    return 0;
}



// ======================================================
