//
// SoftPixel Engine - Tutorial3(Sprites) - (2.2.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

// In this tutorial we must create the main variables out of the main function, so we can use they global
SoftPixelDevice* spDevice       = 0;
video::VideoDriver* spDriver    = 0;
scene::SceneManager* spSmngr    = 0;

const f32 Gravity = 0.0075f;

class SpriteEffect
{
    
    public:
        
        SpriteEffect(video::Texture* hTexture)
        {
            hSprite_    = spSmngr->createSprite(hTexture);
            DegreeYaw_  = f32(math::Random(3600))/10; // To make a float-random value we have to cast and divide it
            Height_     = f32(math::Random(75, 250))/1000;
            Speed_      = f32(math::Random(50, 150))/1000;
            Alpha_      = 255;
        }
        ~SpriteEffect()
        {
        }
        
        bool Update()
        {
            Height_ -= Gravity;
            
            hSprite_->setColor(video::color(255, 255, 255, (u8)Alpha_));
            hSprite_->doTranslate(dim::vector3df(SIN(DegreeYaw_)*Speed_, Height_, COS(DegreeYaw_)*Speed_));
            
            Alpha_ -= 4;
            
            if (hSprite_->getPosition().Y < -10 || Alpha_ <= 0)
            {
                spSmngr->deleteNode(hSprite_);
                return true;
            }
            return false;
        }
        
    private:
        
        scene::Sprite* hSprite_;
        f32 DegreeYaw_, Height_, Speed_;
        s32 Alpha_;
        
};

int main()
{
    
    // The folowing lines should not surpriseful
    
    spDevice = createDevice(
        ChooseRenderer(), dim::size2di(640, 480), 32, "Tutorial3 (Sprites)"
    );
    
    spDriver = spDevice->getVideoDriver();
    spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    video::Texture* sprtex = spDriver->loadTexture("../../../media/DemoImage2.bmp");
    video::Texture* highmap = spDriver->loadTexture("../../../media/DemoImage3.jpg");
    video::Texture* termap = spDriver->loadTexture("../../../media/DemoImage4.jpg");
    
    scene::Camera* cam = spSmngr->createCamera();
    cam->setPosition(dim::vector3df(0, 0, -15));
    
    spDriver->setFogMode(video::FOG_STATIC);
    spDriver->setFogRange(0.025f);
    
    scene::Terrain* terrain = spSmngr->createTerrain(highmap, termap, 1);
    terrain->setScale(dim::vector3df(12, 7, 12));
    terrain->setPosition(dim::vector3df(-6, -5, 6));
    
    scene::Entity* ground = spSmngr->createModel(scene::ENTITY_PLANE, 100);
    ground->setScale(2000);
    ground->doTextureTransform(0, 250);
    ground->addTexture(termap);
    ground->setPosition(dim::vector3df(0, -4.9f, 0));
    
    //spDriver->setClearColor(video::color(0, 0, 64)); // Now we want an other background color: a dark blue
    
    std::vector<SpriteEffect> SpriteEffectList;
    
    s32 RenderMode = 0;
    static f32 Degree;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        spSmngr->renderScene();
        
        if (spControl->isKeyHit(KEY_TAB))
        {
            static u8 RenderMode;
            
            if (++RenderMode > 2)
                RenderMode = 0;
            
            spSmngr->setWireframe(
                RenderMode == 0 ? video::WIREFRAME_SOLID : RenderMode == 1 ? video::WIREFRAME_LINES : video::WIREFRAME_POINTS
            );
        }
        
        ++Degree;
        cam->setPosition(dim::vector3df(SIN(Degree)*8, 3+SIN(Degree/2)*3, COS(Degree)*8));
        cam->lookAt(dim::vector3df(0, 0, 0));
        
        // Adds a new sprite effect object
        SpriteEffectList.push_back(SpriteEffect(sprtex));
        
        // Loop through the sprite effect objects list
        for (std::vector<SpriteEffect>::iterator it = SpriteEffectList.begin(); it != SpriteEffectList.end();)
        {
            if (it->Update())
                it = SpriteEffectList.erase(it);
            else
                ++it;
        }
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}


// Sprite functions




// ======================================================
