//
// SoftPixel Engine - Tutorial4(MultiViewports) - (2.2.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    
    const s32 ScrWidth = 640, ScrHeight = 480;
    
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(ScrWidth, ScrHeight), 32, "Tutorial4 (MultiViewports)", false, false
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver();
    scene::SceneManager* spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    rayEnable(RAY_TEXTURE_SMOOTH);
    
    video::Texture* tex = spDriver->loadTexture("../../../media/HLC.PNG");
    
    // Create a light source
    scene::Light* light = spSmngr->createLight();
    light->setPosition(dim::vector3df(5, 3, -7));
    spSmngr->setLighting();
    
    // The four cameras
    scene::Camera* cam1 = spSmngr->createCamera();
    cam1->setPerspective(dim::rect2di(0, 0, ScrWidth/2, ScrHeight/2), 0.1f, 100.0f);
    cam1->setPosition(dim::vector3df(0, 0, -2));
    
    scene::Camera* cam2 = spSmngr->createCamera();
    cam2->setOrtho(true);
    cam2->setPerspective(dim::rect2di(ScrWidth/2, 0, ScrWidth/2, ScrHeight/2), 0.1f, 100.0f, 200.0f);
    cam2->setPosition(dim::vector3df(0, -2, 0));
    cam2->setRotation(dim::vector3df(-90, 0, 0));
    
    scene::Camera* cam3 = spSmngr->createCamera();
    cam3->setPerspective(dim::rect2di(ScrWidth/2, ScrHeight/2, ScrWidth/2, ScrHeight/2), 0.1f, 100.0f);
    cam3->setPosition(dim::vector3df(2, 0, 0));
    cam3->setRotation(dim::vector3df(0, -90, 0));
    
    scene::Camera* cam4 = spSmngr->createCamera();
    cam4->setPerspective(dim::rect2di(0, ScrHeight/2, ScrWidth/2, ScrHeight/2), 0.1f, 100.0f);
    cam4->setPosition(dim::vector3df(0, 2, 0));
    cam4->setRotation(dim::vector3df(90, 0, 0));
    
    // Create some objects
    scene::Entity* cube = spSmngr->createModel(scene::ENTITY_CUBE);
    cube->addTexture(tex);
    
    scene::Entity* sphere = spSmngr->createModel(scene::ENTITY_SPHERE);
    sphere->setParent(cube); // Sets the parent (more detailed in the next tutorial)
    sphere->setPosition(dim::vector3df(-1, 0, 0));
    sphere->setNormalsType(1);
    sphere->updateModeling();
    
    scene::Entity* cone = spSmngr->createModel(scene::ENTITY_CONE, 18);
    cone->setParent(cube);
    cone->setPosition(dim::vector3df(1, 0, 0));
    cone->setNormalsType(2);
    cone->updateModeling();
    
    const f32 Speed = 2.0f;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE)) {
        spDriver->clearBuffers();
        
        spDriver->beginDrawing2D(true);
        spDriver->draw2DRectangle(
            dim::rect2di(0, 0, ScrWidth, ScrHeight),
            video::color(255, 0, 0), video::color(0, 255, 0),
            video::color(0, 0, 255), video::color(255, 255, 0)
        );
        spDriver->endDrawing2D();
        
        spSmngr->renderScene();
        
        if (spControl->isKeyPressed(KEY_LEFT))
            cube->doTurn(dim::vector3df(0, -Speed, 0));
        if (spControl->isKeyPressed(KEY_RIGHT))
            cube->doTurn(dim::vector3df(0, Speed, 0));
        if (spControl->isKeyPressed(KEY_UP))
            cube->doTurn(dim::vector3df(Speed, 0, 0));
        if (spControl->isKeyPressed(KEY_DOWN))
            cube->doTurn(dim::vector3df(-Speed, 0, 0));
        if (spControl->isKeyPressed(KEY_PAGEUP))
            cube->doTurn(dim::vector3df(0, 0, -Speed));
        if (spControl->isKeyPressed(KEY_PAGEDOWN))
            cube->doTurn(dim::vector3df(0, 0, Speed));
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}


// Sprite functions




// ======================================================
