//
// SoftPixel Engine - Tutorial5(Parents) - (6.2.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    
    // The folowing lines should not surpriseful
    
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(800, 600), 32, "Tutorial5 (Parents)"
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver();
    scene::SceneManager* spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    scene::Camera* cam = spSmngr->createCamera();
    
    scene::Light* light = spSmngr->createLight();
    spSmngr->setLighting();
    
    scene::Entity* obj[3];
    
    // Try this or the other datas
    //obj[0] = spSmngr->createModel(scene::ENTITY_SPIRAL, 9, 0.5f, 0.25f, 360*3, 1.0f, true);
    obj[0] = spSmngr->createModel(scene::ENTITY_SPIRAL, 9, 0.5f, 0.1f, 360*10, 0.5f, true);
    
    obj[1] = spSmngr->createModel(scene::ENTITY_PIPE, 15);
    obj[1]->setParent(obj[0]); // Sets the parent entity (in this sample the previouse entity)
    obj[1]->setPosition(dim::vector3df(-2, 0, 0));
    
    obj[2] = spSmngr->createModel(scene::ENTITY_TORUS);
    obj[2]->setParent(obj[0]); // Sets the parent entity
    obj[2]->setPosition(dim::vector3df( 2, 0, 0));
    
    for (s32 i = 0; i < 3; ++i)
    {
        obj[i]->setNormalsType(scene::NORMALS_SMOOTH);
        obj[i]->updateModeling();
    }
    
    // Move the parent-/ root object forewards
    obj[0]->setPosition(dim::vector3df(0, 0, 4));
    
    spDriver->setClearColor(video::color(0, 0, 64));
    
    dim::point2di CursorSpeed;
    
    f32 LookDegree = 0.0f;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        //spSmngr->renderSceneVirtualReality(cam, LookDegree, 0.25);
        spSmngr->renderScene();
        
        LookDegree += (f32)spControl->getMouseWheel() / 2;
        
        // The 'getCursorSpeed' should call only one times at the top of the main loop.
        // Test what happens if you use this functions more times than one
        CursorSpeed = spControl->getCursorSpeed();
        
        // Rotate all objects in different directions
        obj[0]->doTurn(dim::vector3df(-CursorSpeed.Y, -CursorSpeed.X, 0));
        obj[1]->doTurn(dim::vector3df(1, 0, 0));
        obj[2]->doTurn(dim::vector3df(1, 1, 1));
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}


// Sprite functions




// ======================================================
