//
// SoftPixel Engine - Tutorial6(Fonts) - (22.3.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../common.hpp"

int main()
{
    
    // Again the same header
    
    SoftPixelDevice* spDevice = createDevice(ChooseRenderer(), dim::size2di(640, 480), 32, "Tutorial6 (Fonts)");
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver();
    scene::SceneManager* spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    /*
     * Load a font. You can load another font if this one you do not have.
     * In some 3d engines you must create an image-file with font informations to draw text,
     * but in the 'SoftPixel Engine' fontloading is very easy ;-)
     * 3D Fonts have just more parameters but it is not very difficult too!
     * Description:
     * After all you can call the function 'loadFont();' with no parameters. Then the function loads the standard
     * font with the standard dimension. But to load a font precisely you have to quote some more parameters like the
     * font name, the dimension etc. There are two functions to load a font. The first expect a dimension of 'dim::size2di'
     * whereby you can set the font-height AND width. And the second expect a dimension 's32' which specifies the font-height
     * and their width (this function calls the first function with a dimension of 'dim::size2di(0, FontHeight)').
     * I do not want to explain the function completely so take a look in the documentation which explains all parameters
     * in more detail!
     * To cut a long story short, let's load a font finaly:
     */
    //#define WINGDINGS
    #ifdef WINGDINGS
    video::Font* font = spDriver->loadFont(
        "Wingdings", 0, video::FONT_BOLD | video::FONT_ITALIC, 0, true, true, 0.2f, 0.0f, video::DRAW_SOLID
    );
    #else
    video::Font* font = spDriver->loadFont(
        "Arial", 0, video::FONT_BOLD | video::FONT_ITALIC, 0, false, true, 0.2f, 0.0f, video::DRAW_SOLID
    );
    #endif
    
    // Standard font
    video::Font* font2 = spDriver->loadFont();
    
    // Create a camera and a light
    scene::Camera* cam = spSmngr->createCamera();
    
    scene::Light* light = spSmngr->createLight();
    light->setPosition(dim::vector3df(-5, 0, 0));
    light->setLightingColor(video::color(255, 222, 32), video::color(255), video::color(0));
    spSmngr->setLighting();
    
    spDriver->setRenderMode(video::RENDERMODE_FRONT_AND_BACK);
    
    // Because programers are 'lazy' you can spare some numbers:
    // color(222) = color(222, 222, 222) = color(222, 222, 222, 222)
    spDriver->setClearColor(video::color(222));
    
    f32 Angle = 0.0f;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        spSmngr->renderScene();
        
        spDriver->beginDrawing3D(true);
        {
            Angle += 2.0f;
            spDriver->draw3DText(
                font,
                "SoftPixel Engine",
                dim::vector3df(-3.5, -COS(Angle)*0.4f, 4.0 - SIN(Angle)*0.4f),
                dim::vector3df(Angle, 0, 0)
            );
        }
        spDriver->endDrawing3D();
        
        spDriver->beginDrawing2D();
        {
            // Create a text
            io::stringc Text("Welcome, to the \"SoftPixel Engine\"!");
            
            // Save the dimension of the text in dependence to our second font
            dim::size2di Size(font2->getStringSize(Text));
            
            spDriver->draw2DText(
                font2,
                dim::point2di(320 - Size.Width/2, 100),
                Text,
                video::color(0, 0, 255)
            );
        }
        spDriver->endDrawing2D();
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}


// Sprite functions




// ======================================================
