//
// SoftPixel Engine common file
//

#include <SoftPixelEngine.hpp>
#include <iostream>


enum EChooseRendererInformation
{
    CHOOSE_RENDERER     = 0x01,
    CHOOSE_SCREENSIZE   = 0x02,
    CHOOSE_COLORDEPTH   = 0x04,
    CHOOSE_FULLSCREEN   = 0x08,
    CHOOSE_VSYNC        = 0x10,
    CHOOSE_ANTIALIAS    = 0x20,
};

struct SChooseRendererData
{
    SChooseRendererData()
    {
        Driver      = video::DRIVER_OPENGL;
        ScreenSize  = dim::size2di(640, 480);
        ColorDepth  = 32;
        FullScreen  = false;
        Vsync       = true;
        AntiAlias   = false;
        MultiSample = 2;
    }
    ~SChooseRendererData()
    {
    }
    
    /* Members */
    video::EDriverTypes Driver;
    dim::size2di ScreenSize;
    s32 ColorDepth;
    bool FullScreen;
    bool Vsync;
    bool AntiAlias;
    s32 MultiSample;
};


/*
 * Detailed functions
 */

SChooseRendererData ChooseRenderer(s32 Information)
{
    SChooseRendererData RendererData;
    
    c8 TypeName;
    
    if (Information & CHOOSE_RENDERER)
    {
        TypeName = 0;
        
        io::printMessage("====================");
        io::printMessage("Choose the renderer:");
        io::printMessage("====================");
        io::printMessage("a.) OpenGL");
        io::printMessage("b.) Direct3D9");
        io::printMessage("c.) Software");
        io::printMessage("====================");
        
        while (TypeName < 'a' || TypeName > 'c')
            std::cin >> TypeName;
        
        switch (TypeName)
        {
            case 'a':
                RendererData.Driver = sp::video::DRIVER_OPENGL; break;
            case 'b':
                RendererData.Driver = sp::video::DRIVER_DIRECT3D9; break;
            case 'c':
                RendererData.Driver = sp::video::DRIVER_SOFTWARE; break;
        }
    }
    
    if (Information & CHOOSE_SCREENSIZE)
    {
        TypeName = 0;
        
        io::printMessage("=======================");
        io::printMessage("Choose the screen size:");
        io::printMessage("=======================");
        io::printMessage("a.) 640*480");
        io::printMessage("b.) 800*600");
        io::printMessage("c.) 1024*768");
        io::printMessage("d.) 1280*1024");
        io::printMessage("=======================");
        
        while (TypeName < 'a' || TypeName > 'd')
            std::cin >> TypeName;
        
        switch (TypeName)
        {
            case 'a':
                RendererData.ScreenSize = dim::size2di(640, 480); break;
            case 'b':
                RendererData.ScreenSize = dim::size2di(800, 600); break;
            case 'c':
                RendererData.ScreenSize = dim::size2di(1024, 768); break;
            case 'd':
                RendererData.ScreenSize = dim::size2di(1280, 1024); break;
        }
    }
    
    if (Information & CHOOSE_SCREENSIZE)
    {
        TypeName = 0;
        
        io::printMessage("=======================");
        io::printMessage("Choose the color depth:");
        io::printMessage("=======================");
        io::printMessage("a.) 16 bits");
        io::printMessage("b.) 32 bits");
        io::printMessage("=======================");
        
        while (TypeName < 'a' || TypeName > 'b')
            std::cin >> TypeName;
        
        switch (TypeName)
        {
            case 'a':
                RendererData.ColorDepth = 16; break;
            case 'b':
                RendererData.ColorDepth = 32; break;
        }
    }
    
    if (Information & CHOOSE_FULLSCREEN)
    {
        TypeName = 0;
        
        io::printMessage("=======================");
        io::printMessage("Choose the screen mode:");
        io::printMessage("=======================");
        io::printMessage("a.) windowed screen");
        io::printMessage("b.) full screen");
        io::printMessage("=======================");
        
        while (TypeName < 'a' || TypeName > 'b')
            std::cin >> TypeName;
        
        switch (TypeName)
        {
            case 'a':
                RendererData.FullScreen = false; break;
            case 'b':
                RendererData.FullScreen = true; break;
        }
    }
    
    if (Information & CHOOSE_VSYNC)
    {
        TypeName = 0;
        
        io::printMessage("======================");
        io::printMessage("Choose the vsync mode:");
        io::printMessage("======================");
        io::printMessage("a.) vsync on");
        io::printMessage("b.) vsync off");
        io::printMessage("======================");
        
        while (TypeName < 'a' || TypeName > 'b')
            std::cin >> TypeName;
        
        switch (TypeName)
        {
            case 'a':
                RendererData.Vsync = true; break;
            case 'b':
                RendererData.Vsync = false; break;
        }
    }
    
    if (Information & CHOOSE_ANTIALIAS)
    {
        TypeName = 0;
        
        io::printMessage("===========================");
        io::printMessage("Choose the anti-alias mode:");
        io::printMessage("===========================");
        io::printMessage("a.) no anti-alising");
        io::printMessage("b.) 2x multi-sampler");
        io::printMessage("c.) 4x multi-sampler");
        io::printMessage("d.) 8x multi-sampler");
        io::printMessage("e.) 16x multi-sampler");
        io::printMessage("===========================");
        
        while (TypeName < 'a' || TypeName > 'e')
            std::cin >> TypeName;
        
        switch (TypeName)
        {
            case 'a':
                RendererData.AntiAlias      = false; break;
            case 'b':
                RendererData.AntiAlias      = true;
                RendererData.MultiSample    = 2;
                break;
            case 'c':
                RendererData.AntiAlias      = true;
                RendererData.MultiSample    = 4;
                break;
            case 'd':
                RendererData.AntiAlias      = true;
                RendererData.MultiSample    = 8;
                break;
            case 'e':
                RendererData.AntiAlias      = true;
                RendererData.MultiSample    = 16;
                break;
        }
    }
    
    io::clearConsole();
    
    return RendererData;
}


/*
 * Simple functions
 */

video::EDriverTypes ChooseRenderer()
{
    return ChooseRenderer(CHOOSE_RENDERER).Driver;
}


// ============================
