/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Main header
 *
 */

/*

In the 'SoftPixel Engine' there are some functions which are useless for users such as
'render_minimal()' or 'render_absolute_minimal()'!

the following namspaces are integrated in the 'SoftPixel Engine':

sp              // 'SoftPixel Engine' main namespace
 |- io          // InputOutput
 |- dim         // Dimensions (point2d, size2d, rect2d, vector3d)
 |- math        // Math (sin & cos ...)
 |- video       // Video (drawing, font)
 |  |- movie    // Movie sequence to texture (e.g. AVI files)
 |  |- gui      // Graphics User Interface (GUI)
 |  |- effect   // Video texture graphics effects (e.g. blurr, waves, magnifying glass ...) { Extension }
 |- sound       // Sound & music
 |- scene       // Scene manager (node, entity, camera, light, sprite, terrain ...)
 |  |- anim     // Animator (controls and calculates the animations -> key frame and bone animation)
 |  |- effect   // Special effects (BumpMapping, FurEffect, CelShading, BlurEffect) { Extension }
 |- network     // Network (TCP IP, Sockets ...)
 |- script      // Script extension (e.g. for texture, sound etc. loading) { Extension }

*/

#ifndef __cplusplus
#   error The 'SoftPixel Engine' is only compileable with C++!
#endif
#if !defined(WIN32)
#   error Currently the 'SoftPixel Engine' is only supported under Win32!
#endif

#include "spStandard.hpp"
#include "spDimension.hpp"
#include "spMath.hpp"
#include "spMainDevice.hpp"
#include "spUserControl.hpp"
#include "spInputOutput.hpp"
#include "spVideoDriver.hpp"
#include "spSoftwareDriver.hpp"
#include "spSceneManager.hpp"
#include "spSensitiveSound.hpp"
#include "spNetwork.hpp"
#include "spScriptExtension.hpp"

/*

========== Description: ==========

2D Graphics functions:
    - Image drawing (normal, resized, rotated) *
    - Points *
    - Lines *
    - Circles *
    - Ellipses (normal, rotated, filled) *
    - Arcs *
    - Bzier curves *
    - Splines *
    - Floodfill (*)

Loadable file formats:
    Textures:
        - BMP, DIB (Bitmaps) *
        - JPG, JPEG, JPE, JFIF (Joint Photographic Experts) *
        - TGA (Targa Image Fileformat) *
        - PNG (Portable Network Graphics) *
        - PCX (Picture Exchange) *
        - WAD (Where is All the Data) *
        
    3D Models:
        - SPM (SoftPixel Model) *
        - SPS (SoftPixel Scene)
        - 3DS (3D Studio) (*)
        - MS3D (Milkshape 3D) (*)
        - B3D (Blitz3D) *
        - X (Direct 3D)
        - MD2 (Quake2 Models) *
        - OBJ (Wavefront Objects) (*)
        
    Movies:
        - AVI *

Special effects:
    - Alpha blending (*)
    - Multi textureing *
    - Animation textures *
    - Multi viewports *
    - Terrain rendering (*)
    - Sphere mapping *
    - Lighting *
    - Volumetric fog *
    - Masking *
    - Bumpapping
    - Clipping mirrors (*)
    - Movie to texture (*)
    - Stencil shadows *
    - Keyframe- and bone animation *
    - Collision detection (*)

The following dll-files must be in the system32 directory:
    opengl32.dll
    glew32.dll
    zlib.dll
    smpeg.dll

Intresting addresses:
    Image loading: "http://jeux.developpez.com/sources/opengl/?page=images"

*/

/*

Standard rotation: YXZ

*/



// ================================================================================
