/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * array header (like the vector of the stl.)
 *
 */

#ifndef __SP_DIMENSION_ARRAY_H__
#define __SP_DIMENSION_ARRAY_H__


#include "spStandard.hpp"

#include <algorithm>
#include <cstdlib>
#include <math.h>


namespace sp
{
namespace dim
{


template <typename T> class array
{
    
    public:
        
        /* Type definitions */
        
        typedef T*          iterator;
        typedef T* const    const_iterator;
        
        /* Constructors */
        
        array() : data_(0), len_(0)
        {
        }
        
        array(u32 size) : data_(0), len_(0)
        {
            create(size);
        }
        
        array(const array<T> &other) : data_(0), len_(0)
        {
            *this = other;
        }
        
        /* Destructor */
        
        ~array()
        {
            clear();
        }
        
        /* Operators - comparision */
        
        bool operator == (const array<T> other) const
        {
            /* Check if the arrays are not emtpy */
            if (!data_ && !other.data_)
                return true;
            else if (!data_ || !other.data_)
                return false;
            
            /* Loop the elements */
            for (register u32 i = 0; i < len_; ++i)
            {
                if (data_[i] != other.data_[i])
                    return false;
            }
            
            /* Exit the function */
            return true;
        }
        
        bool operator != (const array<T> other) const
        {
            /* Check if the arrays are not emtpy */
            if (!data_ && !other.data_)
                return false;
            else if (!data_ || !other.data_)
                return true;
            
            /* Loop the elements */
            for (register u32 i = 0; i < len_; ++i)
            {
                if (data_[i] != other.data_[i])
                    return true;
            }
            
            /* Exit the function */
            return false;
        }
        
        bool operator > (const array<T> other) const
        {
            return len_ > other.len_;
        }
        
        bool operator >= (const array<T> other) const
        {
            return len_ >= other.len_;
        }
        
        bool operator < (const array<T> other) const
        {
            return len_ < other.len_;
        }
        
        bool operator <= (const array<T> other) const
        {
            return len_ <= other.len_;
        }
        
        /* Operators - settings / gettings */
        
        T& operator [] (u32 index)
        {
            return data_[index];
        }
        
        const T& operator [] (u32 index) const
        {
            return data_[index];
        }
        
        array<T>& operator = (const array<T> &other)
        {
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Allcoate new memory */
            data_ = new T[other.len_];
            len_ = other.len_;
            
            /* Copy the memory */
            std::copy(other.begin(), other.end(), &data_[0]);
            
            /* Return the reference of this object */
            return *this;
        }
        
        /* Functions */
        
        bool empty() const
        {
            return !data_;
        }
        
        u32 size() const
        {
            return len_;
        }
        
        const_iterator begin() const
        {
            return data_ ? &data_[0] : 0;
        }
        
        const_iterator end() const
        {
            return data_ ? &data_[len_] : 0;
        }
        
        void create(u32 size, T fill = (T)0)
        {
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Allocate new memory */
            data_ = new T[size];
             
            /* Set the length of the memory */
            len_ = size;
            
            /* Initialize the new memory */
            std::fill(begin(), end(), fill);
        }
        
        void clear()
        {
            if (data_)
            {
                delete [] data_;
                data_ = 0;
            }
            len_ = 0;
        }
        
        void push_back(const T &obj)
        {
            /* Allocate a temporary memory */
            T* tmp = new T[len_ + 1];
            
            /* Copy the memory */
            std::copy(begin(), end(), &tmp[0]);
            
            /* Add the new object */
            tmp[len_] = obj;
            
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            ++len_;
        }
        
        void push_back(const T array[], u32 size)
        {
            /* Allocate a temporary memory */
            T* tmp = new T[len_ + size];
            
            /* Copy the memory */
            std::copy(begin(), end(), &tmp[0]);
            std::copy(&array[0], &array[size], &tmp[len_]);
            
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            len_ += size;
        }
        
        void push_front(const T &obj)
        {
            /* Allocate a temporary memory */
            T* tmp = new T[len_ + 1];
            
            /* Copy the memory */
            ++tmp;
            std::copy(begin(), end(), &tmp[0]);
            --tmp;
            
            /* Add the new object */
            tmp[0] = obj;
            
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            ++len_;
        }
        
        void push_front(const T array[], u32 size)
        {
            /* Allocate a temporary memory */
            T* tmp = new T[len_ + size];
            
            /* Copy the memory */
            std::copy(begin(), end(), &tmp[size]);
            std::copy(&array[0], &array[size], &tmp[0]);
            
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            len_ += size;
        }
        
        void insert(const_iterator it, const T &obj)
        {
            /* Allocate a temporary memory */
            T* tmp = new T[len_ + 1];
            
            /* Copy the memory */
            std::copy(begin(), end(), &tmp[0]);
            
            /* Loop the old memory */
            for (register u32 i = len_; i > it - &data_[0]; --i)
                tmp[i] = data_[i-1];
            
            /* Add the new object */
            tmp[it - &data_[0]] = obj;
            
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            ++len_;
        }
        
        void erase(const_iterator it)
        {
            /* Allocate a temporary memory */
            T* tmp = new T[len_ - 1];
            
            /* Copy the memory */
            std::copy(begin(), end() - 1, &tmp[0]);
            
            /* Loop the old memory */
            for (register u32 i = it - &data_[0]; i < len_ - 1; ++i)
                tmp[i] = data_[i+1];
            
            /* Delete the old memory */
            if (data_)
                delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            --len_;
        }
        
        void erase(const_iterator begin, const_iterator end)
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Allocate a temporary memory */
            const u32 less = end - begin + 1;
            T* tmp = new T[len_ - less];
            
            /* Copy the memory */
            std::copy(begin(), end() - less, &tmp[0]);
            
            /* Loop the old memory */
            for (register u32 i = begin - &data_[0]; i < len_ - less; ++i)
                tmp[i] = data_[i+less];
            
            /* Delete the old memory */
            delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            len_ -= less;
        }
        
        void erase_front()
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Allocate a temporary memory */
            T* tmp = new T[len_ - 1];
            
            /* Save the old mamory */
            T* oldarray = data_;
            
            /* Move the old memory */
            ++data_;
            
            /* Copy the memory */
            std::copy(begin(), end() - 1, &tmp[0]);
            
            /* Delete the old memory */
            delete [] oldarray;
            
            /* Use the new memory */
            data_ = tmp;
            --len_;
        }
        
        void erase_back()
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Allocate a temporary memory */
            T* tmp = new T[len_ - 1];
            
            /* Copy the memory */
            std::copy(begin(), end() - 1, &tmp[0]);
            
            /* Delete the old memory */
            delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            --len_;
        }
        
        void move(const_iterator itbegin, const_iterator itend, const_iterator it)
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Allocate a temporary memory */
            T* tmp = new T[len_];
            
            /* Copy the memory */
            std::copy(begin(), end(), &tmp[0]);
            
            /* Move the memory */
            register u32 i;
            for (i = 0; i <= itend - itbegin; ++i)
                data_[i + (it - &data_[0])] = tmp[i + (itbegin - &data_[0])];
            
            /* Move the rest of the memory */
            if (it <= itbegin)
            {
                for (i = 0; i < itbegin - it; ++i)
                    data_[i + (it - &data_[0]) + itend - itbegin + 1] = tmp[i + (it - &data_[0])];
            }
            else
            {
                for (i = 0; i < it - itbegin; ++i)
                    data_[i + (itbegin - &data_[0])] = tmp[i + (itend - &data_[0] + 1)];
            }
            
            /* Delete temporary meory */
            delete [] tmp;
        }
        
        void exchange(const_iterator ita, const_iterator itb)
        {
            /* Move the memory */
            T tmp = data_[itb - begin()];
            data_[itb - begin()] = data_[ita - begin()];
            data_[ita - begin()] = tmp;
        }
        
        void resize(u32 newsize)
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Allocate a temporary memory */
            T* tmp = new T[newsize];
            
            /* Initialize the new memory */
            std::fill(&tmp[0], &tmp[newsize], (T)0);
            
            /* Copy the memory */
            const_iterator itend = (newsize > len_ ? end() : begin() + newsize);
            std::copy(begin(), itend, &tmp[0]);
            
            /* Delete the old memory */
            delete [] data_;
            
            /* Use the new memory */
            data_ = tmp;
            len_ = newsize;
        }
        
        void swap(array<T> &other)
        {
            /* Check if the data are not emtpy */
            if (!data_ && !other.data_)
                return;
            
            /* Allocate temporary memory */
            T* tmp = data_;
            u32 len = len_;
            
            /* Swap the memories */
            data_ = other.data_;
            other.data_ = tmp;
            
            /* Swap the lengths */
            len_ = other.len_;
            other.len_ = len;
        }
        
        void flip()
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Allocate temporary variables */
            T tmp;
            
            /* Loop and exchange the array elements */
            for (register u32 i = 0; i < len_/2; ++i)
            {
                tmp = data_[i];
                data_[i] = data_[len_ - 1 - i];
                data_[len_ - 1 - i] = tmp;
            }
        }
        
        void unique(bool isConsecutive = true)
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Check if the search shall be consecutive */
            if (isConsecutive)
            {
                /* Temporary variables */
                T tmp = data_[0];
                
                /* Loop the data - main */
                for (register u32 i = 1; i < len_;)
                {
                    /* Check if there are multiple elements */
                    if (tmp == data_[i])
                        erase(begin() + i);
                    else
                        ++i;
                    
                    /* Update the temporary memory */
                    tmp = data_[i];
                }
            }
            else
            {
                /* Loop the data - main */
                for (register u32 i = 0, j; i < len_; ++i)
                {
                    /* Loop the data - sub */
                    for (j = 0; j < len_;)
                    {
                        if (i != j && data_[i] == data_[j])
                            erase(begin() + j);
                        else
                            ++j;
                    }
                }
            }
        }
        
        void sort(iterator itbegin = 0, iterator itend = 0)
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Check the default values */
            if (!itbegin)
                itbegin = &data_[0];
            if (!itend)
                itend = &data_[len_ - 1];
            
            /* Allocate temporary variables */
            s32 hasExchanged = 0;
            u32 ibegin = itbegin - &data_[0], iend = itend - &data_[0] - 1, i;
            
            /* Loop until no exchange has happend */
            do
            {
                /* Reset the exchange information */
                hasExchanged = 0;
                
                /* Loop the memory */
                for (i = ibegin; i < iend; ++i)
                {
                    if (data_[i] > data_[i + 1])
                    {
                        hasExchanged = 1;
                        exchange(begin() + i, begin() + i + 1);
                    }
                }
            }
            while (hasExchanged);
        }
        
        void sort(const_iterator itbegin, const_iterator itend, bool (*lpFuncCmp)(const T &obj1, const T &obj2))
        {
            /* Check if the data is not empty */
            if (!data_)
                return;
            
            /* Allocate temporary variables */
            s32 hasExchanged = 0;
            u32 ibegin = itbegin - &data_[0], iend = itend - &data_[0] - 1, i;
            
            /* Loop until no exchange has happend */
            do
            {
                /* Reset the exchange information */
                hasExchanged = 0;
                
                /* Loop the memory */
                for (i = ibegin; i < iend; ++i)
                {
                    if (!lpFuncCmp(data_[i], data_[i + 1]))
                    {
                        hasExchanged = 1;
                        exchange(begin() + i, begin() + i + 1);
                    }
                }
            }
            while (hasExchanged);
        }
        
    private:
        
        /* Members */
        
        T* data_;
        
        u32 len_;
        
};


} // /namespace math

} // /namespace sp


#endif


// =========================================================
