/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * line3d header
 *
 */

#ifndef __SP_DIMENSION_LINE3D_H__
#define __SP_DIMENSION_LINE3D_H__


#include "spStandard.hpp"
#include "spDimensionVector3D.hpp"


namespace sp
{
namespace dim
{


template <typename T> class line3d
{
    
    public:
        
        line3d()
        {
        }
        line3d(vector3d<T> tCorner1, vector3d<T> tCorner2)
        {
            Corner1 = tCorner1; Corner2 = tCorner2;
        }
        line3d(const line3d<T> &tLine)
        {
            Corner1 = tLine.Corner1; Corner2 = tLine.Corner2;
        }
        ~line3d()
        {
        }
        
        /* === Operators === */
        
        inline bool operator == (const line3d<T> &tLine)
        {
            return Corner1 == tLine.Corner1 && Corner2 == tLine.Corner2;
        }
        inline bool operator != (const line3d<T> &tLine)
        {
            return Corner1 != tLine.Corner1 && Corner2 != tLine.Corner2;
        }
        
        inline line3d<T> operator + (const line3d<T> &tLine) const
        {
            return line3d<T>(Corner1+tLine.Corner1, Corner2+tLine.Corner2);
        }
        inline line3d<T>& operator += (const line3d<T> &tLine)
        {
            Corner1 += tLine.Corner1; Corner2 += tLine.Corner2; return *this;
        }
        
        inline line3d<T> operator - (const line3d<T> &tLine) const
        {
            return line3d<T>(Corner1-tLine.Corner1, Corner2-tLine.Corner2);
        }
        inline line3d<T>& operator -= (const line3d<T> &tLine)
        {
            Corner1 -= tLine.Corner1; Corner2 -= tLine.Corner2; return *this;
        }
        
        inline line3d<T> operator / (const line3d<T> &tLine) const
        {
            return line3d<T>(Corner1/tLine.Corner1, Corner2/tLine.Corner2);
        }
        inline line3d<T>& operator /= (const line3d<T> &tLine)
        {
            Corner1 /= tLine.Corner1; Corner2 /= tLine.Corner2; return *this;
        }
        
        inline line3d<T> operator * (const line3d<T> &tLine) const
        {
            return line3d<T>(Corner1*tLine.Corner1, Corner2*tLine.Corner2);
        }
        inline line3d<T>& operator *= (const line3d<T> &tLine)
        {
            Corner1 *= tLine.Corner1; Corner2 *= tLine.Corner2; return *this;
        }
        
        inline line3d<T> operator - () const
        {
            return line3d<T>(-Corner1, -Corner2);
        }
        
        /* === Extra functions === */
        
        inline T getBoxVolume() const
        {
            return (Corner2.X - Corner1.X).getAbs() * (Corner2.Y - Corner1.Y).getAbs() * (Corner2.Z - Corner1.Z).getAbs();
        }
        
        inline vector3d<T> getVector() const
        {
            return Corner2 - Corner1;
        }
        
        inline bool checkBoxBoxIntersection(const line3d<T> &Line) const
        {
            return (Corner1 <= Line.Corner2 && Corner2 >= Line.Corner1);
        }
        
        inline void repair()
        {
            T tmp;
            
            if (Corner1.X > Corner2.X)
                tmp = Corner1.X, Corner1.X = Corner2.X, Corner2.X = tmp;
            if (Corner1.Y > Corner2.Y)
                tmp = Corner1.Y, Corner1.Y = Corner2.Y, Corner2.Y = tmp;
            if (Corner1.Z > Corner2.Z)
                tmp = Corner1.Z, Corner1.Z = Corner2.Z, Corner2.Z = tmp;
        }
        
        /* Members */
        
        vector3d<T> Corner1, Corner2;
        
};

typedef line3d<s32> line3di;
typedef line3d<f32> line3df;


} // /namespace dim

} // /namespace sp


#endif







// ================================================================================
