/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * plane3d header
 *
 */

#ifndef __SP_DIMENSION_PLANE3D_H__
#define __SP_DIMENSION_PLANE3D_H__


#include "spStandard.hpp"
#include "spDimensionVector3D.hpp"
#include "spDimensionTriangle3D.hpp"


namespace sp
{
namespace dim
{


template <typename T> class plane3d
{
    
    public:
        
        plane3d() : Distance(0)
        {
        }
        plane3d(vector3d<T> PlaneNormal, const T Dist) : Normal(PlaneNormal), Distance(Dist)
        {
        }
        plane3d(vector3d<T> PointA, vector3d<T> PointB, vector3d<T> PointC)
        {
            computePlane(PointA, PointB, PointC);
        }
        plane3d(triangle3d<T> Triangle)
        {
            computePlane(Triangle.PointA, Triangle.PointB, Triangle.PointC);
        }
        plane3d(const plane3d<T> &other) : Normal(other.Normal), Distance(other.Distance)
        {
        }
        ~plane3d()
        {
        }
        
        /* === Extra functions === */
        
        inline void computePlane(vector3d<T> PointA, vector3d<T> PointB, vector3d<T> PointC)
        {
            Normal = vector3d<T>(PointB - PointA).getCrossProduct(PointC - PointA).normalize();
            Distance = Normal.getScalarProduct(PointA);
        }
        
        inline bool getLineIntersection(const vector3d<T> &LineCorner1, const vector3d<T> &LineCorner2, vector3d<T> &Intersection)
        {
            vector3d<T> ab(LineCorner2 - LineCorner1);
            T t = (Distance - Normal.getScalarProduct(LineCorner1)) / Normal.getScalarProduct(ab);
            
            if (t >= 0.0 && t <= 1.0)
            {
                Intersection = LineCorner1 + ab * t;
                return true;
            }
            
            return false;
        }
        
        inline bool getPlaneIntersection(plane3d<T> &other, vector3d<T> &Intersection, vector3d<T> &Direction)
        {
            /* Compute the direction of the intersection line */
            Direction = Normal.getCrossProduct(other.Normal);
            
            /*
             * If d is 0 (zero), the planes are parallel (ans separated)
             * or coincident, so they are not considered intersecting
             */
            if (Direction.getScalarProduct(Direction) < 1.0e-8)
                return false;
            
            /* Compute the intersection */
            f32 d11 = Normal.getScalarProduct(Normal);
            f32 d12 = Normal.getScalarProduct(other.Normal);
            f32 d22 = other.Normal.getScalarProduct(other.Normal);
            
            f32 det = d11*d22 - d12*d12;
            f32 k1 = ( Distance*d22 - other.Distance*d12 );
            f32 k2 = ( other.Distance*d11 - Distance*d12 );
            
            Intersection = Normal*k1 + other.Normal*k2;
            
            return true;
        }
        
        inline bool getMultiplePlaneIntersection(plane3d<T> &Plane1, plane3d<T> &Plane2, vector3d<T> &Intersection)
        {
            vector3d<T> Point, Direction;
            
            if (getPlaneIntersection(Plane1, Point, Direction))
                return Plane2.getLineIntersection(Point, Direction, Intersection);
            
            return false;
        }
        
        inline vector3d<T> getClosestPoint(vector3d<T> Point) const
        {
            return Point - Normal * ( (Normal.getScalarProduct(Point) - Distance) / Normal.getScalarProduct(Normal) );
        }
        
        inline vector3d<T> getClosestPointNormalized(vector3d<T> Point) const
        {
            return Point - Normal * ( Normal.getScalarProduct(Point) - Distance );
        }
        
        inline T getPointDistance(vector3d<T> Point) const
        {
            return (Normal.getScalarProduct(Point) - Distance) / Normal.getScalarProduct(Normal);
        }
        
        inline bool isPointFrontSide(vector3d<T> Point) const
        {
            Point = Normal*Distance - Point;
            return Normal.getScalarProduct(Point) <= 0;
        }
        
        template <typename B> inline vector3d<B> cast() const
        {
            return plane3d<B>(Normal.cast<B>(), static_cast<B>(Distance));
        }
        
        /* Members */
        
        vector3d<T> Normal;
        T Distance;
        
};

typedef plane3d<s32> plane3di;
typedef plane3d<f32> plane3df;


} // /namespace dim

} // /namespace sp


#endif







// ================================================================================
