/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * rect2d header
 *
 */

#ifndef __SP_DIMENSION_RECT2D_H__
#define __SP_DIMENSION_RECT2D_H__


#include "spStandard.hpp"
#include "spDimensionSize2D.hpp"


namespace sp
{
namespace dim
{


template <typename T> class rect2d
{
    
    public:
        
        rect2d() : Left(0), Top(0), Right(0), Bottom(0)
        {
        }
        rect2d(T NewLeft, T NewTop) : Right(0), Bottom(0)
        {
            Left = NewLeft; Top = NewTop;
        }
        rect2d(T NewLeft, T NewTop, T NewRight, T NewBottom)
        {
            Left = NewLeft; Top = NewTop; Right = NewRight; Bottom = NewBottom;
        }
        rect2d(const rect2d<T> &other)
        {
            Left = other.Left; Top = other.Top; Right = other.Right; Bottom = other.Bottom;
        }
        ~rect2d()
        {
        }
        
        /* Operators */
        
        inline bool operator == (const rect2d<T> &other) const
        {
            return Left == other.Left && Top == other.Top && Right == other.Right && Bottom == other.Bottom;
        }
        inline bool operator != (const rect2d<T> &other) const
        {
            return Left != other.Left && Top != other.Top && Right != other.Right && Bottom != other.Bottom;
        }
        
        inline rect2d<T> operator + (const rect2d<T> &other) const
        {
            return rect2d<T>(Left+other.Left, Top+other.Top, Right+other.Right, Bottom+other.Bottom);
        }
        inline rect2d<T>& operator += (const rect2d<T> &other)
        {
            Left += other.Left; Top += other.Top; Right += other.Right; Bottom += other.Bottom; return *this;
        }
        
        inline rect2d<T> operator - (const rect2d<T> &other) const
        {
            return rect2d<T>(Left-other.Left, Top-other.Top, Right-other.Right, Bottom-other.Bottom);
        }
        inline rect2d<T>& operator -= (const rect2d<T> &other)
        {
            Left -= other.Left; Top -= other.Top; Right -= other.Right; Bottom -= other.Bottom; return *this;
        }
        
        inline rect2d<T> operator / (const rect2d<T> &other) const
        {
            return rect2d<T>(Left/other.Left, Top/other.Top, Right/other.Right, Bottom/other.Bottom);
        }
        inline rect2d<T>& operator /= (const rect2d<T> &other)
        {
            Left /= other.Left; Top /= other.Top; Right /= other.Right; Bottom /= other.Bottom; return *this;
        }
        
        inline rect2d<T> operator * (const rect2d<T> &other) const
        {
            return rect2d<T>(Left*other.Left, Top*other.Top, Right*other.Right, Bottom*other.Bottom);
        }
        inline rect2d<T>& operator *= (const rect2d<T> &other)
        {
            Left *= other.Left; Top *= other.Top; Right *= other.Right; Bottom *= other.Bottom; return *this;
        }
        
        inline rect2d<T> operator - () const
        {
            return rect2d<T>(-Left, -Top, -Right, -Bottom);
        }
        
        /* Extra functions */
        
        inline size2d<T> getSize() const
        {
            return size2d<T>(Right - Left, Bottom - Top);
        }
        inline point2d<T> getCenter() const
        {
            return point2d<T>((Right - Left)/2, (Bottom - Top)/2);
        }
        
        inline bool isEmpty() const
        {
            return Left == Right == Top == Bottom == 0;
        }
        inline bool isValid() const
        {
            return (Right - Left) >= 0 && (Bottom - Top) >= 0;
        }
        inline T getWidth() const
        {
            return Right - Left;
        }
        inline T getHeight() const
        {
            return Bottom - Top;
        }
        
        inline void repair()
        {
            if (Right < Left)
            {
                T tmp   = Right;
                Right   = Left;
                Left    = tmp;
            }
            if (Bottom < Top)
            {
                T tmp   = Bottom;
                Bottom  = Top;
                Top     = tmp;
            }
        }
        
        inline bool isPointCollided(const point2d<T> &Point) const
        {
            return (Point.X >= Left && Point.X < Right && Point.Y >= Top && Point.Y < Bottom);
        }
        inline bool isRectCollided(const rect2d<T> &other) const
        {
            return (Bottom > other.Top && Top < other.Bottom && Right > other.Left && Left < other.Right);
        }
        
        template <typename B> inline rect2d<B> cast() const
        {
            return rect2d<B>(static_cast<B>(Left), static_cast<B>(Right), static_cast<B>(Top), static_cast<B>(Bottom));
        }
        
        /* Members */
        
        T Left, Right, Top, Bottom;
        
};

typedef rect2d<s32> rect2di;
typedef rect2d<f32> rect2df;


} // /namespace dim

} // /namespace sp


#endif







// ================================================================================
