/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * trianlge3d header
 *
 */

#ifndef __SP_DIMENSION_TRIANGLE3D_H__
#define __SP_DIMENSION_TRIANGLE3D_H__


#include "spStandard.hpp"
#include "spDimensionVector3D.hpp"
#include "spDimensionLine3D.hpp"


namespace sp
{
namespace dim
{


#define __SP_MAX(A, B, C) ( A >= B && A >= C ? A : ( B >= A && B >= C ? B : C ) )
#define __SP_MIN(A, B, C) ( A <= B && A <= C ? A : ( B <= A && B <= C ? B : C ) )


/*
 * ======= Triangle pointer =======
 */

template <typename T> class ptriangle3d
{
    
    public:
        
        ptriangle3d() : PointA(0), PointB(0), PointC(0)
        {
        }
        ~ptriangle3d()
        {
        }
        
        /* Members */
        
        vector3d<T> * PointA, * PointB, * PointC;
        
};

typedef ptriangle3d<s32> ptriangle3di;
typedef ptriangle3d<f32> ptriangle3df;


/*
 * ======= Triangle class =======
 */

template <typename T> class triangle3d
{
    
    public:
        
        triangle3d()
        {
        }
        triangle3d(vector3d<T> NewPointA, vector3d<T> NewPointB, vector3d<T> NewPointC)
        {
            PointA = NewPointA; PointB = NewPointB; PointC = NewPointC;
        }
        triangle3d(const triangle3d<T> &other)
        {
            PointA = other.PointA; PointB = other.PointB; PointC = other.PointC;
        }
        triangle3d(const ptriangle3d<T> &other)
        {
            PointA = *other.PointA; PointB = *other.PointB; PointC = *other.PointC;
        }
        ~triangle3d()
        {
        }
        
        /* === Operators === */
        
        inline void operator = (const ptriangle3d<T> &other)
        {
            PointA = *other.PointA; PointB = *other.PointB; PointC = *other.PointC;
        }
        
        inline triangle3d<T> operator + (const vector3d<T> &Vector) const
        {
            return triangle3d<T>(PointA + Vector, PointB + Vector, PointC + Vector);
        }
        inline triangle3d<T>& operator += (const vector3d<T> &Vector)
        {
            PointA += Vector; PointB += Vector; PointC += Vector; return *this;
        }
        
        inline triangle3d<T> operator - (const vector3d<T> &Vector) const
        {
            return triangle3d<T>(PointA - Vector, PointB - Vector, PointC - Vector);
        }
        inline triangle3d<T>& operator -= (const vector3d<T> &Vector)
        {
            PointA -= Vector; PointB -= Vector; PointC -= Vector; return *this;
        }
        
        inline triangle3d<T> operator * (const vector3d<T> &Vector) const
        {
            return triangle3d<T>(PointA*Vector, PointB*Vector, PointC*Vector);
        }
        inline triangle3d<T>& operator *= (const vector3d<T> &Vector)
        {
            PointA *= Vector; PointB *= Vector; PointC *= Vector; return *this;
        }
        
        inline triangle3d<T> operator / (const vector3d<T> &Vector) const
        {
            return triangle3d<T>(PointA/Vector, PointB/Vector, PointC/Vector);
        }
        inline triangle3d<T>& operator /= (const vector3d<T> &Vector)
        {
            PointA /= Vector; PointB /= Vector; PointC /= Vector; return *this;
        }
        
        inline triangle3d<T> operator - () const
        {
            return triangle3d<T>(-PointA, -PointB, -PointC);
        }
        
        /* === Additional operators === */
        
        inline const vector3d<T> operator [] (s32 i) const
        {
            switch (i)
            {
                case 0: return PointA;
                case 1: return PointB;
                case 2: return PointC;
            }
            return vector3d<T>();
        }
        
        inline vector3d<T>& operator [] (s32 i)
        {
            return *(&PointA + i);
        }
        
        /* === Extra functions === */
        
        inline vector3d<T> getNormal() const
        {
            return dim::vector3d<T>( (PointA - PointB) % (PointB - PointC) ).normalize();
        }
        
        inline bool isPointInside(vector3d<T> Vector) const
        {
            return
                isSameSide(Vector, PointA, PointB, PointC) &&
                isSameSide(Vector, PointB, PointA, PointC) &&
                isSameSide(Vector, PointC, PointA, PointB);
        }
        
        inline bool isSameSide(
            const vector3d<T> &OtherPoint1, const vector3d<T> &OtherPoint2,
            const vector3d<T> &OtherPointA, const vector3d<T> &OtherPointB) const
        {
            vector3d<T> Difference = OtherPointB - OtherPointA;
            vector3d<T> OtherPoint3 = Difference.getCrossProduct(OtherPoint1 - OtherPointA);
            vector3d<T> OtherPoint4 = Difference.getCrossProduct(OtherPoint2 - OtherPointA);
            return OtherPoint3.getScalarProduct(OtherPoint4) >= __SP_FLOAT_ROUND;
        }
        
        inline triangle3d<T> getSwapedTriangle() const
        {
            return triangle3d<T>(PointC, PointB, PointA);
        }
        
        inline line3d<T> getBox() const
        {
            line3d<T> TriangleBox;
            
            TriangleBox.Corner1.X = __SP_MIN(PointA.X, PointB.X, PointC.X);
            TriangleBox.Corner1.Y = __SP_MIN(PointA.Y, PointB.Y, PointC.Y);
            TriangleBox.Corner1.Z = __SP_MIN(PointA.Z, PointB.Z, PointC.Z);
            
            TriangleBox.Corner2.X = __SP_MAX(PointA.X, PointB.X, PointC.X);
            TriangleBox.Corner2.Y = __SP_MAX(PointA.Y, PointB.Y, PointC.Y);
            TriangleBox.Corner2.Z = __SP_MAX(PointA.Z, PointB.Z, PointC.Z);
            
            return TriangleBox;
        }
        
        inline bool isEmpty() const
        {
            return PointA.isEmpty() && PointB.isEmpty() && PointC.isEmpty();
        }
        
        /* Members */
        
        vector3d<T> PointA, PointB, PointC;
        
};

typedef triangle3d<s32> triangle3di;
typedef triangle3d<f32> triangle3df;


} // /namespace dim

} // /namespace sp


#endif







// ================================================================================
