/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for Direct3D9 High Level Shading Language (HLSL)
 *
 */

#ifndef __SP_DIRECT3D9SHADER_H__
#define __SP_DIRECT3D9SHADER_H__


#include "spStandard.hpp"
#include "spVideoDriverShader.hpp"

#if defined(WIN32)
#   include <d3d9.h>
#   include <d3dx9shader.h>
#endif


namespace sp
{
namespace video
{


#if defined(WIN32)

class SP_EXPORT Direct3D9Shader : public Shader
{
    
    public:
        
        Direct3D9Shader();
        Direct3D9Shader(EStandardShaderTypes Type);
        Direct3D9Shader(
            io::stringc FilenameVertex, io::stringc FilenamePixel,
            io::stringc EntryPointVertex, io::stringc EntryPointPixel,
            EHLSLVertexShaderTypes CompilerTargetVertex, EHLSLPixelShaderTypes CompilerTargetPixel,
            bool UseHighLevel
        );
        ~Direct3D9Shader();
        
        /* Binds & unbinds the shader */
        
        void bind();
        void unbind();
        
        /* Set the constants (by name) */
        
        bool setConstant(io::stringc Name, const f32 Value);
        bool setConstant(io::stringc Name, dim::vector3df Position);
        bool setConstant(io::stringc Name, video::color Color);
        bool setConstant(io::stringc Name, dim::matrix4f &Matrix);
        bool setConstant(io::stringc Name, const f32* pFloats, int Count);
        
        /* Set the constants for assembly shaders */
        
        bool setConstant(const f32* pFloats, s32 StartRegister, s32 ConstAmount);
        
    private:
        
        /* Functions */
        
        void init();
        void clear();
        
        void createShader(
            io::stringc FilenameVS, io::stringc FilenamePS,
            io::stringc EntryPointVS, io::stringc EntryPointPS,
            io::stringc CompilerTargetVS, io::stringc CompilerTargetPS,
            bool UseHighLevel
        );
        bool loadShaderFile(io::stringc Filename, c8* &pData);
        
        bool createVertexShader(const c8* strShd);
        bool createPixelShader(const c8* strShd);
        
        bool createVertexShaderHLSL(const c8* strShd, const c8* shdEntryPoint, const c8* shdTargetName);
        bool createPixelShaderHLSL(const c8* strShd, const c8* shdEntryPoint, const c8* shdTargetName);
        
        HRESULT d3dAssembleShader(
            LPCSTR pSrcData, UINT SrcDataLen, CONST D3DXMACRO* pDefines,
            LPD3DXINCLUDE pInclude, DWORD Flags, LPD3DXBUFFER* ppShader, LPD3DXBUFFER* ppErrorMsgs
        );
        
        HRESULT d3dCompileShader(
            LPCSTR pSrcData, UINT SrcDataLen, CONST D3DXMACRO* pDefines,
            LPD3DXINCLUDE pInclude, LPCSTR pFunctionName,
            LPCSTR pProfile, DWORD Flags, LPD3DXBUFFER* ppShader,
            LPD3DXBUFFER* ppErrorMsgs, LPD3DXCONSTANTTABLE* ppConstantTable
        );
        
        /* Members */
        
        LPDIRECT3DDEVICE9 pD3D9Device_;
        
        IDirect3DVertexShader9* VertexShader_;
        IDirect3DPixelShader9* PixelShader_;
        
        LPD3DXCONSTANTTABLE VSConstantsTable_;
        LPD3DXCONSTANTTABLE PSConstantsTable_;
        
};

#endif


} // /namespace scene

} // /namespace sp


#endif







// ================================================================================
