/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model loading B3D-fileformat (Blitz3D)
 *
 */

#ifndef __SP_MODELLOADER_B3D_H__
#define __SP_MODELLOADER_B3D_H__


#include "spStandard.hpp"
#include "spInputOutputStandard.hpp"
#include "spInputOutputFileSystem.hpp"
#include "spDimension.hpp"
#include "spModelLoaderRAW.hpp"

#include <vector>


namespace sp
{
namespace scene
{


class SP_EXPORT ModelLoaderB3D : public ModelLoaderRAW
{
    
    public:
        
        ModelLoaderB3D();
        ~ModelLoaderB3D();
        
        Entity* loadModel(io::stringc &Filename, io::stringc &TexturePath);
        
    private:
        
        /* === Structures === */
        
        struct SBrushSurfaceB3D
        {
            s32 BrushID, Surface;
        };
        
        struct STextureSurfaceB3D
        {
            video::Texture* hTexture;
            dim::point2df Pos;
            dim::size2df Scale;
            bool isSphereMapping;
        };
        
        struct SSkeletonBoneB3D
        {
            // Constructor & destructor
            SSkeletonBoneB3D() : VerticesList(0), KeyframeList(0)
            {
                WeightsCount    = 0;
                KeyframeCount   = 0;
            }
            ~SSkeletonBoneB3D()
            {
                // Delete all memories
                spDeleteArray(VerticesList);
                spDeleteArray(KeyframeList);
            }
            
            // General data
            s32 TabSize; // (later important to get the correct parnet bone)
            u32 ParentBoneID;
            io::stringc Name;
            
            dim::vector3df Translation, Scale;
            dim::quaternion Quaternion;
            
            // Bone vertex data
            struct SBoneVertexInfo
            {
                u32 Surface;
                s32 ID;
                f32 Weight;
            } * VerticesList;
            
            s32 WeightsCount;
            
            // Bone keyframe data
            struct SBoneKeyframeInfo
            {
                SBoneKeyframeInfo() : Position(0), Scale(0, 0, 1), Rotation(0)
                {
                }
                ~SBoneKeyframeInfo()
                {
                }
                
                dim::vector3df Position;
                dim::vector3df Scale;
                dim::vector3df Rotation;
                
                dim::quaternion Quaternion;
            } * KeyframeList;
            
            s32 KeyframeCount;
        };
        
        struct SVertexB3D
        {
            u32 Surface;
            u32 SurfVertexID;
            dim::vector3df Coord;
            dim::vector3df Normal;
            dim::point2df TexCoord;
            video::color Color;
        };
        
        /* === Functions === */
        
        void init();
        void clear();
        
        io::stringc readChunk();
        void breakChunk();
        s32 getChunkSize();
        void dumpChunks(io::stringc Tab = io::stringc(""));
        
        bool readChunkANIM();
        bool readChunkKEYS();
        bool readChunkTEXS();
        bool readChunkBRUS();
        bool readChunkVRTS();
        bool readChunkTRIS();
        bool readChunkMESH();
        bool readChunkBONE(io::stringc &Tab);
        bool readChunkNODE();
        
        void updateSurfaceLocation();
        void updateTexturing();
        
        void buildAnimation();
        
        bool loadModelData();
        
        /* === Members === */
        
        io::FileLoader* hFileLoader_;
        io::File* hFile_;
        
        std::vector<STextureSurfaceB3D> TextureList_;
        std::vector<SBrushSurfaceB3D> BrushSurfaceList_;
        
        std::vector<SSkeletonBoneB3D*> AnimBoneList_;
        
        s32 Surface_;
        io::stringc CurName_;
        
        dim::vector3df SurfPosition_, SurfScale_;
        dim::quaternion SurfQuaternion_;
        
        s32 Stack_[100];
        s32 CurPos_;
        
        // Building information
        
        std::vector<SVertexB3D> VerticesList_;
        std::vector<s32> BrushTextureList_;
        
        s32 CurBone_;
        s32 CurBrushID_;
        bool isExpactNewKeyframes_;
        bool wasLastNodeMesh_; // (was the last node a mesh?)
        
        s32 AnimKeyframeCount_;
        f32 AnimFPS_;
        
};


} // /namespace scene

} // /namespace sp


#endif





// ================================================================================
