/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for loading D3D-fileformat (Direct3D/ "*.x" DirectX)
 *
 */

#ifndef __SP_MODELLOADER_D3D_H__
#define __SP_MODELLOADER_D3D_H__


#include "spStandard.hpp"
#include "spInputOutputStandard.hpp"
#include "spInputOutputFileSystem.hpp"
#include "spDimension.hpp"
#include "spVideoDriverTexture.hpp"
#include "spModelLoaderRAW.hpp"

#include <io.h>
#include <vector>
#include <string>


namespace sp
{
namespace scene
{


class SP_EXPORT ModelLoaderD3D : public ModelLoaderRAW
{
    
    public:
        
        ModelLoaderD3D();
        ~ModelLoaderD3D();
        
        Entity* loadModel(io::stringc &Filename, io::stringc &TexturePath);
        
    private:
        
        /* === Enumerations === */
        
        enum EAnimKeyTypes
        {
            D3D_ANIMKEY_ROT = 0,
            D3D_ANIMKEY_SCL = 1,
            D3D_ANIMKEY_POS = 2,
        };
        
        enum ETokenTypes
        {
            // Record-bearing
            D3D_TOKEN_NAME = 1,
            D3D_TOKEN_STRING,
            D3D_TOKEN_INTEGER,
            D3D_TOKEN_GUID,
            D3D_TOKEN_INTEGER_LIST,
            D3D_TOKEN_FLOAT_LIST,
            
            // Stand-alone
            D3D_TOKEN_OBRACE = 10,
            D3D_TOKEN_CBRACE,
            D3D_TOKEN_OPAREN,
            D3D_TOKEN_CPAREN,
            D3D_TOKEN_OBRACKET,
            D3D_TOKEN_CBRACKET,
            D3D_TOKEN_OANGLE,
            D3D_TOKEN_CANGLE,
            D3D_TOKEN_DOT,
            D3D_TOKEN_COMMA,
            D3D_TOKEN_SEMICOLON,
            D3D_TOKEN_TEMPLATE = 31,
            D3D_TOKEN_WORD = 40,
            D3D_TOKEN_DWORD,
            D3D_TOKEN_FLOAT,
            D3D_TOKEN_DOUBLE,
            D3D_TOKEN_CHAR,
            D3D_TOKEN_UCHAR,
            D3D_TOKEN_SWORD,
            D3D_TOKEN_SDWORD,
            D3D_TOKEN_VOID,
            D3D_TOKEN_LPSTR,
            D3D_TOKEN_UNICODE,
            D3D_TOKEN_CSTRING,
            D3D_TOKEN_ARRAY,
        };
        
        /* === Structures === */
        
        typedef struct STemplate
        {
            io::stringc TypeName, TemplateName;
        }
        templateInfo;
        
        typedef struct SMaterial
        {
            io::stringc MaterialName, TextureFilename;
            video::color FaceColor, SpecularColor, EmissiveColor;
            f32 Power;
        }
        materialInfo;
        
        typedef struct SAnimationKey
        {
            s32 Key; // Key number
            dim::vector3df Position;
            dim::vector3df Rotation;
            dim::vector3df Scale;
        }
        animKeyInfo;
        
        typedef struct SFrame
        {
            io::stringc Name;
            s32 Surface;
        }
        frameInfo;
        
        typedef struct SFrameBone
        {
            // Extra structure
            struct SFrameSeq
            {
                s32 FrameNumber;
                dim::vector3df Vector;
                dim::quaternion Quat;
            };
            
            // Bone information
            io::stringc FrameName;
            s32 Surface;
            s32 TemplateLevel;
            
            // Keyframes
            std::vector<SFrameSeq> KeyframePositionList;
            std::vector<SFrameSeq> KeyframeRotationList;
            std::vector<SFrameSeq> KeyframeScaleList;
        }
        frameBoneInfo;
        
        /* === Functions === */
        
        // Initialization & clearing
        
        void init();
        void clear();
        
        // General data loading
        
        bool loadModelData();
        bool loadModelDataHeader();
        bool loadModelDataBodyText();
        bool loadModelDataBodyBinary();
        
        // Blocks
        
        io::stringc getNextValueLine();
        
        void readInt(s32 &Value, io::stringc Line);
        void readFloat(f32 &Value, io::stringc Line);
        void readBlock(f32* &ValueList, s32 Multiplicator = 1, s32 Count = 0);
        
        void readBlockKeyframe(f32* &ValueList, s32 &Count);
        
        // Templates - text
        
        bool readTemplateTxt(io::stringc &TypeName, io::stringc &TemplateName);
        
        void readTemplateTxtHeader();
        void readTemplateTxtCoords2D();
        void readTemplateTxtColorRGBA();
        void readTemplateTxtColorRGB();
        void readTemplateTxtMaterial();
        void readTemplateTxtTextureFilename();
        void readTemplateTxtMeshFace();
        void readTemplateTxtMeshFaceWraps();
        void readTemplateTxtMeshTextureCoords();
        void readTemplateTxtMeshNormals();
        void readTemplateTxtMeshVertexColors();
        void readTemplateTxtMeshMaterialList();
        void readTemplateTxtMesh();
        void readTemplateTxtFrameTransformMatrix();
        void readTemplateTxtFrame();
        void readTemplateTxtAnimationKey();
        void readTemplateTxtAnimationOptions();
        void readTemplateTxtAnimation();
        void readTemplateTxtAnimationSet();
        
        // Templates - binary
        
        bool readTemplateBin(io::stringc &TypeName);
        
        void readTemplateBinHeader();
        void readTemplateBinCoords2D();
        void readTemplateBinColorRGBA();
        void readTemplateBinColorRGB();
        void readTemplateBinMaterial();
        void readTemplateBinTextureFilename();
        void readTemplateBinMeshFace();
        void readTemplateBinMeshFaceWraps();
        void readTemplateBinMeshTextureCoords();
        void readTemplateBinMeshNormals();
        void readTemplateBinMeshVertexColors();
        void readTemplateBinMeshMaterialList();
        void readTemplateBinMesh();
        void readTemplateBinFrameTransformMatrix();
        void readTemplateBinFrame();
        void readTemplateBinAnimationKey();
        void readTemplateBinAnimationOptions();
        void readTemplateBinAnimation();
        void readTemplateBinAnimationSet();
        
        // Tokens
        
        void readTokenName(io::stringc &Name);
        void readTokenString(io::stringc &String);
        void readTokenInteger(s32 &Value);
        void readTokenGuid(io::stringc &UUID);
        void readTokenIntegerList(s32* &ValueList);
        void readTokenFloat32List(f32* &ValueList);
        void readTokenFloat64List(f64* &ValueList);
        void readTokenFloatList(f32* &FloatList, f64* &DoubleList);
        
        // Building the model
        
        Entity* buildModel();
        
        void buildAnimation();
        
        /* === Members === */
        
        io::FileLoader* FileLoader_;
        io::File* hFile_;
        
        io::stringc CurLine_, TrimLine_;
        
        s32 TemplateLevel_;
        templateInfo CurTemplate_;
        std::vector<templateInfo> TemplateList_;
        
        s8 FloatSize_;
        u32 FileLength_;
        bool isBinary_;
        
        // ( for templates )
        f32* ValueList_;
        s32 ValueListSize_;
        
        // ( for tokens )
        s32* IntegerList_;
        f32* FloatList_;
        f64* DoubleList_;
        
        // Buillding information
        
        s32 CurMaterialIndex_;
        std::vector<materialInfo> MaterialList_;
        
        dim::matrix4f CurTransformation_;
        
        std::vector<animKeyInfo> AnimationKeyList_;
        
        std::vector<frameInfo> FrameList_;
        std::vector<frameBoneInfo> FrameBoneList_;
        std::vector<io::stringc> TempFrameNameList_;
        u32 CurFrameIndex_, CurBoneIndex_;
        u32 HighestFrame_;
        
        // Reading information
        
        s32 LinePos_; // (for reading values & blocks)
        
};


} // /namespace scene

} // /namespace sp


#endif





// ================================================================================
