/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model loading MS3D-fileformat (Milkshape 3D)
 *
 */

#ifndef __SP_MODELLOADER_MS3D_H__
#define __SP_MODELLOADER_MS3D_H__


#include "spStandard.hpp"
#include "spInputOutputStandard.hpp"
#include "spDimension.hpp"
#include "spVideoDriverTexture.hpp"
#include "spModelLoaderRAW.hpp"

#include <io.h>
#include <ios>
#include <vector>
#include <string>
#include <fstream>


namespace sp
{
namespace scene
{


class SP_EXPORT ModelLoaderMS3D : public ModelLoaderRAW
{
    
    public:
        
        ModelLoaderMS3D();
        ~ModelLoaderMS3D();
        
        Entity* loadModel(io::stringc &Filename, io::stringc &TexturePath);
        
    private:
        
        /* === Structures === */
        
        struct SMeshMS3D // Mesh
        {
            s32 MaterialIndex;
            s32 CountOfTriangles;
            s32* pTriangleIndices;
        };
        
        struct SMaterialMS3D // Material properties
        {
            f32 Ambient[4], Diffuse[4], Specular[4], Emissive[4];
            f32 Shininess;
            u32 TextureID;
            video::Texture* hTexture;
            c8* TextureFilename;
        };
        
        struct STriangleMS3D // Triangle structure
        {
            f32 VertexNormals[3][3];
            f32 TexCoordU[3];
            f32 TexCoordV[3];
            s32 VertexIndices[3];
        };
        
        struct SVertexMS3D // Vertex structure
        {
            s8 BoneID; // for skeletal animation
            f32 Location[3];
        };
        
        // Bone/ Skeletal animation
        
        struct SKeyframeMS3D
        {
            s32 JointIndex;
            f32 Time;
            dim::vector3df Vect; // translation or rotation
        };
        
        struct SJointMS3D
        {
            io::stringc Name;
            s32 Index, Parent;
            
            dim::vector3df Rotation, Translation;
            
            dim::matrix4f RelativeTransformation;
            dim::matrix4f AbsoluteTransformation;
            dim::matrix4f FinalTransformation;
            
            std::vector<SKeyframeMS3D> TranslationKeyframes;
            std::vector<SKeyframeMS3D> RotationKeyframes;
        };
        
        /* === Functions === */
        
        void init();
        void clear();
        
        bool loadModelData(io::stringc Filename);
        void buildAnimation();
        Entity* buildModel();
        
        /* === Members === */
        
        // Meshes
        s32 CountOfMeshes_;
        SMeshMS3D* pMeshes_;
        
        // Materials
        s32 CountOfMaterials_;
        SMaterialMS3D* pMaterials_;
        
        // Triangles
        s32 CountOfTriangles_;
        STriangleMS3D* pTriangles_;
        
        // Vertices
        s32 CountOfVertcies_;
        SVertexMS3D* pVertices_;
        
        // Joints (for bone/ skeletal animation)
        s32 CountOfJoints_;
        SJointMS3D* pJoints_;
        
        //f32* pTimer_;
        f64 TotalTime_;
        
        f32 TotalFrames_, FPS_;
        
};


} // /namespace scene

} // /namespace sp


#endif





// ================================================================================
