/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model loading OBJ-fileformat (Wavefront)
 *
 */

#ifndef __SP_MODELLOADER_OBJ_H__
#define __SP_MODELLOADER_OBJ_H__


#include "spStandard.hpp"
#include "spInputOutputStandard.hpp"
#include "spInputOutputFileSystem.hpp"
#include "spDimension.hpp"
#include "spModelLoaderRAW.hpp"

#include <vector>


namespace sp
{
namespace scene
{


class SP_EXPORT ModelLoaderOBJ : public ModelLoaderRAW
{
    
    public:
        
        ModelLoaderOBJ();
        ~ModelLoaderOBJ();
        
        Entity* loadModel(io::stringc &Filename, io::stringc &TexturePath);
        
    private:
        
        /* Enumerations */
        
        enum EModelCreateStates
        {
            OBJ_MODELCREATE_NONE,
            OBJ_MODELCREATE_NEWSURFACE,
            OBJ_MODELCREATE_VERTICES,
            OBJ_MODELCREATE_TRIANGLES,
        };
        
        /* Structures */
        
        struct SVertexOBJ
        {
            dim::vector3df Coord;
            dim::vector3df Normal;
            dim::point2df TexCoord;
        };
        
        struct SSurfaceOBJ
        {
            SSurfaceOBJ() : NormalCount(0), TexCoordCount(0)
            {
            }
            ~SSurfaceOBJ()
            {
            }
            
            io::stringc Name;
            
            s32 NormalCount, TexCoordCount;
            
            std::vector<SVertexOBJ> VerticesList;
            std::vector<s32> IndicesList;
        };
        
        /* Functions */
        
        void init();
        void clear();
        
        void readLine();
        
        //readVector3();
        //readVector4();
        //readVertices(); // -> coord/ normal/ mapping
        
        s32 getFloatList(f32 pList[4]);
        s32 getIntegerList(s32 pList[4]);
        
        void createNewSurface();
        void buildModel();
        
        bool loadModelData();
        
        /* Members */
        
        io::FileLoader* FileLoader_;
        io::File* hFile_;
        
        io::stringc CurLine_;
        s32 CurSurface_;
        
        EModelCreateStates ModelCreateStatus_;
        
        std::vector<SSurfaceOBJ> SurfacesList_;
        
};


} // /namespace scene

} // /namespace sp


#endif





// ================================================================================
