/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model loading RAW
 *
 */

#ifndef __SP_MODELLOADER_RAW_H__
#define __SP_MODELLOADER_RAW_H__


#include "spStandard.hpp"
#include "spInputOutput.hpp"
#include "spSceneManagerEntity.hpp"
#include "spSceneManagerAnimator.hpp"
#include "spVideoDriverTexture.hpp"


namespace sp
{
namespace scene
{


extern anim::Animator* __spAnimator;


/*
 * ======= ModelLoaderRAW class =======
 */

class SP_EXPORT ModelLoaderRAW
{
    
    public:
        
        virtual ~ModelLoaderRAW()
        {
        }
        
        virtual Entity* loadModel(io::stringc &Filename, io::stringc &TexturePath) = 0;
        
    protected:
        
        /* Protected functions */
        
        ModelLoaderRAW() : hEntity_(0), Anim_(0)
        {
        }
        
        /* Members */
        
        Entity* hEntity_;
        
        anim::Animation* Anim_;
        
        io::stringc Filename_, TexturePath_;
        
};


/*
 * ======= SceneLoaderRAW class =======
 */

class SceneManager;

class SP_EXPORT SceneLoaderRAW
{
    
    public:
        
        virtual ~SceneLoaderRAW()
        {
            /* Delete the file loader */
            spDeletePointer(hFileLoader_);
        }
        
        virtual Entity* loadScene(SceneManager* SceneMngr, io::stringc &Filename, io::stringc &TexturePath, s32 Tesselation) = 0;
        
    protected:
        
        /* Protected fucntions */
        
        SceneLoaderRAW()
            : hEntity_(0), hEntityBase_(0), hEntityTrans_(0), hFile_(0), hFileLoader_(0), SceneMngr_(0)
        {
            /* Allocate a new file loader */
            hFileLoader_ = new io::FileLoader();
        }
        
        /* Members */
        
        Entity* hEntity_;
        Entity* hEntityBase_;
        Entity* hEntityTrans_;
        
        io::File* hFile_;
        io::FileLoader* hFileLoader_;
        
        SceneManager* SceneMngr_;
        
        io::stringc Filename_, TexturePath_;
        
};


} // /namespace scene

} // /namespace sp


#endif





// ================================================================================
