/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for all network operations (server, client)
 *
 */

#ifndef __SP_NETWORK_H__
#define __SP_NETWORK_H__


#include "spStandard.hpp"
#include "spInputOutputString.hpp"
#include "spInputOutputStandard.hpp"

#if defined(WIN32)
#   include <windows.h>
#   include <winsock2.h>
#elif defined(LINUX)

#endif

#include <unistd.h>
#include <list>


namespace sp
{
namespace network
{


#define __SP_NETWORK_STDPORT        80
#define __SP_NETWORK_CLIENTID_ALL   0

#define __SP_NETMSG_LEN             256

typedef struct SNetworkMessageInfo
{
    io::stringc Message;
    u32 SenderClientID;
}
messageInfo;

typedef struct SClientInfo
{
    u32 ID;
    SOCKET Socket;
}
clientInfo;

// External lists
extern std::list<clientInfo> ClientList;
extern std::list<u32> NewClientList;
extern std::list<messageInfo> SendMessageList;
extern std::list<io::stringc> ReceiveMessageList;
extern bool isServer;
extern SOCKET sockMain;


class SP_EXPORT NetworkManager
{
    
    public:
        
        NetworkManager();
        ~NetworkManager();
        
        bool createServer(io::stringc ServerIPAddress, u32 Port = __SP_NETWORK_STDPORT);
        bool connectServer(io::stringc ServerIPAddress, u32 Port = __SP_NETWORK_STDPORT);
        
        std::list<u32> checkNewClients();
        bool addClient(u32 NetworkClientID);
        void removeClient(u32 NetworkClientID);
        
        void sendMessage(io::stringc Message, u32 NetworkClientID = __SP_NETWORK_CLIENTID_ALL);
        std::list<io::stringc> receiveMessages();
        
        void closeConnection();
        
        inline std::list<clientInfo> getClientList()
        {
            return ClientList;
        }
        
    private:
        
        void init();
        bool createSocket(SOCKET &Socket);
        void createThreads();
        void closeThreads();
        
        WSADATA WSA_;
        HANDLE hThread_Connections_;
        HANDLE hThread_MessageSend_;
        HANDLE hThread_MessageRecv_;
        unsigned long ThreadIDConnections_;
        unsigned long ThreadIDMessageSend_;
        unsigned long ThreadIDMessageRecv_;
        
        SOCKET MainSocket_;
        io::stringc ServerIPAddress_;
        
};

DWORD WINAPI Thread_Connections(PVOID pParam);
DWORD WINAPI Thread_MessageSend(PVOID pParam);
DWORD WINAPI Thread_MessageRecv(PVOID pParam);


} // /namespace network

} // /namespace sp


#endif





// ================================================================================
