/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for the "Ray" standard types
 *
 */

#ifndef __SP_RAYSTANDARDTYPES_H__
#define __SP_RAYSTANDARDTYPES_H__


#include "spRayCore.h"


/*
 * ========== Internal macros & static members ==========
 */

static const RAYint RAY_RENDER_PIPELINE_SIZE    = 4;
static const RAYint RAY_TEXTURE_LAYER_SIZE      = 2;
static const RAYint RAY_MAX_MIPMAPLEVELS        = 32;
static const RAYint RAY_MAX_LIGHT_COUNT         = 1;

#define mcr_rayGetTexelColor(c, o)                                      \
    c.r = rcTexture[rcTextureLayer]->image[rcTextureMipMapLevel][o+0];  \
    c.g = rcTexture[rcTextureLayer]->image[rcTextureMipMapLevel][o+1];  \
    c.b = rcTexture[rcTextureLayer]->image[rcTextureMipMapLevel][o+2];  \
    c.a = rcTexture[rcTextureLayer]->image[rcTextureMipMapLevel][o+3];

#define mcr_rayComputeTriangleFaceSize(a, b, c)                     \
    ( 0.5*( a.x*(b.y - c.y) + b.x*(c.y - a.y) + c.x*(a.y - b.y) ) )


/*
 * ========== Internal structures ==========
 */

struct SRayVector2f
{
    RAYfloat x, y;
};

struct SRayVector3f
{
    RAYfloat x, y, z;
};

struct SRayVector4f
{
    RAYfloat x, y, z, w;
};

struct SRayVector2i
{
    RAYint x, y;
};

struct SRayVector3i
{
    RAYint x, y, z;
};

struct SRayVector4i
{
    RAYint x, y, z, w;
};

struct SRayColor4i
{
    RAYcolor_t r, g, b, a;
};

struct SRayColor4f
{
    RAYfloat r, g, b, a;
};

struct SRayMatrix3f
{
    RAYfloat m[9];
};

struct SRayMatrix4f
{
    RAYfloat m[16];
};

struct SRayMappingGen
{
    RAYint genu, genv;
};

struct SRayTexture2D
{
    RAYint width;
    RAYint height;
    
    RAYpixel_t** image; // Image data in integer and RGBA format ( [width*height*4]*mipcount )
    RAYint mipcount;    // Count of MIP-map levels
};

struct SRayPolygonSide
{
    // Components for linear interpolation between the polygon edges
    RAYfloat z, u, v;   // Depth and texcoords
    SRayColor4i clr;    // Color
};

struct SRayVertex
{
    SRayVector4f pos;   // Coordinate
    SRayVector3f nml;   // Normal
    SRayColor4i clr;    // Color
    SRayVector2f map[RAY_TEXTURE_LAYER_SIZE];   // Texture map coordinate
};

struct SRayLight
{
    RAYenum type;       // ERayLightTypes
    SRayVector3f dir;
    SRayMatrix4f mat;
    SRayColor4i diffuse;
    SRayColor4i ambient;
};


/*
 * ========== Internal typedefintions ==========
 */

#define RAY_BLENDPROC_PARAMETERS                                            \
    RAYcolor_t &srcR, RAYcolor_t &srcG, RAYcolor_t &srcB, RAYcolor_t &srcA, \
    RAYpixel_t &dstR, RAYpixel_t &dstG, RAYpixel_t &dstB, RAYpixel_t &dstA

typedef void (*PFNRAYBLENDPROC)(RAY_BLENDPROC_PARAMETERS);


/*
 * ========== Internal variable procedures declerations ==========
 */

void rayBlendProc_Normal(RAY_BLENDPROC_PARAMETERS);
void rayBlendProc_Bright(RAY_BLENDPROC_PARAMETERS);
void rayBlendProc_Dark  (RAY_BLENDPROC_PARAMETERS);
void rayBlendProc_Light (RAY_BLENDPROC_PARAMETERS);


/*
 * ========== Internal declerations ==========
 */

extern RAYenum rayLastError;

// Ray current members (rc's)
extern SRayVertex*      rcVertex    [RAY_RENDER_PIPELINE_SIZE];
extern SRayVertex*      rcMemVertex [RAY_RENDER_PIPELINE_SIZE];

extern SRayTexture2D*   rcTexture   [RAY_TEXTURE_LAYER_SIZE];

extern SRayLight*       rcLight     [RAY_MAX_LIGHT_COUNT];

extern RAYint rcPLIndex; // render pipe line index
extern RAYint rcRenderType;

extern RAYint rcMatrixMode;
extern RAYint rcTextureLayer;
extern RAYint rcMaxTextureLayer;
extern RAYint rcTextureMipMapLevel;
extern RAYint rcUpdateVerticesMatrix;

// Ray status members
extern RAYint rayRenderStatus[RAY_STATUS_COUNT];

extern RAYint rayPixelRect     [4];
extern RAYint rayViewportRect  [4];
extern RAYint rayScissorRect   [4];

extern RAYfloat rayPlaneNear, rayRangeNear;
extern RAYfloat rayPlaneFar, rayRangeFar;
extern RAYfloat rayFieldOfView;

extern SRayColor4i rayBgClearColor;

extern SRayMatrix4f rayRenderMatrix[RAY_COUNT_OF_MATRICES];

// Ray render state functions
extern PFNRAYBLENDPROC rayBlendProc;
extern RAYint rayRenderFace;

extern SRayMappingGen rayMappingGenType[RAY_TEXTURE_LAYER_SIZE];

// Screen interface
extern RAYuint rayDCTexID;

extern RAYpixel_t* rayPixelBuffer;
extern RAYfloat* rayDepthBuffer;

extern RAYint rayScreenWidth;
extern RAYint rayScreenHeight;

extern RAYint rayDepthBufferSize;
extern RAYint rayPixelBufferSize;


/* 
 * ========== Internal inline functions ==========
 */

inline void rayClamp(RAYfloat &val, const RAYfloat min, const RAYfloat max)
{
    if (val < min)
        val = min;
    else if (val > max)
        val = max;
}

inline void rayClamp(RAYint &val, const RAYint min, const RAYint max)
{
    if (val < min)
        val = min;
    else if (val > max)
        val = max;
}

inline RAYint rayIsPixelIndside2D(const RAYint x, const RAYint y)
{
    return x >= rayPixelRect[0] && y >= rayPixelRect[1] && x < rayPixelRect[2] && y < rayPixelRect[3];
}

inline RAYint rayIsPixelIndside3D(const RAYint x, const RAYint y, const RAYfloat z, const RAYint i)
{
    return
        ( x >= rayPixelRect[0] && y >= rayPixelRect[1] && x < rayPixelRect[2] && y < rayPixelRect[3] ) &&
        ( !rayRenderStatus[RAY_DEPTH_TEST] || ( z < rayRangeNear && z > rayRangeFar && z >= rayDepthBuffer[i] ) );
}

inline RAYint rayCheckPlotDepth(const RAYint i, const RAYfloat z)
{
    // Check if the pixel is able to render
    if ( rayRenderStatus[RAY_DEPTH_TEST] && !( z < rayRangeNear && z > rayRangeFar && z >= rayDepthBuffer[i] - 1.0e-6 ) )
        return 0;
    
    return 1;
}

inline void rayPlotPixel(RAYint i, const RAYcolor_t r, const RAYcolor_t g, const RAYcolor_t b)
{
    i <<= 2;
    rayPixelBuffer[i+0] = r;
    rayPixelBuffer[i+1] = g;
    rayPixelBuffer[i+2] = b;
}

inline void rayPlotPixel(RAYint i, const RAYcolor_t r, const RAYcolor_t g, const RAYcolor_t b, const RAYcolor_t a)
{
    i <<= 2;
    rayPixelBuffer[i+0] = r * a / 255   +   rayPixelBuffer[i+0] * ( 255 - a ) / 255;
    rayPixelBuffer[i+1] = g * a / 255   +   rayPixelBuffer[i+1] * ( 255 - a ) / 255;
    rayPixelBuffer[i+2] = b * a / 255   +   rayPixelBuffer[i+2] * ( 255 - a ) / 255;
}

inline void rayPlotDepth(const RAYint i, const RAYfloat z)
{
    rayDepthBuffer[i] = z;
}

inline void rayPlotIndexed(RAYint i, const RAYfloat z, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a)
{
    // Check if the pixel is able to render
    if (a == 0)
        return;
    
    // Plot the depth
    rayDepthBuffer[i] = z;
    
    // Get the pixel buffer index
    i <<= 2;
    
    // Blend test
    if (rayRenderStatus[RAY_BLEND_TEST])
    {
        rayBlendProc(
            r, g, b, a,
            rayPixelBuffer[i+0], rayPixelBuffer[i+1], rayPixelBuffer[i+2], rayPixelBuffer[i+3]
        );
    }
    
    // Plot the pixel
    if (a == 255 || !rayRenderStatus[RAY_BLEND])
    {
        rayPixelBuffer[i+0] = r;
        rayPixelBuffer[i+1] = g;
        rayPixelBuffer[i+2] = b;
    }
    else
    {
        rayPixelBuffer[i+0] = r * a / 255   +   rayPixelBuffer[i+0] * ( 255 - a ) / 255;
        rayPixelBuffer[i+1] = g * a / 255   +   rayPixelBuffer[i+1] * ( 255 - a ) / 255;
        rayPixelBuffer[i+2] = b * a / 255   +   rayPixelBuffer[i+2] * ( 255 - a ) / 255;
    }
}

inline void rayPlotLocated(RAYint x, RAYint y, RAYfloat z, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a)
{
    if (x < rayPixelRect[0] || y < rayPixelRect[1] || x >= rayPixelRect[2] || y >= rayPixelRect[3])
        return;
    
    rayPlotIndexed(y * rayScreenWidth + x, z, r, g, b, a);
}


inline RAYint rayTriangleFaceVisible()
{
    if (rayRenderFace != RAY_RENDER_BOTH)
    {
        // Compute if the triangle is drawn in the clock- or counter clock wise
        RAYfloat d =
            (rcVertex[1]->pos.x - rcVertex[0]->pos.x)*(rcVertex[2]->pos.y - rcVertex[0]->pos.y) -
            (rcVertex[1]->pos.y - rcVertex[0]->pos.y)*(rcVertex[2]->pos.x - rcVertex[0]->pos.x);
        
        if ( ( rayRenderFace == RAY_RENDER_FRONT && d <= 0.0f ) || ( rayRenderFace == RAY_RENDER_BACK && d >= 0.0f ) )
            return 0;
    }
    
    return 1;
}

inline RAYint rayTriangleVisible()
{
    RAYfloat far = 1.0 / rayRangeFar;
    return
        !( ( rcVertex[0]->pos.x < 0 && rcVertex[1]->pos.x < 0 && rcVertex[2]->pos.x < 0 ) ||
        ( rcVertex[0]->pos.x > rayScreenWidth && rcVertex[1]->pos.x > rayScreenWidth && rcVertex[2]->pos.x > rayScreenWidth ) ||
        ( rcVertex[0]->pos.y < 0 && rcVertex[1]->pos.y < 0 && rcVertex[2]->pos.y < 0 ) ||
        ( rcVertex[0]->pos.y > rayScreenHeight && rcVertex[1]->pos.y > rayScreenHeight && rcVertex[2]->pos.y > rayScreenHeight ) ||
        ( rcVertex[0]->pos.z > far && rcVertex[1]->pos.z > far && rcVertex[2]->pos.z > far ) );
}

inline void rayGetTriangleArea(RAYint &VerticalStart, RAYint &VerticalEnd)
{
    #define MIN(A, B, C) \
        ( A <= B && A <= C ? A : ( B <= A && B <= C ? B : C ) )
    #define MAX(A, B, C) \
        ( A >= B && A >= C ? A : ( B >= A && B >= C ? B : C ) )
    
    // Get the area of y-axis
    VerticalStart   = (RAYint)MIN(rcVertex[0]->pos.y, rcVertex[1]->pos.y, rcVertex[2]->pos.y);
    VerticalEnd     = (RAYint)MAX(rcVertex[0]->pos.y, rcVertex[1]->pos.y, rcVertex[2]->pos.y);
    
    // Check the maximal area
    if (VerticalStart < rayPixelRect[1])
        VerticalStart = rayPixelRect[1];
    if (VerticalEnd > rayPixelRect[3])
        VerticalEnd = rayPixelRect[3];
    
    #undef MIN
    #undef MAX
}


inline void rayMultMatrix(const SRayMatrix4f &Matrix, SRayVector3f &Vector)
{
    RAYfloat x, y, z;
    
    x = Vector.x*Matrix.m[0] + Vector.y*Matrix.m[4] + Vector.z*Matrix.m[ 8] + Matrix.m[12];
    y = Vector.x*Matrix.m[1] + Vector.y*Matrix.m[5] + Vector.z*Matrix.m[ 9] + Matrix.m[13];
    z = Vector.x*Matrix.m[2] + Vector.y*Matrix.m[6] + Vector.z*Matrix.m[10] + Matrix.m[14];
    
    Vector.x = x, Vector.y = y, Vector.z = z;
}

inline void rayMultMatrix(const SRayMatrix4f &Matrix, SRayVector4f &Vector)
{
    RAYfloat x, y, z, w;
    
    x = Vector.x*Matrix.m[0] + Vector.y*Matrix.m[4] + Vector.z*Matrix.m[ 8] + Vector.w*Matrix.m[12];
    y = Vector.x*Matrix.m[1] + Vector.y*Matrix.m[5] + Vector.z*Matrix.m[ 9] + Vector.w*Matrix.m[13];
    z = Vector.x*Matrix.m[2] + Vector.y*Matrix.m[6] + Vector.z*Matrix.m[10] + Vector.w*Matrix.m[14];
    w = Vector.x*Matrix.m[3] + Vector.y*Matrix.m[7] + Vector.z*Matrix.m[11] + Vector.w*Matrix.m[15];
    
    Vector.x = x, Vector.y = y, Vector.z = z, Vector.w = w;
}

inline void rayNormalizeVec3(SRayVector3f &v)
{
    const RAYfloat l = 1.0 / sqrtf(v.x*v.x + v.y*v.y + v.z*v.z);
    v.x *= l;
    v.y *= l;
    v.z *= l;
}

inline void rayProject2Dto3D(RAYfloat &X, RAYfloat &Y)
{
    X = (RAYfloat)( (RAYfloat)( X - rayScreenWidth /2) / (rayScreenWidth /2) );
    Y = (RAYfloat)( (RAYfloat)(-Y + rayScreenHeight/2) / (rayScreenHeight/2) );
}


inline void rayProject3Dto2D(SRayVector4f &v)
{
    #if 0
    
    RAYint HalfWidth    = ( rayViewportRect[2] - rayViewportRect[0] ) / 2;
    RAYint HalfHeight   = ( rayViewportRect[3] - rayViewportRect[1] ) / 2;
    
    v.x =   v.x / v.z * HalfWidth + rayViewportRect[0] + HalfWidth;
    v.y = - v.y / v.z * HalfWidth + rayViewportRect[1] + HalfHeight;
    
    #else
    
    RAYfloat z = v.z;
    
    rayMultMatrix(rayRenderMatrix[RAY_PROJECTION_MATRIX], v);
    
    const RAYfloat invw = 1.0f / v.w;
    
    v.x = invw * ( v.x * rayRenderMatrix[RAY_VIEWPORT_MATRIX].m[0] + v.w * rayRenderMatrix[RAY_VIEWPORT_MATRIX].m[12] );
    v.y = invw * ( v.y * rayRenderMatrix[RAY_VIEWPORT_MATRIX].m[5] + v.w * rayRenderMatrix[RAY_VIEWPORT_MATRIX].m[13] );
    v.z = z;
    
    #endif
}

inline void raySwapVertex(SRayVertex* &v1, SRayVertex* &v2)
{
    SRayVertex* t;
    t = v1;
    v1 = v2;
    v2 = t;
}

inline RAYfloat rayComputeAngle(const SRayVector4f &v1, const SRayVector4f &v2)
{
    const RAYfloat len1 = sqrtf(v1.x*v1.x + v1.y*v1.y + v1.z*v1.z);
    const RAYfloat len2 = sqrtf(v2.x*v2.x + v2.y*v2.y + v2.z*v2.z);
    
    const RAYfloat scalar = v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
    
    return acosf( scalar / (len1*len2) )*180.0f/M_PI;
}


inline void rayInterpolateColor(SRayColor4i &clr, const SRayColor4i &clra, const SRayColor4i &clrb, const RAYfloat &t)
{
    clr.r = clra.r + (RAYint)( (RAYfloat)( clrb.r - clra.r ) * t );
    clr.g = clra.g + (RAYint)( (RAYfloat)( clrb.g - clra.g ) * t );
    clr.b = clra.b + (RAYint)( (RAYfloat)( clrb.b - clra.b ) * t );
    clr.a = clra.a + (RAYint)( (RAYfloat)( clrb.a - clra.a ) * t );
}

inline void rayClampTexCoords(RAYint w, RAYint h, RAYfloat &u, RAYfloat &v)
{
    u -= (RAYint)(u / w) * w;
    v -= (RAYint)(v / h) * h;
    
    if (u < 0)
        u = w + u;
    if (v < 0)
        v = h + v;
}

inline void rayGetTexturePixelLinear(SRayColor4i &clr, SRayVector2f &map)
{
    rayClampTexCoords(
        rcTexture[rcTextureLayer]->width, rcTexture[rcTextureLayer]->height, map.x, map.y
    );
    
    RAYint i = (
        ( ((RAYint)map.y) >> rcTextureMipMapLevel ) * ( rcTexture[rcTextureLayer]->width >> rcTextureMipMapLevel ) +
        ( ((RAYint)map.x) >> rcTextureMipMapLevel )
    ) << 2;
    
    #if 0
    RAYint DataSize = ( rcTexture[rcTextureLayer]->width * rcTexture[rcTextureLayer]->height ) << 2;
    if (i >= DataSize)
        i = DataSize - 1;
    #endif
    
    mcr_rayGetTexelColor(clr, i);
}

inline void rayGetTexturePixelSmooth(SRayColor4i &clr, SRayVector2f &map)
{
    RAYint i;
    SRayVector2f map2;
    
    // Clamp the texture coordinates
    rayClampTexCoords(
        rcTexture[rcTextureLayer]->width, rcTexture[rcTextureLayer]->height, map.x, map.y
    );
    
    // Get the next texture coordinates
    map2.x = map.x + (1 << rcTextureMipMapLevel);
    map2.y = map.y + (1 << rcTextureMipMapLevel);
    
    if (map2.x > rcTexture[rcTextureLayer]->width)
        map2.x = 0;
    if (map2.y > rcTexture[rcTextureLayer]->height)
        map2.y = 0;
    
    RAYint offset1 = ( ((RAYint)map.y) >> rcTextureMipMapLevel ) * ( rcTexture[rcTextureLayer]->width >> rcTextureMipMapLevel );
    RAYint offset2 = ( ((RAYint)map2.y) >> rcTextureMipMapLevel ) * ( rcTexture[rcTextureLayer]->width >> rcTextureMipMapLevel );
    
    // Get the texels color
    #define GET_TEXEL_COLOR(c, o, p)                                \
        i = ( o + ( ((RAYint)p) >> rcTextureMipMapLevel ) ) << 2;   \
        SRayColor4i c;                                              \
        mcr_rayGetTexelColor(c, i);
    
    GET_TEXEL_COLOR(clr1, offset1, map.x);
    GET_TEXEL_COLOR(clr2, offset1, map2.x);
    GET_TEXEL_COLOR(clr3, offset2, map2.x);
    GET_TEXEL_COLOR(clr4, offset2, map.x);
    
    // Compute the mix value
    map.x /= (RAYfloat)(1 << rcTextureMipMapLevel);
    map.y /= (RAYfloat)(1 << rcTextureMipMapLevel);
    
    RAYint ratiox = (RAYint)((map.x - (RAYint)map.x)*255);
    RAYint ratioy = (RAYint)((map.y - (RAYint)map.y)*255);
    
    RAYint invratiox = 255 - ratiox;
    RAYint invratioy = 255 - ratioy;
    
    #define INTERPOLATE_COLOR(c)                                                        \
        clr.c =                                                                         \
            ( clr1.c * invratiox / 255 + clr2.c * ratiox / 255 ) * invratioy / 255 +    \
            ( clr4.c * invratiox / 255 + clr3.c * ratiox / 255 ) * ratioy / 255;
    
    INTERPOLATE_COLOR(r);
    INTERPOLATE_COLOR(g);
    INTERPOLATE_COLOR(b);
    INTERPOLATE_COLOR(a);
    
    #undef INTERPOLATE_COLOR
    #undef GET_TEXEL_COLOR
}

inline RAYint rayCheckColorEquality(const SRayColor4i &clr1, const SRayColor4i &clr2, const SRayColor4i &clr3)
{
    if (clr1.r != clr2.r || clr1.r != clr3.r ||
        clr1.g != clr2.g || clr1.g != clr3.g ||
        clr1.b != clr2.b || clr1.b != clr3.b ||
        clr1.a != clr2.a || clr1.a != clr3.a)
    {
        return 0;
    }
    
    return 1;
}

inline void rayGetFreeVisibleVertexIndex(const RAYint &index1, RAYint vi[2])
{
    switch (index1)
    {
        case 0:
            vi[0] = 1, vi[1] = 2; break;
        case 1:
            vi[0] = 2, vi[1] = 0; break;
        case 2:
            vi[0] = 0, vi[1] = 1; break;
    }
}

inline RAYint rayGetFreeVisibleVertexIndex(const RAYint &index1, const RAYint &index2)
{
    switch (index1)
    {
        case 0:
            switch (index2)
            {
                case 1: return 2;
                case 2: return 1;
            }
            break;
        case 1:
            switch (index2)
            {
                case 0: return 2;
                case 2: return 0;
            }
            break;
        case 2:
            switch (index2)
            {
                case 0: return 1;
                case 1: return 0;
            }
            break;
    }
    
    return -1;
}

inline void rayGetVertexClippingPlane(SRayVertex* &vtxA, const SRayVertex* vtxB)
{
    RAYfloat distA      = vtxB->pos.z - vtxA->pos.z;    // Distance between vertex A and B
    RAYfloat distB      = vtxB->pos.z - rayPlaneNear;   // Distance between vertex B and near plane
    RAYfloat distFactor = distB / distA;
    
    SRayVector2f dir;
    
    dir.x = vtxA->pos.x - vtxB->pos.x;
    dir.y = vtxA->pos.y - vtxB->pos.y;
    
    vtxA->pos.x = vtxB->pos.x + dir.x * distFactor;
    vtxA->pos.y = vtxB->pos.y + dir.y * distFactor;
    vtxA->pos.z = rayPlaneNear;
    
    dir.x = vtxA->map[0].x - vtxB->map[0].x;
    dir.y = vtxA->map[0].y - vtxB->map[0].y;
    
    vtxA->map[0].x = vtxB->map[0].x + dir.x * distFactor;
    vtxA->map[0].y = vtxB->map[0].y + dir.y * distFactor;
}

inline void rayComputeMipMapLevel()
{
    // Pixel face size
    RAYfloat psize = fabs(mcr_rayComputeTriangleFaceSize(
        rcVertex[0]->pos, rcVertex[1]->pos, rcVertex[2]->pos
    ));
    
    SRayVector2f mapa, mapb, mapc;
    
    mapa.x = rcVertex[0]->map[rcTextureLayer].x * rcTexture[rcTextureLayer]->width;
    mapa.y = rcVertex[0]->map[rcTextureLayer].y * rcTexture[rcTextureLayer]->height;
    
    mapb.x = rcVertex[1]->map[rcTextureLayer].x * rcTexture[rcTextureLayer]->width;
    mapb.y = rcVertex[1]->map[rcTextureLayer].y * rcTexture[rcTextureLayer]->height;
    
    mapc.x = rcVertex[2]->map[rcTextureLayer].x * rcTexture[rcTextureLayer]->width;
    mapc.y = rcVertex[2]->map[rcTextureLayer].y * rcTexture[rcTextureLayer]->height;
    
    // Texel face size
    RAYfloat tsize = fabs(mcr_rayComputeTriangleFaceSize(mapa, mapb, mapc));
    
    // MIP-map level
    rcTextureMipMapLevel = (RAYint)(tsize / psize);
    
    rayClamp(rcTextureMipMapLevel, 0, 1024);
    
    // !TODO: optimize this part!
    RAYint c = 0;
    while ((rcTextureMipMapLevel >> 2) > 0)
    {
        rcTextureMipMapLevel >>= 2;
        ++c;
    }
    rcTextureMipMapLevel = c;
    
    rayClamp(rcTextureMipMapLevel, 0, rcTexture[rcTextureLayer]->mipcount - 1);
}

inline void rayUpdateMappingGen(const RAYint &GenType, const RAYfloat &pos, const RAYfloat &nml, RAYfloat &map)
{
    switch (GenType)
    {
        case RAY_OBJECT_LINEAR:
            break;
        case RAY_EYE_LINEAR:
            map = pos;
            break;
        case RAY_SPHERE_MAP:
            map = nml / 2 + 0.5;
            break;
    }
}




#endif







// ================================================================================
