/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for cameras
 *
 */

#ifndef __SP_SCENEMANAGER_CAMERA_H__
#define __SP_SCENEMANAGER_CAMERA_H__


#include "spStandard.hpp"
#include "spDimension.hpp"
#include "spSceneManagerNode.hpp"
#include "spSceneManagerEntity.hpp"

#include <vector>


namespace sp
{
namespace scene
{


static const f32 CAMERADEF_PICKING_LENGTH   = -1.0f;
static const f32 CAMERADEF_RANGE_NEAR       = 1.0f;
static const f32 CAMERADEF_RANGE_FAR        = 1000.0f;


class SP_EXPORT Camera : public Node
{
    
    public:
        
        Camera();
        ~Camera();
        
        void updatePerspective();
        
        void setPerspective(dim::rect2di Viewport, f32 NearRange, f32 FarRange, f32 ZoomAngle = DEF_PERSPECTIVE_FOV);
        void getPerspective(dim::rect2di &Viewport, f32 &NearRange, f32 &FarRange, f32 &ZoomAngle);
        
        inline void setRange(f32 NearRange, f32 FarRange)
        {
            setPerspective(Viewport_, NearRange, FarRange, ZoomAngle_);
        }
        inline void getRange(f32 &NearRange, f32 &FarRange) const
        {
            NearRange   = NearRange_;
            FarRange    = FarRange_;
        }
        
        inline void setZoom(f32 ZoomAngle)
        {
            setPerspective(Viewport_, NearRange_, FarRange_, ZoomAngle);
        }
        inline f32 getZoom() const
        {
            return ZoomAngle_;
        }
        
        dim::point2di getProjection(dim::vector3df Position);
        
        dim::line3df getPickingLine(dim::point2di Position, f32 Length = CAMERADEF_PICKING_LENGTH);
        
        void lookAt(dim::vector3df Position, bool isGlobal = false);
        
        /* Matrix transformation functions */
        
        void updateViewmode();
        
        void updateModelviewMatrix();
        
        /* Inline functions */
        
        inline void setPerspectiveMatrix(const dim::matrix4f &Matrix)
        {
            PerspectiveMatrix_ = Matrix;
        }
        inline dim::matrix4f getPerspectiveMatrix() const
        {
            return PerspectiveMatrix_;
        }
        
        inline void setOrtho(bool isOrtho)
        {
            isOrtho_ = isOrtho;
        }
        inline bool getOrtho() const
        {
            return isOrtho_;
        }
        
        inline void setMirrorMatrix(const dim::matrix4f &Matrix)
        {
            MirrorMatrix_ = Matrix;
        }
        inline dim::matrix4f getMirrorMatrix() const
        {
            return MirrorMatrix_;
        }
        
        inline void setMirror(bool isMirror)
        {
            isMirror_ = isMirror;
        }
        inline bool getMirror() const
        {
            return isMirror_;
        }
        
    private:
        
        /* Members */
        
        dim::matrix4f PerspectiveMatrix_;
        dim::matrix4f MirrorMatrix_;
        
        dim::rect2di Viewport_;
        f32 NearRange_, FarRange_;
        f32 ZoomAngle_;
        bool isOrtho_;
        bool isMirror_;
        
};


} // /namespace scene

} // /namespace sp


#endif







// ================================================================================
