/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for script extension (SoftPixel loading script 1.1)
 *
 */

#ifndef __SP_SCRIPTEXTENSION_H__
#define __SP_SCRIPTEXTENSION_H__


#include "spStandard.hpp"
#include "spInputOutputString.hpp"
#include "spInputOutputFileSystem.hpp"
#include "spVideoDriverTexture.hpp"
#include "spSensitiveSound.hpp"
#include "spSceneManager.hpp"

#include <list>
#include <vector>


namespace sp
{
namespace script
{


class SP_EXPORT ScriptLoader
{
    
    public:
        
        ScriptLoader();
        ScriptLoader(u32 TexMax, u32 SoundMax, u32 ModelMax);
        ~ScriptLoader();
        
        bool loadScriptFile(
            io::stringc Filename,
            scene::SceneManager* sceneMngr
        );
        
        inline void getTextureList(video::Texture** &TextureList)
        {
            TextureList = TextureList_;
        }
        inline void getSoundList(sound::Sound** &SoundList)
        {
            SoundList = SoundList_;
        }
        inline void getEntityList(scene::Entity** &EntityList)
        {
            EntityList = EntityList_;
        }
        
        void clearLists();
        
    private:
        
        /* Structures */
        
        struct SAttribute
        {
            io::stringc Name;
            f32 Value;
        };
        
        /* Functions */
        
        void readScript();
        
        bool readPartMessages();
        bool readPartIndices();
        bool readPartPathes();
        bool readPartLoading();
        
        /*void readPartLoadingTextures();
        void readPartLoadingSounds();
        void readPartLoadingModels();*/
        
        bool readAttributeList();
        
        /* Members */
        
        video::Texture** TextureList_;
        sound::Sound** SoundList_;
        scene::Entity** EntityList_;
        
        std::vector<SAttribute> AttribtueList_;
        
        io::stringc strLine_, trimLine_, stdPath_;
        
        sound::SensitiveSoundDevice* SoundLoader_;
        scene::SceneManager* SceneMngr_;
        
        io::FileLoader* hFileLoader_;
        io::File* hFile_;
        
        s32 texIndex_;
        s32 soundIndex_;
        s32 modelIndex_;
        s32 soundCount_;
        
        u32 TexMax_, SoundMax_, ModelMax_;
        
};


} // /namespace video

} // /namespace sp


#endif






// ================================================================================
