/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for GUI objects (buttons, text, lists ...)
 *
 */

#ifndef __SP_VIDEODRIVER_GUI_OBJECTS_H__
#define __SP_VIDEODRIVER_GUI_OBJECTS_H__


#include "spStandard.hpp"
#include "spDimension.hpp"
#include "spVideoDriverFont.hpp"
#include "spInputOutputString.hpp"
#include "spUserControl.hpp"

#include <list>


namespace sp
{
namespace video
{
namespace gui
{


extern Font* __spGUIfont;
extern UserControl* __spGUIcontrol;
extern dim::point2di __spGUIcurpos;
extern bool __spGUIraster;


/*
 * ========== GUI Objects ==========
 */

enum EGUIEventTypes
{
    // Window events
    WINDOW_CLOSE = 1,
    WINDOW_MINIMIZE,
    WINDOW_MAXIMIZE,
    WINDOW_NORMALIZE,
    
    // Object events
    BUTTON_CLICKED,
    
};

class SP_EXPORT GUIObject
{
    
    public:
        
        /* Title */
        
        inline void setTitle(io::stringc Title)
        {
            Title_ = Title;
        }
        inline io::stringc getTitle()
        {
            return Title_;
        }
        
        /* Position */
        
        inline void setPosition(dim::point2di Position)
        {
            OrigPosition_ = Position_ = Position;
        }
        inline dim::point2di getPosition()
        {
            return OrigPosition_;
        }
        
        /* Size */
        
        inline void setSize(dim::size2di Size)
        {
            Size_ = Size;
        }
        inline dim::size2di getSize()
        {
            return Size_;
        }
        
        /* Parameter */
        
        inline void setParam(s32 Param)
        {
            Param_ = Param;
        }
        inline s32 getParam()
        {
            return Param_;
        }
        
        /* Enable/ Disable */
        
        inline void setEnable(bool isEnable)
        {
            isEnable_ = isEnable;
        }
        inline bool getEnable()
        {
            return isEnable_;
        }
        
        /* Parents */
        
        inline void setParent(GUIObject* hParent)
        {
            hParent_ = hParent;
        }
        inline GUIObject* getParent()
        {
            return hParent_;
        }
        
        inline s32 getZAxis()
        {
            return ZAxis_;
        }
        
    protected:
        
        GUIObject();
        GUIObject(io::stringc Title, dim::point2di Position, dim::size2di Size, s32 Param);
        ~GUIObject();
        
        io::stringc Title_;
        dim::point2di OrigPosition_, Position_;
        dim::size2di Size_;
        s32 Event_, Param_;
        s32 ZAxis_;
        bool isEnable_;
        
    private:
        
        void initGUIObject();
        
        GUIObject* hParent_;
        
};


/*
 * ========== GUI Buttons (Normal, text, checkbox, option, frame) ==========
 */

#define __SP_GUIBTN_HIGHLIGHT_COLOR_NORMAL  color(64, 64, 255)
#define __SP_GUIBTN_HIGHLIGHT_COLOR_OVER    color(255, 222, 0)
#define __SP_GUIBTN_HIGHLIGHT_COLOR_CLICKED color(0, 222, 0)

enum EGUIButtonTypes
{
    BUTTON_NORMAL = 0,
    BUTTON_TEXT,
    BUTTON_CHECKBOX,
    BUTTON_OPTION,
    BUTTON_FRAME,
    BUTTON_IMAGE
};

enum EGUIButtonParamTypes
{
    BUTTON_TOGGLE   = 0x0001,
    TEXT_RIGHT      = 0x0002,
    TEXT_CENTER     = 0x0004
};

class SP_EXPORT GUIButton : public GUIObject
{
    
    public:
        
        GUIButton();
        GUIButton(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param);
        ~GUIButton();
        
        s32 update(dim::point2di WndPosition, bool isWinRect);
        void render();
        
        // Option functions
        
        /* Types */
        
        void setType(s32 Type)
        {
            Type_ = Type;
        }
        s32 getType()
        {
            return Type_;
        }
        
        /* Color */
        
        inline void setColor(color Color)
        {
            bgColor_ = Color;
        }
        inline color getColor()
        {
            return bgColor_;
        }
        
        /* Image */
        
        void setImage(Texture* hTexture)
        {
            hTexture_ = hTexture;
        }
        Texture* getImage()
        {
            return hTexture_;
        }
        
        void setImageRect(dim::rect2df ClipRect)
        {
            TextureClip_ = ClipRect;
        }
        dim::rect2df getImageRect()
        {
            return TextureClip_;
        }
        
    private:
        
        // Functions
        
        void init();
        
        bool drawButton(dim::point2di Position, dim::size2di Size);
        bool drawCheckbox(dim::point2di Position, dim::size2di Size);
        bool drawOption(dim::point2di Position, dim::size2di Size);
        
        // Members
        
        s32 Type_;
        bool isToggled_;
        color bgColor_, HighLightColor_;
        Texture* hTexture_;
        dim::rect2df TextureClip_;
        
};


/*
 * ========== GUI Editbox (string, editor) ==========
 */

class SP_EXPORT GUIEditbox : public GUIObject
{
    
    public:
        
        GUIEditbox();
        GUIEditbox(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param);
        ~GUIEditbox();
        
        s32 update(dim::point2di WndPosition, bool isWinRect);
        void render();
        
    private:
        
        // Members
        
        std::list<io::stringc> ItemList_;
        
        u8 Type_;
        s32 Scroll_X_Thumb_, Scroll_X_Max_;
        s32 Scroll_Y_Thumb_, Scroll_Y_Max_;
        
};


/*
 * ========== GUI List (normal, iconed) ==========
 */

class SP_EXPORT GUIList : public GUIObject
{
    
    public:
        
        GUIList();
        GUIList(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param);
        ~GUIList();
        
        void addItem(io::stringc Title, s32 Position = -1, s32 SubItem = 0);
        void removeItem(s32 Position);
        void setItemTitle(s32 Position, io::stringc Title);
        io::stringc getItemTitle(s32 Position);
        
        s32 update(dim::point2di WndPosition, bool isWinRect);
        void render();
        
    private:
        
        // Structure
        
        typedef struct SGUIListItemInfo
        {
            u32 ID;
            io::stringc Title;
        }
        itemInfo;
        
        // Functions
        
        void thumbInfoUpdate();
        
        // Members
        
        std::list<itemInfo> ItemList_;
        
        u8 Type_;
        s32 Scroll_X_Thumb_, Scroll_X_Max_;
        s32 Scroll_Y_Thumb_, Scroll_Y_Max_;
        
};


/*
 * ========== GUI Menu ==========
 */

#define __SP_GUIMENU_BORDERWIDTH    5
#define __SP_GUIMENU_HEIGHT         20
#define __SP_GUIMENU_LASTITEM       0

enum EGUIMenuTypes
{
    MENU_CHAPTER = 0,
    MENU_SUBITEM,
    MENU_ITEM,
};

class SP_EXPORT GUIMenu
{
    
    public:
        
        GUIMenu();
        GUIMenu(s32 Type, io::stringc Title, GUIMenu* hParent);
        ~GUIMenu();
        
        s32 update(dim::point2di WndPosition, bool isWinRect);
        void render();
        
        // Option functions
        
        inline void setParent(GUIMenu* hParent)
        {
            hParent_ = hParent;
        }
        inline GUIMenu* getParent()
        {
            return hParent_;
        }
        
        // (The following functions are bot very useful for users)
        
        inline dim::point2di getPosition()
        {
            return Position_;
        }
        inline dim::size2di getSize()
        {
            return Size_;
        }
        inline s32 getType()
        {
            return Type_;
        }
        inline s32 getCategorySub() const
        {
            return CategorySub_;
        }
        inline s32 getCategoryChapter() const
        {
            return CategoryChapter_;
        }
        
        static void calcLocation(std::list<GUIMenu*> &WndMenuList);
        
    private:
        
        // Functions
        
        void init();
        
        // Members
        
        bool isVisible_;
        s32 Type_;
        s32 CategorySub_, CategoryChapter_;
        io::stringc Title_;
        dim::point2di OrigPosition_, Position_;
        dim::size2di Size_;
        GUIMenu* hParent_;
        
};


/*
 * ========== GUI Windows ==========
 */

#define __SP_GUIWND_TITLE_HEIGHT    20
#define __SP_GUIWND_TITLE_POSX      10
#define __SP_GUIWND_TITLE_POSY      3
#define __SP_GUIWND_TITLE_COLOR_LF  color(128, 128, 255)
#define __SP_GUIWND_TITLE_COLOR_RT  color(222, 222, 255)
#define __SP_GUIWND_MIN_SIZEX       50
#define __SP_GUIWND_BTN_SIZE        10
#define __SP_GUIWND_BTN_DIST        3
#define __SP_GUIWND_RESIZE_SIZE     15
#define __SP_GUIWND_STATUS_HEIGHT   15
#define __SP_GUIWND_RASTER_SIZE     10

enum EGUIWindowTypes
{
    WINDOW_EMPTY        = 0x0000,
    WINDOW_BUTTON_X     = 0x0001,
    WINDOW_BUTTON_MIN   = 0x0002,
    WINDOW_BUTTON_MAX   = 0x0004,
    WINDOW_BUTTON_SIZE  = 0x0008,
    WINDOW_STATUSBAR    = 0x0010,
    WINDOW_SCROLL_VERT  = 0x0020,
    WINDOW_SCROLL_HORZ  = 0x0040,
    WINDOW_CENTERED     = 0x0080,
    WINDOW_MINIMIZED    = 0x0100,
    WINDOW_MAXIMIZED    = 0x0200
};

enum EGUIStatusBarTypes
{
    STATUSBAR_LEFT = 0,
    STATUSBAR_RIGHT,
    STATUSBAR_CENTER
};

class SP_EXPORT GUIWindow : public GUIObject
{
    
    public:
        
        GUIWindow();
        GUIWindow(io::stringc Title, dim::point2di Position, dim::size2di Size, s32 Param);
        ~GUIWindow();
        
        GUIButton* createButton(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param = 0);
        GUIEditbox* createEditbox(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param = 0);
        GUIList* createList(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param = 0);
        
        bool deleteButton(GUIButton* hButton);
        bool deleteEditbox(GUIEditbox* hEditlist);
        bool deleteList(GUIList* hList);
         
        void createMenu();
        void deleteMenu();
        GUIMenu* addMenuItem(s32 Type, io::stringc Title, GUIMenu* hParent = __SP_GUIMENU_LASTITEM);
        bool removeMenuItem(GUIMenu* hMenuItem);
        
        s32 update();
        void render();
        
        // Option functions
        
        /* Dimensions */
        
        inline void setMinDimension(dim::size2di Size)
        {
            MinSize_ = Size;
        }
        inline dim::size2di getMinDimension()
        {
            return MinSize_;
        }
        
        inline void setMaxDimension(dim::size2di Size)
        {
            MaxSize_ = Size;
        }
        inline dim::size2di getMaxDimension()
        {
            return MaxSize_;
        }
        
        /* Color */
        
        inline void setColor(color Color)
        {
            bgColor_ = Color;
        }
        inline color getColor()
        {
            return bgColor_;
        }
        
        /* Statusbar */
        
        void setStatusbarType(u8 Type);
        inline u8 getStatusbarType()
        {
            return StatusBarType_;
        }
        
        inline void setStatusbarText(io::stringc Text)
        {
            StatusBarText_ = Text;
        }
        inline io::stringc getStatusbarText()
        {
            return StatusBarText_;
        }
        
        void updateZAxis(s32 FocusedWindowZ); // (Not very useful for users)
        
    private:
        
        /* Enumerations */
        
        enum EWndEventTypes
        {
            WNDEVENT_NONE = 0,
            WNDEVENT_MOVING,
            WNDEVENT_RESIZE
        };
        
        /* Private functions */
        
        void init();
        void clear();
        void renderEmptyWindow();
        void updateWindowEvents();
        void updateResize();
        void setParameter(s32 Param);
        bool drawButton(dim::point2di Position);
        
        /* Members */
        
        std::list<GUIButton*>   ButtonList_;
        std::list<GUIEditbox*>  EditboxList_;
        std::list<GUIList*>     ListItemList_;
        std::list<GUIMenu*>     MenuList_;
        
        s32 WndEventMode_;
        dim::point2di WndMovingPos_;
        
        bool isButtonX_, isButtonMin_, isButtonMax_, isButtonSize_;
        bool isStatusBar_, isMenu_, isScrollVert_, isScrollHorz_;
        
        s32 TitleBarLength_, MaxTitleBarLength_;
        s32 ScrollVertState_, ScrollHorzState_;
        s32 ScrollVertMax_, ScrollHorzMax_;
        s32 ScrollVertLength_, ScrollHorzLength_;
        u8 StatusBarType_;
        s32 StatusBarPos_;
        io::stringc StatusBarText_;
        
        color bgColor_;
        color titleLfColor_, titleRtColor_;
        
        dim::size2di LastSize_;
        dim::point2di LastPosition_;
        dim::size2di MinSize_, MaxSize_;
        dim::rect2di RenderRect_;
        
};


// Sorting functions
bool sortGUIButtons(GUIButton* &obj1, GUIButton* &obj2);
bool sortGUIEditboxes(GUIEditbox* &obj1, GUIEditbox* &obj2);
bool sortGUILists(GUIList* &obj1, GUIList* &obj2);

extern GUIWindow* __spGUIForegroundWindow; // ! actually unused !
extern GUIWindow* __spGUIFocusedWindow;
extern GUIObject* __spGUIFocusedObject;


} // /namespace gui

} // /namespace video

} // /namespace sp


#endif






// ================================================================================
