//
// SoftPixel Engine - Fur effect (8.10.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp;

#include "../../help/tutorials/common.hpp"

//#define FLAGTEST

void recreateFlag(scene::Entity* obj, const s32 c, const f32 r, const f32 h = 0.025);

int main()
{
    SoftPixelDevice* spDevice = createDevice(
        ChooseRenderer(), dim::size2di(800, 600), 32, "Fur effect (25 layer)", 0, SDeviceFlags(0, 1, 1, 8)
    );
    
    video::VideoDriver* spDriver = spDevice->getVideoDriver();
    scene::SceneManager* spSmngr = spDevice->getSceneManager();
    UserControl* spControl = spDevice->getUserControl();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    scene::Camera* cam = spSmngr->createCamera();
    scene::Light* light = spSmngr->createLight();
    spSmngr->setLighting();
    cam->setRange(0.1, 100.0);
    
    video::Texture* tex = spDriver->loadTexture("Fur5.jpg");
    video::Texture* tex2 = spDriver->copyTexture(tex);
    
    video::Font* font = spDriver->loadFont();
    
    #ifndef FLAGTEST
    
    math::SeedRandom();
    tex2->setType(video::TEXTURE_RGBA);
    for (s32 y = 0, x; y < tex2->getSize().Height; ++y)
    {
        for (x = 0; x < tex2->getSize().Width; ++x)
        {
            if (math::Rnd(2) != 1) // Rnd: 1 - 3
            {
                video::color Color( tex2->getPixelColor(dim::point2di(x, y)) );
                Color.Alpha = 0;
                tex2->setPixelColor(dim::point2di(x, y), Color);
            }
        }
    }
    tex2->updateImageData(video::SET_IMAGEDATA);
    
    scene::Entity* srcobj = spSmngr->createModel(scene::ENTITY_TORUS, 20);
    srcobj->setPosition(dim::vector3df(0, 0, 4));
    srcobj->setScale(2);
    
    srcobj->setNormalsType(scene::NORMALS_SMOOTH);
    srcobj->updateNormals();
    
    srcobj->addTexture(tex);
    srcobj->doTextureTransform(0, dim::size2df(4, 2));
    
    scene::Entity* obj[25];
    
    dim::vector3df Coord, Normal;
    s32 LayerCount = 25;
    f32 Radius = 0.2;
    
    for (s32 i = 0, j; i < LayerCount; ++i)
    {
        obj[i] = spSmngr->copyNode(srcobj);
        obj[i]->setParent(srcobj);
        obj[i]->setPosition(0);
        obj[i]->setScale(1);
        
        obj[i]->setColorMaterial(false);
        
        if (spDriver->getDriverType() == video::DRIVER_SOFTWARE)
            obj[i]->setColor(video::color(255, 255, 255, 255 - 255 * (i+1) / LayerCount), 0, true);
        else
            obj[i]->getThisMaterial().ColorDiffuse.Alpha = 255 - 255 * (i+1) / LayerCount;
        
        for (j = 0; j < obj[i]->getMeshVerticesCount(); ++j)
        {
            Coord   = srcobj->getVertexCoord(j);
            Normal  = srcobj->getVertexNormal(j);
            
            Coord += Normal * (Radius * (f32)(i+1) / LayerCount);
            
            obj[i]->setVertexCoord(j, Coord);
            obj[i]->setTexture(0, tex2);
        }
        
        obj[i]->updateModeling(false);
        
        //obj[i]->setWireframe(video::WIREFRAME_POINTS);
        
        obj[i]->setVisible(false);
    }
    
    //spSmngr->createModel(scene::ENTITY_CUBE, 0, 1)->doMove(dim::vector3df(0, -1, 4));
    
    #else
    
    const s32 c = 100;
    
    scene::Entity* srcobj = spSmngr->createModel(scene::ENTITY_PLANE, c);
    
    srcobj->setPosition(dim::vector3df(0, 0, 3));
    srcobj->setScale(3);
    
    srcobj->setRenderFace(video::RENDER_BOTH);
    srcobj->setWireframe(video::WIREFRAME_SOLID, video::WIREFRAME_LINES);
    
    recreateFlag(srcobj, c, 35);
    
    //srcobj->setColor(video::color(255, 255, 0));
    
    //srcobj->setWireframe(video::WIREFRAME_LINES);
    
    scene::Light* spot = spSmngr->createLight();
    spot->setPosition(dim::vector3df(1, 0, 2));
    spot->setVolumetric(true);
    spot->setLightingColor(video::color(255, 0, 0));
    spot->setVolumetricRadius(1.0);
    
    scene::Light* spot2 = spSmngr->createLight();
    spot2->setPosition(dim::vector3df(-1, 1, 3.5));
    spot2->setVolumetric(true);
    spot2->setLightingColor(video::color(255, 255, 0));
    spot2->setVolumetricRadius(1.0);
    
    #   if 0
    light->setVisible(false);
    #   else
    spot->setVisible(false);
    spot2->setVisible(false);
    #   endif
    
    //#   define SEQUENCE
    #   ifdef SEQUENCE
    
    sound::SensitiveSoundDevice* spListener = sound::createListener();
    
    video::movie::MovieManager* spCinema = spDriver->getMovieManager();
    
    s32 seqImgSize = 384;//256
    video::Texture* texMovie = spDriver->createTexture(dim::size2di(seqImgSize));
    video::movie::Movie* movie = spCinema->loadMovie(
        //"D:/Lukas Daten/Filme/LangFilme/Spongebob Schwammkopf/Season 2 - 2000/SpongeBob_SquarePants_-_2x13_-_B_-_Die_Wanne_Ist_Voll.german.dreitausend1.[www.the-realworld.de]..avi", seqImgSize);
        "D:/Lukas Daten/Filme/LangFilme/Spongebob Schwammkopf/Season 1 - 1999/SpongeBob_SquarePants_-_1x01_-_A_-_Aushilfe_Gesucht.german.3001.[www.the-realworld.de].divx.avi", seqImgSize);
    spCinema->playMovie(movie);
    
    sound::Sound* seqMusic = spListener->loadSound("D:/Lukas Daten/Filme/LangFilme/Spongebob Schwammkopf/SpongeBobMusicBackground2.mp3");
    spListener->setSoundVolume(seqMusic, 0.5);
    spListener->playSound(seqMusic);
    //spListener->setSoundSpeed(seqMusic, 1.0);
    
    texMovie->setMipMapFilter(video::TEXTURE_SMOOTH, video::TEXTURE_SMOOTH, false);//setMipmapping(false);
    
    srcobj->addTexture(texMovie);
    
    dim::point2df map;
    for (s32 y = 0, x, i = 0; y <= c; ++y)
    {
        for (x = 0; x <= c; ++x, ++i)
        {
            map = srcobj->getVertexTexCoord(i);
            map.Y = -map.Y;
            srcobj->setVertexTexCoord(i, map);
        }
    }
    
    #   endif
    
    #endif
    
    spDriver->setClearColor(255);
    
    dim::vector3df Rot;
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        spSmngr->renderScene();
        
        #ifdef SEQUENCE
        spCinema->renderMovieTexture(movie, texMovie);
        #endif
        
        #ifndef FLAGTEST
        
        if (spControl->isKeyHit(KEY_SPACE))
        {
            static bool isShow;
            isShow = !isShow;
            for (s32 i = 0; i < LayerCount; ++i)
                obj[i]->setVisible(isShow);
        }
        
        for (s32 i = 0; i < LayerCount; ++i)
        {
            obj[i]->setRotation(
                dim::vector3df(
                    Rot.X * (f32)i / LayerCount,
                    Rot.Y * (f32)i / LayerCount,
                    Rot.Z * (f32)i / LayerCount
                )
            );
        }
        
        #endif
        
        #ifdef FLAGTEST
        static f32 d;
        #   if 0
        d += 2.5;
        recreateFlag(srcobj, c, SIN(d)*15+25);
        #   else
        d += 1.5;
        recreateFlag(srcobj, c, SIN(d)*25+25);
        #   endif
        #endif
        
        if (spControl->isMouseButtonPressed(MOUSE_LEFT))
            srcobj->doTurn(dim::vector3df(1, 1, 1));
        else if (spControl->isMouseButtonPressed(MOUSE_RIGHT))
            Rot.Y -= (f32)spControl->getCursorSpeed().X / 25;
        else
            srcobj->doTurn(dim::vector3df(-(f32)spControl->getCursorSpeed().Y / 2, -(f32)spControl->getCursorSpeed().X / 2, 0));
        
        spDriver->beginDrawing2D();
        {
            io::stringc Text("See the power of the \"SoftPixel Engine\"");
            dim::size2di TexDim(font->getStringSize(Text));
            spDriver->draw2DText(font, dim::point2di(400 - TexDim.Width/2, 50), Text, 0);
            
            Text = "(left mouse -> turn auto. / right mouse -> turn hairs / space -> fur on/off)";
            TexDim = font->getStringSize(Text);
            spDriver->draw2DText(font, dim::point2di(400 - TexDim.Width/2, 75), Text, 0);
            
            spDriver->draw2DText(font, dim::point2di(25, 50), io::stringc("FPS: ") + spDevice->getFPS(), 0);
            spDriver->draw2DText(font, dim::point2di(25, 70), io::stringc("Count of triangles: ") + spSmngr->getSceneTriangleCount(), 0);
        }
        spDriver->endDrawing2D();
        
        spDriver->flipBuffers();
    }
    
    deleteDevice();
    
    return 0;
}

void recreateFlag(scene::Entity* obj, const s32 c, const f32 r, const f32 h)
{
    s32 x, y, i = 0;
    f32 posx, posy;
    f32 d;
    
    dim::vector3df v;
    
    for (y = 0; y <= c; ++y)
    {
        for (x = 0; x <= c; ++x, ++i)
        {
            v = obj->getVertexCoord(i);
            
            posx = ( (f32)x - (f32)c/2 ) / c * 100;
            posy = ( (f32)y - (f32)c/2 ) / c * 100;
            
            d = sqrt(posx*posx + posy*posy)*r;
            
            v.Y = SIN(d) * h;
            
            obj->setVertexCoord(i, v);
        }
    }
    
    obj->updateNormals();
}





// ================================
