//
// SoftPixel Engine - BoxPong5 - (31.3.2008)
//

#include <SoftPixelEngine.hpp>

using namespace sp; // Main namespace

#include "../../help/tutorials/common.hpp"

// INITIALIZATION

SoftPixelDevice* spDevice                   = 0;

video::VideoDriver* spDriver                = 0;
scene::SceneManager* spSmngr                = 0;
UserControl* spControl                      = 0;
scene::CollisionDetector* spCmngr           = 0;
scene::Collision* spColl                    = 0;
scene::StencilManager* spSclmngr            = 0;

// BALLS

#define BOX_SIZE    5.0f
#define WALL_RADIUS (BOX_SIZE/2 - 0.5f)

class Ball
{
    
    public:
        
        Ball(dim::vector3df Direction, dim::vector3df Position);
        ~Ball();
        
        void update();
        
    private:
        
        scene::Entity* Object_;
        dim::vector3df Direction_;
        
};

Ball::Ball(dim::vector3df Direction, dim::vector3df Position)
{
    Object_ = spSmngr->createModel(scene::ENTITY_SPHERE, 15);
    
    Object_->setScale(0.75f);
    
    Object_->setColorMaterial(false);
    Object_->setColor(video::color(math::Rnd(255), math::Rnd(255), math::Rnd(255)));
    Object_->setShininess(0.75);
    
    spSclmngr->addEntityCastCloud(Object_);
    
    if (Direction == dim::vector3df())
    {
        Direction_ = dim::vector3df(
            (f32)math::Random(200) / 100 - 1.0f,
            (f32)math::Random(200) / 100 - 1.0f,
            (f32)math::Random(200) / 100 - 1.0f
        );
    }
    else
    {
        Direction_ = Direction;
        if (Position != dim::vector3df())
        {
            Object_->setPosition(Position);
        }
    }
    
    Direction.normalize();
    Object_->doTranslate(Direction_*2);
    spCmngr->addCollisionEntity(Object_, spColl, 0.75f/2);
}
Ball::~Ball()
{
    spSmngr->deleteNode(Object_);
}

void Ball::update()
{
    Object_->doTranslate(Direction_ / 15);
    
    /* Check the wall collision */
    if (Object_->getPosition().X < -WALL_RADIUS)
    {
        Direction_.X = -Direction_.X;
        Object_->setPosition(dim::vector3df(-WALL_RADIUS, Object_->getPosition().Y, Object_->getPosition().Z));
    }
    else if (Object_->getPosition().X > WALL_RADIUS)
    {
        Direction_.X = -Direction_.X;
        Object_->setPosition(dim::vector3df(WALL_RADIUS, Object_->getPosition().Y, Object_->getPosition().Z));
    }
    
    if (Object_->getPosition().Y < -WALL_RADIUS)
    {
        Direction_.Y = -Direction_.Y;
        Object_->setPosition(dim::vector3df(Object_->getPosition().X, -WALL_RADIUS, Object_->getPosition().Z));
    }
    else if (Object_->getPosition().Y > WALL_RADIUS)
    {
        Direction_.Y = -Direction_.Y;
        Object_->setPosition(dim::vector3df(Object_->getPosition().X, WALL_RADIUS, Object_->getPosition().Z));
    }
    
    if (Object_->getPosition().Z < -WALL_RADIUS)
    {
        Direction_.Z = -Direction_.Z;
        Object_->setPosition(dim::vector3df(Object_->getPosition().X, Object_->getPosition().Y, -WALL_RADIUS));
    }
    else if (Object_->getPosition().Z > WALL_RADIUS)
    {
        Direction_.Z = -Direction_.Z;
        Object_->setPosition(dim::vector3df(Object_->getPosition().X, Object_->getPosition().Y, WALL_RADIUS));
    }
    
    /* Check the collision */
    if (Object_->detectCollision(0))
    {
        Direction_ = - Object_->getCollisionNormal();
    }
}

std::vector<Ball*> BallsList;

void AddBall(dim::vector3df Direction = dim::vector3df(), dim::vector3df Position = dim::vector3df())
{
    BallsList.push_back(new Ball(Direction, Position));
}
void UpdateBalls()
{
    for (std::vector<Ball*>::iterator it = BallsList.begin(); it != BallsList.end(); ++it)
        (*it)->update();
}
void ClearBalls()
{
    for (std::vector<Ball*>::iterator it = BallsList.begin(); it != BallsList.end(); ++it)
        spDeletePointer(*it);
}

// /BALLS

int main()
{
    
    const s32 ScrWidth = 640, ScrHeight = 480;
    
    spDevice    = createDevice(
        ChooseRenderer(), dim::size2di(ScrWidth, ScrHeight), 32, "Rumble Balls", false
        //,SDeviceFlags(false, true, true, 8)
    );
    
    spDriver    = spDevice->getVideoDriver();
    spSmngr     = spDevice->getSceneManager();
    spControl   = spDevice->getUserControl();
    spCmngr     = spDevice->getCollisionDetector();
    
    spSclmngr   = spSmngr->getStencilManager();
    
    spDevice->setWindowTitle(
        spDevice->getWindowTitle() + io::stringc(" [ ") + spDriver->getVersion() + io::stringc(" ]")
    );
    
    scene::Camera* cam = spSmngr->createCamera();
    cam->setPosition(dim::vector3df(0, 3.5, -7));
    cam->lookAt(0);
    cam->doMove(dim::vector3df(0, 0, 3.5));
    cam->setRange(0.1, 100.0);
    
    scene::Light* light = spSmngr->createLight(scene::LIGHT_POINT);
    light->setPosition(dim::vector3df(5, 8, -5));
    //spSclmngr->addLightSource(light);
    
    spSmngr->setLighting();
    spSmngr->setStencilEffects(); // !!! WARNING: BIG ERROR WITH TO MUCH CAST CLOUD OBJECTS !!!
    
    light->setLightingColor(255, 222, 255);
    
    scene::Entity* cube = spSmngr->createModel(scene::ENTITY_CUBE);
    cube->setScale(BOX_SIZE);
    cube->flipMesh();
    
    math::SeedRandom();
    
    std::vector<u32> DestIDList; DestIDList.push_back(0);
    std::vector<s32> CollTypeList; CollTypeList.push_back(scene::COLLISION_SPHERE_TO_SPHERE);
    spColl = spCmngr->createCollision(0, CollTypeList, DestIDList);
    
    for (s32 i = 0; i < 25; ++i)
        AddBall();
    
    spSclmngr->addLightSource(new scene::SShadowLightSource(light));
    
    video::Texture* spTex = spDriver->createTexture(dim::size2di(1024));
    
    while (spDevice->updateEvent() && !spControl->isKeyPressed(KEY_ESCAPE))
    {
        spDriver->clearBuffers();
        
        spCmngr->updateScene();
        spSmngr->renderScene();
        
        #if 0
        if (spTex && spDriver->getDriverType() == video::DRIVER_OPENGL)
        {
            spDriver->beginDrawing2D();
            spDriver->draw2DImage(
                spTex, dim::rect2di(0, 0, 1024, 1024), dim::rect2df(0, 1, 1, 0), video::color(255, 255, 255, 160)
            );
            spDriver->endDrawing2D();
            spDriver->createScreenShot(spTex);
        }
        #endif
        
        UpdateBalls();
        
        spDriver->flipBuffers();
    }
    ClearBalls();
    
    spDevice->deleteDevice();
    
    return 0;
}




// ======================================================
