/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * ViewFrustum header
 *
 */

#ifndef __SP_DIMENSION_VIEWFRUSTUM_H__
#define __SP_DIMENSION_VIEWFRUSTUM_H__


#include "spStandard.hpp"
#include "spDimensionVector3D.hpp"
#include "spDimensionLine3D.hpp"
#include "spDimensionPlane3D.hpp"


namespace sp
{
namespace dim
{


enum EViewFrustumTypes
{
    VIEWFRUSTUM_FAR = 0,
    VIEWFRUSTUM_NEAR,
    VIEWFRUSTUM_LEFT,
    VIEWFRUSTUM_RIGHT,
    VIEWFRUSTUM_BOTTOM,
    VIEWFRUSTUM_TOP,
    
    VIEWFRUSTUM_PLANE_COUNT,
};


class ViewFrustum
{
    
    public:
        
        ViewFrustum()
        {
        }
        ViewFrustum(matrix4f &Matrix)
        {
            setFrustum(Matrix);
        }
        ~ViewFrustum()
        {
        }
        
        /* Extra functions */
        
        inline void setFrustum(matrix4f &Matrix)
        {
            // Left plane
            Planes[VIEWFRUSTUM_LEFT].Normal.X   = Matrix[ 3] + Matrix[ 0];
            Planes[VIEWFRUSTUM_LEFT].Normal.Y   = Matrix[ 7] + Matrix[ 4];
            Planes[VIEWFRUSTUM_LEFT].Normal.Z   = Matrix[11] + Matrix[ 8];
            Planes[VIEWFRUSTUM_LEFT].Distance   = Matrix[15] + Matrix[12];
            
            // Right plane
            Planes[VIEWFRUSTUM_RIGHT].Normal.X  = Matrix[ 3] - Matrix[ 0];
            Planes[VIEWFRUSTUM_RIGHT].Normal.Y  = Matrix[ 7] - Matrix[ 4];
            Planes[VIEWFRUSTUM_RIGHT].Normal.Z  = Matrix[11] - Matrix[ 8];
            Planes[VIEWFRUSTUM_RIGHT].Distance  = Matrix[15] - Matrix[12];
            
            // Top plane
            Planes[VIEWFRUSTUM_TOP].Normal.X    = Matrix[ 3] - Matrix[ 1];
            Planes[VIEWFRUSTUM_TOP].Normal.Y    = Matrix[ 7] - Matrix[ 5];
            Planes[VIEWFRUSTUM_TOP].Normal.Z    = Matrix[11] - Matrix[ 9];
            Planes[VIEWFRUSTUM_TOP].Distance    = Matrix[15] - Matrix[13];
            
            // Bottom plane
            Planes[VIEWFRUSTUM_BOTTOM].Normal.X = Matrix[ 3] + Matrix[ 1];
            Planes[VIEWFRUSTUM_BOTTOM].Normal.Y = Matrix[ 7] + Matrix[ 5];
            Planes[VIEWFRUSTUM_BOTTOM].Normal.Z = Matrix[11] + Matrix[ 9];
            Planes[VIEWFRUSTUM_BOTTOM].Distance = Matrix[15] + Matrix[13];
            
            // Far plane
            Planes[VIEWFRUSTUM_FAR].Normal.X    = Matrix[ 3] - Matrix[ 2];
            Planes[VIEWFRUSTUM_FAR].Normal.Y    = Matrix[ 7] - Matrix[ 6];
            Planes[VIEWFRUSTUM_FAR].Normal.Z    = Matrix[11] - Matrix[10];
            Planes[VIEWFRUSTUM_FAR].Distance    = Matrix[15] - Matrix[14];
            
            // Near plane
            Planes[VIEWFRUSTUM_NEAR].Normal.X   = Matrix[ 2];
            Planes[VIEWFRUSTUM_NEAR].Normal.Y   = Matrix[ 6];
            Planes[VIEWFRUSTUM_NEAR].Normal.Z   = Matrix[10];
            Planes[VIEWFRUSTUM_NEAR].Distance   = Matrix[14];
            
            // Normalize all plane normals
            for (s32 i = 0; i < VIEWFRUSTUM_PLANE_COUNT; ++i)
                Planes[i].Normal.normalize();
        }
        
        inline vector3df getLeftUp()
        {
            vector3df Point;
            Planes[VIEWFRUSTUM_FAR].getMultiplePlaneIntersection(Planes[VIEWFRUSTUM_TOP], Planes[VIEWFRUSTUM_LEFT], Point);
            return Point;
        }
        
        inline vector3df getLeftDown()
        {
            vector3df Point;
            Planes[VIEWFRUSTUM_FAR].getMultiplePlaneIntersection(Planes[VIEWFRUSTUM_BOTTOM], Planes[VIEWFRUSTUM_LEFT], Point);
            return Point;
        }
        
        inline vector3df getRightUp()
        {
            vector3df Point;
            Planes[VIEWFRUSTUM_FAR].getMultiplePlaneIntersection(Planes[VIEWFRUSTUM_TOP], Planes[VIEWFRUSTUM_RIGHT], Point);
            return Point;
        }
        
        inline vector3df getRightDown()
        {
            vector3df Point;
            Planes[VIEWFRUSTUM_FAR].getMultiplePlaneIntersection(Planes[VIEWFRUSTUM_BOTTOM], Planes[VIEWFRUSTUM_RIGHT], Point);
            return Point;
        }
        
        /* Members */
        
        plane3df Planes[VIEWFRUSTUM_PLANE_COUNT];
        
        matrix4f Matrices[VIEWFRUSTUM_PLANE_COUNT];
        
};


} // /namespace dim

} // /namespace sp


#endif







// ================================================================================
