/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Direct3D9 Textures
 *
 */

#include "spDirect3D9Texture.hpp"
#include "spDirect3D9Driver.hpp"
#include "spTextureManager.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace video
{


#if defined(WIN32)

Direct3D9Texture::Direct3D9Texture()
    : Texture(), pDirect3DTexture_(0), pDirect3DVolumeTexture_(0)
{
}
Direct3D9Texture::Direct3D9Texture(
    LPDIRECT3DTEXTURE9 d3dTexture, io::stringc Filename, dim::size2di Size, u8* ImageData,
    s32 RenderMode, s32 FilterMode, s32 Format, s32 Type, bool isEdgeSmooth, bool isMipMapping)
    : Texture(1, Filename, Size, ImageData, RenderMode, FilterMode, Format, Type, isEdgeSmooth, isMipMapping),
    pDirect3DTexture_(d3dTexture), pDirect3DVolumeTexture_(0)
{
    /* Initialization */
    updateFormat();
    
    if (OriginalImageData_)
    {
        /* Update the image data */
        makeIdentityImageData();
        
        /* Recreate the image data */
        recreateImageData();
    }
}
Direct3D9Texture::~Direct3D9Texture()
{
    clear();
}

void Direct3D9Texture::changeAttributes(s32 RenderMode, s32 FilterMode, bool isEdgeSmooth)
{
    /* Set the new render mode */
    RenderMode_ = RenderMode;
    
    /* Set the new filter mode */
    FilterMode_ = FilterMode;
}

void Direct3D9Texture::setColorIntensity(f32 Red, f32 Green, f32 Blue)
{
    
}

void Direct3D9Texture::flipColors()
{
    /* Check if the image is not empty */
    if (!ImageData_ || hasAnimation())
        return;
    
    /* Determine the image data size */
    s32 ImageDataSize = Size_.Width * Size_.Height * InternalFormat_;
    
    /* Flip the image data */
    for (s32 i = 0; i < ImageDataSize; ++i)
        ImageData_[i] = 255 - ImageData_[i];
    
    /* Recreate the image */
    recreateImageData();
}

void Direct3D9Texture::setMipMapFilter(s32 MinFilter, s32 MagFilter, bool isMipMapping)
{
    /* Set the mipmapping state */
    isMipMapping_ = isMipMapping;
    
    /* Recreate the image */
    recreateImageData();
}

void Direct3D9Texture::updateImageData(EUpdateDataFunc UpdateDataFunc)
{
    /* Check if the image is not empty */
    if (ImageData_)
    {
        if (UpdateDataFunc == GET_IMAGEDATA)
            receiveImageData();
        else if (UpdateDataFunc == SET_IMAGEDATA)
            recreateImageData();
    }
}

void Direct3D9Texture::bind(s32 Level)
{
    static_cast<Direct3D9Driver*>(__spVideoDriver)->CurSamplerLevel_ = Level;
    
    static_cast<Direct3D9Driver*>(__spVideoDriver)->setTextureParameters(
        DimensionType_, RenderMode_, FilterMode_, AnisotropicSamples_, isMipMapping_, isEdgeSmooth_
    );
    
    static_cast<Direct3D9Driver*>(__spVideoDriver)->getDirect3DDevice()->SetTexture(Level, pDirect3DTexture_);
}

void Direct3D9Texture::unbind(s32 Level)
{
    static_cast<Direct3D9Driver*>(__spVideoDriver)->getDirect3DDevice()->SetTexture(Level, 0);
}


/*
 * ======= Private: =======
 */


void Direct3D9Texture::updateFormat()
{
    /* Use the default 1 component */
    InternalFormat_ = 1;
    
    /* Determine the SoftPixel type -> and set the renderer format */
    switch (Type_)
    {
        /* 1 component */
        case TEXTURE_INDEX:
            /*Format_ = ; */break;
        case TEXTURE_STENCIL:
            /*Format_ = ; */break;
        case TEXTURE_DEPTH:
            /*Format_ = ; */break;
        case TEXTURE_RED:
            /*Format_ = ; */break;
        case TEXTURE_GREEN:
            /*Format_ = ; */break;
        case TEXTURE_BLUE:
            /*Format_ = ; */break;
        case TEXTURE_ALPHA:
            Format_ = D3DFMT_A8; break;
        case TEXTURE_GRAY:
            Format_ = D3DFMT_L8; break;
            
        /* 2 components */
        case TEXTURE_GRAYALPHA:
            Format_         = D3DFMT_A8L8;
            InternalFormat_ = 2;
            break;
            
        /* 3 components */
        case TEXTURE_RGB:
            Format_         = D3DFMT_R8G8B8;
            InternalFormat_ = 3;
            break;
            
        /* 4 components */
        case TEXTURE_RGBA:
            Format_         = D3DFMT_A8R8G8B8;
            InternalFormat_ = 4;
            break;
    }
}

void Direct3D9Texture::makeEmptyImageData()
{
    /* Delete the Direct3D9 texture */
    clear();
    
    /* Create a new Direct3D9 texture */
    static_cast<Direct3D9Driver*>(__spVideoDriver)->getDirect3DDevice()->CreateTexture(
        Size_.Width,
        Size_.Height,
        isMipMapping_ ? 0 : 1,
        isMipMapping_ ? D3DUSAGE_AUTOGENMIPMAP : 0,
        D3DFMT_A8R8G8B8, // !!! (D3DFMT_X8R8G8B8)
        D3DPOOL_MANAGED,
        &pDirect3DTexture_,
        0
    );
}

void Direct3D9Texture::receiveImageData()
{
    
}

void Direct3D9Texture::recreateImageData()
{
    
    #if 1 // !no other type supported yet!
    DimensionType_  = TEXTURE_2D;
    Depth_          = 1;
    #endif
    
    /* Update the correct format */
    updateFormat();
    
    /* Clear the image data */
    makeEmptyImageData();
    
    if (!ImageData_) // !!!!!!!
    {
        return;
    }
    
    /* Temporary variables */
    s32 x, y, i = 0;
    D3DLOCKED_RECT Rect;
    
    /* Lock the texture */
    HRESULT d3dError = pDirect3DTexture_->LockRect(0, &Rect, 0, 0);
    
    /* Check for error */
    if (d3dError)
    {
        io::printError("could not lock Direct3D9 texture");
        return;
    }
    
    /* Get the image data */
    s32* DestImageData = (s32*)Rect.pBits;
    
    /* Check if the destination image data is not empty image data */
    if (!DestImageData)
        return;
    
    /* Loop for the image data */
    switch (InternalFormat_)
    {
        case 1:
        {
            for (y = 0; y < Size_.Height; ++y)
            {
                for (x = 0; x < Size_.Width; ++x, ++i)
                {
                    DestImageData[i] = getColor(
                        ImageData_[i], ImageData_[i], ImageData_[i], 255
                    );
                }
            }
        }
        break;
        
        case 2:
        {
            for (y = 0; y < Size_.Height; ++y)
            {
                for (x = 0; x < Size_.Width; ++x, ++i)
                {
                    DestImageData[i] = getColor(
                        ImageData_[(i << 1)], ImageData_[(i << 1)], ImageData_[(i << 1)], ImageData_[(i << 1)+1]
                    );
                }
            }
        }
        break;
        
        case 3:
        {
            for (y = 0; y < Size_.Height; ++y)
            {
                for (x = 0; x < Size_.Width; ++x, ++i)
                {
                    DestImageData[i] = getColor(
                        ImageData_[(i * 3)], ImageData_[(i * 3)+1], ImageData_[(i * 3)+2], 255
                    );
                }
            }
        }
        break;
        
        case 4:
        {
            for (y = 0; y < Size_.Height; ++y)
            {
                for (x = 0; x < Size_.Width; ++x, ++i)
                {
                    DestImageData[i] = getColor(
                        ImageData_[(i << 2)], ImageData_[(i << 2)+1], ImageData_[(i << 2)+2], ImageData_[(i << 2)+3]
                    );
                }
            }
        }
        break;
    }
    
    /* Unlock the texture */
    pDirect3DTexture_->UnlockRect(0);
    
    /* Check if mipmaps are used */
    #if 0
    if (isMipMapping_)
        createMipMaps();
    #endif
    
}

bool Direct3D9Texture::createMipMaps(s32 Level)
{
    /* Temporary variables */
    IDirect3DSurface9* UpperSurface = 0;
    IDirect3DSurface9* LowerSurface = 0;
    
    /* Get the upper level */
    if (pDirect3DTexture_->GetSurfaceLevel(Level - 1, &UpperSurface))
    {
        io::printError("could not get the upper surface level");
        return false;
    }
    
    /* Get the lower level */
    if (pDirect3DTexture_->GetSurfaceLevel(Level, &LowerSurface))
    {
        io::printError("could not get the lower surface level");
        return false;
    }
    
    /* Get the descriptions */
    D3DSURFACE_DESC UpperDesc, LowerDesc;
    
    UpperSurface->GetDesc(&UpperDesc);
    LowerSurface->GetDesc(&LowerDesc);
    
    /* Locked rectangles */
    D3DLOCKED_RECT UpperRect, LowerRect;
    
    /* Lock upper surface */
    if (UpperSurface->LockRect(&UpperRect, 0, 0))
    {
        io::printError("could not lock upper texture rectangle");
        return false;
    }
    
    /* Lock lower surface */
    if (LowerSurface->LockRect(&LowerRect, 0, 0))
    {
        io::printError("could not lock lower texture rectangle");
        return false;
    }
    
    /* Generate the current mip map level */
    generateMipMapLevel((s32*)UpperRect.pBits, (s32*)LowerRect.pBits, LowerDesc.Width, LowerDesc.Height);
    
    /* Unlock surfaces */
    if (UpperSurface->UnlockRect())
        return false;
    if (LowerSurface->UnlockRect())
        return false;
    
    /* Release the surfaces */
    UpperSurface->Release();
    LowerSurface->Release();
    
    /* Check if the mip map generation has been finished */
    if (LowerDesc.Width == 1 && LowerDesc.Height == 1)
        return true;
    
    /* Generate the next mip map level */
    return createMipMaps(Level + 1);
}

void Direct3D9Texture::generateMipMapLevel(s32* src, s32* dst, s32 Width, s32 Height)
{
    /* Temporary variables */
    s32 x, y, px, py;
    s32 r, g, b, a;
    s32 i, j;
    
    /* Loop for the new mipmap texture */
    for (y = 0; y < Height; ++y)
    {
        for (x = 0; x < Width; ++x)
        {
            /* Initialize the color values */
            r = g = b = a = 0;
            
            /* Loop for the old pixel area */
            for (py = 0; py < 2; ++py)
            {
                for (px = 0; px < 2; ++px)
                {
                    /* Compute the source index */
                    i = ((y << 1) + py) * (Width << 1) + (x << 1) + px;
                    
                    /* Add the color values */
                    r += getRed(src[i]);
                    g += getGreen(src[i]);
                    b += getBlue(src[i]);
                    a += getAlpha(src[i]);
                }
            }
            
            /* Divide the color values */
            r >>= 2;
            g >>= 2;
            b >>= 2;
            a >>= 2;
            
            /* Compute the destination index */
            j = y * Width + x;
            
            /* Set the new color value */
            dst[j] = video::color(r, g, b, a).getSingle();
        }
    }
}

void Direct3D9Texture::clear()
{
    if (pDirect3DTexture_)
        pDirect3DTexture_->Release();
}


#endif


} // /namespace video

} // /namespace sp







// ================================================================================
