/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for Direct3D9 textures
 *
 */

#ifndef __SP_VIDEODRIVER_DIRECT3D9TEXTURE_H__
#define __SP_VIDEODRIVER_DIRECT3D9TEXTURE_H__


#include "spStandard.hpp"
#include "spVideoDriverTexture.hpp"

#if defined(WIN32)
#   include <d3d9.h>
#   include <d3dx9.h>
#   include <dxerr9.h>
#endif


namespace sp
{
namespace video
{


#if defined(WIN32)

class SP_EXPORT Direct3D9Texture : public Texture
{
    
    public:
        
        Direct3D9Texture();
        Direct3D9Texture(
            LPDIRECT3DTEXTURE9 d3dTexture, io::stringc Filename, dim::size2di Size, u8* ImageData,
            s32 RenderMode = TEXTURE_SMOOTH, s32 FilterMode = TEXTURE_TRILINEAR,
            s32 Format = 3, s32 Type = TEXTURE_RGB, bool isEdgeSmooth = false, bool isMipMapping = true
        );
        ~Direct3D9Texture();
        
        /* Extra option functions */
        
        void changeAttributes(s32 RenderMode, s32 FilterMode = TEXTURE_TRILINEAR, bool isEdgeSmooth = false);
        
        void setColorIntensity(f32 Red, f32 Green, f32 Blue);
        
        void flipColors();
        
        /* Image data updating and recreating */
        
        void setMipMapFilter(s32 MinFilter, s32 MagFilter, bool isMipMapping = true);
        
        // Update the image buffer (Direct3D9 texture -> image data / image data - > Direct3D9 texture)
        void updateImageData(EUpdateDataFunc UpdateDataFunc);
        
        /* Binding & unbinding */
        
        void bind(s32 Level = 0);
        void unbind(s32 Level = 0);
        
        /* Special renderer functions */
        
        inline LPDIRECT3DTEXTURE9 getDirect3DTexture() const
        {
            return pDirect3DTexture_;
        }
        
    private:
        
        /* Private functions */
        
        void updateFormat();
        
        void makeEmptyImageData();
        
        void receiveImageData();
        void recreateImageData();
        
        bool createMipMaps(s32 Level = 1);
        void generateMipMapLevel(s32* src, s32* dst, s32 Width, s32 Height);
        
        void clear();
        
        /* Members */
        
        IDirect3DTexture9* pDirect3DTexture_;
        IDirect3DVolumeTexture9* pDirect3DVolumeTexture_;
        
        D3DFORMAT Format_;
        
};

#endif


} // /namespace video

} // /namespace sp


#endif






// ================================================================================
