/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * File system
 *
 */

#include "spInputOutputFileSystem.hpp"

#if defined(WIN32)
#   include <windows.h>
#endif


namespace sp
{
namespace io
{


FileLoader::FileLoader()
{
}
FileLoader::~FileLoader()
{
}

bool FileLoader::doesFileExist(stringc Filename)
{
    FILE* hFile;
    
    if (hFile = fopen(Filename.c_str(), "rb"))
    {
        fclose(hFile);
        return true;
    }
    
    return false;
}

File* FileLoader::openFile(stringc Filename, s32 OpenType)
{
    FILE* pFile = 0;
    u32 Length  = 0;
    
    stringc OpenTypeStr;
    
    switch (OpenType)
    {
        case FILE_READ_WRITE_ASCII:
            OpenTypeStr = "a+"; break;
        case FILE_READ_BINARY:
            OpenTypeStr = "rb"; break;
        case FILE_READ_ASCII:
            OpenTypeStr = "r"; break;
        case FILE_WRITE_BINARY:
            OpenTypeStr = "wb"; break;
        case FILE_WRITE_ASCII:
            OpenTypeStr = "w"; break;
        case FILE_READ_WRITE_BINARY:
        default:
            OpenTypeStr = "r+"; break;
    }
    
    /* Open the file */
    if ( !( pFile = fopen(Filename.c_str(), OpenTypeStr.c_str()) ) )
    {
        /* Error message */
        io::printError(io::stringc("could not open file: \"")+Filename+io::stringc("\""));
        return 0;
    }
    
    /* Get length of file */
	fseek(pFile, 0, SEEK_END);
	Length = ftell(pFile);
	fseek(pFile, 0, SEEK_SET);
	
    return new File(pFile, Length);
}

void FileLoader::closeFile(File* &hFile)
{
    if (hFile)
    {
        fclose(hFile->getID());
        spDeletePointer(hFile);
    }
}
bool FileLoader::isFile(File* hFile)
{
    return hFile->getID() != 0;
}
bool FileLoader::isEOF(File* hFile)
{
    return feof(hFile->getID());
}

/* Write strings */

s32 FileLoader::writeString(File* hFile, stringc Str, u32 Count)
{
    s32 c = 0;
    
    for (u32 i = 0; i < Count; ++i)
        c += fwrite(Str.c_str(), 1, Str.size(), hFile->getID());
    
    return c;
}

s32 FileLoader::writeStringN(File* hFile, stringc Str, u32 Count)
{
    s32 c = 0;
    
    #if defined(WIN32)
    
    c8 nlChar1 = 13, nlChar2 = 10;
    
    for (u32 i = 0; i < Count; ++i)
    {
        c += fwrite(Str.c_str(), 1, Str.size(), hFile->getID());
        c += fwrite(&nlChar1, 1, sizeof(nlChar1), hFile->getID());
        c += fwrite(&nlChar2, 1, sizeof(nlChar2), hFile->getID());
    }
    
    #elif defined(LINUX)
    
    c8 nlChar = 10;
    
    for (u32 i = 0; i < Count; ++i)
    {
        c += fwrite(Str.c_str(), 1, Str.size(), hFile->getID());
        c += fwrite(&nlChar, 1, sizeof(nlChar), hFile->getID());
    }
    
    #endif
    
    return c;
}

s32 FileLoader::writeStringData(File* hFile, stringc Str, u32 Count)
{
    s32 c = 0;
    u32 Len = Str.size();
    
    for (u32 i = 0; i < Count; ++i)
    {
        c += fwrite(&Len, 1, sizeof(Len), hFile->getID());
        c += fwrite(Str.c_str(), 1, Len, hFile->getID());
    }
    
    return c;
}

s32 FileLoader::writeStringANSIC(File* hFile, stringc Str, u32 Count)
{
    s32 c = 0;
    c8 CharZero = 0;
    
    for (u32 i = 0; i < Count; ++i)
    {
        c += fwrite(Str.c_str(), 1, Str.size(), hFile->getID());
        c += fwrite(&CharZero, 1, sizeof(CharZero), hFile->getID());
    }
    
    return c;
}

/* Read strings */

stringc FileLoader::readString(File* hFile, s32 Length)
{
    c8* cStr = new c8[Length + 1];
    
    fread(cStr, Length, 1, hFile->getID());
    cStr[Length] = 0;
    
    io::stringc Str = cStr;
    
    delete [] cStr;
    
    return Str;
}

stringc FileLoader::readString(File* hFile)
{
    io::stringc Str;
    c8 nlChar = 0;
    
    while ( !feof(hFile->getID()) )
    {
        fread(&nlChar, sizeof(nlChar), 1, hFile->getID());
        
        if (nlChar == 13)
            continue;
        if (nlChar == 10)
            break;
        
        Str += nlChar;
    }
    
    return Str;
}

stringc FileLoader::readStringData(File* hFile)
{
    u32 Len = 0;
    
    fread(&Len, sizeof(Len), 1, hFile->getID());
    
    c8* cStr = new c8[Len + 1];
    
    fread(cStr, Len, 1, hFile->getID());
    cStr[Len] = 0;
    
    io::stringc Str = cStr;
    
    delete [] cStr;
    
    return Str;
}

stringc FileLoader::readStringANSIC(File* hFile)
{
    io::stringc Str;
    c8 CharZero;
    
    while ( !feof(hFile->getID()) )
    {
        fread(&CharZero, sizeof(CharZero), 1, hFile->getID());
        
        if (CharZero == 0)
            break;
        
        Str += CharZero;
    }
    
    return Str;
}

/* Read/ write streams */

u32 FileLoader::readStream(File* hFile, void* pBuffer, u32 Size, s32 Count)
{
    return fread(pBuffer, (size_t)Count, (size_t)Size, hFile->getID()) * Count;
}
u32 FileLoader::writeStream(File* hFile, const void* pBuffer, u32 Size, s32 Count)
{
    return fwrite(pBuffer, (size_t)Count, (size_t)Size, hFile->getID()) * Count;
}

u32 FileLoader::getFileSize(io::stringc Filename)
{
    FILE* hFile;
    
    if (hFile = fopen(Filename.c_str(), "rb"))
    {
        fseek(hFile, 0, SEEK_END);
        
        long len = ftell(hFile);
        
        fclose(hFile);
        
        return static_cast<u32>(len);
    }
    
    return 0;
}

void FileLoader::setFileSeek(File* hFile, s32 Pos, s32 PosType)
{
    fseek(hFile->getID(), (long)Pos, PosType);
}
s32 FileLoader::getFileSeek(File* hFile)
{
    fpos_t fPos;
    
    fgetpos(hFile->getID(), &fPos);
    
    return static_cast<s32>(fPos);
}

bool FileLoader::createFile(stringc Filename)
{
    FILE* hFile = 0;
    
    if (hFile = fopen(Filename.c_str(), "w"))
        fclose(hFile);
    
    return hFile != 0;
}

#if defined(WIN32)

bool FileLoader::deleteFile(stringc Filename)
{
    return DeleteFile(Filename.c_str());
}
bool FileLoader::copyFile(stringc Filename, stringc NewFilename)
{
    return CopyFile(Filename.c_str(), NewFilename.c_str(), false);
}
bool FileLoader::moveFile(stringc Filename, stringc NewFilename)
{
    return MoveFile(Filename.c_str(), NewFilename.c_str());
}

stringc FileLoader::getCurrentDirectory()
{
    char path[__SP_MAXSTRLEN];
    
    GetCurrentDirectory(__SP_MAXSTRLEN, path);
    
    return stringc(std::string(path));
}
void FileLoader::setCurrentDirectory(stringc Path)
{
    SetCurrentDirectory(Path.c_str());
}

bool FileLoader::createDirectory(stringc Path)
{
    if (Path.mid(1, 1) == ":")
        Path = io::stringc("//?/") + Path;
    else
        Path = io::stringc("//?/") + getCurrentDirectory() + io::stringc("/") + Path;
    
    SECURITY_ATTRIBUTES lpAttributes;
    {
        lpAttributes.nLength                = sizeof(SECURITY_ATTRIBUTES);
        lpAttributes.lpSecurityDescriptor   = 0;
        lpAttributes.bInheritHandle         = false;
    }
    return CreateDirectory(Path.c_str(), &lpAttributes);
}
bool FileLoader::deleteDirectory(stringc Path)
{
    return RemoveDirectory(Path.c_str());
}

#elif defined(LINUX)

#endif


} // /namespace io

} // /namespace sp











// ================================================================================
