/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * File system header
 *
 */

#ifndef __SP_INPUTOUTPUT_FILESYSTEM_H__
#define __SP_INPUTOUTPUT_FILESYSTEM_H__


#include "spStandard.hpp"
#include "spMemoryManagement.hpp"
#include "spInputOutputFile.hpp"
#include "spInputOutputStandard.hpp"


namespace sp
{
namespace io
{


#define __SP_MAX_FILE_COUNT 100
#define __SP_MAXSTRLEN      512

enum EFileSeekTypes
{
    FILEPOS_BEGIN   = SEEK_SET,
    FILEPOS_CURRENT = SEEK_CUR,
    FILEPOS_END     = SEEK_END
};

enum EFileSystemTypes
{
    FILE_READ_WRITE_BINARY = 0,
    FILE_READ_WRITE_ASCII,
    FILE_READ_BINARY,
    FILE_READ_ASCII,
    FILE_WRITE_BINARY,
    FILE_WRITE_ASCII
};


class SP_EXPORT FileLoader
{
    
    public:
        
        FileLoader();
        ~FileLoader();
        
        bool doesFileExist(stringc Filename);
        File* openFile(stringc Filename, s32 OpenType = FILE_READ_WRITE_BINARY);
        void closeFile(File* &hFile);
        bool isFile(File* hFile);
        bool isEOF(File* hFile);
        
        /* Write strings */
        s32 writeString(File* hFile, stringc Str, u32 Count = 1);      // (only the characters)
        s32 writeStringN(File* hFile, stringc Str, u32 Count = 1);     // (with binary '13' and binary '10' at the end)
        s32 writeStringData(File* hFile, stringc Str, u32 Count = 1);  // (integer at the begin)
        s32 writeStringANSIC(File* hFile, stringc Str, u32 Count = 1); // (with a binary 0 (zero) at the end)
        
        /* Read strings */
        stringc readString(File* hFile, s32 Length);
        stringc readString(File* hFile); // readStringN
        stringc readStringData(File* hFile);
        stringc readStringANSIC(File* hFile);
        
        /* Read/ write values */
        template <typename T> s32 writeValue(File* hFile, T Value, u32 Count = 1)
        {
            return fwrite(&Value, Count, sizeof(Value), hFile->getID()) * Count;
        }
        template <typename T> T readValue(File* hFile)
        {
            T Value;
            fread(&Value, 1, sizeof(Value), hFile->getID());
            return Value;
        }
        
        /* Read/ write streams */
        u32 readStream(File* hFile, void* pBuffer, u32 Size, s32 Count = 1);
        u32 writeStream(File* hFile, const void* pBuffer, u32 Size, s32 Count = 1);
        
        u32 getFileSize(io::stringc Filename);
        
        void setFileSeek(File* hFile, s32 Pos, s32 PosType = FILEPOS_BEGIN); // Sets the read/write position
        s32 getFileSeek(File* hFile);
        
        bool createFile(stringc Filename); // Not useable for read and write operations
        bool deleteFile(stringc Filename);
        bool copyFile(stringc Filename, stringc NewFilename);
        bool moveFile(stringc Filename, stringc NewFilename);
        
        // Directories
        stringc getCurrentDirectory();
        void setCurrentDirectory(stringc Path);
        
        bool createDirectory(stringc Path);
        bool deleteDirectory(stringc Path);
        
};


} // /namespace io

} // /namespace sp


#endif









// ================================================================================
