/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * OperatingSystem informator
 *
 */

#include "spInputOutputOSInformator.hpp"

#include <iostream>

#if defined(WIN32)
#   include <windows.h>
#endif


namespace sp
{
namespace io
{


#if defined(WIN32)

OSInformator::OSInformator() : OSVersion_(allocOSVersion())
{
}
OSInformator::~OSInformator()
{
}

void OSInformator::setClipboardText(stringc Text)
{
    const char* cText = Text.c_str();
    if (strlen(cText) == 0)
        return;
    
    if (!OpenClipboard(0) || cText == 0)
        return;
    
    EmptyClipboard();
    
    HGLOBAL clipbuffer;
    char* buffer;
    
    clipbuffer = GlobalAlloc(GMEM_DDESHARE, strlen(cText)+1);
    buffer = (char*)GlobalLock(clipbuffer);
    
    strcpy(buffer, cText);
    
    GlobalUnlock(clipbuffer);
    SetClipboardData(CF_TEXT, clipbuffer);
    CloseClipboard();
}

stringc OSInformator::getClipboardText()
{
    if (!OpenClipboard(0))
        return io::stringc();
    
    char* buffer;
    
    HANDLE hData = GetClipboardData(CF_TEXT);
    buffer = (char*)GlobalLock(hData);
    GlobalUnlock(hData);
    CloseClipboard();
    
    return stringc(std::string(buffer));
}

u32 OSInformator::getProcessorSpeed() // (in MHz)
{
    u32 MHz = 0;
    LONG Error;
    
    HKEY Key;
    Error = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", 0, KEY_READ, &Key);
    
    if (Error != ERROR_SUCCESS)
        return 0;
    
    DWORD Speed = 0;
    DWORD Size = sizeof(Speed);
    Error = RegQueryValueEx(Key, "~MHz", 0, 0, (LPBYTE)&Speed, &Size);
    
    RegCloseKey(Key);
    
    return MHz = static_cast<u32>(Speed);
}

void OSInformator::getDiskSpace(stringc PartitionName, u32 &Total, u32 &Free)
{
    
    // ! unfinished !
    
}

void OSInformator::getVirtualMemory(f32 &Total, f32 &Free, s32 SizeType)
{
    MEMORYSTATUS memStatus;
    GlobalMemoryStatus(&memStatus);
    // Select size type
    switch (SizeType)
    {
        case SIZE_BYTE:
            Total = (f32)memStatus.dwTotalPhys;
            Free  = (f32)memStatus.dwAvailPhys;
            break;
        case SIZE_KB:
            Total = (f32)memStatus.dwTotalPhys / 1024.0f;
            Free  = (f32)memStatus.dwAvailPhys / 1024.0f;
            break;
        case SIZE_MB:
            Total = (f32)memStatus.dwTotalPhys / 1024.0f / 1024.0f;
            Free  = (f32)memStatus.dwAvailPhys / 1024.0f / 1024.0f;
            break;
        case SIZE_GB:
            Total = (f32)memStatus.dwTotalPhys / 1024.0f / 1024.0f / 1024.0f;
            Free  = (f32)memStatus.dwAvailPhys / 1024.0f / 1024.0f / 1024.0f;
            break;
    }
}


/*
 * ========== private: ==========
 */


stringc OSInformator::allocOSVersion()
{
    stringc Version;
    
    OSVERSIONINFOEX osvi;
    BOOL bOsVersionInfoEx;
    
    ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
    
    if ( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO*)&osvi)) )
    {
        osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
        if ( !GetVersionEx((OSVERSIONINFO*)&osvi) )
            return "";
    }
    
    switch (osvi.dwPlatformId)
    {
        case VER_PLATFORM_WIN32_NT:
        {
            if (osvi.dwMajorVersion <= 4)
                Version += "Microsoft Windows NT ";
            if (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0)
                Version += "Microsoft Windows 2000 ";
            if (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1)
                Version += "Microsoft Windows XP ";
            
            if(bOsVersionInfoEx)
            {
                #ifdef VER_SUITE_ENTERPRISE
                if (osvi.wProductType == VER_NT_WORKSTATION)
                {
                    if( osvi.wSuiteMask & VER_SUITE_PERSONAL )
                        Version += "Personal ";
                    else
                        Version += "Professional ";
                }
                else if (osvi.wProductType == VER_NT_SERVER)
                {
                    if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                        Version += "DataCenter Server ";
                    else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                        Version += "Advanced Server ";
                    else
                        Version += "Server ";
                }
                #endif
            }
            else
            {
                HKEY hKey;
                char szProductType[80];
                DWORD dwBufLen;
                
                RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SYSTEM\\CurrentControlSet\\Control\\ProductOptions", 0, KEY_QUERY_VALUE, &hKey );
                RegQueryValueEx(hKey, "ProductType", 0, 0, (LPBYTE) szProductType, &dwBufLen);
                RegCloseKey(hKey);
                
                if (lstrcmpi("WINNT", szProductType) == 0)
                    Version += "Professional ";
                if (lstrcmpi("LANMANNT", szProductType) == 0)
                    Version += "Server " ;
                if (lstrcmpi("SERVERNT", szProductType) == 0)
                    Version += "Advanced Server ";
            }
            
            // Display version, service pack (if any), and build number.
            
            char tmp[255];
            
            if (osvi.dwMajorVersion <= 4 )
            {
                sprintf(tmp, "version %ld.%ld %s (Build %ld)",
                osvi.dwMajorVersion,
                osvi.dwMinorVersion,
                osvi.szCSDVersion,
                osvi.dwBuildNumber & 0xFFFF);
            }
            else
            {
                sprintf(tmp, "%s (Build %ld)", osvi.szCSDVersion,
                osvi.dwBuildNumber & 0xFFFF);
            }
            
            Version += tmp;
        }
        break;
            
        case VER_PLATFORM_WIN32_WINDOWS:
        {
            if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
            {
                Version += "Microsoft Windows 95 ";
                if ( osvi.szCSDVersion[1] == 'C' || osvi.szCSDVersion[1] == 'B' )
                    Version += "OSR2 ";
            }
            
            if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10)
            {
                Version += "Microsoft Windows 98 ";
                if ( osvi.szCSDVersion[1] == 'A' )
                    Version += "SE ";
            }
            
            if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90)
                Version += "Microsoft Windows Me ";
        }
        break;
            
        case VER_PLATFORM_WIN32s:
        {
            Version += "Microsoft Win32s ";
        }
        break;
    }
    
    return Version;
}

#elif defined(LINUX)

#endif


} // /namespace io

} // /namespace sp







// ================================================================================
