/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Standard InputOutput
 *
 */

#include "spInputOutputStandard.hpp"

#include <iostream>
#include <windows.h>


namespace sp
{
namespace io
{


/*
 * ======= Standard S.P. inout & output operation functions =======
 */

std::ofstream __spLogStream;
bool __spLogUsed = false;


#if defined(WIN32)

SP_EXPORT void openConsole(stringc Title)
{
    AllocConsole();
    if (Title.str() != "")
        SetConsoleTitle(Title.c_str());
}
SP_EXPORT void closeConsole()
{
    FreeConsole();
}
SP_EXPORT void clearConsole()
{
    system("CLS");
}
SP_EXPORT void pauseConsole()
{
    system("PAUSE");
}

#elif defined(LINUX)

#endif

SP_EXPORT void printMessage(stringc Message)
{
    if (__spLogUsed)
        logMessage(Message);
    else
        std::cout << Message.str() << std::endl;
}

#if defined(WIN32)

SP_EXPORT void printInfo(stringc Message, EInfoColorTypes Color)
{
    if (__spLogUsed)
        logMessage(Message);
    else
    {
        static HANDLE hConsoleOut = GetStdHandle(STD_OUTPUT_HANDLE);
        
        if (Color == INFO_WARNING)
            SetConsoleTextAttribute(hConsoleOut, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY);
        else if (Color == INFO_ERROR)
            SetConsoleTextAttribute(hConsoleOut, FOREGROUND_RED | FOREGROUND_INTENSITY);
        
        std::cout << Message.str() << std::endl;
        SetConsoleTextAttribute(hConsoleOut, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
    }
}

SP_EXPORT void printWarning(stringc Message)
{
    if (__spLogUsed)
        logMessage(io::stringc("Warning: ") + Message + io::stringc("!"));
    else
    {
        static HANDLE hConsoleOut = GetStdHandle(STD_OUTPUT_HANDLE);
        SetConsoleTextAttribute(hConsoleOut, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY);
        std::cout << "Warning: " << Message.str() << "!" << std::endl;
        SetConsoleTextAttribute(hConsoleOut, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
    }
}
SP_EXPORT void printError(stringc Message)
{
    if (__spLogUsed)
        logMessage(io::stringc("Error: ") + Message + io::stringc("!"));
    else
    {
        static HANDLE hConsoleOut = GetStdHandle(STD_OUTPUT_HANDLE);
        SetConsoleTextAttribute(hConsoleOut, FOREGROUND_RED | FOREGROUND_INTENSITY);
        std::cerr << "Error: " << Message.str() << "!" << std::endl;
        SetConsoleTextAttribute(hConsoleOut, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
    }
}

SP_EXPORT void getMessage(stringc Message)
{
    MessageBox(0, TEXT(Message.c_str()), TEXT("Information"), MB_OK | MB_ICONQUESTION); // !!!
    //MessageBox(0, TEXT(Message.c_str()), TEXT("Information"), MB_OK | MB_ICONINFORMATION);
}
SP_EXPORT void getWarning(stringc Message)
{
    MessageBox(0, TEXT(Message.c_str()), TEXT("Warning"), MB_OK | MB_ICONWARNING);
}
SP_EXPORT void getError(stringc Message)
{
    MessageBox(0, TEXT(Message.c_str()), TEXT("Error"), MB_OK | MB_ICONERROR);
}

#elif defined(LINUX)

SP_EXPORT void printInfo(stringc Message, s32 Color)
{
    if (__spLogUsed)
        logMessage(Message);
    else
        std::cout << Message.str() << std::endl;
}

SP_EXPORT void printWarning(stringc Message)
{
    if (__spLogUsed)
        logMessage(io::stringc("Warning: ") + Message + io::stringc("!"));
    else
        std::cout << "Warning: " << Message.str() << "!" << std::endl;
}
SP_EXPORT void printError(stringc Message)
{
    if (__spLogUsed)
        logMessage(io::stringc("Error: ") + Message + io::stringc("!"));
    else
        std::cerr << "Error: " << Message.str() << "!" << std::endl;
}

#endif

SP_EXPORT void setDebugLog(bool isDebugLog, io::stringc Filename)
{
    if (isDebugLog)
    {
        logOpen(Filename);
        logMessage("SoftPixel Engine - debug log file:");
        logMessage("==================================\n");
    }
    else
    {
        logMessage("\n===========");
        logMessage("END-OF-LINE");
        logClose();
    }
}

SP_EXPORT void logOpen(stringc Filename)
{
    if (!__spLogUsed)
    {
        __spLogStream.open(Filename.c_str(), std::ios::trunc);
        __spLogUsed = true;
    }
}
SP_EXPORT void logMessage(stringc Message)
{
    if (__spLogUsed)
        __spLogStream << Message.str() << std::endl;
}
SP_EXPORT void logClose()
{
    if (__spLogUsed)
    {
        __spLogStream.close();
        __spLogUsed = false;
    }
}


/*
 * ======= Some text functions (dor debugging) =======
 */

SP_EXPORT stringc getDebugString(dim::vector3df Value)
{
    return
        stringc("( ") + stringc(Value.X) +
        stringc(" | ") + stringc(Value.Y) +
        stringc(" | ") + stringc(Value.Z) + stringc(" )");
}

SP_EXPORT stringc getDebugString(dim::quaternion Value)
{
    return
        stringc("( ") + stringc(Value.X) +
        stringc(" | ") + stringc(Value.Y) +
        stringc(" | ") + stringc(Value.Z) +
        stringc(" | ") + stringc(Value.W) + stringc(" )");
}

SP_EXPORT stringc getDebugString(dim::matrix4f Value)
{
    stringc str[16];
    
    for (s32 i = 0; i < 16; ++i)
    {
        str[i] = Value[i];
        if (str[i].size() < 9)
            str[i] += stringc("\t");
    }
    
    return
        stringc("[\t") + str[ 0] + stringc("\t|\t") + str[ 4] + stringc("\t|\t") + str[ 8] + stringc("\t|\t") + str[12] + stringc("\t]\n") +
        stringc("[\t") + str[ 1] + stringc("\t|\t") + str[ 5] + stringc("\t|\t") + str[ 9] + stringc("\t|\t") + str[13] + stringc("\t]\n") +
        stringc("[\t") + str[ 2] + stringc("\t|\t") + str[ 6] + stringc("\t|\t") + str[10] + stringc("\t|\t") + str[14] + stringc("\t]\n") +
        stringc("[\t") + str[ 3] + stringc("\t|\t") + str[ 7] + stringc("\t|\t") + str[11] + stringc("\t|\t") + str[15] + stringc("\t]");
}


} // /namespace io

} // /namespace sp







// ================================================================================
