/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * String header
 *
 */

#if 1

#ifndef __SP_IO_STRING_H__
#define __SP_IO_STRING_H__


#include "spStandard.hpp"

#include <cstdlib>
#include <sstream>
#include <string>


namespace sp
{
namespace io
{


/* SoftPixel- String class */

template <typename T> class string
{
    
    public:
        
        string<T>() : Str_("")
        {
        }
        string<T>(const string<T> &Str)
        {
            Str_ = Str.Str_;
        }
        string<T>(const std::string Str)
        {
            Str_ = Str;
        }
        string<T>(const std::stringstream Str)
        {
            Str_ = Str.str();
        }
        template <typename B> string<T>(const B Value)
        {
            std::stringstream SStr; SStr << Value; Str_ = SStr.str();
        }
        ~string<T>()
        {
        }
        
        /* Operators */
        
        string<T> operator = (const string<T> &Str)
        {
            Str_ = Str.Str_; return Str_;
        }
        string<T> operator = (const std::string Str)
        {
            Str_ = Str; return Str_;
        }
        string<T> operator = (const std::stringstream Str)
        {
            Str_ = Str.str(); return Str_;
        }
        template <typename B> string<T> operator = (const B Value)
        {
            std::stringstream SStr; SStr << Value; Str_ = SStr.str(); return Str_;
        }
        
        bool operator == (const string<T> &Str) const
        {
            return Str_ == Str.Str_;
        }
        bool operator == (const std::string Str) const
        {
            return Str_ == Str;
        }
        bool operator == (const std::stringstream Str) const
        {
            return Str_ == Str.str();
        }
        template <typename B> bool operator == (const B Value) const
        {
            std::stringstream SStr; SStr << Value; return Str_ == SStr.str();
        }
        
        bool operator != (const string<T> &Str) const
        {
            return Str_ != Str.Str_;
        }
        bool operator != (const std::string Str) const
        {
            return Str_ != Str;
        }
        bool operator != (const std::stringstream Str) const
        {
            return Str_ != Str.str();
        }
        template <typename B> bool operator != (const B Value) const
        {
            std::stringstream SStr; SStr << Value; return Str_ != SStr.str();
        }
        
        string<T> operator + (const string<T> &Str) const
        {
            return Str_ + Str.Str_;
        }
        string<T> operator + (const std::string Str) const
        {
            return Str_ + Str;
        }
        string<T> operator + (const std::stringstream Str) const
        {
            return Str_ + Str.str();
        }
        template <typename B> string<T> operator + (const B Value) const
        {
            std::stringstream SStr; SStr << Value; return Str_ + SStr.str();
        }
        
        string<T> operator += (const string<T> &Str)
        {
            return Str_ += Str.Str_;
        }
        string<T> operator += (const std::string Str)
        {
            return Str_ += Str;
        }
        string<T> operator += (const std::stringstream Str)
        {
            return Str_ += Str.str();
        }
        template <typename B> string<T> operator += (const B Value)
        {
            std::stringstream SStr; SStr << Value; return Str_ += SStr.str();
        }
        
        /* Indexed value */
        
        inline T& operator [] (s32 index)
        {
            return Str_[index];
        }
        
        inline const T& operator [] (s32 index) const
        {
            return Str_[index];
        }
        
        /* Extra functions */
        
        inline std::string str() const
        {
            return Str_;
        }
        inline const char* c_str() const
        {
            return Str_.c_str();
        }
        inline char* cstr() const
        {
            return (char*)Str_.c_str();
        }
        inline size_t size() const
        {
            return Str_.size();
        }
        
        template <typename B> B val() const
        {
            char* offset;
            return static_cast<B>(strtod(Str_.c_str(), &offset));
        }
        
        string<T> left(size_t Len) const
        {
            return string<T>(Str_.substr(0, Len));
        }
        string<T> right(size_t Len) const
        {
            return string<T>(Str_.substr(Str_.size() - Len, Len));
        }
        string<T> mid(size_t Pos, size_t Len) const
        {
            return string<T>(Str_.substr(Pos, Len));
        }
        string<T> section(size_t Pos1, size_t Pos2) const
        {
            return string<T>(Str_.substr(Pos1, Pos2 - Pos1));
        }
        string<T> trim() // trims the whole string
        {
            std::string tStr = Str_;
            for (size_t i = 0; i < tStr.size();)
            {
                if (tStr[i] == ' ' || tStr[i] == '\t')
                    tStr = tStr.substr(0, i) + tStr.substr(i+1, tStr.size()-1);
                else
                    ++i;
            }
            return string<T>(tStr);
        }
        string<T> ltrim() // trims only the left side
        {
            std::string tStr = Str_;
            while (tStr[0] == ' ' || tStr[0] == '\t')
                tStr = tStr.substr(1, tStr.size()-1);
            return string<T>(tStr);
        }
        string<T> rtrim() // trims only the right side
        {
            std::string tStr = Str_;
            while (tStr[tStr.size()-1] == ' ' || tStr[tStr.size()-1] == '\t')
                tStr = tStr.substr(0, tStr.size()-2);
            return string<T>(tStr);
        }
        
        size_t find(string<T> &Str, size_t PosBegin) const
        {
            return Str_.find(Str.Str_, PosBegin);
        }
        size_t find(std::string Str, size_t PosBegin) const
        {
            return Str_.find(Str, PosBegin);
        }
        size_t find(std::stringstream Str, size_t PosBegin) const
        {
            return Str_.find(Str.str(), PosBegin);
        }
        template <typename B> size_t find(B Value, size_t PosBegin) const
        {
            std::stringstream SStr;
            SStr << Value;
            return Str_.find(SStr.str(), PosBegin);
        }
        
        string<T> upper() const
        {
            std::string NewStr = Str_;
            
            for (u32 i = 0; i < NewStr.size(); ++i)
            {
                if (NewStr[i] >= 97 && NewStr[i] <= 122)
                    NewStr[i] -= 32;
            }
            
            return string<T>(NewStr);
        }
        string<T> lower() const
        {
            std::string NewStr = Str_;
            
            for (u32 i = 0; i < NewStr.size(); ++i)
            {
                if (NewStr[i] >= 65 && NewStr[i] <= 90)
                    NewStr[i] += 32;
            }
            
            return string<T>(NewStr);
        }
        
        string<T> replace(string<T> &StrFind, string<T> &StrReplace, size_t PosBegin = 0)
        {
            std::string tStr = Str_;
            u32 Pos = (u32)PosBegin;
            
            while ( ( Pos = (u32)tStr.find(StrFind.str(), Pos+1) ) != -1)
                tStr = tStr.substr(0, Pos) + StrReplace.str() + tStr.substr( Pos+StrFind.size(), tStr.size()-(Pos+StrFind.size()) );
            
            return string<T>(tStr);
        }
        string<T> replace(std::string StrFind, std::string StrReplace, size_t PosBegin = 0)
        {
            std::string tStr = Str_;
            u32 Pos = (u32)PosBegin;
            
            while ( ( Pos = (u32)tStr.find(StrFind, Pos+1) ) != -1)
                tStr = tStr.substr(0, Pos) + StrReplace + tStr.substr( Pos+StrFind.size(), tStr.size()-(Pos+StrFind.size()) );
            
            return string<T>(tStr);
        }
        string<T> replace(std::stringstream StrFind, std::stringstream StrReplace, size_t PosBegin = 0)
        {
            std::string tStr = Str_;
            u32 Pos = (u32)PosBegin;
            
            while ( ( Pos = (u32)tStr.find(StrFind.str(), Pos+1) ) != -1)
                tStr = tStr.substr(0, Pos) + StrReplace.str() + tStr.substr( Pos+StrFind.str().size(), tStr.size()-(Pos+StrFind.str().size()) );
            
            return string<T>(tStr);
        }
        
        static string<T> ascii(u8 tChar)
        {
            return string<T>(std::string(1, tChar));
        }
        
        static string<T> space(size_t Count, c8 Ascii = ' ')
        {
            return string<T>(std::string(Count, Ascii));
        }
        
        static string<T> create(string<T> &Str, size_t Count = 1)
        {
            std::string tStr;
            for (size_t i = 0; i < Count; ++i)
                tStr += Str.Str_;
            return string<T>(tStr);
        }
        static string<T> create(std::string Str, size_t Count = 1)
        {
            std::string tStr;
            for (size_t i = 0; i < Count; ++i)
                tStr += Str;
            return string<T>(tStr);
        }
        static string<T> create(std::stringstream Str, size_t Count = 1) {
            std::string tStr;
            for (size_t i = 0; i < Count; ++i)
                tStr += Str.str();
            return string<T>(tStr);
        }
        template <typename B> static string<T> create(B Value, size_t Count = 1)
        {
            std::stringstream SStr;
            SStr << Value;
            std::string tStr, Str = SStr.str();
            for (size_t i = 0; i < Count; ++i)
                tStr += Str;
            return string<T>(tStr);
        }
        
        /* Extra operation functions */
        
        string<T> getFilePath() const
        {
            string<T> tStr = string<T>(Str_).replace("\\", "/");
            s32 tPos = 0, tLastPost = -1;
            while ( (s32)(tPos = tStr.find("/", tPos+1)) != -1 )
                tLastPost = tPos;
            if (tLastPost != -1)
                tStr = tStr.left(tLastPost+1);
            return tStr;
        }
        string<T> getFileExtension() const
        {
            size_t pos = 0, lastpos;
            while ((signed)pos != -1)
            {
                lastpos = pos;
                pos = Str_.find(".", pos+1);
            }
            pos = lastpos;
            return string<T>(Str_).section(lastpos+1, Str_.size());
        }
        
        static string<T> makeIdNum(f32 Number, u32 IdentityLen = 10)
        {
            string<T> str(Number), val(Number);
            
            if (static_cast<s32>(val.find(".", 1)) == -1)
                val = val + string<T>(".0");
            
            if (Number < 0)
            {
                str = val + space(IdentityLen - val.size(), '0');
            }
            else
            {
                str = string<T>(" ") + val + space(IdentityLen - val.size(), '0');
            }
            
            return str;
        }
        
    private:
        
        /* Members */
        
        std::basic_string<T> Str_;
        
};

typedef string<c8>  stringc;
typedef string<c16> stringw;


/* Extendet template function */

template <typename T> stringc getHex(T Numeric)
{
    stringc str;
    
    for (s32 i = 2*sizeof(T) - 1; i >= 0; --i)
    {
        str += "0123456789ABCDEF"[((Numeric >> i*4) & 0xF)];
    }
    
    return str;
}


} // /namespace io

} // /namespace sp


#endif

#else

#ifndef __SP_IO_STRING_H__
#define __SP_IO_STRING_H__


#include <cstdlib>
#include <algorithm>
#include <string>
#include <sstream>


namespace sp
{
namespace io
{


/* SoftPixel- String class */

template <typename T> class string
{
    
    public:
        
        /* Type definitions */
        
        typedef T*          iterator;
        typedef T* const    const_iterator;
        
        /* Constructors */
        
        string() : str_(0), len_(0)
        {
            create(0);
        }
        
        string(const string<T> &other) : str_(0), len_(0)
        {
            /* Create new memory */
            create(other.len_);
            
            /* Copy memory */
            std::copy(other.begin(), other.end(), begin());
        }
        
        string(const T character, u32 count = 1) : str_(0), len_(0)
        {
            /* Create new memory */
            create(count);
            
            /* Fill memory */
            std::fill(begin(), end(), character);
        }
        
        string(const T* cstr, u32 len) : str_(0), len_(0)
        {
            /* Create new memory */
            create(len);
            
            /* Copy memory */
            std::copy(&cstr[0], &cstr[len], begin());
        }
        
        string(const T* cstr) : str_(0), len_(0)
        {
            register u32 len = 0;
            
            /* Loop for the length */
            while (cstr[len++]);
            
            /* Create new memory */
            create(len - 1);
            
            /* Copy memory */
            std::copy(&cstr[0], &cstr[len - 1], begin());
        }
        
        string(const std::string str) : str_(0), len_(0)
        {
            /* Create new memory */
            create(str.size());
            
            /* Copy memory */
            std::copy(str.begin(), str.end(), begin());
        }
        
        string(const std::wstring wstr) : str_(0), len_(0)
        {
            /* Create new memory */
            create(wstr.size());
            
            /* Copy memory */
            std::copy(wstr.begin(), wstr.end(), begin());
        }
        
        template <typename B> string(const B Value) : str_(0), len_(0)
        {
            /* Create temporary variable */
            std::stringstream sstr;
            std::string str;
            
            /* Fill the temporary variable */
            sstr << Value;
            str = sstr.str();
            
            /* Create new memory */
            create(str.size());
            
            /* Copy memory */
            std::copy(str.begin(), str.end(), begin());
        }
        
        /* Destructor */
        
        ~string()
        {
            clear();
        }
        
        /* Operators */
        
        // Comparision operators
        
        bool operator == (string<T> str) const
        {
            /* Check if the arrays are not emtpy */
            if (!str_ && !str.str_)
                return true;
            else if (!str_ || !str.str_)
                return false;
            
            /* Loop the elements */
            for (register u32 i = 0; i < len_; ++i)
            {
                if (str_[i] != str.str_[i])
                    return false;
            }
            
            /* Exit the function */
            return true;
        }
        
        bool operator != (string<T> str) const
        {
            /* Check if the arrays are not emtpy */
            if (!str_ && !str.str_)
                return false;
            else if (!str_ || !str.str_)
                return true;
            
            /* Loop the elements */
            for (register u32 i = 0; i < len_; ++i)
            {
                if (str_[i] != str.str_[i])
                    return true;
            }
            
            /* Exit the function */
            return false;
        }
        
        // Gettings operators
        
        inline T& operator [] (u32 index)
        {
            return str_[index];
        }
        
        inline const T& operator [] (u32 index) const
        {
            return str_[index];
        }
        
        // Settings operators
        
        string<T>& operator = (const string<T> &str)
        {
            /* Create new memory */
            create(str.len_);
            
            /* Copy memory */
            std::copy(str.begin(), str.end(), begin());
            
            /* Return the result */
            return *this;
        }
        
        // Manipulation operators
        
        string<T> operator + (const string<T> &str) const
        {
            /* Create temporary variable */
            string<T> tmp(str_, len_);
            
            /* Append the new string */
            tmp.append(str);
            
            /* Return the result */
            return tmp;
        }
        
        string<T>& operator += (const string<T> &str)
        {
            /* Append the new string */
            append(str);
            
            /* Return the result */
            return *this;
        }
        
        /* Statics */
        
        static inline const c8 ascSpace()
        {
            return ' ';
        }
        
        static inline const c8 ascZero()
        {
            return '0';
        }
        
        static inline const c8 ascUpperA()
        {
            return 'A';
        }
        
        static inline const c8 ascLowerA()
        {
            return 'a';
        }
        
        /* Extendet functions */
        
        // Memory allocation and deletion functions
        
        void create(u32 len)
        {
            /* Check if the memory is not empty */
            if (str_)
                delete [] str_;
            
            /* Use the new length */
            len_ = len;
            
            /* Allocate new memory */
            str_ = new T[len_ + 1];
            
            /* Set the length */
            str_[len_] = (T)0;
        }
        
        void clear()
        {
            if (str_)
            {
                delete [] str_;
                str_ = 0;
            }
            len_ = 0;
        }
        
        void append(const string<T> &str)
        {
            /* Allocate temporary memory */
            T* tmp = new T[len_ + str.len_ + 1];
            
            /* Copy memory */
            std::copy(begin(), end(), &tmp[0]);
            std::copy(str.begin(), str.end(), &tmp[len_]);
            
            /* Delete the old memory */
            if (str_)
                delete [] str_;
            
            /* Use the new memory */
            str_ = tmp;
            len_ += str.len_;
            
            /* Set the termination character */
            str_[len_] = (T)0;
        }
        
        void resize(u32 newsize)
        {
            /* Allocate temporary memory */
            T* tmp = new T[newsize + 1];
            
            /* Check if the new size is larger than the old */
            if (newsize >= len_)
            {
                /* Copy memory */
                std::copy(begin(), end(), &tmp[0]);
                
                /* Fill the rest */
                std::fill(&tmp[end() - begin()], &tmp[newsize], (T)(' '));
            }
            else
            {
                /* Copy memory */
                std::copy(begin(), begin() + newsize, &tmp[0]);
            }
            
            /* Delete the old memory */
            if (str_)
                delete [] str_;
            
            /* Use the new memory */
            str_ = tmp;
            len_ = newsize;
            
            /* Set the termination character */
            str_[newsize] = (T)0;
        }
        
        // Value getting functions
        
        inline const T* c_str() const
        {
            return str_;
        }
        
        const char* cast8() const
        {
            char cstr[len_ + 1];
            
            for (register u32 i = 0; i < len_; ++i)
                cstr[i] = static_cast<char>(str_[i]);
            cstr[len_] = 0;
            
            return std::string(cstr).c_str();
        }
        
        const wchar_t* cast16() const
        {
            wchar_t wstr[len_ + 1];
            
            for (register u32 i = 0; i < len_; ++i)
                wstr[i] = static_cast<wchar_t>(str_[i]);
            wstr[len_] = 0;
            
            return std::wstring(wstr).c_str();
        }
        
        std::string str() const
        {
            return std::string(c_str());
        }
        
        std::stringstream sstr() const
        {
            std::stringstream sstr;
            sstr << str_;
            return sstr;
        }
        
        // Getting functions
        
        inline bool empty() const
        {
            return !str_;
        }
        
        inline u32 size() const
        {
            return len_;
        }
        
        const_iterator begin() const
        {
            return &str_[0];
        }
        
        const_iterator end() const
        {
            return &str_[len_];
        }
        
        u32 count(string<T> str) const
        {
            /* Allocate temporary variables */
            u32 cnt = 0, pos = 0;
            
            /* Loop the memory */
            while ( ( pos = find(str, pos + 1) ) != 0 )
                ++cnt;
            
            /* Return the count */
            return cnt;
        }
        
        // Working functions
        
        inline template <typename B> B val() const
        {
            /* Create temporary memory */
            char* offset;
            
            /* Create a double and cast to the specified typename */
            return static_cast<B>(strtod(c_str(), &offset));
        }
        
        string<T> mid(u32 pos, u32 len) const
        {
            /* Check if pos is out of range */
            if (pos >= len_)
                return string<T>();
            
            /* Clamp the length */
            if (pos + len > len_)
                len = len_ - pos;
            
            /* Alocate temporary memory */
            T* cstr = new T[len_ + 1];
            T* oldcstr = cstr;
            
            /* Copy the memory */
            std::copy(begin(), end(), &cstr[0]);
            
            /* Move the memory */
            cstr += pos;
            cstr[len] = 0;
            
            /* Create the result */
            string<T> result(cstr, len);
            
            /* Delete the old temporary memory */
            delete [] oldcstr;
            
            /* Return a new string */
            return result;
        }
        
        string<T> part(u32 begin, u32 end) const
        {
            return mid(begin, end - begin);
        }
        
        string<T> left(u32 len) const
        {
            return mid(0, len);
        }
        
        string<T> right(u32 len) const
        {
            return mid(len_ - len, len);
        }
        
        string<T> upper() const
        {
            /* Allocate temporary variables */
            string<T> str(str_, len_);
            
            /* Loop the string */
            for (register u32 i = 0; i < len_; ++i)
            {
                if (str.str_[i] >= ascLowerA() && str.str_[i] <= ascLowerA() + 26)
                    str.str_[i] += (ascUpperA() - ascLowerA());
            }
            
            /* Return result */
            return str;
        }
        
        string<T> lower() const
        {
            /* Allocate temporary variables */
            string<T> str(str_, len_);
            
            /* Loop the string */
            for (register u32 i = 0; i < len_; ++i)
            {
                if (str.str_[i] >= ascUpperA() && str.str_[i] <= ascUpperA() + 26)
                    str.str_[i] += (ascLowerA() - ascUpperA());
            }
            
            /* Return result */
            return str;
        }
        
        s32 find(const string<T> &str, u32 start = 0) const
        {
            /* Check if the search string is not longer than this string */
            if (str.len_ > len_)
                return -1;
            
            /* Allocate temporary memory */
            T* cmp = new T[len_ + 1];
            T* oldcmp = cmp;
            T tmp;
            
            /* Copy memory */
            std::copy(begin(), end(), &cmp[0]);
            
            /* Move memory */
            cmp += start;
            
            /* Loop the memory */
            for (register u32 i = start; i < len_ - str.len_ + 1; ++i, ++cmp)
            {
                /* Fill the temporary variable */
                tmp = cmp[str.len_];
                cmp[str.len_] = (T)0;
                
                /* Compare memories */
                if (memcmp(cmp, str.str_, str.len_ * sizeof(T)) == 0)
                {
                    /* Delete the old temporary memory */
                    delete [] oldcmp;
                    
                    /* Return the found position */
                    return i;
                }
                
                /* Fill the compare variable */
                cmp[str.len_] = tmp;
            }
            
            /* Delete the old temporary memory */
            delete [] oldcmp;
            
            /* Not found */
            return -1;
        }
        
        string<T> trim(const string<T> &TrimCandidates = " ") const // (e.g. " \t" -> will cut the ' ' char and the '\t' char)
        {
            /* Allocate temporary variables */
            string<T> tmp, str;
            T TrimChar;
            u32 len = 0;
            bool use = false;
            
            /* Create temporary memory */
            tmp.create(len_);
            
            /* Loop memory */
            for (register u32 i = 0, j; i < len_; ++i)
            {
                use = true;
                for (j = 0; j < TrimCandidates.size(); ++j)
                {
                    if (str_[i] == TrimCandidates.str_[j])
                    {
                        use = false;
                        break;
                    }
                }
                if (use)
                {
                    tmp.str_[len] = str_[i];
                    ++len;
                }
            }
            
            /* Create finaly string */
            tmp.str_[len] = (T)0;
            str.create(len);
            str = tmp;
            
            return str;
        }
        
        string<T> replace(const string<T> &Search, const string<T> &Replace, bool HoldSize = false) const
        {
            /* Check if length of the search string
               is not longer than this string */
            if (Search.len_ > len_)
                return *this;
            
            /* Allocate temporary memory */
            string<T> str;
            
            /* Select the type of replaceing */
            if (HoldSize)
            {
                /* Allocate temporary variables */
                u32 j = 0;
                
                /* Copy the memory */
                str = *this;
                
                /* Loop the string */
                while ( ( j = find(Search, j+1) ) != -1 )
                {
                    /* Copy the replaced part */
                    std::copy(Replace.begin(), Replace.end(), str.begin() + j);
                }
            }
            else
            {
                /* Allocate temporary variables */
                u32 i = 0, j = 0;
                
                /* Loop the string */
                while ( ( j = find(Search, j+1) ) != -1 )
                {
                    /* Append the used part */
                    str.append(part(i, j));
                    str.append(Replace);
                    i = j + Search.size();
                }
                
                /* Append the rest of the string */
                str.append(part(i, len_));
            }
            
            return str;
        }
        
        /* Extendet functions */
        
        inline c8* cstr() const
        {
            static_cast<c8*>(str_);
        }
        
        string<T> section(u32 begin, u32 end) const
        {
            return part(begin, end);
        }
        
        string<T> getFilePath() const
        {
            string<T> str = replace("\\", "/");
            s32 i = 0, j = 0;
            
            while ( ( i = str.find("/", i+1) ) != -1 )
                j = i;
            
            if (j != -1)
                str = left(j + 1);
            
            return str;
        }
        
        string<T> getFileExtension() const
        {
            s32 i = 1, j = 0;
            
            do
            {
                j = i;
                i = find(".", i + 1);
            }
            while (i != -1);
            
            i = j;
            
            return part(j + 1, size());
        }
        
        string<T> ltrim() // trims only the left side
        {
            std::string str = str_;
            
            while (str[0] == ' ' || str[0] == '\t')
                str = str.substr(1, str.size() - 1);
            
            return string<T>(str);
        }
        string<T> rtrim() // trims only the right side
        {
            std::string str = str_;
            
            while (str[str.size() - 1] == ' ' || str[str.size() - 1] == '\t')
                str = str.substr(0, str.size() - 2);
            
            return string<T>(str);
        }
        
        static string<T> space(u32 count, c8 Ascii = ' ')
        {
            /* Allocate memory */
            string<T> str;
            
            /* Resize memory */
            str.create(count);
            
            /* Fill memory */
            std::fill(str.begin(), str.end(), (T)Ascii);
            
            /* Return the result */
            return str;
        }
        
        static string<c8> makeIdNum(f32 Number, u32 IdentityLen = 10)
        {
            string<c8> str(Number), val(Number);
            
            if (static_cast<s32>(val.find(".", 1)) == -1)
                val = val + string<c8>(".0");
            
            if (Number < 0)
            {
                str = val + space(IdentityLen - val.size(), '0');
            }
            else
            {
                str = string<c8>(" ") + val + space(IdentityLen - val.size(), '0');
            }
            
            return str;
        }
        
    private:
        
        /* Members */
        
        T* str_;
        
        u32 len_;
        
};


typedef string<c8>  stringc;
typedef string<c16> stringw;


/* Extendet template function */

template <typename T> stringc getHex(T Numeric)
{
    stringc str;
    
    for (s32 i = 2*sizeof(T) - 1; i >= 0; --i)
    {
        str += "0123456789ABCDEF"[((Numeric >> i*4) & 0xF)];
    }
    
    return str;
}


} // /namespace io

} // /namespace sp


#endif



#endif







// ================================================================================
