/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Main device
 *
 */

#include "spMainDevice.hpp"
#include "spOpenGLDriver.hpp"
#include "spSoftwareDriver.hpp"
#include "spDirect3D9Driver.hpp"
#include "spInternalDeclerations.hpp"

#include <unistd.h>

#if defined(WIN32)
#   include <windows.h>
#elif defined(LINUX)
#   include <sys/utsname.h>
#endif


namespace sp
{


/*
 * Internal member
 */

SoftPixelDevice*            __spDevice              = 0;
video::VideoDriver*         __spVideoDriver         = 0;
scene::SceneManager*        __spSceneManager        = 0;
scene::CollisionDetector*   __spCollisionDetector   = 0;
UserControl*                __spUserControl         = 0;
io::FileLoader*             __spFileSystem          = 0;
io::OSInformator*           __spOSInformator        = 0;
script::ScriptLoader*       __spScriptLoader        = 0;

io::stringc spUserCharList;

static const s32 DIRSTR_MAXLEN  = 256;


void spPrintConsoleHeader();

#if defined(WIN32)

SP_EXPORT SoftPixelDevice* createDevice(
    video::EDriverTypes VideoDriverType,
    dim::size2di Dimension, s32 ColorDepth, io::stringc Title,
    bool isFullScreen, SDeviceFlags Flags)
{
	/* General settings */
    if (Title == "")
    {
        c8 DirName[DIRSTR_MAXLEN];
        GetCurrentDirectory(DIRSTR_MAXLEN, DirName);
        Title = DirName;
    }
    if (isFullScreen)
        ShowCursor(false);
    
    /* Create the SoftPixelDevice */
    __spDevice = new SoftPixelDevice(
        VideoDriverType, 0, Dimension, ColorDepth, Title, isFullScreen,
        Flags.isResizAble, Flags.isVsync, Flags.isAntiAlias, Flags.MultiSample
    );
    
    /* Print the header */
    spPrintConsoleHeader();
    
    /* Return the device */
    return __spDevice;
}

SP_EXPORT SoftPixelDevice* createDeviceWnd(
    video::EDriverTypes VideoDriverType,
    HWND hWndParent, dim::size2di Dimension, s32 ColorDepth, io::stringc Title, SDeviceFlags Flags)
{
	/* General settings */
    if (Title == "")
    {
        c8 DirName[DIRSTR_MAXLEN];
        GetCurrentDirectory(DIRSTR_MAXLEN, DirName);
        Title = DirName;
    }
    
    /* Create the SoftPixelDevice */
    __spDevice = new SoftPixelDevice(
        VideoDriverType, hWndParent, Dimension, ColorDepth, Title, false, false,
        Flags.isVsync, Flags.isAntiAlias, Flags.MultiSample
    );
    
    /* Print the header */
    spPrintConsoleHeader();
    
    /* Return the device */
    return __spDevice;
}

SP_EXPORT void deleteDevice()
{
    spDeletePointer(__spDevice);
}

/*
 * Window call back
 * All window messages (key & cursor input) are received in this function
 */

SP_EXPORT LRESULT CALLBACK spWindowCallback(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
    static s32 LastCursorX, LastCursorY;
    
    switch (Message)
    {
        case WM_CLOSE:
        {
            __spDevice->clearDevice();
            PostQuitMessage(0);
        }
        return 0;
        
        case WM_KEYDOWN:
        {
            if (!__isKey[(s32)wParam])
                __hitKey[(s32)wParam] = true;
            __isKey[(s32)wParam] = true;
        }
        return 0;
        
        case WM_KEYUP:
        {
            __wasKey[(s32)wParam] = true;
            __isKey[(s32)wParam] = false;
        }
        return 0;
        
        case WM_CHAR:
        {
            spUserCharList += (TCHAR)wParam;
        }
        return 0;
        
        case WM_LBUTTONDOWN:
        {
            if (!__isMouseKey[MOUSE_LEFT])
                __hitMouseKey[MOUSE_LEFT] = true;
            __isMouseKey[MOUSE_LEFT] = true;
        }
        return 0;
        
        case WM_LBUTTONUP:
        {
            __wasMouseKey[MOUSE_LEFT] = true;
            __isMouseKey[MOUSE_LEFT] = false;
        }
        return 0;
        
        case WM_RBUTTONDOWN:
        {
            if (!__isMouseKey[MOUSE_RIGHT])
                __hitMouseKey[MOUSE_RIGHT] = true;
            __isMouseKey[MOUSE_RIGHT] = true;
        }
        return 0;
        
        case WM_RBUTTONUP:
        {
            __wasMouseKey[MOUSE_RIGHT] = true;
            __isMouseKey[MOUSE_RIGHT] = false;
        }
        return 0;
        
        case WM_MBUTTONDOWN:
        {
            if (!__isMouseKey[MOUSE_MIDDLE])
                __hitMouseKey[MOUSE_MIDDLE] = true;
            __isMouseKey[MOUSE_MIDDLE] = true;
        }
        return 0;
        
        case WM_MBUTTONUP:
        {
            __wasMouseKey[MOUSE_MIDDLE] = true;
            __isMouseKey[MOUSE_MIDDLE] = false;
        }
        return 0;
        
        case WM_MOUSEMOVE:
        {
            __CursorSpeedX = LOWORD(lParam) - LastCursorX;
            __CursorSpeedY = HIWORD(lParam) - LastCursorY;
            LastCursorX = LOWORD(lParam);
            LastCursorY = HIWORD(lParam);
        }
        return 0;
        
        case WM_ERASEBKGND:
        {
        }
        return 0;
        
        case WM_SIZE:
        {
            if (__spDevice)
                __spDevice->resizeScreen(dim::size2di(LOWORD(lParam), HIWORD(lParam)));
        }
        return 0;
        
        case WM_MOUSEWHEEL:
        {
            __MouseWheel = (s16)(((f32)((s16)HIWORD(wParam))) / (f32)WHEEL_DELTA);
        }
        return 0;
        
        case WM_SYSCOMMAND:
        {
            // Prevent for a powersave mode of monitor or the screensaver
            if ((wParam & 0xFFF0) == SC_SCREENSAVE || (wParam & 0xFFF0) == SC_MONITORPOWER)
                return 0;
        }
        break;
    }
    
    return DefWindowProc(hWnd, Message, wParam, lParam);
}

// 'SoftPixel Engine' main device class

SoftPixelDevice::SoftPixelDevice(
    video::EDriverTypes VideoDriverType,
    HWND hWndParent, dim::size2di Dimension, s32 ColorDepth, io::stringc Title,
    bool isFullScreen, bool isResizAble, bool isVsync, bool isAntiAlias, s32 MultiSample)
    : hMainWnd_(0), MultiSampleFormat_(0), MultiSamplePixelFormat_(0)
{
    
    /* General settings */
    isWindowOpened_     = true;
    isResizeUpdate_     = true;
    isAntiAlias_        = isAntiAlias;
    MultiSampleFormat_  = MultiSample;
    FrameRate_          = 0;
    
    /* Get the video driver type */
    DriverType_ = VideoDriverType;
    
    /* Renderer diferences */
    if (DriverType_ == video::DRIVER_DIRECT3D9)
        MultiSamplePixelFormat_ = MultiSample;
    
    /* Other checks */
    if (isFullScreen)
        isResizAble = false;
    if (!isAntiAlias_)
        MultiSampleFormat_ = 0;
    
    if (DriverType_ == video::DRIVER_AUTODETECT)
        autoDetectVideoDriver();
    
    /* Create the video driver */
    switch (DriverType_)
    {
        case video::DRIVER_SOFTWARE:
            __spVideoDriver = new video::SoftwareDriver(); break;
        case video::DRIVER_OPENGL:
            __spVideoDriver = new video::OpenGLDriver(); break;
        case video::DRIVER_DIRECT3D9:
            #if defined(WIN32)
            __spVideoDriver = new video::Direct3D9Driver(); break;
            #elif defined(LINUX)
            io::printError("Direct3D9 is only supported under Win32");
            #endif
    }
    
    /* Rengister window class */
    initEngine();
    
    /*
     * Open the screen
     * Pixel format, device- & render context etc.
     */
    if (!openScreen(hWndParent, Dimension, ColorDepth, Title, isFullScreen, isResizAble, isAntiAlias_))
        return;
    
    /* Initialzie the renderer */
    __spVideoDriver->initRenderer();
    
    /* Vertical Synchronisation */
    if (isVsync)
        __spVideoDriver->setVsync(true);
    
}

SoftPixelDevice::~SoftPixelDevice()
{
    deleteDevice();
}

#elif defined(LINUX)

#endif

/* Video driver for the video buffer */
video::VideoDriver* SoftPixelDevice::getVideoDriver()
{
    return __spVideoDriver;
}

/* Scene manager for 3d rendering operations */
scene::SceneManager* SoftPixelDevice::getSceneManager()
{
    return __spSceneManager ? __spSceneManager : __spSceneManager = new scene::SceneManager();
}

/* Collision detector for collision detection and intersection arithmetic */
scene::CollisionDetector* SoftPixelDevice::getCollisionDetector()
{
    return __spCollisionDetector ? __spCollisionDetector : __spCollisionDetector = new scene::CollisionDetector();
}

/* User control for user input-/ output communication */
UserControl* SoftPixelDevice::getUserControl()
{
    return __spUserControl ? __spUserControl : __spUserControl = new UserControl();
}

/* Filesystem for read-/ write files */
io::FileLoader* SoftPixelDevice::getFileSystem()
{
    return __spFileSystem ? __spFileSystem : __spFileSystem = new io::FileLoader();
}

/* OS informator for informations about the operating-system */
io::OSInformator* SoftPixelDevice::getOSInformator()
{
    return __spOSInformator ? __spOSInformator : __spOSInformator = new io::OSInformator();
}

/* Script loader (script extension) for simple file loading (textures, sounds, 3d models) */
script::ScriptLoader* SoftPixelDevice::getScriptLoader(u32 TexMax, u32 SoundMax, u32 ModelMax)
{
    return __spScriptLoader ? __spScriptLoader : __spScriptLoader = new script::ScriptLoader(TexMax, SoundMax, ModelMax);
}

void SoftPixelDevice::deleteDevice()
{
    
    if (isWindowOpened_)
    {
        
        isWindowOpened_ = false;
        
        /* Delete all objects before deleting the screen */
        __spVideoDriver->clearGUI();
        __spVideoDriver->clearTextureList();
        __spVideoDriver->clearBuffers();
        
        /* Close the screen & release all main contexts */
        closeScreen();
        
        /* Delete all poiters */
        spDeletePointer(__spSceneManager);
        spDeletePointer(__spCollisionDetector);
        spDeletePointer(__spUserControl);
        spDeletePointer(__spScriptLoader);
        spDeletePointer(__spVideoDriver);
        
        #if defined(WIN32)
        /* Unregister window class */
        if (!UnregisterClass(ClassName_, GetModuleHandle(0)))
            io::printError("release window class failed");
        #endif
        
        /* Close the possible debug log file */
        io::setDebugLog(false);
        
    }
    
}

#if defined(WIN32)

bool SoftPixelDevice::updateEvent()
{
    // Resettings
    memset(__wasKey, 0, sizeof(__wasKey));
    memset(__wasMouseKey, 0, sizeof(__wasMouseKey));
    memset(__hitKey, 0, sizeof(__hitKey));
    memset(__hitMouseKey, 0, sizeof(__hitMouseKey));
    
    __MouseWheel = 0;
    if (__spUserControl)
        __spUserControl->resetEvents();
    
    // Framerate delay
    if (FrameRate_ > 0)
        Sleep((DWORD)FrameRate_);
    
    // Update window event
    while (PeekMessage(&Msg_, 0, 0, 0, PM_REMOVE))
    {
        TranslateMessage(&Msg_);
        DispatchMessage(&Msg_);
    }
    
    return isWindowOpened_;
}

void SoftPixelDevice::changeGFXMode(
    HWND hWndParent, dim::size2di Dimension, s32 ColorDepth, io::stringc Title,
    bool isFullScreen, bool isResizAble)
{
    closeScreen();
    openScreen(hWndParent, Dimension, ColorDepth, Title, isFullScreen, isResizAble, false);
}

#elif defined(LINUX)

#endif

void SoftPixelDevice::setResizeMode(bool isResizeUpdate)
{
    isResizeUpdate_ = isResizeUpdate;
}

void SoftPixelDevice::setAntiAlias(bool isAntiAlias, s32 MultiSample)
{
    /* Check the video driver type */
    if (DriverType_ == video::DRIVER_SOFTWARE)
    {
        io::printWarning("The \"Ray\" SoftwareRenderer does not support anti-aliasing");
        return;
    }
    
    /* General settings */
    isAntiAlias_            = isAntiAlias;
    if (isAntiAlias_)
        MultiSampleFormat_  = MultiSample;
    else
        MultiSampleFormat_  = 0;
    MultiSamplePixelFormat_ = 0;
    
    /* Initialize anitalias */
    __spVideoDriver->initAntiAlias(hMainWnd_, isAntiAlias_, MultiSamplePixelFormat_, MultiSampleFormat_, ColorDepth_);
    
    /* Close the screen */
    closeScreen();
    
    /* Reopen a new screen */
    openScreen(GetParent(hMainWnd_), ScreenDimension_, ColorDepth_, Title_, isFullScreen_, isResizAble_, true);
    
    /* Reinitialzie the renderer */
    __spVideoDriver->initRenderer();
    
    /* Reload all textures */
    __spVideoDriver->reloadTextureList();
}

void SoftPixelDevice::manipulateScreenSize(dim::size2di ScreenSize)
{
    ScreenDimension_    = ScreenSize;
    __Screen_Width      = ScreenDimension_.Width;
    __Screen_Height     = ScreenDimension_.Height;
}

void SoftPixelDevice::setFrameRate(f32 FrameRate)
{
    if (FrameRate > 0)
        FrameRate_ = __SP_SEC / FrameRate;
    else
        FrameRate_ = 0;
}

f32 SoftPixelDevice::getFPS()
{
    static clock_t lasttime, fps;
    static s32 frames;
    
    ++frames;
    
    clock_t curtime = clock();
    
    if ( ( curtime - lasttime ) > __SP_SEC/2 )
    {
        fps         = (curtime - lasttime) / frames;
        lasttime    = curtime;
        frames      = 0;
    }
    
    return __SP_SEC / fps;
}

u64 SoftPixelDevice::getMilliseconds()
{
    return static_cast<u64>(clock());
}

#if defined(WIN32)

u16 SoftPixelDevice::getTime(u8 Type)
{
    WORD TimeVal = 0;
    SYSTEMTIME sTime;
    
    GetLocalTime(&sTime);
    
    switch (Type)
    {
        case TIME_SECOND:
            TimeVal = sTime.wSecond; break;
        case TIME_MINUTE:
            TimeVal = sTime.wMinute; break;
        case TIME_HOURE:
            TimeVal = sTime.wHour; break;
        case TIME_DAY:
            TimeVal = sTime.wDay; break;
        case TIME_MONTH:
            TimeVal = sTime.wMonth; break;
        case TIME_YEAR:
            TimeVal = sTime.wYear; break;
    }
    
    return (u16)TimeVal;
}

void SoftPixelDevice::setWindowTitle(io::stringc Title)
{
    SetWindowText(hMainWnd_, Title.c_str());
    Title_ = Title;
}
io::stringc SoftPixelDevice::getWindowTitle() const
{
    char lpTitle[256];
    GetWindowText(hMainWnd_, lpTitle, 256);
    return io::stringc(std::string(lpTitle));
}

bool SoftPixelDevice::hasWindowResized() const
{
    return hasWindowResized_;
}

bool SoftPixelDevice::isWindowActive() const
{
    return GetForegroundWindow() == hMainWnd_;
}

io::stringc SoftPixelDevice::getUserCharList() const
{
    io::stringc Result = spUserCharList;
    spUserCharList = "";
    return Result;
}

#elif defined(LINUX)

#endif

io::stringc SoftPixelDevice::getVersion()
{
    return io::stringc("SoftPixel Engine - v.1.7");
}

#if defined(WIN32)

void SoftPixelDevice::sleep(u32 Milliseconds)
{
    Sleep((DWORD)Milliseconds);
}
void SoftPixelDevice::beep(u32 Milliseconds, u32 Frequency)
{
    Beep(Frequency, Milliseconds);
}

#elif defined(LINUX)

void SoftPixelDevice::sleep(u32 Milliseconds)
{
    usleep(Milliseconds * 1000);
}

#endif


// ===== Private: =====


#if defined(WIN32)

void SoftPixelDevice::initEngine()
{
    sprintf(ClassName_, "SoftPixelEngine_Class");
    
    WNDCLASS wincl;
    
    /* Fill the window class */
    wincl.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wincl.hInstance     = GetModuleHandle(0);
    wincl.lpfnWndProc   = (WNDPROC)sp::spWindowCallback;
    wincl.hIcon         = LoadIcon(0, IDI_APPLICATION);
    wincl.hCursor       = LoadCursor(0, IDC_ARROW);
    wincl.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wincl.cbClsExtra    = 0;
    wincl.cbWndExtra    = 0;
    wincl.lpszMenuName  = 0;
    wincl.lpszClassName = ClassName_;
    
    /* Register window class */
    if (!RegisterClass(&wincl))
        io::printError("could not register window-class!");
    
    /* Initialize standard memories */
    memset(__isKey, 0, sizeof(__isKey));
    memset(__isMouseKey, 0, sizeof(__isMouseKey));
    memset(__wasKey, 0, sizeof(__wasKey));
    memset(__wasMouseKey, 0, sizeof(__wasMouseKey));
    memset(__hitKey, 0, sizeof(__hitKey));
    memset(__hitMouseKey, 0, sizeof(__hitMouseKey));
}

bool SoftPixelDevice::openScreen(
    HWND hWndParent, dim::size2di Dimension, s32 ColorDepth, io::stringc Title,
    bool isFullScreen, bool isResizAble, bool isAntiAlias)
{
    
    /* Temporary variables */
    
    s32 WinBorderWidth, WinBorderHeight, WinCaptionHeight;
    
    if (isFullScreen)
    {
        WinBorderWidth      = 0;
        WinBorderHeight     = 0;
        WinCaptionHeight    = 0;
    }
    else
    {
        WinBorderWidth      = GetSystemMetrics(SM_CXFIXEDFRAME);
        WinBorderHeight     = GetSystemMetrics(SM_CYFIXEDFRAME);
        WinCaptionHeight    = GetSystemMetrics(SM_CYCAPTION);
    }
    
    /* General settings */
    
    __Screen_Width      = Dimension.Width;
    __Screen_Height     = Dimension.Height;
    
    ScreenDimension_    = Dimension;
    ColorDepth_         = ColorDepth;
    Title_              = Title;
    isFullScreen_       = isFullScreen;
    isResizAble_        = isResizAble;
    
    dim::point2di Position;
    
    if (!isFullScreen)
    {
        Position.set(
            GetSystemMetrics(SM_CXSCREEN)/2 - Dimension.Width/2,
            GetSystemMetrics(SM_CYSCREEN)/2 - Dimension.Height/2
        );
    }
    
    /* Display settings */
    
    __spVideoDriver->changeDisplaySettings(isFullScreen, Dimension, ColorDepth);
    
    /* Windowed Screen */
    
    if (hWndParent) // Owner draw
    {
        hMainWnd_ = hWndParent;
    }
    else // Own window
    {
        hMainWnd_ = CreateWindowEx(
            WS_EX_APPWINDOW,
            ClassName_,
            TEXT(Title.c_str()),
            isFullScreen ?
                WS_POPUP :
                isResizAble_ ?
                    WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_SIZEBOX :
                    WS_SYSMENU | WS_MINIMIZEBOX,
            Position.X - WinBorderWidth, Position.Y - WinBorderHeight - WinCaptionHeight/2,
            Dimension.Width + WinBorderWidth*2, Dimension.Height + WinBorderHeight*2 + WinCaptionHeight,
            0,
            0,
            GetModuleHandle(0),
            0
        );
        
        if (!hMainWnd_)
        {
            io::printError("could not create main window");
            return false;
        }
    }
    
    /* Init the Screen using the video driver */
    if (!__spVideoDriver->initScreen(hMainWnd_, isAntiAlias, MultiSamplePixelFormat_, ColorDepth))
    {
        io::printError("initialization of video driver failed");
        return false;
    }
    
    /* Anti alias */
    if (isAntiAlias && DriverType_ == video::DRIVER_OPENGL)
    {
        if (MultiSamplePixelFormat_ == 0)
        {
            /* Initialize anti alias */
            __spVideoDriver->initAntiAlias(hMainWnd_, isAntiAlias_, MultiSamplePixelFormat_, MultiSampleFormat_, ColorDepth_);
            
            /* Close the screen */
            closeScreen();
            
            /* Reopen the screen */
            openScreen(hWndParent, ScreenDimension_, ColorDepth_, Title_, isFullScreen_, isResizAble_, true);
            
            /* Exit the function */
            return true;
        }
    }
    
    /* Main windows visible */
    ShowWindow(hMainWnd_, SW_NORMAL);
    
    if (!hWndParent)
        SetForegroundWindow(hMainWnd_);
    
    return true;
    
}

void SoftPixelDevice::closeScreen()
{
    
    /* Release the device & resource context */
    __spVideoDriver->clearScreen(hMainWnd_);
    
    /* Close the used display settings */
    if (isFullScreen_)
        ChangeDisplaySettings(0, 0);
    
}

void SoftPixelDevice::autoDetectVideoDriver()
{
    #if defined(WIN32)
    
    c8 SysDir[256];
    GetSystemDirectory(SysDir, 256);
    
    DriverType_ = video::DRIVER_DIRECT3D9;
    
    if (!io::FileLoader().doesFileExist(io::stringc(SysDir) + io::stringc("\\d3d9.dll")))
        DriverType_ = video::DRIVER_OPENGL;
    
    #elif defined(LINUX)
    
    DriverType_ = video::DRIVER_OPENGL;
    
    #endif
    
    if (DriverType_ == video::DRIVER_OPENGL && io::stringc(glGetString(GL_VERSION)).left(1).val<s32>() < 2)
        DriverType_ = video::DRIVER_SOFTWARE;
}

#elif defined(LINUX)

#endif

void SoftPixelDevice::resizeScreen(dim::size2di Dimension)
{
    if (isResizAble_)
    {
        /* General settings */
        if (isResizeUpdate_)
        {
            ScreenDimension_ = Dimension;
            ScreenDimension_.get(__Screen_Width, __Screen_Height);
        }
        
        WindowDimension_ = Dimension;
        
        /* Temporary height swaping */
        s32 tmp = __Screen_Height;
        __Screen_Height = Dimension.Height;
        
        /* Viewport */
        __spVideoDriver->setViewport(0, 0, Dimension.Width, Dimension.Height);
        
        /* Back settings */
        __Screen_Height = tmp;
    }
}

void SoftPixelDevice::clearDevice()
{
    if (isWindowOpened_)
        isWindowOpened_ = false;
}

/* Other function (no part of the SoftPixelDevice) */

void spPrintConsoleHeader()
{
    if (__spDevice)
    {
        io::OSInformator tOSInfo;
        
        io::printMessage(__spDevice->getVersion());
        io::printMessage("Copyright (c) 2008 - Lukas Hermanns");
        io::printMessage(tOSInfo.getOSVersion());
        io::printMessage(io::stringc("Using renderer: ")+__spVideoDriver->getVersion());
        io::printMessage(__spVideoDriver->getRenderer()+io::stringc(": ")+__spVideoDriver->getVendor());
        
        __spVideoDriver->printWarning();
        
        io::printMessage("");
    }
    else
        io::printError("could not create 'SoftPixel Engine' device");
}


} // /namespace sp


// DLL main function

#ifdef __SP_EXPORT_TO_LIBRARY

#if defined(WIN32)

BOOL APIENTRY DllMain(
    HANDLE hModule,
    DWORD dwMessage,
    LPVOID lpReserved)
{
    switch (dwMessage)
    {
        case DLL_PROCESS_ATTACH:
        case DLL_THREAD_ATTACH:
        case DLL_THREAD_DETACH:
        case DLL_PROCESS_DETACH:
            break;
    }
    return TRUE;
}

#elif defined(LINUX)

#endif


#endif






// ================================================================================
