/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for all standard memory management functions
 *
 */

#ifndef __SP_MEMORY_MANAGEMENT_H__
#define __SP_MEMORY_MANAGEMENT_H__


#include "spInputOutputStandard.hpp"

#include <vector>
#include <stdarg.h>


namespace sp
{


template <typename T> void spDeletePointer(T* &pBuffer)
{
    if (pBuffer)
    {
        delete pBuffer;
        pBuffer = 0;
    }
}

template <typename T> void spDeleteArray(T* &pBuffer)
{
    if (pBuffer)
    {
        delete [] pBuffer;
        pBuffer = 0;
    }
}

template <typename T> T* spAllocatePointer()
{
    T* pBuffer = new T;
    
    if (!pBuffer)
        io::printError(io::stringc("Could not allocate memory for ")+io::stringc(sizeof(T))+io::stringc(" byte(s)"));
    
    return pBuffer;
}

template <typename T> T* spAllocateArray(const s32 Count)
{
    T* pBuffer = new T[Count];
    
    if (!pBuffer)
        io::printError(io::stringc("Could not allocate memory for ")+io::stringc(sizeof(T)*Count)+io::stringc(" byte(s)"));
    
    return pBuffer;
}

template <typename T> std::vector<T> spGetList(u32 Count, ...)
{
    std::vector<T> tmpList;
    
    va_list vl;
    va_start(vl, Count);
    
    for (u32 i = 0; i < Count; ++i)
        tmpList.push_back(va_arg(vl, T));
    
    va_end(vl);
    
    return tmpList;
}

template <typename T> void spRemoveElement(std::vector<T*> &List, T* &Object, bool Delete = false, bool MultipleObjects = false)
{
    for (typename std::vector< T*, std::allocator<T*> >::iterator it = List.begin(); it != List.end();)
    {
        if (*it == Object)
        {
            if (Delete)
                spDeletePointer(Object);
            
            it = List.erase(it);
            
            if (!MultipleObjects)
                break;
        }
        else
            ++it;
    }
}



} // /namespace sp


#endif









// ================================================================================
