/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model loading MD2-fileformat (Quake II)
 *
 */

#ifndef __SP_MODELLOADER_MD2_H__
#define __SP_MODELLOADER_MD2_H__


#include "spStandard.hpp"
#include "spInputOutputStandard.hpp"
#include "spDimension.hpp"
#include "spVideoDriverTexture.hpp"
#include "spModelLoaderRAW.hpp"

#include <fstream>
#include <vector>
#include <string>


namespace sp
{
namespace scene
{


#define __SP_SHADEDOT_QUANT         16
#define __SP_MD2_NUMVERTEXNORMALS   162


enum EMD2AnimTypes
{
    MD2ANIM_STAND = 0,
    MD2ANIM_RUN,
    MD2ANIM_ATTACK,
    MD2ANIM_PAIN_A,
    MD2ANIM_PAIN_B,
    MD2ANIM_PAIN_C,
    MD2ANIM_JUMP,
    MD2ANIM_FLIP,
    MD2ANIM_SALUTE,
    MD2ANIM_FALLBACK,
    MD2ANIM_WAVE,
    MD2ANIM_POINT,
    MD2ANIM_CROUCH_STAND,
    MD2ANIM_CROUCH_WALK,
    MD2ANIM_CROUCH_ATTACK,
    MD2ANIM_CROUCH_PAIN,
    MD2ANIM_CROUCH_DEATH,
    MD2ANIM_DEATH_FALLBACK,
    MD2ANIM_DEATH_FALLFORWARD,
    MD2ANIM_DEATH_FALLBACKSLOW,
    MD2ANIM_BOOM,
    
    MD2ANIM_MAX_ANIMATIONS
};


class SP_EXPORT ModelLoaderMD2 : public ModelLoaderRAW
{
    
    public:
        
        ModelLoaderMD2();
        ~ModelLoaderMD2();
        
        Entity* loadModel(io::stringc &Filename, io::stringc &TexturePath);
        
        static void getAnimationSeq(s32 Type, u32 &Start, u32 &End);
        
    private:
        
        /* === Structures === */
        
        typedef struct SHeaderMD2Info
        {
            s32 ident;      // magic number. must be equal to "IDP2"
            s32 version;    // md2 version. must be equal to 8
            
            s32 skinwidth;  // width of the texture
            s32 skinheight; // height of the texture
            s32 framesize;  // size of one frame in bytes
            
            s32 num_skins;  // number of textures
            s32 num_xyz;    // number of vertices
            s32 num_st;     // number of texture coordinates
            s32 num_tris;   // number of triangles
            s32 num_glcmds; // number of opengl commands
            s32 num_frames; // total number of frames
            
            s32 ofs_skins;  // offset to skin names (64 bytes each)
            s32 ofs_st;     // offset to s-t texture coordinates
            s32 ofs_tris;   // offset to triangles
            s32 ofs_frames; // offset to frame data
            s32 ofs_glcmds; // offset to opengl commands
            s32 ofs_end;    // offset to end of file
        }
        headerMD2Info;
        
        typedef struct SMD2Vertex
        {
            dim::vector3d<u8> Vertex;
            u8 LightNormalIndex;
        }
        md2VertexInfo;
        
        //typedef dim::point2d<s16> md2TexCoordInfo;
        
        typedef struct SMD2KeyFrame
        {
            dim::vector3df Scale;
            dim::vector3df Translate;
            s8 Name[16];
            md2VertexInfo Vertices[1];
        }
        md2KeyFrameInfo;
        
        typedef struct SMD2Triangle
        {
            s16 IndexVertex[3];
            s16 IndexTexCoord[3];
        }
        md2TriangleInfo;
        
        typedef struct SMD2AnimInfo
        {
            s32 FirstFrame; // first frame of animation
            s32 LastFrame;  // number of frames
            s32 FPS;        // number of frames
        }
        md2AnimInfo;
        
        typedef struct SMD2AnimStateInfo
        {
            s32 StartFrame;
            s32 EndFrame;
            s32 FPS;
            
            f32 CurTime;
            f32 OldTime;
            f32 Interpol;   // percent of interpolation
            
            s32 Type;       // animation type
            s32 CurFrame;
            s32 NextFrame;
        }
        md2AnimStateInfo;
        
        /* === Private functions === */
        
        void init();
        void clear();
        
        bool loadModelData(io::stringc Filename);
        void buildAnimation();
        Entity* buildModel();
        
        void interpolate(dim::vector3df* VerticesList);
        
        /* === Members === */
        
        std::ifstream hFile_;
        
        s32 KeyFramesCount_;
        s32 VerticesCount_;
        s32 CommandsCount_;
        
        dim::vector3df* pVertices_;
        s32* pCommands_;
        s32* pLightNormals_;
        
        md2AnimStateInfo Animation_;
        f32 Scale_;
        
        //static dim::vector3df Anorms_[__SP_MD2_NUMVERTEXNORMALS];
        //static f32 AnormsDots_[__SP_SHADEDOT_QUANT][256];
        static md2AnimInfo AnimList_[21];
        
};


} // /namespace scene

} // /namespace sp


#endif





// ================================================================================
