/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Model loading OBJ-fileformat (Wavefront)
 *
 */

#include "spModelLoaderOBJ.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace scene
{


ModelLoaderOBJ::ModelLoaderOBJ()
{
    init();
}
ModelLoaderOBJ::~ModelLoaderOBJ()
{
    clear();
}

Entity* ModelLoaderOBJ::loadModel(io::stringc &Filename, io::stringc &TexturePath)
{
    Filename_       = Filename;
    TexturePath_    = TexturePath;
    
    hEntity_        = new Entity();
    
    if ( !( hFile_ = FileLoader_->openFile(Filename, io::FILE_READ_BINARY) ) )
        return hEntity_;
    
    if (!loadModelData())
        io::printError("loading OBJ model failed");
    
    FileLoader_->closeFile(hFile_);
    
    return hEntity_;
}


/*
 * ========== Private: ==========
 */


void ModelLoaderOBJ::init()
{
    /* General settings */
    CurSurface_         = 0;
    ModelCreateStatus_  = OBJ_MODELCREATE_NEWSURFACE;
    
    /* Allocate all pointers */
    FileLoader_         = new io::FileLoader();
}
void ModelLoaderOBJ::clear()
{
    /* Delete all pointers */
    spDeletePointer(FileLoader_);
}

void ModelLoaderOBJ::readLine()
{
    
    /* Temporary variables */
    io::stringc LineTrim(CurLine_.trim());
    
    const c8* str = CurLine_.c_str();
    
    f32 pFloatList[4];
    s32 pIntegerList[4];
    
    /* Examine the line */
    switch (str[0])
    {
        case '#': // comment
            return;
        
        case 'v': // vertex
        {
            switch (str[1])
            {
                case ' ': // coordinate
                {
                    if (ModelCreateStatus_ == OBJ_MODELCREATE_TRIANGLES)
                        createNewSurface();
                    
                    ModelCreateStatus_ = OBJ_MODELCREATE_VERTICES;
                    
                    getFloatList(pFloatList);
                    
                    SVertexOBJ VertexData;
                    VertexData.Coord.set(pFloatList[0], pFloatList[1], -pFloatList[2]);
                    SurfacesList_[CurSurface_].VerticesList.push_back(VertexData);
                }
                break;
                
                case 'n': // normal
                {
                    ModelCreateStatus_ = OBJ_MODELCREATE_TRIANGLES;
                    
                    getFloatList(pFloatList);
                    
                    SurfacesList_[CurSurface_].VerticesList[ SurfacesList_[CurSurface_].NormalCount++ ].Normal =
                        dim::vector3df(pFloatList[0], pFloatList[1], pFloatList[2]);
                }
                break;
                
                case 't': // texture coordinate
                {
                    getFloatList(pFloatList);
                    
                    SurfacesList_[CurSurface_].VerticesList[ SurfacesList_[CurSurface_].TexCoordCount++ ].TexCoord =
                        dim::point2df(pFloatList[0], pFloatList[1]);
                }
                break;
            }
        }
        break;
        
        case 'p': // point
        {
            
        }
        break;
        
        case 'l': // line
        {
            
        }
        break;
        
        case 'f': // face
        {
            
            ModelCreateStatus_ = OBJ_MODELCREATE_TRIANGLES;
            
            s32 Count = getIntegerList(pIntegerList);
            
            if (Count == 3)
            {
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[0] - 1);
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[1] - 1);
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[2] - 1);
            }
            else if (Count == 4)
            {
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[0] - 1);
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[1] - 1);
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[2] - 1);
                
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[0] - 1);
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[2] - 1);
                SurfacesList_[CurSurface_].IndicesList.push_back(pIntegerList[3] - 1);
            }
        }
        break;
        
        case 'g': // group name
        {
            
            SurfacesList_[CurSurface_].Name = CurLine_.right(CurLine_.size() - 2);
            
        }
        break;
        
        case 'o': // object name
        {
            
            /* Set the object name */
            hEntity_->setName(CurLine_.right(CurLine_.size() - 2));
            
        }
        break;
        
        case 'c': // color interpolation (C_INTERP)
        {
            
        }
        break;
        
        case 'd': // dissolve interpolation (D_INTERP)
        {
            
        }
        break;
        
        case 'u': // material name (USEMTL)
        {
            
        }
        break;
        
        case 'm': // material library (MTLLIB)
        {
            
        }
        break;
    }
    
}

template <typename T> s32 getValueList(io::stringc Line, T pList[4])
{
    
    /* Temporary variables */
    io::stringc strVal;
    s32 pos = 0, len, c = 0;
    
    /* Cut the begin of the line */
    s32 BlankCount = 1;
    while (Line.mid(BlankCount + 1, 1) == " ")
        ++BlankCount;
    Line = Line.right(Line.size() - BlankCount + 1);
    
    /* Loop while reading the values */
    do
    {
        
        pos = Line.find(" ", pos + 1);
        
        if (pos)
        {
            len = Line.find(" ", pos + 1);
            
            if (len == 0)
                len = Line.size() - pos;
            
            strVal = Line.mid(pos, len - pos);
            
            pList[c++] = strVal.val<T>();
        }
        
    }
    while (len < Line.size() && pos != 0 && c < 4);
    
    /* Return the count of values */
    return c;
    
}

s32 ModelLoaderOBJ::getFloatList(f32 pList[4])
{
    return getValueList<f32>(CurLine_, pList);
}

s32 ModelLoaderOBJ::getIntegerList(s32 pList[4])
{
    return getValueList<s32>(CurLine_, pList);
}

void ModelLoaderOBJ::createNewSurface()
{
    ModelCreateStatus_ = OBJ_MODELCREATE_NONE;
    
    SurfacesList_.push_back(SSurfaceOBJ());
    
    CurSurface_ = SurfacesList_.size() - 1;
}

void ModelLoaderOBJ::buildModel()
{
    
    s32 i = 0, c = 0;
    
    for (std::vector<SSurfaceOBJ>::iterator surf = SurfacesList_.begin(); surf != SurfacesList_.end(); ++surf, ++i)
    {
        /* Create a new surface */
        hEntity_->createSurface();
        
        /* Set the surface name */
        hEntity_->setSurfaceName(i, surf->Name);
        
        /* Create all vertices */
        for (std::vector<SVertexOBJ>::iterator it = surf->VerticesList.begin(); it != surf->VerticesList.end(); ++it)
            hEntity_->addVertex(it->Coord, it->Normal, it->TexCoord);
        
        /* Create all triangles */
        for (std::vector<s32>::iterator it = surf->IndicesList.begin(); it != surf->IndicesList.end();)
            hEntity_->addTriangle(*it++ - c, *it++ - c, *it++ - c);
        
        /* Count the vertices indices size */
        c += surf->VerticesList.size();
    }
    
}

bool ModelLoaderOBJ::loadModelData()
{
    
    /* Loop while reading the file */
    while (!FileLoader_->isEOF(hFile_))
    {
        /* Read the next line in file */
        CurLine_ = FileLoader_->readString(hFile_);
        
        /* Check if a new surface must created */
        if (ModelCreateStatus_ == OBJ_MODELCREATE_NEWSURFACE)
            createNewSurface();
        
        /* Examine the line */
        readLine();
    }
    
    /* Build the model */
    buildModel();
    
    /* Update the model */
    hEntity_->updateModeling(false);
    
    return true;
}


} // /namespace scene

} // /namespace sp








// ================================================================================
