/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model loading SPM-fileformat
 *
 */

#ifndef __SP_MODELLOADER_SPM_H__
#define __SP_MODELLOADER_SPM_H__


#include "spStandard.hpp"
#include "spInputOutputStandard.hpp"
#include "spDimension.hpp"
#include "spVideoDriverTexture.hpp"
#include "spModelLoaderRAW.hpp"

#include <io.h>
#include <vector>
#include <string>


namespace sp
{
namespace scene
{


static const u16 SPM_VERSION_NUMBER = 0x0003;

enum EModelSPMChunkIDTypes
{
    MODEL_SPM_PRIMARY       = 0x0001,
    MODEL_SPM_VERSION       = 0x0002,
    MODEL_SPM_OBJECTNAME    = 0x0003,
    
    MODEL_SPM_NEWSURFACE    = 0xA001,
    
    MODEL_SPM_NEWMATERIAL   = 0xB001,
    
    MODEL_SPM_NEWVERTEX     = 0xC001,
    MODEL_SPM_NEWTRIANGLE   = 0xC002,
    
    MODEL_SPM_ANIMATION     = 0xD001,
    MODEL_SPM_NEWNODE       = 0xD002,
    MODEL_SPM_NEWKEYFRAME   = 0xD003,
    MODEL_SPM_NEWBONE       = 0xD0A1,
    MODEL_SPM_NEWBONEFRAME  = 0xD0A2,
};


// !!! MUST BE UPDATED !!!

class SP_EXPORT ModelLoaderSPM : public ModelLoaderRAW
{
    
    public:
        
        ModelLoaderSPM();
        ~ModelLoaderSPM();
        
        Entity* loadModel(io::stringc &Filename, io::stringc &TexturePath);
        
    private:
        
        /* Structures */
        
        typedef struct SChunkInfo
        {
            u16 ID;
            u32 Length;
        }
        chunkInfo;
        
        /* Functions */
        
        void init();
        void clear();
        
        void readChunk(chunkInfo* pChunk);
        
        void processAllChunks();
        
        /* Member */
        
        io::FileLoader* FileLoader_;
        io::File* hFile_;
        
        chunkInfo* CurrentChunk_;
        
};


} // /namespace scene

} // /namespace sp


#endif





// ================================================================================
