/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * File for model loading TMF-fileformat
 *
 */

#include "spModelLoaderTMF.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace scene
{


ModelLoaderTMF::ModelLoaderTMF() : hFileLoader_(0), hFile_(0)
{
    init();
}
ModelLoaderTMF::~ModelLoaderTMF()
{
    clear();
}

Entity* ModelLoaderTMF::loadModel(io::stringc &Filename, io::stringc &TexturePath)
{
    Filename_       = Filename;
    TexturePath_    = TexturePath;
    
    hEntity_        = new Entity();
    
    if ( !( hFile_ = hFileLoader_->openFile(Filename, io::FILE_READ_BINARY) ) )
        return hEntity_;
    
    loadModelData();
    
    hFileLoader_->closeFile(hFile_);
    
    return hEntity_;
}

void ModelLoaderTMF::init()
{
    hFileLoader_ = new io::FileLoader();
}
void ModelLoaderTMF::clear()
{
    spDeletePointer(hFileLoader_);
}

void ModelLoaderTMF::loadModelData()
{
    
    /* Temporary variables */
    
    std::vector<SVertexTMF> VerticesList;
    std::vector<SSurfaceTMF> SurfacesList;
    
    dim::vector3df tmpVec;
    dim::point2df tmpPoint;
    
    s32 i, j;
    s32 Surface;
    s32 SurfaceCount, MaterialCount;
    s32 VerticesCount, TrianglesCount;
    s32 v0, v1, v2;
    
    video::Texture* Texture;
    video::Texture* Normalmap;
    
    io::stringc TextureFilename;
    io::stringc NormalmapFilename;
    
    s8 SphereMapping, AlphaMap;
    f32 Shininess, Alpha;
    
    video::color Color(255);
    
    /* Read the first elements */
    hFileLoader_->readString(hFile_);
    hFileLoader_->readValue<s32>(hFile_);
    hFileLoader_->readValue<s32>(hFile_);
    
    /* Determine if the normals does exist */
    bool isNormals = (bool)hFileLoader_->readValue<s8>(hFile_);
    
    /* Read the count of surfaces */
    SurfaceCount = hFileLoader_->readValue<s32>(hFile_);
    
    /* Resize the surface list */
    SurfacesList.resize(SurfaceCount);
    
    /* Loop for all surfaces */
    for (i = 0; i < SurfaceCount; ++i)
    {
        /* Create a new surface */
        Surface = hEntity_->createSurface();
        
        /* Read the count of vertices */
        VerticesCount = hFileLoader_->readValue<s32>(hFile_);
        
        /* Resize the vertices list */
        VerticesList.resize(VerticesCount);
        
        /* Loop for all vertices */
        for (j = 0; j < VerticesCount; ++j)
        {
            /* Read the next vector */
            tmpVec.X = hFileLoader_->readValue<f32>(hFile_);
            tmpVec.Y = hFileLoader_->readValue<f32>(hFile_);
            tmpVec.Z = hFileLoader_->readValue<f32>(hFile_);
            
            /* Set the vertex coordinate */
            VerticesList[j].Coord = tmpVec;
        }
        
        /* Loop for all vertices */
        for (j = 0; j < VerticesCount; ++j)
        {
            /* Read the next point */
            tmpPoint.X = hFileLoader_->readValue<f32>(hFile_);
            tmpPoint.Y = hFileLoader_->readValue<f32>(hFile_);
            
            /* Set the vertex coordinate */
            VerticesList[j].TexCoord = tmpPoint;
        }
        
        /* Check if the normals does exist */
        if (isNormals)
        {
            /* Loop for all vertices */
            for (j = 0; j < VerticesCount; ++j)
            {
                /* Read the next vector */
                tmpVec.X = hFileLoader_->readValue<f32>(hFile_);
                tmpVec.Y = hFileLoader_->readValue<f32>(hFile_);
                tmpVec.Z = hFileLoader_->readValue<f32>(hFile_);
                
                /* Set the vertex coordinate */
                VerticesList[j].Normal = tmpVec;
            }
        }
        
        /* Loop for all vertices */
        for (j = 0; j < VerticesCount; ++j)
        {
            /* Create a new vertex */
            hEntity_->addVertex(VerticesList[j].Coord, VerticesList[j].Normal, VerticesList[j].TexCoord);
        }
        
        /* Read the count of triangles */
        TrianglesCount = hFileLoader_->readValue<s32>(hFile_);
        
        /* Loop for all triangles */
        for (j = 0; j < TrianglesCount; ++j)
        {
            /* Read the triangle indices */
            v0 = hFileLoader_->readValue<s32>(hFile_);
            v1 = hFileLoader_->readValue<s32>(hFile_);
            v2 = hFileLoader_->readValue<s32>(hFile_);
            
            /* Create a new triangle */
            hEntity_->addTriangle(v0, v1, v2);
        }
        
        /* Set the information for the current surface */
        SurfacesList[i].ID          = Surface;
        SurfacesList[i].MaterialID  = hFileLoader_->readValue<s32>(hFile_);
        
    } // next surface
    
    /* Read the count of materials */
    MaterialCount = hFileLoader_->readValue<s32>(hFile_);
    
    /* Loop for all materials */
    for (i = 0; i < MaterialCount; ++i)
    {
        /* Read the material attributes */
        TextureFilename     = hFileLoader_->readString(hFile_);
        NormalmapFilename   = hFileLoader_->readString(hFile_);
        
        SphereMapping       = hFileLoader_->readValue<s8>(hFile_);
        Alpha               = hFileLoader_->readValue<f32>(hFile_);
        Shininess           = hFileLoader_->readValue<f32>(hFile_);
        AlphaMap            = hFileLoader_->readValue<s8>(hFile_);
        
        Color.Red           = hFileLoader_->readValue<s32>(hFile_);
        Color.Green         = hFileLoader_->readValue<s32>(hFile_);
        Color.Blue          = hFileLoader_->readValue<s32>(hFile_);
        
        /* Correct the alphamap value */
        if (AlphaMap == 1)
            AlphaMap = 2;
        
        /* Load the texture */
        Texture = __spVideoDriver->loadTexture(TexturePath_ + TextureFilename);
        
        /* Check if the normal map filename is not empty */
        if (NormalmapFilename != "")
        {
            if (hFileLoader_->getFileSize(TexturePath_ + NormalmapFilename) > 0)
            {
                /* Load the normal map */
                Normalmap = __spVideoDriver->loadTexture(TexturePath_ + NormalmapFilename);
            }
            else
                NormalmapFilename = "";
        }
        
        /* Loop for all surfaces */
        for (j = 0; j < SurfaceCount; ++j)
        {
            if (SurfacesList[j].MaterialID == i)
            {
                /* Select the current surface */
                hEntity_->selectSurface(j);
                
                /* Add the current texture */
                hEntity_->addTexture(Texture);
                
                /* Check if sphere mapping is using */
                //if (SphereMapping)
                //    hEntity_->setSphereMapping();
                
                /* Set the color */
                //hEntity_->setColor(Color, j, true);
                
                /* Set the shininess */
                hEntity_->setShininess(Shininess);
            }
        } // next surface
        
    } // next material
    
    /* Update the model */
    hEntity_->updateModeling(!isNormals);
    
}


} // /namespace scene

} // /namespace sp






// ================================================================================
