/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model saving B3D-fileformat (Blitz3D)
 *
 */

#include "spModelSaverB3D.hpp"


namespace sp
{
namespace scene
{


ModelSaverB3D::ModelSaverB3D() : ModelSaverRAW()
{
    init();
}
ModelSaverB3D::~ModelSaverB3D()
{
    clear();
}

bool ModelSaverB3D::saveModel(Entity* hEntity, io::stringc &Filename)
{
    if (!openSaveFile(hEntity, Filename));
        return false;
    
    saveModelData();
    
    FileLoader_->closeFile(hFile_);
    
    return true;
}


/*
 * ========== Private: ==========
 */


void ModelSaverB3D::init()
{
    /* General settings */
    CurPos_ = 0;
    
    // Initialize pointer
    memset(Stack_, 0, sizeof(Stack_));
    
    /* Allocate memories */
    FileLoader_     = new io::FileLoader();
}
void ModelSaverB3D::clear()
{
    /* Delete all memories */
    spDeletePointer(FileLoader_);
}

void ModelSaverB3D::beginChunk(io::stringc Tag)
{
    ++CurPos_;
    FileLoader_->writeStream(hFile_, Tag.cstr(), 4);
    FileLoader_->writeValue<s32>(hFile_, 0);
    Stack_[CurPos_] = FileLoader_->getFileSeek(hFile_);
}
void ModelSaverB3D::endChunk()
{
    s32 Pos = FileLoader_->getFileSeek(hFile_);
    FileLoader_->setFileSeek(hFile_, Stack_[CurPos_] - 4);
    FileLoader_->writeValue<s32>(hFile_, Pos - Stack_[CurPos_]);
    FileLoader_->setFileSeek(hFile_, Pos);
    --CurPos_;
}

/*
 * Texture flags:
 * 1: Color
 * 2: Alpha
 * 4: Masked
 * 8: Mipmapped
 * 16: Clamp U
 * 32: Clamp V
 * 64: Spherical reflection map
 */

void ModelSaverB3D::saveModelData()
{
    
    video::Texture* hTexture = 0;
    s32 TextureFlags = 0;
    s32 TextureCounter = 0;
    f32 EntityColor[4];
    hEntity_->getColor().getFloatArray(EntityColor);
    
    beginChunk("BB3D");
    {
        FileLoader_->writeValue<s32>(hFile_, 1); // Version
        
        beginChunk("TEXS");
        {
            for (s32 surf = 0; surf < hEntity_->getMeshSurfaceCount(); ++surf)
            {
                hEntity_->selectSurface(surf);
                if (hTexture = hEntity_->getTexture())
                {
                    ++TextureCounter;
                    if (hTexture->getTransFilter().Alpha != 255)
                        TextureFlags = 4;
                    
                    // Writing
                    FileLoader_->writeStringANSIC(hFile_, hTexture->getFilename());
                    FileLoader_->writeValue<s32>(hFile_, TextureFlags);
                    FileLoader_->writeValue<s32>(hFile_, 0);
                    FileLoader_->writeValue<f32>(hFile_, 0.0f);
                    FileLoader_->writeValue<f32>(hFile_, 0.0f);
                    FileLoader_->writeValue<f32>(hFile_, 1.0f);
                    FileLoader_->writeValue<f32>(hFile_, 1.0f);
                    FileLoader_->writeValue<f32>(hFile_, 0.0f);
                }
            }
        }
        endChunk();
        
        beginChunk("BRUS"); // !!! INCOMPLETE !!!
        {
            FileLoader_->writeValue<s32>(hFile_, TextureCounter);
            FileLoader_->writeStringANSIC(hFile_, "Brush1");
            FileLoader_->writeValue<f32>(hFile_, EntityColor[0]);
            FileLoader_->writeValue<f32>(hFile_, EntityColor[1]);
            FileLoader_->writeValue<f32>(hFile_, EntityColor[2]);
            FileLoader_->writeValue<f32>(hFile_, EntityColor[3]);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getShininess());
            FileLoader_->writeValue<s32>(hFile_, 0);
            FileLoader_->writeValue<s32>(hFile_, 0);
            
            for (s32 i = 0; i < TextureCounter; ++i)
            {
                FileLoader_->writeValue<s32>(hFile_, i);
            }
        }
        endChunk();
        
        beginChunk("NODE");
        {
            FileLoader_->writeStringANSIC(hFile_, hEntity_->getName());
            
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getPosition().X);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getPosition().Y);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getPosition().Z);
            
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getScale().X);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getScale().Y);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getScale().Z);
            
            FileLoader_->writeValue<f32>(hFile_, 0.0f);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getRotation().X);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getRotation().Y);
            FileLoader_->writeValue<f32>(hFile_, hEntity_->getRotation().Z);
            
            writeMesh();
        }
        endChunk();
        
    }
    endChunk();
    
}

void ModelSaverB3D::writeMesh()
{
    
    s32 SurfCount = hEntity_->getMeshSurfaceCount();
    s32 TriangleCount = 0;
    u32 Vertices[3];
    dim::vector3df Coord;
    dim::point2df TexCoord;
    video::color Color;
    
    beginChunk("MESH");
    {
        FileLoader_->writeValue<s32>(hFile_, 0); // Brush ID
        
        beginChunk("VRTS");
        {
            FileLoader_->writeValue<s32>(hFile_, 3); // Flags
            FileLoader_->writeValue<s32>(hFile_, 1); // 1 TexCoords sets
            FileLoader_->writeValue<s32>(hFile_, 2); // 2 Coords per set
            
            for (s32 i = 0, j; i < SurfCount; ++i)
            {
                for (j = 0; j < hEntity_->getMeshVerticesCount(i); ++j)
                {
                    // Coord
                    Coord = hEntity_->getVertexCoord(j);
                    FileLoader_->writeValue<f32>(hFile_, Coord.X);
                    FileLoader_->writeValue<f32>(hFile_, Coord.Y);
                    FileLoader_->writeValue<f32>(hFile_, Coord.Z);
                    
                    // Normal
                    Coord = hEntity_->getVertexNormal(j);
                    FileLoader_->writeValue<f32>(hFile_, Coord.X);
                    FileLoader_->writeValue<f32>(hFile_, Coord.Y);
                    FileLoader_->writeValue<f32>(hFile_, Coord.Z);
                    
                    // Color
                    Color = hEntity_->getVertexColor(j);
                    FileLoader_->writeValue<f32>(hFile_, (f32)Color.Red   / 255);
                    FileLoader_->writeValue<f32>(hFile_, (f32)Color.Green / 255);
                    FileLoader_->writeValue<f32>(hFile_, (f32)Color.Blue  / 255);
                    FileLoader_->writeValue<f32>(hFile_, (f32)Color.Alpha / 255);
                    
                    // TexCoord
                    TexCoord = hEntity_->getVertexTexCoord(j).vec2<f32>();
                    FileLoader_->writeValue<f32>(hFile_, TexCoord.X);
                    FileLoader_->writeValue<f32>(hFile_, TexCoord.Y);
                }
            }
        }
        endChunk();
        
        beginChunk("TRIS");
        {
            FileLoader_->writeValue<s32>(hFile_, 0); // Bursh for these triangles
            
            for (s32 i = 0, j; i < SurfCount; ++i)
            {
                for (j = 0; j < hEntity_->getMeshTriangleCount(i); ++j)
                {
                    hEntity_->getTriangleVertices(j, Vertices);
                    FileLoader_->writeValue<s32>(hFile_, TriangleCount + Vertices[0]);
                    FileLoader_->writeValue<s32>(hFile_, TriangleCount + Vertices[1]);
                    FileLoader_->writeValue<s32>(hFile_, TriangleCount + Vertices[2]);
                }
                TriangleCount += hEntity_->getMeshTriangleCount(i);
            }
        }
        endChunk();
        
    }
    endChunk();
    
}






} // /namespace scene

} // /namespace sp








// ================================================================================
