/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Model saving OBJ-fileformat
 *
 */

#include "spModelSaverOBJ.hpp"


namespace sp
{
namespace scene
{


ModelSaverOBJ::ModelSaverOBJ() : ModelSaverRAW()
{
}
ModelSaverOBJ::~ModelSaverOBJ()
{
}

bool ModelSaverOBJ::saveModel(Entity* hEntity, io::stringc &Filename)
{
    if (!openSaveFile(hEntity, Filename))
        return false;
    
    saveModelData();
    
    FileLoader_->closeFile(hFile_);
    
    return true;
}


/*
 * ========== Private: ==========
 */


void ModelSaverOBJ::saveModelData()
{
    
    /* === Temporary variables === */
    
    dim::vector3df Coord, Normal;
    dim::point2df TexCoord;
    u32 Vertices[3];
    
    u32 i, j, c;
    
    u32 CountOfVertices = 1;
    
    /* === Loop for all surfaces === */
    
    FileLoader_->writeStringN(hFile_, "# Wavefront Object [ generated by the SoftPixel Engine ]");
    
    if (hEntity_->getName() != "")
        FileLoader_->writeStringN(hFile_, io::stringc("o ") + io::stringc(hEntity_->getName()));
    
    for (i = 0; i < hEntity_->getMeshSurfaceCount(); ++i)
    {
        
        hEntity_->selectSurface(i);
        
        c = hEntity_->getMeshVerticesCount(i);
        
        /* === Loop for all vertices coordinates === */
        for (j = 0; j < c; ++j)
        {
            Coord = hEntity_->getVertexCoord(j);
            
            Coord.Z = -Coord.Z;
            
            writeValueList<f32>("v", &Coord.X, 3);
        }
        
        /* === Loop for all vertices texture coordinates === */
        for (j = 0; j < c; ++j)
        {
            TexCoord = hEntity_->getVertexTexCoord(j).vec2<f32>();
            
            writeValueList<f32>("vt", &TexCoord.X, 2);
        }
        
        /* === Loop for all vertices normals === */
        for (j = 0; j < c; ++j)
        {
            Normal = hEntity_->getVertexNormal(j);
            
            writeValueList<f32>("vn", &Normal.X, 3);
        }
        
        /* === Loop for all triangles === */
        for (j = 0; j < hEntity_->getMeshTriangleCount(i); ++j)
        {
            hEntity_->getTriangleVertices(j, Vertices);
            
            Vertices[0] += CountOfVertices;
            Vertices[1] += CountOfVertices;
            Vertices[2] += CountOfVertices;
            math::Swap(Vertices[0], Vertices[2]);
            
            writeValueList<u32>("f", Vertices, 3);
        }
        
        CountOfVertices += c;
        
    } // next surface
    
}



} // /namespace scene

} // /namespace sp






// ================================================================================
