/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for model saving SPM-fileformat
 *
 */

#include "spModelSaverSPM.hpp"


namespace sp
{
namespace scene
{


ModelSaverSPM::ModelSaverSPM() : ModelSaverRAW()
{
}
ModelSaverSPM::~ModelSaverSPM()
{
}

bool ModelSaverSPM::saveModel(Entity* hEntity, io::stringc &Filename)
{
    if (!openSaveFile(hEntity, Filename))
        return false;
    
    writeAllChunks();
    
    FileLoader_->closeFile(hFile_);
    
    return true;
}


/*
 * ========== Private: ==========
 */

void ModelSaverSPM::writeChunk(u16 ID, u32 Length)
{
    FileLoader_->writeValue<u16>(hFile_, ID);
    FileLoader_->writeValue<u32>(hFile_, Length);
}

/*
 * Main chunk
 */

void ModelSaverSPM::writeAllChunks()
{
    
    /* === Primary === */
    writeChunk(MODEL_SPM_PRIMARY, 0);
    
    /* === Version === */
    writeChunk(MODEL_SPM_VERSION, sizeof(u16));
    FileLoader_->writeValue<u16>(hFile_, SPM_VERSION_NUMBER);
    
    /* === Object name === */
    writeChunk(MODEL_SPM_OBJECTNAME, (u32)(hEntity_->getName().size()));
    if (hEntity_->getName().size() > 0)
    {
        const c8* Name = hEntity_->getName().c_str();
        FileLoader_->writeStream(hFile_, Name, sizeof(c8), hEntity_->getName().size());
    }
    
    /* === All surfaces === */
    
    for (u32 i = 0, j; i < hEntity_->getMeshSurfaceCount(); ++i)
    {
        
        /* === New surface === */
        
        hEntity_->selectSurface(i);
        writeChunk(MODEL_SPM_NEWSURFACE, 0);
        
        /* === New material === */
        
        u32 MatNameLength = (u32)io::stringc(io::stringc("Material")+io::stringc(i+1)).size();
        u32 TexFileNameLength = (u32)hEntity_->getTexture()->getFilename().size();
        
        // Write chunk
        writeChunk(MODEL_SPM_NEWMATERIAL, sizeof(u32)*2 + sizeof(u8) + MatNameLength + TexFileNameLength);
        
        // Material name
        FileLoader_->writeValue<u32>(hFile_, MatNameLength);
        const c8* MatName = io::stringc(io::stringc("Material")+io::stringc(i+1)).c_str();
        FileLoader_->writeStream(hFile_, MatName, sizeof(c8), MatNameLength);
        
        // Material filename
        FileLoader_->writeValue<u32>(hFile_, TexFileNameLength);
        const c8* TexFileName = hEntity_->getTexture()->getFilename().c_str();
        FileLoader_->writeStream(hFile_, TexFileName, sizeof(c8), TexFileNameLength);
        
        // Material properties
        FileLoader_->writeValue<video::EMappingGenTypes>(hFile_, video::MAPGEN_DISABLE);
        
        /* === New vertices === */
        
        for (j = 0; j < hEntity_->getMeshVerticesCount(i); ++j)
        {
            
            writeChunk(MODEL_SPM_NEWVERTEX, sizeof(f32)*8 + sizeof(u8)*4);
            
            dim::vector3df VertexCoord  ( hEntity_->getVertexCoord   (j) );
            dim::vector3df VertexNormal ( hEntity_->getVertexNormal  (j) );
            dim::point2df TexCoord      ( hEntity_->getVertexTexCoord(j).vec2<f32>() );
            video::color Color          ( hEntity_->getVertexColor   (j) );
            
            FileLoader_->writeValue<f32>(hFile_, VertexCoord.X);
            FileLoader_->writeValue<f32>(hFile_, VertexCoord.Y);
            FileLoader_->writeValue<f32>(hFile_, VertexCoord.Z);
            FileLoader_->writeValue<f32>(hFile_, VertexNormal.X);
            FileLoader_->writeValue<f32>(hFile_, VertexNormal.Y);
            FileLoader_->writeValue<f32>(hFile_, VertexNormal.Z);
            FileLoader_->writeValue<f32>(hFile_, TexCoord.X);
            FileLoader_->writeValue<f32>(hFile_, TexCoord.Y);
            FileLoader_->writeValue<u8>(hFile_, Color.Red);
            FileLoader_->writeValue<u8>(hFile_, Color.Green);
            FileLoader_->writeValue<u8>(hFile_, Color.Blue);
            FileLoader_->writeValue<u8>(hFile_, Color.Alpha);
            
        }
        
        /* === New triangles === */
        
        for (j = 0; j < hEntity_->getMeshTriangleCount(i); ++j)
        {
            
            writeChunk(MODEL_SPM_NEWTRIANGLE, sizeof(u32)*3);
            
            u32 Vertices[3];
            hEntity_->getTriangleVertices(j, Vertices);
            
            FileLoader_->writeStream(hFile_, Vertices, sizeof(u32), 3);
            
        }
        
    }
    
}


} // /namespace scene

} // /namespace sp








// ================================================================================
