/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Network operations (server, client)
 *
 */

#include "spNetwork.hpp"


namespace sp
{
namespace network
{


// External lists
std::list<clientInfo> ClientList;
std::list<u32> NewClientList;
std::list<messageInfo> SendMessageList;
std::list<io::stringc> ReceiveMessageList;
bool isServer = false;
SOCKET sockMain;


NetworkManager::NetworkManager() : ServerIPAddress_(""),
    hThread_Connections_(0), hThread_MessageSend_(0), hThread_MessageRecv_(0)
{
    io::printMessage("");
    io::printMessage("Network system - v.1.0");
    io::printMessage("(c) Copyright 2008 by Lukas hermanns");
    io::printMessage("");
    // General
    isServer = false;
    init();
}
NetworkManager::~NetworkManager()
{
    closeConnection();
}

// Server/ Client

bool NetworkManager::createServer(io::stringc ServerIPAddress, u32 Port)
{
    
    // Creates the main socket
    if (!createSocket(MainSocket_))
        return false;
    
    // Create network address structure
    sockaddr_in tAddress;
    memset(&tAddress, 0, sizeof(tAddress));
    tAddress.sin_family = AF_INET;
    tAddress.sin_port = htons(Port);
    tAddress.sin_addr.s_addr = inet_addr(ServerIPAddress.c_str());
    
    // Create server
    if (bind(MainSocket_, (SOCKADDR*)&tAddress, sizeof(tAddress)) == SOCKET_ERROR)
    {
        io::printError(io::stringc("could not create server: ")+io::stringc(ServerIPAddress)+
                       io::stringc(" with port: ")+io::stringc(Port));
        closeConnection();
        return false;
    }
    else
        io::printMessage("Network info: created server successful");
    
    isServer = true;
    sockMain = MainSocket_;
    
    createThreads();
    
    return true;
    
}

bool NetworkManager::connectServer(io::stringc ServerIPAddress, u32 Port)
{
    
    // Creates the main socket
    if (!createSocket(MainSocket_))
        return false;
    
    // Create network address structure
    sockaddr_in tAddress;
    memset(&tAddress, 0, sizeof(tAddress));
    tAddress.sin_family         = AF_INET;
    tAddress.sin_port           = htons(Port);
    tAddress.sin_addr.s_addr    = inet_addr(ServerIPAddress.c_str());
    
    // Connection to server
    if (connect(MainSocket_, (sockaddr*)&tAddress, sizeof(sockaddr)) == -1)
    {
        io::printError(io::stringc("could not connect to server: ")+io::stringc(ServerIPAddress)+
                       io::stringc(" with port: ")+io::stringc(Port));
        closeConnection();
        return false;
    }
    else
        io::printMessage("Network info: server connection successful");
    
    isServer = false;
    sockMain = MainSocket_;
    
    createThreads();
    
    return true;
    
}

// Clients

std::list<u32> NetworkManager::checkNewClients()
{
    std::list<u32> tClientList = NewClientList;
    NewClientList.clear();
    return tClientList;
}

bool NetworkManager::addClient(u32 NetworkClientID)
{
    clientInfo ClientData;
    {
        ClientData.ID = NetworkClientID;
        if (!createSocket(ClientData.Socket))
            return false;
    }
    ClientList.push_back(ClientData);
    return true;
}

void NetworkManager::removeClient(u32 NetworkClientID)
{
    for (std::list<clientInfo>::iterator it = ClientList.begin(); it != ClientList.end(); ++it)
    {
        if (it->ID == NetworkClientID)
        {
            close(it->Socket);
            ClientList.erase(it);
            break;
        }
    }
}

// Messages

void NetworkManager::sendMessage(io::stringc Message, u32 NetworkClientID)
{
    messageInfo MessageData;
    {
        MessageData.SenderClientID = NetworkClientID;
        MessageData.Message = Message;
    }
    SendMessageList.push_back(MessageData);
}

std::list<io::stringc> NetworkManager::receiveMessages()
{
    std::list<io::stringc> tMessageList = ReceiveMessageList;
    ReceiveMessageList.clear();
    return tMessageList;
}

// Disconnection

void NetworkManager::closeConnection()
{
    
    io::printMessage("Network info: disconnection");
    
    // Close all threads
    closeThreads();
    
    // Settings
    ClientList.clear();
    ServerIPAddress_ = "";
    isServer = false;
    
    // Close all sockets
    close(MainSocket_); // Main socket
    for (std::list<clientInfo>::iterator it = ClientList.begin(); it != ClientList.end(); ++it)
        close(it->Socket);
    ClientList.clear();
    
    // Wsa clean up
    WSACleanup();
    
}


// ========== Private: ==========


void NetworkManager::init()
{
    
    s32 tResult = WSAStartup(0x0002, &WSA_);
    if (tResult)
    {
        switch (tResult)
        {
            case WSASYSNOTREADY:
                io::printError("'WSAStartup', WinSock not ready"); break;
            case WSAVERNOTSUPPORTED:
                io::printError("'WSAStartup', requested WinSock version not supported"); break;
            case WSAEINPROGRESS:
                io::printError("'WSAStartup', blocking WinSock 1.1 operation in progress"); break;
            case WSAEPROCLIM:
                io::printError("'WSAStartup', maximum WinSock tasks reached"); break;
            case WSAEFAULT:
                io::printError("'WSAStartup', the lpWSAData is not a valid pointer"); break;
            default:
                io::printError(io::stringc("'WSAStartup', unknown error code: ")+io::stringc(tResult)); break;
        }
    }
    
}

bool NetworkManager::createSocket(SOCKET &Socket)
{
    
    Socket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);//SOCK_STREAM, IPPROTO_TCP);
    if (Socket == INVALID_SOCKET)
    {
        io::printError(io::stringc("could not create new socket. Error ID = ")+io::stringc(WSAGetLastError()));
        closeConnection();
        return false;
    }
    else
        io::printMessage("Network info: created new socket successful");
    
    return true;
}

void NetworkManager::createThreads()
{
    
    io::printMessage("Network info: create network-system multi-threading");
    
    s32 tThreadParam = 0;
    
    // Connection thread function
    if (isServer)
        hThread_Connections_ = CreateThread(0, 0, Thread_Connections, &tThreadParam, 0, &ThreadIDConnections_);
    
    // Message sending thread function
    hThread_MessageSend_ = CreateThread(0, 0, Thread_MessageSend, &tThreadParam, 0, &ThreadIDMessageSend_);
    
    // Message receiving thread function
    hThread_MessageRecv_ = CreateThread(0, 0, Thread_MessageRecv, &tThreadParam, 0, &ThreadIDMessageRecv_);
    
}

void NetworkManager::closeThreads()
{
    
    io::printMessage("Network info: close network-system multi-threading");
    
    if (hThread_Connections_ != 0)
    {
        CloseHandle(hThread_Connections_);
        hThread_Connections_ = 0;
    }
    if (hThread_MessageSend_ != 0)
    {
        CloseHandle(hThread_MessageSend_);
        hThread_MessageSend_ = 0;
    }
    if (hThread_MessageRecv_ != 0)
    {
        CloseHandle(hThread_MessageRecv_);
        hThread_MessageRecv_ = 0;
    }
    
}


// ========== Thread functions: ==========


// Check for connections

DWORD WINAPI Thread_Connections(PVOID pParam)
{
    
    static u32 CountID = 1; // 1 is the server
    SOCKET tSocket;
    
    while (true)
    {
        
        // Listen for a new client connection
        listen(tSocket, 1);
        
        if (tSocket != -1)
        {
            ++CountID;
            io::printMessage(io::stringc("Network info: a new client (ID = ")+io::stringc(CountID)+io::stringc(") has been connected"));
            clientInfo ClientData;
            {
                ClientData.ID = CountID;
                ClientData.Socket = tSocket;
            }
            ClientList.push_back(ClientData);
            NewClientList.push_back(CountID);
            // Informate all clients
            messageInfo MessageData;
            {
                MessageData.SenderClientID = 0;
                MessageData.Message = io::stringc("~nc~(0)")+io::stringc(CountID);
            }
            SendMessageList.push_back(MessageData);
        }
        
    }
    
    return 0;
}

// Sending messages

DWORD WINAPI Thread_MessageSend(PVOID pParam)
{
    
    while (true)
    {
        
        if (isServer) // Server
        {
            
            for (std::list<messageInfo>::iterator itMsg = SendMessageList.begin(); itMsg != SendMessageList.end(); ++itMsg)
            {
                itMsg->Message = itMsg->Message;
                for (std::list<clientInfo>::iterator itClient = ClientList.begin(); itClient != ClientList.end(); ++itClient)
                {
                    if (itMsg->SenderClientID == 0)
                    {
                        send(itClient->Socket, itMsg->Message.c_str(), itMsg->Message.size(), 0);
                    }
                    else if (itMsg->SenderClientID == itClient->ID)
                    {
                        send(itClient->Socket, itMsg->Message.c_str(), itMsg->Message.size(), 0);
                        break;
                    }
                }
            }
            
        }
        else // Normal client
        {
            
            for (std::list<messageInfo>::iterator itMsg = SendMessageList.begin(); itMsg != SendMessageList.end(); ++itMsg)
            {
                itMsg->Message = io::stringc("~to~(")+io::stringc(itMsg->SenderClientID)+io::stringc(")")+itMsg->Message;
                send(sockMain, itMsg->Message.c_str(), itMsg->Message.size(), 0);
            }
            
        }
        
        SendMessageList.clear();
        
    }
    
    return 0;
}

// Receive messages

DWORD WINAPI Thread_MessageRecv(PVOID pParam)
{
    
    char cStr[__SP_NETMSG_LEN];
    u32 SenderID = 0;
    size_t tLen = 0;
    io::stringc Message;
    
    while (true)
    {
        
        if (isServer) // Server
        {
            
            for (std::list<clientInfo>::iterator itClient = ClientList.begin(); itClient != ClientList.end(); ++itClient)
            {
                recv(itClient->Socket, cStr, __SP_NETMSG_LEN, 0);
                
                Message = std::string(cStr);
                if (Message.left(5) == "~to~(")
                {
                    tLen = Message.find(")", 0);
                    SenderID = Message.mid(5, tLen-5).val<u32>();
                    Message = Message.right(Message.size()-tLen);
                    if (SenderID == 0 || SenderID > 1)
                    {
                        messageInfo MessageData;
                        {
                            MessageData.SenderClientID = 0;
                            MessageData.Message = io::stringc(itClient->ID);
                        }
                        SendMessageList.push_back(MessageData);
                        if (SenderID > 1)
                            continue;
                    }
                }
                
                ReceiveMessageList.push_back(Message);
            }
            
        }
        else // Normal client
        {
            
            recv(sockMain, cStr, __SP_NETMSG_LEN, 0);
            ReceiveMessageList.push_back(std::string(cStr));
            
        }
        
    }
    
    return 0;
}





} // /namespace network

} // /namespace sp







// ================================================================================
