/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for OpenGL Shading Language (GLSL)
 *
 */

#ifndef __SP_OPENGLSHADER_H__
#define __SP_OPENGLSHADER_H__


#include "spStandard.hpp"
#include "spVideoDriverShader.hpp"

#include <gl/gl.h>
#include <gl/glext.h>


namespace sp
{
namespace video
{


class SP_EXPORT OpenGLShader : public Shader
{
    
    public:
        
        OpenGLShader();
        OpenGLShader(EStandardShaderTypes Type);
        OpenGLShader(io::stringc &FilenameVertex, io::stringc &FilenameFragment, bool UseHighLevel);
        ~OpenGLShader();
        
        /* Binds & unbinds the shader */
        
        void bind();
        void unbind();
        
        /* Set the constants (by number) */
        
        void setConstant(s32 Number, EConstantTypes Type, const f32 Value);
        void setConstant(s32 Number, EConstantTypes Type, dim::vector3df Position);
        void setConstant(s32 Number, EConstantTypes Type, video::color Color);
        void setConstant(s32 Number, EConstantTypes Type, dim::matrix4f &Matrix);
        void setConstant(s32 Number, EConstantTypes Type, const f32* pFloats, int Count);
        
        /* Set the constants (by name) */
        
        bool setConstant(io::stringc Name, const f32 Value);
        bool setConstant(io::stringc Name, dim::vector3df Position);
        bool setConstant(io::stringc Name, video::color Color);
        bool setConstant(io::stringc Name, dim::matrix4f &Matrix);
        bool setConstant(io::stringc Name, const f32* pFloats, int Count);
        
        /* Set the constants for assembly shaders */
        
        bool setConstant(const f32* pFloats, s32 StartRegister, s32 ConstAmount);
        
    private:
        
        /* Functions */
        
        bool loadShaderFile(io::stringc Filename, c8** &pData, u32 &LinesCount);
        
        void createShader(c8** pDataVert, c8** pDataFrag, u32 LinesCountVert, u32 LinesCountFrag);
        void createShaderGLSL(c8** pDataVert, c8** pDataFrag, u32 LinesCountVert, u32 LinesCountFrag);
        void createShaderVertex(c8** pDataShd, u32 LinesCount);
        void createShaderPixel(c8** pDataShd, u32 LinesCount);
        
        bool checkForErrorsGLSL();
        bool checkForErrorsShader(const io::stringc ShaderName);
        
        void receiveAllUniforms();
        
        bool switchShaderTypes(
            EStandardShaderTypes Type, c8** &pDataVert, c8** &pDataFrag, u32 &LinesCountVert, u32 &LinesCountFrag
        );
        
        void convertShaderText(const c8** pData, c8* &pDataSource, u32 LinesCount);
        void deleteShaderText(c8** &pData, u32 LinesCount);
        
        void checkConstantType(GLenum &Type);
        
        /* Structures */
        
        struct SUniformInfo
        {
            io::stringc Name;
            GLenum Type;
        };
        
        /* Members */
        
        GLuint VertexShader_;
        GLuint PixelShader_;
        
        GLhandleARB ProgramObject_;
        GLhandleARB VertexShaderObject_;
        GLhandleARB FragmentShaderObject_;
        
        std::vector<SUniformInfo> UniformList_;
        
};


} // /namespace scene

} // /namespace sp


#endif







// ================================================================================
