/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * SpecialEffectsActor (for special effects implementation)
 *
 */

#include "spOpenGLSpecialEffectsActor.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace scene
{
namespace effect
{


OpenGLSpecialEffectsActor::OpenGLSpecialEffectsActor() : SpecialEffectsActor()
{
}
OpenGLSpecialEffectsActor::~OpenGLSpecialEffectsActor()
{
}

/*
 * ========== Bump mapping ==========
 */

/*void OpenGLSpecialEffectsActor::renderBumpMapping(Entity* hEntity, video::Texture* BumpTexture, f32 BumpHeight, Light* hLight)
{
    
}*/


/*
 * ========== Cel shading ==========
 */

// CelShading functions

CelShadow* OpenGLSpecialEffectsActor::createCelShadow(s32 LevelsCount, video::color ShadowColor, video::color DarknessColor)
{
    u8 ShadowData[LevelsCount][3];
    
    if (LevelsCount < 2)
        LevelsCount = 2;
    
    for (s32 i = 0; i < LevelsCount; ++i)
    {
        ShadowData[i][0] = (s32)(ShadowColor.Red   - DarknessColor.Red  ) * i / (LevelsCount-1) + DarknessColor.Red;
        ShadowData[i][1] = (s32)(ShadowColor.Green - DarknessColor.Green) * i / (LevelsCount-1) + DarknessColor.Green;
        ShadowData[i][2] = (s32)(ShadowColor.Blue  - DarknessColor.Blue ) * i / (LevelsCount-1) + DarknessColor.Blue;
    }
    
    u32 tID = 0;
    glGenTextures(1, &tID);
    glBindTexture(GL_TEXTURE_1D, tID);
    
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);	
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    
    glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, LevelsCount, 0, GL_RGB, GL_UNSIGNED_BYTE, ShadowData);
    
    return new CelShadow(tID);
}

CelShadow* OpenGLSpecialEffectsActor::createCelShadow(s32 LevelsCount, video::color* LevelsColor)
{
    u8 ShadowData[LevelsCount][3];
    
    if (LevelsCount < 2)
        LevelsCount = 2;
    
    for (s32 i = 0; i < LevelsCount; ++i)
    {
        ShadowData[i][0] = LevelsColor[i].Red;
        ShadowData[i][1] = LevelsColor[i].Green;
        ShadowData[i][2] = LevelsColor[i].Blue;
    }
    
    u32 tID = 0;
    glGenTextures(1, &tID);
    glBindTexture(GL_TEXTURE_1D, tID);
    
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);	
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    
    glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, LevelsCount, 0, GL_RGB, GL_UNSIGNED_BYTE, ShadowData);
    
    return new CelShadow(tID);
}

void OpenGLSpecialEffectsActor::deleteCelShadow(CelShadow* &hCelShadow)
{
    /* Get the texture name */
    u32 TexID = hCelShadow->getID();
    
    /* Check if the OpenGL textures had been registered */
    if (glIsTexture(TexID))
    {
        /* Delete the texture */
        glDeleteTextures(1, &TexID);
    }
    
    /* Delete the cel shadow */
    spDeletePointer(hCelShadow);
}

void OpenGLSpecialEffectsActor::renderCelShading(Entity* hEntity, CelShadow* hCelShadow, Light* hLight)
{
    
    // Receive settings
    
    GLboolean isLight;
        glGetBooleanv(GL_LIGHTING, &isLight);
    s32 PolygonMode[2];
        glGetIntegerv(GL_POLYGON_MODE, PolygonMode);
    
    glDisable(GL_LIGHTING);
    
    // Loop through all registered camera
    
    for (std::list<Camera*>::iterator itCam = RegisteredCameras_.begin(); itCam != RegisteredCameras_.end(); ++itCam)
    {
        
        if (!(*itCam)->getVisible())
            continue;
        
        (*itCam)->updateViewmode();
        (*itCam)->updateModelviewMatrix();
        
        // Rendering finaly
        
        glEnable(GL_TEXTURE_1D);
        glBindTexture(GL_TEXTURE_1D, hCelShadow->getID());
        
        renderCelShaderSolid(hEntity, hLight); // Solid shader
        
        glDisable(GL_TEXTURE_1D);
        
        renderCelShaderWire(hEntity); // Wire outlined
        
    }
    
    // Settings back
    
    glPolygonMode(GL_BACK, GL_FILL); // !!!
    if (isLight)
        glEnable(GL_LIGHTING);
    
}

/* ======= Private: ======= */

void OpenGLSpecialEffectsActor::renderCelShaderSolid(Entity* hEntity, Light* hLight)
{
    
    dim::matrix4f Matrix;
    dim::vector3df Normal;
    dim::vector3df LightAngle = hLight->getDirection();
    f32 Shadow = 0.0f;
    u32 VerticesIndices[3];
    
    // Calculate matrix
    
    hEntity->updateModelviewMatrix();
    Matrix = spWorldMatrix;
    
    __spVideoDriver->updateModelviewMatrix();
    
    // Render cel shader
    
    glColor4ub(255, 255, 255, 255);
    
    glPolygonMode(GL_FRONT, GL_FILL);
    
    glBegin(GL_TRIANGLES);
    
    for (s32 surf = 0, tri, vert; surf < hEntity->getMeshSurfaceCount(); ++surf)
    {
        
        hEntity->selectSurface(surf);
        
        for (tri = 0; tri < hEntity->getMeshTriangleCount(surf); ++tri)
        {
            
            hEntity->getTriangleVertices(tri, VerticesIndices);
            
            for (vert = 0; vert < 3; ++vert)
            {
                
                Normal = Matrix.getRotationMatrix() * hEntity->getVertexNormal(VerticesIndices[vert]);
                Normal.normalize();
                
                Shadow = - Normal.getScalarProduct(LightAngle);
                
                if (Shadow < 0.0f)
                    Shadow = 0.0f;
                
                glTexCoord1f(Shadow);
                glVertex3f(
                    hEntity->getVertexCoord(VerticesIndices[vert]).X,
                    hEntity->getVertexCoord(VerticesIndices[vert]).Y,
                    hEntity->getVertexCoord(VerticesIndices[vert]).Z
                );
                
            }
            
        }
        
    }
    
    glEnd();
    
}

void OpenGLSpecialEffectsActor::renderCelShaderWire(Entity* hEntity)
{
    
    u32 VerticesIndices[3];
    
    // Receive settings
    
    s32 LineWidth;
        glGetIntegerv(GL_LINE_WIDTH, &LineWidth);
    GLboolean isCullFace;
        glGetBooleanv(GL_CULL_FACE, &isCullFace);
    s32 CullFaceMode;
        glGetIntegerv(GL_CULL_FACE_MODE, &CullFaceMode);
    
    // Settings on
    
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    
    glPolygonMode(GL_FRONT, GL_LINE);
    glLineWidth(CSLineWidth_);
    
    glColor4ub(CSBorderColor_.Red, CSBorderColor_.Green, CSBorderColor_.Blue, CSBorderColor_.Alpha);
    
    // Render cel shader
    
    glBegin(GL_TRIANGLES);
    
    for (s32 surf = 0, tri, vert; surf < hEntity->getMeshSurfaceCount(); ++surf)
    {
        hEntity->selectSurface(surf);
        for (tri = 0; tri < hEntity->getMeshTriangleCount(surf); ++tri)
        {
            hEntity->getTriangleVertices(tri, VerticesIndices);
            for (vert = 0; vert < 3; ++vert)
            {
                glVertex3f(
                    hEntity->getVertexCoord(VerticesIndices[vert]).X,
                    hEntity->getVertexCoord(VerticesIndices[vert]).Y,
                    hEntity->getVertexCoord(VerticesIndices[vert]).Z
                );
            }
        }
    }
    
    glEnd();
    
    // Settings off
    
    if (!isCullFace)
        glDisable(GL_CULL_FACE);
    glCullFace(CullFaceMode);
    
    glLineWidth(LineWidth);
    
}


/*
 * ========== Blur effect ==========
 */

void OpenGLSpecialEffectsActor::renderBlurEffect(Entity* hEntity, s32 LevelsCount, f32 LevelsDist, u8 ColorAlpha)
{
    
    f32 Dist;
    u8 Alpha;
    const u8 AlphaLess = ColorAlpha / LevelsCount;
    
    // Loop through all registered camera
    
    for (std::list<Camera*>::iterator itCam = RegisteredCameras_.begin(); itCam != RegisteredCameras_.end(); ++itCam)
    {
        
        if (!(*itCam)->getVisible())
            continue;
        if (RegisteredCameras_.size() > 1)
            (*itCam)->updateViewmode();
        
        glLoadIdentity();
        
        (*itCam)->updateModelviewMatrix();
        
        // Rendering finaly
        
        glEnableClientState(GL_VERTEX_ARRAY);
        glEnableClientState(GL_NORMAL_ARRAY);
        glEnableClientState(GL_FOG_COORDINATE_ARRAY);
        glEnableClientState(GL_COLOR_ARRAY);
        
        __spVideoDriver->beginTextureDrawing(BEBlurTexture_);
        {
            hEntity->render();
        }
        __spVideoDriver->endTextureDrawing();
        
        //hEntity->render();
        
        glDisableClientState(GL_VERTEX_ARRAY);
        glDisableClientState(GL_NORMAL_ARRAY);
        glDisableClientState(GL_FOG_COORDINATE_ARRAY);
        glDisableClientState(GL_COLOR_ARRAY);
        
        // Render
        
        Dist = 0.0f;
        Alpha = ColorAlpha;
        
        __spVideoDriver->beginDrawing2D();
        
        for (s32 i = 0; i < LevelsCount; ++i)
        {
            __spVideoDriver->draw2DImage(
                BEBlurTexture_,
                dim::rect2di(0, 0, __Screen_Width, __Screen_Height),
                dim::rect2df(Dist, 1.0f-Dist, 1.0f-Dist, Dist),
                video::color(255, 255, 255, Alpha)
            );
            Alpha -= AlphaLess;
            Dist += LevelsDist;
        }
        
        __spVideoDriver->endDrawing2D();
        
    }
    
}





} // /namespace effect

} // /namespace scene

} // /namespace sp







// ================================================================================
