/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for OpenGL textures
 *
 */

#ifndef __SP_VIDEODRIVER_OPENGLTEXTURE_H__
#define __SP_VIDEODRIVER_OPENGLTEXTURE_H__


#include "spStandard.hpp"
#include "spVideoDriverTexture.hpp"

#include <gl/gl.h>


namespace sp
{
namespace video
{


class OpenGLDriver;

class SP_EXPORT OpenGLTexture : public Texture
{
    
    public:
        
        OpenGLTexture();
        OpenGLTexture(
            u32 ID, io::stringc Filename, dim::size2di Size, u8* ImageData,
            s32 RenderMode = TEXTURE_SMOOTH, s32 FilterMode = TEXTURE_TRILINEAR,
            s32 Format = 3, s32 Type = TEXTURE_RGB, bool isEdgeSmooth = false, bool isMipMapping = true
        );
        ~OpenGLTexture();
        
        /* Extra option functions */
        
        void changeAttributes(s32 RenderMode, s32 FilterMode = TEXTURE_TRILINEAR, bool isEdgeSmooth = false);
        
        void setColorIntensity(f32 Red, f32 Green, f32 Blue);
        
        void flipColors();
        
        /* Image data updating and recreating */
        
        void setMipMapFilter(s32 MinFilter, s32 MagFilter, bool isMipMapping = true);
        
        // Update the image buffer (OpenGL texture -> image data / image data - > OpenGL texture)
        void updateImageData(EUpdateDataFunc UpdateDataFunc);
        
        /* Binding & unbinding */
        
        void bind(s32 Level = 0);
        void unbind(s32 Level = 0);
        
    private:
        
        /* Private functions */
        
        void updateFormat();
        
        void makeEmptyImageData();
        
        void receiveImageData();
        void recreateImageData();
        
        void clear();
        
        /* Members */
        
        GLenum Format_, Dimension_;
        
};


} // /namespace video

} // /namespace sp


#endif






// ================================================================================
