/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * "Ray" Software renderer
 *
 */

#include "spRaySoftwareRenderer.h"
#include "spRayStandardTypes.h"

#include <gl/gl.h>


/*
 * ========== Internal variable procedures ==========
 */

void rayBlendProc_Normal(RAY_BLENDPROC_PARAMETERS)
{
}

void rayBlendProc_Bright(RAY_BLENDPROC_PARAMETERS)
{
    srcR = dstR + (srcR * srcA / 255);
    srcG = dstG + (srcG * srcA / 255);
    srcB = dstB + (srcB * srcA / 255);
    
    rayClamp(srcR, 0, 255);
    rayClamp(srcG, 0, 255);
    rayClamp(srcB, 0, 255);
}

void rayBlendProc_Dark(RAY_BLENDPROC_PARAMETERS)
{
    srcR = dstR - 255 + (srcR * srcA / 255);
    srcG = dstG - 255 + (srcG * srcA / 255);
    srcB = dstB - 255 + (srcB * srcA / 255);
    
    rayClamp(srcR, 0, 255);
    rayClamp(srcG, 0, 255);
    rayClamp(srcB, 0, 255);
}

void rayBlendProc_Light(RAY_BLENDPROC_PARAMETERS)
{
    srcR = dstR * srcR / 255;
    srcG = dstG * srcG / 255;
    srcB = dstB * srcB / 255;
}


/*
 * ========== Internal members ==========
 */

RAYenum rayLastError        = RAY_ERROR_NONE;

// Ray current members (rc's)
SRayVertex*     rcVertex    [RAY_RENDER_PIPELINE_SIZE]  = { 0 };
SRayVertex*     rcMemVertex [RAY_RENDER_PIPELINE_SIZE]  = { 0 };

SRayTexture2D*  rcTexture   [RAY_TEXTURE_LAYER_SIZE]    = { 0 };

SRayLight*      rcLight     [RAY_MAX_LIGHT_COUNT]       = { 0 };

RAYint rcPLIndex                = 0; // render pipe line index
RAYint rcRenderType             = 0;

RAYint rcMatrixMode             = RAY_MODELVIEW_MATRIX;
RAYint rcTextureLayer           = 0;
RAYint rcMaxTextureLayer        = 1;
RAYint rcTextureMipMapLevel     = 0;
RAYint rcUpdateVerticesMatrix   = RAY_TRUE;

// Ray status members
RAYint rayRenderStatus[RAY_STATUS_COUNT] = { RAY_FALSE };

RAYint rayPixelRect     [4] = { 0 };
RAYint rayViewportRect  [4] = { 0 };
RAYint rayScissorRect   [4] = { 0 };

RAYfloat rayPlaneNear   = 1.0f,     rayRangeNear    = 1.0 / rayPlaneNear;
RAYfloat rayPlaneFar    = 100.0f,   rayRangeFar     = 1.0 / rayPlaneFar;
RAYfloat rayFieldOfView = 640 / 2;

SRayColor4i rayBgClearColor;

SRayMatrix4f rayRenderMatrix[RAY_COUNT_OF_MATRICES];

// Ray render state functions
PFNRAYBLENDPROC rayBlendProc    = rayBlendProc_Normal;
RAYint rayRenderFace            = RAY_RENDER_FRONT;

SRayMappingGen rayMappingGenType[RAY_TEXTURE_LAYER_SIZE];

// Screen interface
RAYuint rayDCTexID          = 0;

RAYpixel_t* rayPixelBuffer  = 0;
RAYfloat* rayDepthBuffer    = 0;

RAYint rayScreenWidth       = 0;
RAYint rayScreenHeight      = 0;

RAYint rayDepthBufferSize   = 0;
RAYint rayPixelBufferSize   = 0;


/*
 * ========== Internal functions ==========
 */

static void rayUpdatePixelRect()
{
    if (rayRenderStatus[RAY_SCISSOR_TEST])
    {
        rayPixelRect[0] = ( rayViewportRect[0] < rayScissorRect[0] ? rayViewportRect[0] : rayScissorRect[0] );
        rayPixelRect[1] = ( rayViewportRect[1] < rayScissorRect[1] ? rayViewportRect[1] : rayScissorRect[1] );
        rayPixelRect[2] = ( rayViewportRect[2] < rayScissorRect[2] ? rayViewportRect[2] : rayScissorRect[2] );
        rayPixelRect[3] = ( rayViewportRect[3] < rayScissorRect[3] ? rayViewportRect[3] : rayScissorRect[3] );
    }
    else
    {
        for (RAYint i = 0; i < 4; ++i)
            rayPixelRect[i] = rayViewportRect[i];
    }
    
    rayClamp(rayPixelRect[0], 0, rayScreenWidth);
    rayClamp(rayPixelRect[1], 0, rayScreenHeight);
    rayClamp(rayPixelRect[2], 0, rayScreenWidth);
    rayClamp(rayPixelRect[3], 0, rayScreenHeight);
    
    if (rayPixelRect[0] > rayPixelRect[2])
    {
        RAYint tmp = rayPixelRect[0]; rayPixelRect[0] = rayPixelRect[2]; rayPixelRect[2] = tmp;
    }
    if (rayPixelRect[1] > rayPixelRect[3])
    {
        RAYint tmp = rayPixelRect[1]; rayPixelRect[1] = rayPixelRect[3]; rayPixelRect[3] = tmp;
    }
}

static RAYint rayRound2(RAYint Value)
{
    RAYint i;
    
    for (i = 1; i < Value; i <<= 1);
    
    if (i - Value <= Value - i/2)
        return i;
    
    return i/2;
}

static void rayClampTextureSize(SRayTexture2D* tex, RAYuint Level)
{
    RAYint Width    = rayRound2(tex->width);
    RAYint Height   = rayRound2(tex->height);
    
    if (Width != tex->width || Height != tex->height)
    {
        RAYvoid* image = (RAYvoid*)tex->image[Level];
        
        rayScaleImage2D(
            image, tex->width, tex->height, Width, Height, 4, RAY_UNSIGNED_BYTE
        );
        
        tex->image[Level] = (RAYpixel_t*)image;
        
        tex->width  = Width;
        tex->height = Height;
    }
}



/*
 * ========== Static functions ==========
 */


/*
 * ===== Identities =====
 */

RAYAPI const RAYbyte* rayGetString(RAYenum Name)
{
    switch (Name)
    {
        case RAY_VENDOR:
            return "SoftPixel Engine";
        case RAY_RENDERER:
            return "Ray";
        case RAY_VERSION:
            return "2.0";
    }
    
    return "";
}

RAYAPI RAYenum rayGetError()
{
    return rayLastError;
}

RAYAPI void rayGetValue(RAYenum Type, RAYvoid* data)
{
    switch (Type)
    {
        case RAY_SCISSOR:
        {
            RAYint* pInt = (RAYint*)data;
            
            pInt[0] = rayScissorRect[0];
            pInt[1] = rayScissorRect[1];
            pInt[2] = rayScissorRect[2];
            pInt[3] = rayScissorRect[3];
        }
        break;
        
        case RAY_VIEWPORT:
        {
            RAYint* pInt = (RAYint*)data;
            
            pInt[0] = rayViewportRect[0];
            pInt[1] = rayViewportRect[1];
            pInt[2] = rayViewportRect[2];
            pInt[3] = rayViewportRect[3];
        }
        break;
        
        case RAY_PERSPECTIVE:
        {
            RAYfloat* pFloat = (RAYfloat*)data;
            
            pFloat[0] = rayPlaneNear;
            pFloat[1] = rayPlaneFar;
            pFloat[2] = rayFieldOfView;
        }
        break;
    }
}


/*
 * ===== Render states =====
 */

RAYAPI void rayEnable(RAYenum Status)
{
    rayRenderStatus[Status] = RAY_TRUE;
}
RAYAPI void rayDisable(RAYenum Status)
{
    rayRenderStatus[Status] = RAY_FALSE;
}

RAYAPI void raySetStatus(RAYenum Status, RAYint Enable)
{
    rayRenderStatus[Status] = Enable;
}
RAYAPI RAYint rayGetStatus(RAYenum Status)
{
    return rayRenderStatus[Status];
}

RAYAPI void rayScissor(RAYint X, RAYint Y, RAYint Width, RAYint Height)
{
    rayScissorRect[0] = X;
    rayScissorRect[1] = Y;
    rayScissorRect[2] = X + Width;
    rayScissorRect[3] = Y + Height;
    rayUpdatePixelRect();
}
RAYAPI void rayViewport(RAYint X, RAYint Y, RAYint Width, RAYint Height)
{
    rayViewportRect[0] = X;
    rayViewportRect[1] = Y;
    rayViewportRect[2] = X + Width;
    rayViewportRect[3] = Y + Height;
    rayUpdatePixelRect();
}
RAYAPI void rayPerspective(RAYfloat Near, RAYfloat Far, RAYfloat FieldOfView)
{
    rayPlaneNear    = Near;
    rayPlaneFar     = Far;
    
    rayRangeNear    = 1.0 / Near;
    rayRangeFar     = 1.0 / Far;
    
    rayFieldOfView  = FieldOfView;
}

RAYAPI void rayRenderMode(RAYenum Mode)
{
    rayRenderFace = Mode;
}
RAYAPI void rayBlendFunc(RAYenum Mode)
{
    if (Mode == RAY_BLEND_NORMAL)
        rayBlendProc = rayBlendProc_Normal;
    else if (Mode == RAY_BLEND_BRIGHT)
        rayBlendProc = rayBlendProc_Bright;
    else if (Mode == RAY_BLEND_DARK)
        rayBlendProc = rayBlendProc_Dark;
}

RAYAPI void rayLighti(RAYenum Index, RAYenum Type, const RAYint Param)
{
    // Get the light handle
    Index -= RAY_LIGHT0;
    
    if (Index < 0 || Index >= RAY_MAX_LIGHT_COUNT)
        return;
    
    SRayLight* light = rcLight[Index];
    
    // Set the attributes
    switch (Type)
    {
        case RAY_LIGHT_TYPE:
            light->type = Param;
            break;
    }
}

RAYAPI void rayLightfv(RAYenum Index, RAYenum Type, const RAYfloat* Params)
{
    // Get the light handle
    Index -= RAY_LIGHT0;
    
    if (Index < 0 || Index >= RAY_MAX_LIGHT_COUNT)
        return;
    
    SRayLight* light = rcLight[Index];
    
    // Set the attributes
    switch (Type)
    {
        case RAY_DIFFUSE:
            light->diffuse.r = (RAYcolor_t)(Params[0]*255);
            light->diffuse.g = (RAYcolor_t)(Params[1]*255);
            light->diffuse.b = (RAYcolor_t)(Params[2]*255);
            light->diffuse.a = (RAYcolor_t)(Params[3]*255);
            break;
        case RAY_AMBIENT:
            light->ambient.r = (RAYcolor_t)(Params[0]*255);
            light->ambient.g = (RAYcolor_t)(Params[1]*255);
            light->ambient.b = (RAYcolor_t)(Params[2]*255);
            light->ambient.a = (RAYcolor_t)(Params[3]*255);
            break;
        case RAY_TRANSFORMATION:
            memcpy(&light->mat.m[0], Params, sizeof(light->mat));
            break;
        case RAY_DIRECTION:
            memcpy(&light->dir.x, Params, sizeof(light->dir));
            break;
    }
}


/*
 * ===== Texturing =====
 */

RAYAPI void raySetMaxTextureLayer(RAYint MaxLayer)
{
    rcMaxTextureLayer = MaxLayer;
    rayClamp(rcMaxTextureLayer, 0, RAY_TEXTURE_LAYER_SIZE);
}
RAYAPI void rayUseTextureLayer(RAYint Layer)
{
    rcTextureLayer = Layer;
}

RAYAPI void rayGenTexture(RAYuint &ID)
{
    SRayTexture2D* tex = (SRayTexture2D*)malloc(sizeof(SRayTexture2D));
    
    tex->width      = 0;
    tex->height     = 0;
    tex->mipcount   = 0;
    tex->image      = (RAYpixel_t**)calloc(RAY_MAX_MIPMAPLEVELS, sizeof(RAYpixel_t*));
    
    for (RAYint i = 0; i < RAY_MAX_MIPMAPLEVELS; ++i)
        tex->image[i] = 0;
    
    ID = (RAYuint)tex;
}
RAYAPI void rayDeleteTexture(RAYuint &ID)
{
    if (ID)
    {
        SRayTexture2D* tex = (SRayTexture2D*)ID;
        
        if (tex)
        {
            if (tex->image)
            {
                for (RAYint i = 0; i < tex->mipcount; ++i)
                {
                    if (tex->image[i])
                        free(tex->image[i]);
                }
                
                free(tex->image);
            }
            
            free(tex);
        }
        
        ID = 0;
    }
}

RAYAPI void rayBindTexture(RAYuint ID)
{
    if (ID)
        rcTexture[rcTextureLayer] = (SRayTexture2D*)ID;
}
RAYAPI void rayUnbindTexture()
{
    rcTexture[rcTextureLayer] = 0;
}

RAYAPI RAYuint rayGetCurTexture()
{
    return (RAYuint)rcTexture[rcTextureLayer];
}

RAYAPI void rayTexGen(RAYint TexCoordType, RAYint Type)
{
    switch (TexCoordType)
    {
        case RAY_TEXTURE_GEN_U:
            rayMappingGenType[rcTextureLayer].genu = Type;
            break;
        case RAY_TEXTURE_GEN_V:
            rayMappingGenType[rcTextureLayer].genv = Type;
            break;
    }
}


RAYAPI void rayTexSubImage2D(
    RAYuint ID, RAYuint Level, RAYint Format, RAYenum Type, const RAYvoid* ImageData)
{
    
    // Check for correct parameter values
    if (!ID || !ImageData || Format < 1 || Format > 4 || Type < RAY_INTEGER || Type > RAY_FLOAT)
        return;
    
    // Get the image data
    SRayTexture2D* tex = (SRayTexture2D*)ID;
    
    if (!tex->image[Level])
    {
        rayLastError = RAY_ERROR_EMPTYIMAGE;
        return;
    }
    
    RAYint Width    = tex->width >> Level;
    RAYint Height   = tex->height >> Level;
    
    const RAYint ImageDataSize = (Width * Height) << 2;
    
    // Temporary variables
    register RAYint x, y, i = 0, j = 0;
    
    // Fill the image data
    switch (Type)
    {
        case RAY_INTEGER:
        {
            const RAYint* refImage = (const RAYint*)ImageData;
            
            for (y = 0; y < Height; ++y)
            {
                for (x = 0; x < Width; ++x, i += 4, j += Format)
                {
                    switch (Format)
                    {
                        case 1:
                            tex->image[Level][i+0] = refImage[j];
                            tex->image[Level][i+1] = refImage[j];
                            tex->image[Level][i+2] = refImage[j];
                            tex->image[Level][i+3] = 255;
                            break;
                            
                        case 2:
                            tex->image[Level][i+0] = refImage[j];
                            tex->image[Level][i+1] = refImage[j];
                            tex->image[Level][i+2] = refImage[j];
                            tex->image[Level][i+3] = refImage[j+1];
                            break;
                            
                        case 3:
                            tex->image[Level][i+0] = refImage[j+0];
                            tex->image[Level][i+1] = refImage[j+1];
                            tex->image[Level][i+2] = refImage[j+2];
                            tex->image[Level][i+3] = 255;
                            break;
                            
                        case 4:
                            tex->image[Level][i+0] = refImage[j+0];
                            tex->image[Level][i+1] = refImage[j+1];
                            tex->image[Level][i+2] = refImage[j+2];
                            tex->image[Level][i+3] = refImage[j+3];
                            break;
                    }
                } // next texel
            } // next line
        }
        break;
        
        case RAY_FLOAT:
        {
            const RAYfloat* refImage = (const RAYfloat*)ImageData;
            
            for (y = 0; y < Height; ++y)
            {
                for (x = 0; x < Width; ++x, i += 4, j += Format)
                {
                    switch (Format)
                    {
                        case 1:
                            tex->image[Level][i+0] = (RAYint)(refImage[j] * 255);
                            tex->image[Level][i+1] = (RAYint)(refImage[j] * 255);
                            tex->image[Level][i+2] = (RAYint)(refImage[j] * 255);
                            tex->image[Level][i+3] = 255;
                            break;
                            
                        case 2:
                            tex->image[Level][i+0] = (RAYint)(refImage[j+0] * 255);
                            tex->image[Level][i+1] = (RAYint)(refImage[j+0] * 255);
                            tex->image[Level][i+2] = (RAYint)(refImage[j+0] * 255);
                            tex->image[Level][i+3] = (RAYint)(refImage[j+1] * 255);
                            break;
                            
                        case 3:
                            tex->image[Level][i+0] = (RAYint)(refImage[j+0] * 255);
                            tex->image[Level][i+1] = (RAYint)(refImage[j+1] * 255);
                            tex->image[Level][i+2] = (RAYint)(refImage[j+2] * 255);
                            tex->image[Level][i+3] = 255;
                            break;
                            
                        case 4:
                            tex->image[Level][i+0] = (RAYint)(refImage[j+0] * 255);
                            tex->image[Level][i+1] = (RAYint)(refImage[j+1] * 255);
                            tex->image[Level][i+2] = (RAYint)(refImage[j+2] * 255);
                            tex->image[Level][i+3] = (RAYint)(refImage[j+3] * 255);
                            break;
                    }
                } // next texel
            } // next line
        }
        break;
        
        case RAY_UNSIGNED_BYTE:
        {
            const RAYubyte* refImage = (const RAYubyte*)ImageData;
            
            for (y = 0; y < Height; ++y)
            {
                for (x = 0; x < Width; ++x, i += 4, j += Format)
                {
                    switch (Format)
                    {
                        case 1:
                            tex->image[Level][i+0] = refImage[j];
                            tex->image[Level][i+1] = refImage[j];
                            tex->image[Level][i+2] = refImage[j];
                            tex->image[Level][i+3] = 255;
                            break;
                            
                        case 2:
                            tex->image[Level][i+0] = refImage[j];
                            tex->image[Level][i+1] = refImage[j];
                            tex->image[Level][i+2] = refImage[j];
                            tex->image[Level][i+3] = refImage[j+1];
                            break;
                            
                        case 3:
                            tex->image[Level][i+0] = refImage[j+0];
                            tex->image[Level][i+1] = refImage[j+1];
                            tex->image[Level][i+2] = refImage[j+2];
                            tex->image[Level][i+3] = 255;
                            break;
                            
                        case 4:
                            tex->image[Level][i+0] = refImage[j+0];
                            tex->image[Level][i+1] = refImage[j+1];
                            tex->image[Level][i+2] = refImage[j+2];
                            tex->image[Level][i+3] = refImage[j+3];
                            break;
                    }
                } // next texel
            } // next line
        }
        break;
        
        default:
        {
            memset(tex->image[Level], 0, sizeof(RAYpixel_t)*ImageDataSize);
        }
        break;
    }
    
    #ifdef __RAY_CLAMP_TEXTURESIZE__
    // Clamp the image
    rayClampTextureSize(tex, Level);
    #endif
    
}


RAYAPI void rayTexImage2D(
    RAYuint ID, RAYuint Level, RAYint Width, RAYint Height, RAYint Format, RAYenum Type, const RAYvoid* ImageData)
{
    if (!ID || !ImageData || Width <= 0 || Height <= 0 || Format < 1 || Format > 4 || Type < RAY_INTEGER || Type > RAY_FLOAT)
        return;
    
    SRayTexture2D* tex = (SRayTexture2D*)ID;
    
    // Delete old image data
    if (tex->image[Level])
        free(tex->image[Level]);
    
    const RAYint ImageDataSize = (Width * Height) << 2;
    
    if (Level == 0)
    {
        tex->width  = Width;
        tex->height = Height;
    }
    
    if (tex->mipcount < Level + 1)
        tex->mipcount = Level + 1;
    
    // Allocate new image data
    tex->image[Level] = (RAYpixel_t*)calloc(ImageDataSize, sizeof(RAYpixel_t));
    
    // Fill the image data
    rayTexSubImage2D(ID, Level, Format, Type, ImageData);
}


RAYAPI void rayGetImage2D(
    RAYuint ID, RAYuint Level, RAYint Format, RAYenum Type, RAYvoid* ImageData)
{
    
    // Get the image dat
    SRayTexture2D* tex = (SRayTexture2D*)ID;
    
    RAYint Width    = tex->width >> Level;
    RAYint Height   = tex->height >> Level;
    
    if (!tex || !tex->image[Level])
    {
        rayLastError = RAY_ERROR_EMPTYIMAGE;
        return;
    }
    
    // Temporary variables
    register RAYint x, y, i = 0, j = 0;
    
    // Fill the image data
    switch (Type)
    {
        case RAY_INTEGER:
        {
            RAYint* refImage = (RAYint*)ImageData;
            
            for (y = 0; y < Height; ++y)
            {
                for (x = 0; x < Width; ++x, i += 4, j += Format)
                {
                    switch (Format)
                    {
                        case 1:
                            refImage[j] = ( tex->image[Level][i+0] + tex->image[Level][i+1] + tex->image[Level][i+2] ) / 3;
                            break;
                            
                        case 2:
                            refImage[j+0] = ( tex->image[Level][i+0] + tex->image[Level][i+1] + tex->image[Level][i+2] ) / 3;
                            refImage[j+1] = tex->image[Level][i+3];
                            break;
                            
                        case 3:
                            refImage[j+0] = tex->image[Level][i+0];
                            refImage[j+1] = tex->image[Level][i+1];
                            refImage[j+2] = tex->image[Level][i+2];
                            break;
                            
                        case 4:
                            refImage[j+0] = tex->image[Level][i+0];
                            refImage[j+1] = tex->image[Level][i+1];
                            refImage[j+2] = tex->image[Level][i+2];
                            refImage[j+3] = tex->image[Level][i+3];
                            break;
                    }
                } // next texel
            } // next line
        }
        break;
        
        case RAY_FLOAT:
        {
            RAYfloat* refImage = (RAYfloat*)ImageData;
            
            for (y = 0; y < Height; ++y)
            {
                for (x = 0; x < Width; ++x, i += 4, j += Format)
                {
                    switch (Format)
                    {
                        case 1:
                            refImage[j] = ( tex->image[Level][i+0] + tex->image[Level][i+1] + tex->image[Level][i+2] ) / 3;
                            refImage[j] /= 255;
                            break;
                            
                        case 2:
                            refImage[j+0] = ( tex->image[Level][i+0] + tex->image[Level][i+1] + tex->image[Level][i+2] ) / 3;
                            refImage[j+1] = tex->image[Level][i+3];
                            refImage[j+0] /= 255;
                            refImage[j+1] /= 255;
                            break;
                            
                        case 3:
                            refImage[j+0] = tex->image[Level][i+0];
                            refImage[j+1] = tex->image[Level][i+1];
                            refImage[j+2] = tex->image[Level][i+2];
                            refImage[j+0] /= 255;
                            refImage[j+1] /= 255;
                            refImage[j+2] /= 255;
                            break;
                            
                        case 4:
                            refImage[j+0] = tex->image[Level][i+0];
                            refImage[j+1] = tex->image[Level][i+1];
                            refImage[j+2] = tex->image[Level][i+2];
                            refImage[j+3] = tex->image[Level][i+2];
                            refImage[j+0] /= 255;
                            refImage[j+1] /= 255;
                            refImage[j+2] /= 255;
                            refImage[j+3] /= 255;
                            break;
                    }
                } // next texel
            } // next line
        }
        break;
        
        case RAY_UNSIGNED_BYTE:
        {
            RAYubyte* refImage = (RAYubyte*)ImageData;
            
            for (y = 0; y < Height; ++y)
            {
                for (x = 0; x < Width; ++x, i += 4, j += Format)
                {
                    switch (Format)
                    {
                        case 1:
                            refImage[j] = ( tex->image[Level][i+0] + tex->image[Level][i+1] + tex->image[Level][i+2] ) / 3;
                            break;
                            
                        case 2:
                            refImage[j+0] = ( tex->image[Level][i+0] + tex->image[Level][i+1] + tex->image[Level][i+2] ) / 3;
                            refImage[j+1] = tex->image[Level][i+3];
                            break;
                            
                        case 3:
                            refImage[j+0] = tex->image[Level][i+0];
                            refImage[j+1] = tex->image[Level][i+1];
                            refImage[j+2] = tex->image[Level][i+2];
                            break;
                            
                        case 4:
                            refImage[j+0] = tex->image[Level][i+0];
                            refImage[j+1] = tex->image[Level][i+1];
                            refImage[j+2] = tex->image[Level][i+2];
                            refImage[j+3] = tex->image[Level][i+3];
                            break;
                    }
                } // next texel
            } // next line
        }
        break;
    }
    
}


RAYAPI void rayCopyTexImage2D(
    RAYuint ID, RAYuint Level, RAYint OffsetX, RAYint OffsetY, RAYint X, RAYint Y, RAYint Width, RAYint Height)
{
    // Get the image data
    SRayTexture2D* tex = (SRayTexture2D*)ID;
    
    if (!tex)
    {
        rayLastError = RAY_ERROR_EMPTYIMAGE;
        return;
    }
    
    // Delete old image data
    if (tex->image[Level])
        free(tex->image[Level]);
    
    Width   >>= Level;
    Height  >>= Level;
    
    const RAYint ImageDataSize = (Width * Height) << 2;
    
    if (Level == 0)
    {
        tex->width  = Width;
        tex->height = Height;
    }
    
    if (tex->mipcount < Level)
        tex->mipcount = Level;
    
    // Allocate new image data
    tex->image[Level] = (RAYpixel_t*)calloc(ImageDataSize, sizeof(RAYpixel_t));
    
    rayCopyTexSubImage2D(ID, Level, OffsetX, OffsetY, X, Y, RAY_RGBA);
}

RAYAPI void rayCopyTexSubImage2D(
    RAYuint ID, RAYuint Level, RAYint OffsetX, RAYint OffsetY, RAYint X, RAYint Y, RAYenum Components)
{
    // Get the image data
    SRayTexture2D* tex = (SRayTexture2D*)ID;
    
    if (!tex)
    {
        rayLastError = RAY_ERROR_EMPTYIMAGE;
        return;
    }
    
    // Temporary variables
    RAYint ix, iy, i, j = 0, k = 0;
    RAYfloat z;
    
    RAYint Width    = tex->width >> Level;
    RAYint Height   = tex->height >> Level;
    
    // Clamp the screen dimensions
    rayClamp(X, 0, rayScreenWidth);
    rayClamp(Y, 0, rayScreenHeight);
    rayClamp(Width, 0, rayScreenWidth);
    rayClamp(Height, 0, rayScreenHeight);
    
    if (X + Width > rayScreenWidth)
        Width -= rayScreenWidth - ( X + Width );
    if (Y + Height > rayScreenHeight)
        Height -= rayScreenHeight - ( Y + Height );
    
    // Clamp the image dimensions
    rayClamp(OffsetX, 0, tex->width);
    rayClamp(OffsetY, 0, tex->height);
    rayClamp(Width, 0, tex->width);
    rayClamp(Height, 0, tex->height);
    
    if (OffsetX + Width > tex->width)
        Width -= tex->width - ( OffsetX + Width );
    if (OffsetY + Height > tex->height)
        Height -= tex->height - ( OffsetY + Height );
    
    // Loop for all lines
    if (Components == RAY_RGBA)
    {
        for (iy = Y; iy < Y + Height; ++iy)
        {
            // Get the offsets
            i = ( iy * rayScreenWidth + X ) << 2;
            j = ( ( iy - Y + OffsetY ) * tex->width + OffsetX ) << 2;
            
            // Loop for all texels
            for (ix = X; ix < ( X + Width ) << 2; ++ix, ++i, ++j)
                tex->image[Level][j] = rayPixelBuffer[i];
        }
    }
    else if (Components == RAY_DEPTH)
    {
        for (iy = Y; iy < Y + Height; ++iy)
        {
            // Get the offsets
            i = iy * rayScreenWidth + X;
            j = ( ( iy - Y + OffsetY ) * tex->width + OffsetX ) << 2;
            
            z = rayDepthBuffer[i] * 255.0f;
            rayClamp(z, 0.0f, 255.0f);
            
            // Loop for all texels
            for (ix = X; ix < ( X + Width ) << 2; ++ix, ++j)
            {
                if (k == 3)
                    tex->image[Level][j] = 255;
                else
                    tex->image[Level][j] = (RAYcolor_t)z;
                
                if (++k >= 4)
                {
                    k = 0;
                    z = rayDepthBuffer[++i] * 255.0f;
                    rayClamp(z, 0.0f, 255.0f);
                }
            }
        }
    }
    
    #ifdef __RAY_CLAMP_TEXTURESIZE__
    // Clamp the image
    rayClampTextureSize(tex, Level);
    #endif
}


RAYAPI void rayScaleImage2D(
    RAYvoid* &ImageData, RAYint Width, RAYint Height, RAYint NewWidth, RAYint NewHeight, RAYint Format, RAYenum Type)
{
    
    // Check if the memory is not empty
    if ( !ImageData || ( Width == NewWidth && Height == NewHeight ) )
        return;
    
    // Temporary variables
    register RAYint x, y, i, j, k;
    
    // Fill the image data
    switch (Type)
    {
        case RAY_INTEGER:
        {
            // Allocate new memory
            RAYint* NewImageData = (RAYint*)calloc(NewWidth * NewHeight * Format, sizeof(RAYint));
            
            RAYint* refImage = (RAYint*)ImageData;
            
            for (y = 0; y < NewHeight; ++y)
            {
                for (x = 0; x < NewWidth; ++x)
                {
                    for (i = 0; i < Format; ++i)
                    {
                        // Get the image data indices
                        j = y * NewWidth + x;
                        k = ( y * Height / NewHeight ) * Width + ( x * Width / NewWidth );
                        
                        // Fill the new image data
                        NewImageData[ j * Format + i ] = refImage[ k * Format + i ];
                    }
                }
            }
            
            // Use the new image data
            free(ImageData);
            ImageData = NewImageData;
        }
        break;
        
        case RAY_FLOAT:
        {
            // Allocate new memory
            RAYfloat* NewImageData = (RAYfloat*)calloc(NewWidth * NewHeight * Format, sizeof(RAYfloat));
            
            RAYfloat* refImage = (RAYfloat*)ImageData;
            
            for (y = 0; y < NewHeight; ++y)
            {
                for (x = 0; x < NewWidth; ++x)
                {
                    for (i = 0; i < Format; ++i)
                    {
                        // Get the image data indices
                        j = y * NewWidth + x;
                        k = ( y * Height / NewHeight ) * Width + ( x * Width / NewWidth );
                        
                        // Fill the new image data
                        NewImageData[ j * Format + i ] = (RAYint)(refImage[ k * Format + i ] * 255);
                    }
                }
            }
            
            // Use the new image data
            free(ImageData);
            ImageData = NewImageData;
        }
        break;
        
        case RAY_UNSIGNED_BYTE:
        {
            // Allocate new memory
            RAYubyte* NewImageData = (RAYubyte*)calloc(NewWidth * NewHeight * Format, sizeof(RAYubyte));
            
            RAYubyte* refImage = (RAYubyte*)ImageData;
            
            for (y = 0; y < NewHeight; ++y)
            {
                for (x = 0; x < NewWidth; ++x)
                {
                    for (i = 0; i < Format; ++i)
                    {
                        // Get the image data indices
                        j = y * NewWidth + x;
                        k = ( y * Height / NewHeight ) * Width + ( x * Width / NewWidth );
                        
                        // Fill the new image data
                        NewImageData[ j * Format + i ] = refImage[ k * Format + i ];
                    }
                }
            }
            
            // Use the new image data
            free(ImageData);
            ImageData = NewImageData;
        }
        break;
    }
    
}



/*
 * ===== Vertex information =====
 */

RAYAPI void rayVertex2i(RAYint X, RAYint Y)
{
    rcVertex[rcPLIndex]->pos.x = X, rcVertex[rcPLIndex]->pos.y = Y, rcVertex[rcPLIndex]->pos.z = 0, rcVertex[rcPLIndex]->pos.w = 1;
    ++rcPLIndex;
}
RAYAPI void rayVertex3i(RAYint X, RAYint Y, RAYint Z)
{
    rcVertex[rcPLIndex]->pos.x = X, rcVertex[rcPLIndex]->pos.y = Y, rcVertex[rcPLIndex]->pos.z = Z, rcVertex[rcPLIndex]->pos.w = 1;
    ++rcPLIndex;
}
RAYAPI void rayVertex4i(RAYint X, RAYint Y, RAYint Z, RAYint W)
{
    rcVertex[rcPLIndex]->pos.x = X, rcVertex[rcPLIndex]->pos.y = Y, rcVertex[rcPLIndex]->pos.z = Z, rcVertex[rcPLIndex]->pos.w = W;
    ++rcPLIndex;
}

RAYAPI void rayVertex2f(RAYfloat X, RAYfloat Y)
{
    rcVertex[rcPLIndex]->pos.x = X, rcVertex[rcPLIndex]->pos.y = Y, rcVertex[rcPLIndex]->pos.z = 0, rcVertex[rcPLIndex]->pos.w = 1;
    ++rcPLIndex;
}
RAYAPI void rayVertex3f(RAYfloat X, RAYfloat Y, RAYfloat Z)
{
    rcVertex[rcPLIndex]->pos.x = X, rcVertex[rcPLIndex]->pos.y = Y, rcVertex[rcPLIndex]->pos.z = Z, rcVertex[rcPLIndex]->pos.w = 1;
    ++rcPLIndex;
}
RAYAPI void rayVertex4f(RAYfloat X, RAYfloat Y, RAYfloat Z, RAYfloat W)
{
    rcVertex[rcPLIndex]->pos.x = X, rcVertex[rcPLIndex]->pos.y = Y, rcVertex[rcPLIndex]->pos.z = Z, rcVertex[rcPLIndex]->pos.w = W;
    ++rcPLIndex;
}

RAYAPI void rayNormal3i(RAYint X, RAYint Y, RAYint Z)
{
    rcVertex[rcPLIndex]->nml.x = X; rcVertex[rcPLIndex]->nml.y = Y; rcVertex[rcPLIndex]->nml.z = Z;
}
RAYAPI void rayNormal3f(RAYfloat X, RAYfloat Y, RAYfloat Z)
{
    rcVertex[rcPLIndex]->nml.x = X; rcVertex[rcPLIndex]->nml.y = Y; rcVertex[rcPLIndex]->nml.z = Z;
}

RAYAPI void rayTexCoord2i(RAYint Layer, RAYint X, RAYint Y)
{
    rcVertex[rcPLIndex]->map[Layer].x = X, rcVertex[rcPLIndex]->map[Layer].y = Y;
}
RAYAPI void rayTexCoord2f(RAYint Layer, RAYfloat X, RAYfloat Y)
{
    const RAYfloat* m = rayRenderMatrix[RAY_TEXTURE_MATRIX].m;
    
    rcVertex[rcPLIndex]->map[Layer].x = X*m[0] + Y*m[4] + m[12];
    rcVertex[rcPLIndex]->map[Layer].y = X*m[1] + Y*m[5] + m[13];
}

RAYAPI void rayColor3i(RAYint Red, RAYint Green, RAYint Blue)
{
    rcVertex[rcPLIndex]->clr.r = Red, rcVertex[rcPLIndex]->clr.g = Green, rcVertex[rcPLIndex]->clr.b = Blue, rcVertex[rcPLIndex]->clr.a = 255;
}
RAYAPI void rayColor4i(RAYint Red, RAYint Green, RAYint Blue, RAYint Alpha)
{
    rcVertex[rcPLIndex]->clr.r = Red, rcVertex[rcPLIndex]->clr.g = Green, rcVertex[rcPLIndex]->clr.b = Blue, rcVertex[rcPLIndex]->clr.a = Alpha;
}

RAYAPI void rayColor3f(RAYfloat Red, RAYfloat Green, RAYfloat Blue)
{
    rcVertex[rcPLIndex]->clr.r = RAYint(Red*255), rcVertex[rcPLIndex]->clr.g = RAYint(Green*255), rcVertex[rcPLIndex]->clr.b = RAYint(Blue*255), rcVertex[rcPLIndex]->clr.a = 255;
}
RAYAPI void rayColor4f(RAYfloat Red, RAYfloat Green, RAYfloat Blue, RAYfloat Alpha)
{
    rcVertex[rcPLIndex]->clr.r = RAYint(Red*255), rcVertex[rcPLIndex]->clr.g = RAYint(Green*255), rcVertex[rcPLIndex]->clr.b = RAYint(Blue*255), rcVertex[rcPLIndex]->clr.a = RAYint(Alpha*255);
}


/*
 * ===== Matrices =====
 */

RAYAPI void rayMatrixMode(RAYint Matrix)
{
    rcMatrixMode = Matrix;
}
RAYAPI void rayLoadIdentity()
{
    rayRenderMatrix[rcMatrixMode].m[0] = 1; rayRenderMatrix[rcMatrixMode].m[4] = 0; rayRenderMatrix[rcMatrixMode].m[ 8] = 0; rayRenderMatrix[rcMatrixMode].m[12] = 0;
    rayRenderMatrix[rcMatrixMode].m[1] = 0; rayRenderMatrix[rcMatrixMode].m[5] = 1; rayRenderMatrix[rcMatrixMode].m[ 9] = 0; rayRenderMatrix[rcMatrixMode].m[13] = 0;
    rayRenderMatrix[rcMatrixMode].m[2] = 0; rayRenderMatrix[rcMatrixMode].m[6] = 0; rayRenderMatrix[rcMatrixMode].m[10] = 1; rayRenderMatrix[rcMatrixMode].m[14] = 0;
    rayRenderMatrix[rcMatrixMode].m[3] = 0; rayRenderMatrix[rcMatrixMode].m[7] = 0; rayRenderMatrix[rcMatrixMode].m[11] = 0; rayRenderMatrix[rcMatrixMode].m[15] = 1;
}
RAYAPI void rayLoadMatrix(const RAYfloat* Matrix)
{
    rayRenderMatrix[rcMatrixMode].m[0] = Matrix[0]; rayRenderMatrix[rcMatrixMode].m[4] = Matrix[4]; rayRenderMatrix[rcMatrixMode].m[ 8] = Matrix[ 8]; rayRenderMatrix[rcMatrixMode].m[12] = Matrix[12];
    rayRenderMatrix[rcMatrixMode].m[1] = Matrix[1]; rayRenderMatrix[rcMatrixMode].m[5] = Matrix[5]; rayRenderMatrix[rcMatrixMode].m[ 9] = Matrix[ 9]; rayRenderMatrix[rcMatrixMode].m[13] = Matrix[13];
    rayRenderMatrix[rcMatrixMode].m[2] = Matrix[2]; rayRenderMatrix[rcMatrixMode].m[6] = Matrix[6]; rayRenderMatrix[rcMatrixMode].m[10] = Matrix[10]; rayRenderMatrix[rcMatrixMode].m[14] = Matrix[14];
    rayRenderMatrix[rcMatrixMode].m[3] = Matrix[3]; rayRenderMatrix[rcMatrixMode].m[7] = Matrix[7]; rayRenderMatrix[rcMatrixMode].m[11] = Matrix[11]; rayRenderMatrix[rcMatrixMode].m[15] = Matrix[15];
}
RAYAPI void rayGetMatrix(RAYfloat* Matrix)
{
    Matrix[0] = rayRenderMatrix[rcMatrixMode].m[0]; Matrix[4] = rayRenderMatrix[rcMatrixMode].m[4]; Matrix[ 8] = rayRenderMatrix[rcMatrixMode].m[ 8]; Matrix[12] = rayRenderMatrix[rcMatrixMode].m[12];
    Matrix[1] = rayRenderMatrix[rcMatrixMode].m[1]; Matrix[5] = rayRenderMatrix[rcMatrixMode].m[5]; Matrix[ 9] = rayRenderMatrix[rcMatrixMode].m[ 9]; Matrix[13] = rayRenderMatrix[rcMatrixMode].m[13];
    Matrix[2] = rayRenderMatrix[rcMatrixMode].m[2]; Matrix[6] = rayRenderMatrix[rcMatrixMode].m[6]; Matrix[10] = rayRenderMatrix[rcMatrixMode].m[10]; Matrix[14] = rayRenderMatrix[rcMatrixMode].m[14];
    Matrix[3] = rayRenderMatrix[rcMatrixMode].m[3]; Matrix[7] = rayRenderMatrix[rcMatrixMode].m[7]; Matrix[11] = rayRenderMatrix[rcMatrixMode].m[11]; Matrix[15] = rayRenderMatrix[rcMatrixMode].m[15];
}

RAYAPI void rayUpdateNormalMatrix()
{
    #define src     rayRenderMatrix[RAY_MODELVIEW_MATRIX].m
    #define dest    rayRenderMatrix[RAY_NORMAL_MATRIX].m
    
    dest[0] = src[0]; dest[4] = src[4]; dest[ 8] = src[ 8]; dest[12] = 0.0f;
    dest[1] = src[1]; dest[5] = src[5]; dest[ 9] = src[ 9]; dest[13] = 0.0f;
    dest[2] = src[2]; dest[6] = src[6]; dest[10] = src[10]; dest[14] = 0.0f;
    dest[3] = src[3]; dest[7] = src[7]; dest[11] = src[11]; dest[15] = src[15];
    
    #undef src
    #undef dest
}


/*
 * ===== Device context =====
 */

RAYAPI void rayCreateDevice(RAYint ScreenWidth, RAYint ScreenHeight)
{
    // General settings
    rayScreenWidth      = ScreenWidth;
    rayScreenHeight     = ScreenHeight;
    
    rayDepthBufferSize  = rayScreenWidth * rayScreenHeight;
    rayPixelBufferSize  = rayDepthBufferSize << 2;
    
    // Allocate the screen buffers
    rayPixelBuffer = (RAYpixel_t*)calloc(rayPixelBufferSize, sizeof(RAYpixel_t));
    memset(rayPixelBuffer, 0, sizeof(RAYpixel_t)*rayPixelBufferSize);
    
    rayDepthBuffer = (RAYfloat*)calloc(rayDepthBufferSize, sizeof(RAYfloat));
    memset(rayDepthBuffer, 0, sizeof(RAYfloat)*rayDepthBufferSize);
    
    // Allocate the vertices for the render (pixel) pipeline
    for (RAYint i = 0; i < RAY_RENDER_PIPELINE_SIZE; ++i)
    {
        // Vertices for the render pipeline
        rcVertex[i] = (SRayVertex*)malloc(sizeof(SRayVertex));
        memset(rcVertex[i], 0, sizeof(SRayVertex));
        
        // Vertices for memorization (temporary memories)
        rcMemVertex[i] = (SRayVertex*)malloc(sizeof(SRayVertex));
        memset(rcMemVertex[i], 0, sizeof(SRayVertex));
    }
    
    // Allocate the lights for the render (vertex) pipeline
    for (RAYint i = 0; i < RAY_MAX_LIGHT_COUNT; ++i)
    {
        rcLight[i] = (SRayLight*)malloc(sizeof(SRayLight));
        memset(rcLight[i], 0, sizeof(SRayLight));
    }
    
    // Default settings for each texture layer
    for (RAYint i = 0; i < RAY_TEXTURE_LAYER_SIZE; ++i)
    {
        rayMappingGenType[i].genu = RAY_OBJECT_LINEAR;
        rayMappingGenType[i].genv = RAY_OBJECT_LINEAR;
    }
    
    // Default settings
    memset(&rayBgClearColor.r, 0, sizeof(RAYint)*4);
    
    // Default render states
    rayRenderMode(RAY_RENDER_FRONT);
    rayBlendFunc(RAY_BLEND_NORMAL);
    rayViewport(0, 0, rayScreenWidth, rayScreenHeight);
    rayPerspective(1.0f, 100.0f, rayScreenWidth/2);
    
    // Default matrices states
    for (RAYint i = RAY_MODELVIEW_MATRIX; i <= RAY_TEXTURE_MATRIX; ++i)
    {
        rayMatrixMode(RAY_MODELVIEW_MATRIX);
        rayLoadIdentity();
    }
    
    // Create the device context
    glGenTextures(1, &rayDCTexID);
    glBindTexture(GL_TEXTURE_2D, rayDCTexID);
    
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    
    glTexImage2D(GL_TEXTURE_2D, 0, 4, rayScreenWidth, rayScreenHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, rayPixelBuffer);
    glEnable(GL_TEXTURE_2D);
    
    glBindTexture(GL_TEXTURE_2D, rayDCTexID);
}

RAYAPI void rayDeleteDevice()
{
    
    // Delete the device context
    if (glIsTexture(rayDCTexID))
    {
        glBindTexture(GL_TEXTURE_2D, 0);
        glDeleteTextures(GL_TEXTURE_2D, &rayDCTexID);
    }
    
    // Delete the screen buffers
    free(rayPixelBuffer);
    free(rayDepthBuffer);
    
    // Delete the vertices of the render pipeline
    for (RAYint i = 0; i < RAY_RENDER_PIPELINE_SIZE; ++i)
    {
        free(rcVertex[i]);
        free(rcMemVertex[i]);
    }
    
}

RAYAPI void rayClearColor(RAYint Red, RAYint Green, RAYint Blue)
{
    rayBgClearColor.r = Red;
    rayBgClearColor.g = Green;
    rayBgClearColor.b = Blue;
}

RAYAPI void rayClear(RAYbyte Target)
{
    // Temporary counter
    register RAYint i;
    
    // Clear the pixel buffer
    if (Target & RAY_CLEAR_COLOR)
    {
        i = 0;
        while (i < rayPixelBufferSize)
        {
            rayPixelBuffer[i+0] = rayBgClearColor.r;
            rayPixelBuffer[i+1] = rayBgClearColor.g;
            rayPixelBuffer[i+2] = rayBgClearColor.b;
            rayPixelBuffer[i+3] = 255;
            i += 4;
        }
    }
    
    // Clear the depth buffer
    if (Target & RAY_CLEAR_DEPTH)
    {
        i = 0;
        while (i < rayDepthBufferSize)
        {
            rayDepthBuffer[i] = 0;
            ++i;
        }
    }
}

RAYAPI void rayFlipBuffers()
{
    // Put onto the screen
    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, rayScreenWidth, rayScreenHeight, GL_RGBA, GL_UNSIGNED_BYTE, rayPixelBuffer);
    
    RAYfloat pos1x = 0, pos1y = 0;
    RAYfloat pos2x = rayScreenWidth, pos2y = rayScreenHeight;
    
    rayProject2Dto3D(pos1x, pos1y);
    rayProject2Dto3D(pos2x, pos2y);
    
    // Draw the result
    glBegin(GL_TRIANGLE_STRIP);
    {
        glTexCoord2i(0, 0);
        glVertex2f(pos1x, pos1y);
        
        glTexCoord2i(0, 1);
        glVertex2f(pos1x, pos2y);
        
        glTexCoord2i(1, 0);
        glVertex2f(pos2x, pos1y);
        
        glTexCoord2i(1, 1);
        glVertex2f(pos2x, pos2y);
    }
    glEnd();
}


/*
 * ===== Render pipeline =====
 */

RAYAPI void rayWrite2DPixel(RAYint x, RAYint y, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a)
{
    // Check if the pixel is inside the screen
    if (!rayIsPixelIndside2D(x, y))
        return;
    
    // Get the pixel buffer index
    RAYint i = ( y * rayScreenWidth + x ) << 2;
    
    // Plot the pixel
    if (a == 255)
    {
        rayPixelBuffer[i+0] = r;
        rayPixelBuffer[i+1] = g;
        rayPixelBuffer[i+2] = b;
    }
    else
    {
        rayPixelBuffer[i+0] = r * a / 255   +   rayPixelBuffer[i+0] * ( 255 - a ) / 255;
        rayPixelBuffer[i+1] = g * a / 255   +   rayPixelBuffer[i+1] * ( 255 - a ) / 255;
        rayPixelBuffer[i+2] = b * a / 255   +   rayPixelBuffer[i+2] * ( 255 - a ) / 255;
    }
}
RAYAPI void rayWrite3DPixel(RAYint x, RAYint y, RAYfloat z, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a)
{
    // Get the pixel buffer index
    RAYint i = y * rayScreenWidth + x;
    
    // Check if the pixel is inside the screen
    if (!rayIsPixelIndside3D(x, y, z, i))
        return;
    
    // Plot the depth
    rayDepthBuffer[i] = z;
    
    // Resize the pixel buffer index
    i <<= 2;
    
    // Plot the pixel
    if (a == 255)
    {
        rayPixelBuffer[i+0] = r;
        rayPixelBuffer[i+1] = g;
        rayPixelBuffer[i+2] = b;
    }
    else
    {
        rayPixelBuffer[i+0] = r * a / 255   +   rayPixelBuffer[i+0] * ( 255 - a ) / 255;
        rayPixelBuffer[i+1] = g * a / 255   +   rayPixelBuffer[i+1] * ( 255 - a ) / 255;
        rayPixelBuffer[i+2] = b * a / 255   +   rayPixelBuffer[i+2] * ( 255 - a ) / 255;
    }
}

RAYAPI void rayWrite2DPixelFast(RAYint x, RAYint y, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a)
{
    RAYint i = ( y * rayScreenWidth + x ) << 2;
    
    rayPixelBuffer[i+0] = r;
    rayPixelBuffer[i+1] = g;
    rayPixelBuffer[i+2] = b;
}
RAYAPI void rayWrite3DPixelFast(RAYint x, RAYint y, RAYfloat z, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a)
{
    RAYint i = y * rayScreenWidth + x;
    
    rayDepthBuffer[i] = z;
    i <<= 2;
    
    rayPixelBuffer[i+0] = r;
    rayPixelBuffer[i+1] = g;
    rayPixelBuffer[i+2] = b;
}

RAYAPI void rayRead2DPixel(RAYint x, RAYint y, RAYcolor_t &r, RAYcolor_t &g, RAYcolor_t &b, RAYcolor_t &a)
{
    RAYint i = ( y * rayScreenWidth + x ) << 2;
    
    r = rayPixelBuffer[i+0];
    g = rayPixelBuffer[i+1];
    r = rayPixelBuffer[i+2];
}
RAYAPI void rayRead3DPixel(RAYint x, RAYint y, RAYfloat &z, RAYcolor_t &r, RAYcolor_t &g, RAYcolor_t &b, RAYcolor_t &a)
{
    RAYint i = y * rayScreenWidth + x;
    
    z = rayDepthBuffer[i];
    i <<= 2;
    
    r = rayPixelBuffer[i+0];
    g = rayPixelBuffer[i+1];
    r = rayPixelBuffer[i+2];
}










// ================================================================================
