/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for the "Ray" Software renderer
 *
 */

#ifndef __SP_RAYSOFTWARERENDERER_H__
#define __SP_RAYSOFTWARERENDERER_H__


#include "spRayCore.h"


/*
 * ===== Macros =====
 */

static const RAYint RAY_TRUE   = 1;
static const RAYint RAY_FALSE  = 0;


// ERayErrorTypes
#define RAY_ERROR_NONE          0
#define RAY_ERROR_EMPTYIMAGE    1

// ERaySatusTypes
#define RAY_DEPTH_TEST          0
#define RAY_ALPHA_TEST          1
#define RAY_SCISSOR_TEST        2
#define RAY_BLEND_TEST          3

#define RAY_NORMALIZE           4
#define RAY_LIGHTING            5
#define RAY_BLEND               6
#define RAY_TEXGEN              7

#define RAY_COLOR_SMOOTH        8

#define RAY_TEXTURE_SMOOTH      9
#define RAY_TEXTURE_MIPMAPS     10

#define RAY_LIGHT0              11

#define RAY_STATUS_COUNT        12

// ERayClearTypes
#define RAY_CLEAR_COLOR         0x01
#define RAY_CLEAR_DEPTH         0x02

// ERayRenderModes
#define RAY_RENDER_FRONT        0
#define RAY_RENDER_BACK         1
#define RAY_RENDER_BOTH         2

// ERayBlendModes
#define RAY_BLEND_NORMAL        0
#define RAY_BLEND_BRIGHT        1
#define RAY_BLEND_DARK          2

// ERayRenderMatrices
#define RAY_MODELVIEW_MATRIX    0
#define RAY_PROJECTION_MATRIX   1
#define RAY_NORMAL_MATRIX       2
#define RAY_TEXTURE_MATRIX      3
#define RAY_VIEWPORT_MATRIX     4

#define RAY_COUNT_OF_MATRICES   5

// ERayImageDataTypes
#define RAY_INTEGER             0
#define RAY_UNSIGNED_BYTE       1
#define RAY_FLOAT               2

// ERayImageDataComponents
#define RAY_DEPTH               0
#define RAY_GRAY                1
#define RAY_GRAYALPHA           2
#define RAY_RGB                 3
#define RAY_RGBA                4

// ERayTexCoordTypes
#define RAY_TEXTURE_GEN_U       0
#define RAY_TEXTURE_GEN_V       1

// ERayMappingGenTypes
#define RAY_OBJECT_LINEAR       0
#define RAY_EYE_LINEAR          1
#define RAY_SPHERE_MAP          2
//#define RAY_NORMAL_MAP          3
//#define RAY_REFLECTION_MAP      4

// ERayStringNames
#define RAY_VENDOR              0
#define RAY_RENDERER            1
#define RAY_VERSION             2

// ERayRectangleTypes
#define RAY_RECT_LINEAR         0
#define RAY_RECT_SMOOTH         1
#define RAY_RECT_TEXTURELINEAR  2
#define RAY_RECT_TEXTURESMOOTH  3

// ERayInformationTypes
#define RAY_SCISSOR             0
#define RAY_VIEWPORT            1
#define RAY_PERSPECTIVE         2

// ERayLightTypes
#define RAY_LIGHT_DIRECTIONAL   0
#define RAY_LIGHT_POINT         1
#define RAY_LIGHT_SPOT          2

// ERayLightAttributes
#define RAY_LIGHT_TYPE          0
#define RAY_DIFFUSE             1
#define RAY_AMBIENT             2
#define RAY_TRANSFORMATION      3
#define RAY_DIRECTION           4


/*
 * ===== Static functions =====
 */

RAYAPI const RAYbyte* rayGetString(RAYenum Name);
RAYAPI RAYenum rayGetError();

RAYAPI void rayGetValue(RAYenum Type, RAYvoid* data);

RAYAPI void rayEnable(RAYenum Status);
RAYAPI void rayDisable(RAYenum Status);

RAYAPI void raySetStatus(RAYenum Status, RAYint Enable);
RAYAPI RAYint rayGetStatus(RAYenum Status);

RAYAPI void rayScissor(RAYint X, RAYint Y, RAYint Width, RAYint Height);
RAYAPI void rayViewport(RAYint X, RAYint Y, RAYint Width, RAYint Height);
RAYAPI void rayPerspective(RAYfloat Near, RAYfloat Far, RAYfloat FieldOfView);

RAYAPI void rayRenderMode(RAYenum Mode);
RAYAPI void rayBlendFunc(RAYenum Mode);

RAYAPI void rayLighti(RAYenum Index, RAYenum Type, const RAYint Param);
RAYAPI void rayLightfv(RAYenum Index, RAYenum Type, const RAYfloat* Params);


RAYAPI void raySetMaxTextureLayer(RAYint MaxLayer);
RAYAPI void rayUseTextureLayer(RAYint Layer);

RAYAPI void rayGenTexture(RAYuint &ID);
RAYAPI void rayDeleteTexture(RAYuint &ID);

RAYAPI void rayBindTexture(RAYuint ID);
RAYAPI void rayUnbindTexture();
RAYAPI RAYuint rayGetCurTexture();

RAYAPI void rayTexGen(RAYint TexCoordType, RAYint Type);


RAYAPI void rayTexSubImage2D(
    RAYuint ID, RAYuint Level, RAYint Format, RAYenum Type, const RAYvoid* ImageData
);
RAYAPI void rayCopyTexSubImage2D(
    RAYuint ID, RAYuint Level, RAYint OffsetX, RAYint OffsetY, RAYint X, RAYint Y, RAYenum Components
);

RAYAPI void rayTexImage2D(
    RAYuint ID, RAYuint Level, RAYint Width, RAYint Height, RAYint Format, RAYenum Type, const RAYvoid* ImageData
);
RAYAPI void rayCopyTexImage2D(
    RAYuint ID, RAYuint Level, RAYint OffsetX, RAYint OffsetY, RAYint X, RAYint Y, RAYint Width, RAYint Height
);

RAYAPI void rayGetImage2D(RAYuint ID, RAYuint Level, RAYint Format, RAYenum Type, RAYvoid* ImageData);

RAYAPI void rayScaleImage2D(
    RAYvoid* &ImageData, RAYint Width, RAYint Height, RAYint NewWidth, RAYint NewHeight, RAYint Format, RAYenum Type
);


RAYAPI void rayVertex2i(RAYint X, RAYint Y);
RAYAPI void rayVertex3i(RAYint X, RAYint Y, RAYint Z);
RAYAPI void rayVertex4i(RAYint X, RAYint Y, RAYint Z, RAYint W);

RAYAPI void rayVertex2f(RAYfloat X, RAYfloat Y);
RAYAPI void rayVertex3f(RAYfloat X, RAYfloat Y, RAYfloat Z);
RAYAPI void rayVertex4f(RAYfloat X, RAYfloat Y, RAYfloat Z, RAYfloat W);

RAYAPI void rayNormal3i(RAYint X, RAYint Y, RAYint Z);
RAYAPI void rayNormal3f(RAYfloat X, RAYfloat Y, RAYfloat Z);

RAYAPI void rayTexCoord2i(RAYint Layer, RAYint X, RAYint Y);
RAYAPI void rayTexCoord2f(RAYint Layer, RAYfloat X, RAYfloat Y);

RAYAPI void rayColor3i(RAYint Red, RAYint Green, RAYint Blue);
RAYAPI void rayColor4i(RAYint Red, RAYint Green, RAYint Blue, RAYint Alpha);

RAYAPI void rayColor3f(RAYfloat Red, RAYfloat Green, RAYfloat Blue);
RAYAPI void rayColor4f(RAYfloat Red, RAYfloat Green, RAYfloat Blue, RAYfloat Alpha);


RAYAPI void rayMatrixMode(RAYint Matrix);

RAYAPI void rayLoadIdentity();
RAYAPI void rayLoadMatrix(const RAYfloat* Matrix);

RAYAPI void rayGetMatrix(RAYfloat* Matrix);

RAYAPI void rayUpdateNormalMatrix();


RAYAPI void rayCreateDevice(RAYint ScreenWidth, RAYint ScreenHeight);
RAYAPI void rayDeleteDevice();

RAYAPI void rayClearColor(RAYint Red, RAYint Green, RAYint Blue);

RAYAPI void rayClear(RAYbyte Target);
RAYAPI void rayFlipBuffers();


RAYAPI void rayWrite2DPixel(RAYint x, RAYint y, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a);
RAYAPI void rayWrite3DPixel(RAYint x, RAYint y, RAYfloat z, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a);

RAYAPI void rayWrite2DPixelFast(RAYint x, RAYint y, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a);
RAYAPI void rayWrite3DPixelFast(RAYint x, RAYint y, RAYfloat z, RAYcolor_t r, RAYcolor_t g, RAYcolor_t b, RAYcolor_t a);

RAYAPI void rayRead2DPixel(RAYint x, RAYint y, RAYcolor_t &r, RAYcolor_t &g, RAYcolor_t &b, RAYcolor_t &a);
RAYAPI void rayRead3DPixel(RAYint x, RAYint y, RAYfloat &z, RAYcolor_t &r, RAYcolor_t &g, RAYcolor_t &b, RAYcolor_t &a);



#endif







// ================================================================================
