/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * "Ray" textured geometry
 *
 */

#include "spRayTexturedGeometry.h"
#include "spRayStandardTypes.h"
#include "spRaySoftwareRenderer.h"

#include "spInputOutputStandard.hpp"


/*
 * ========== Internal members ==========
 */

RAYint optxStart, optxEnd;
RAYint optyStart, optyEnd;

RAYint xStart, xEnd;
RAYint yStart, yEnd, yMiddle;

RAYint yMiddleStart;
RAYint yEndMiddle;
RAYint yEndStart;

RAYfloat factor;

RAYint tmps32;
RAYfloat tmpf32;

RAYint v0 = 0, v1 = 1, v2 = 2;

SRayPolygonSide lside, rside, step;


/*
 * ========== Internal macros ==========
 */

#define uvm map[rcTextureLayer]

#define mcr_rayStandardMembers  \
    /* Temporary variables */   \
    RAYint x, y, i;             \
    RAYfloat z;

#define mcr_rayComputeScanlineDimension                                                                                         \
    /* Determine the line range start */                                                                                        \
    if (y < yMiddle)                                                                                                            \
        xStart = (RAYint)( rcVertex[0]->pos.x + ( rcVertex[1]->pos.x - rcVertex[0]->pos.x ) * ( y - yStart ) / yMiddleStart );  \
    else if (y > yMiddle)                                                                                                       \
        xStart = (RAYint)( rcVertex[1]->pos.x + ( rcVertex[2]->pos.x - rcVertex[1]->pos.x ) * ( y - yMiddle ) / yEndMiddle );   \
    else                                                                                                                        \
        xStart = (RAYint)( rcVertex[1]->pos.x );                                                                                \
                                                                                                                                \
    /* Determine the line range end */                                                                                          \
    xEnd = (RAYint)( rcVertex[0]->pos.x + ( rcVertex[2]->pos.x - rcVertex[0]->pos.x ) * ( y - yStart ) / yEndStart );

#define mcr_rayClampScanlineDimension                                       \
    /* Determine the optimized x start- & end location */                   \
    optxStart   = (xStart < rayPixelRect[0] ? rayPixelRect[0] : xStart);    \
    optxEnd     = (xEnd >= rayPixelRect[2] ? rayPixelRect[2] : xEnd);

#define mcr_raySwapSingleInteger(a, b)  \
    tmps32 = a; a = b; b = tmps32;

#define mcr_raySwapSingleFloat(a, b)    \
    tmpf32 = a; a = b; b = tmpf32;

#define mcr_rayPointInside(p)                                               \
    p.x < 0 || p.x >= rayScreenWidth || p.y < 0 || p.y > rayScreenHeight

#define mcr_rayCheckPlotDepth               \
    if (!rayCheckPlotDepth(i, z)) continue;

#define mcr_rayComputePolygonSides_Depth(a, b)                                                  \
{                                                                                               \
    factor  = (RAYfloat)( y - yStart ) / ( rcVertex[2]->pos.y - rcVertex[0]->pos.y );           \
    a.z     = rcVertex[0]->pos.z + ( rcVertex[2]->pos.z - rcVertex[0]->pos.z ) * factor;        \
                                                                                                \
    if (y < yMiddle)                                                                            \
    {                                                                                           \
        factor  = (RAYfloat)( y - yStart ) / ( rcVertex[1]->pos.y - rcVertex[0]->pos.y );       \
        b.z     = rcVertex[0]->pos.z + ( rcVertex[1]->pos.z - rcVertex[0]->pos.z ) * factor;    \
    }                                                                                           \
    else                                                                                        \
    {                                                                                           \
        factor  = (RAYfloat)( y - yMiddle ) / ( rcVertex[2]->pos.y - rcVertex[1]->pos.y );      \
        b.z     = rcVertex[1]->pos.z + ( rcVertex[2]->pos.z - rcVertex[1]->pos.z ) * factor;    \
    }                                                                                           \
}

#define mcr_rayComputePolygonSides_Depth_Color(a, b)                                            \
{                                                                                               \
    factor  = (RAYfloat)( y - yStart ) / ( rcVertex[2]->pos.y - rcVertex[0]->pos.y );           \
                                                                                                \
    a.z     = rcVertex[0]->pos.z + ( rcVertex[2]->pos.z - rcVertex[0]->pos.z ) * factor;        \
    rayInterpolateColor(a.clr, rcVertex[0]->clr, rcVertex[2]->clr, factor);                     \
                                                                                                \
    if (y < yMiddle)                                                                            \
    {                                                                                           \
        factor  = (RAYfloat)( y - yStart ) / ( rcVertex[1]->pos.y - rcVertex[0]->pos.y );       \
                                                                                                \
        b.z     = rcVertex[0]->pos.z + ( rcVertex[1]->pos.z - rcVertex[0]->pos.z ) * factor;    \
        rayInterpolateColor(b.clr, rcVertex[0]->clr, rcVertex[1]->clr, factor);                 \
    }                                                                                           \
    else                                                                                        \
    {                                                                                           \
        factor  = (RAYfloat)( y - yMiddle ) / ( rcVertex[2]->pos.y - rcVertex[1]->pos.y );      \
                                                                                                \
        b.z     = rcVertex[1]->pos.z + ( rcVertex[2]->pos.z - rcVertex[1]->pos.z ) * factor;    \
        rayInterpolateColor(b.clr, rcVertex[1]->clr, rcVertex[2]->clr, factor);                 \
    }                                                                                           \
}

#define mcr_rayComputePolygonSides_Depth_TexCoords(a, b)                                        \
{                                                                                               \
    factor  = (RAYfloat)( y - yStart ) / ( rcVertex[2]->pos.y - rcVertex[0]->pos.y );           \
                                                                                                \
    a.z     = rcVertex[0]->pos.z + ( rcVertex[2]->pos.z - rcVertex[0]->pos.z ) * factor;        \
    a.u     = rcVertex[0]->uvm.x + ( rcVertex[2]->uvm.x - rcVertex[0]->uvm.x ) * factor;        \
    a.v     = rcVertex[0]->uvm.y + ( rcVertex[2]->uvm.y - rcVertex[0]->uvm.y ) * factor;        \
                                                                                                \
    if (y < yMiddle)                                                                            \
    {                                                                                           \
        factor  = (RAYfloat)( y - yStart ) / ( rcVertex[1]->pos.y - rcVertex[0]->pos.y );       \
                                                                                                \
        b.z     = rcVertex[0]->pos.z + ( rcVertex[1]->pos.z - rcVertex[0]->pos.z ) * factor;    \
        b.u     = rcVertex[0]->uvm.x + ( rcVertex[1]->uvm.x - rcVertex[0]->uvm.x ) * factor;    \
        b.v     = rcVertex[0]->uvm.y + ( rcVertex[1]->uvm.y - rcVertex[0]->uvm.y ) * factor;    \
    }                                                                                           \
    else                                                                                        \
    {                                                                                           \
        factor  = (RAYfloat)( y - yMiddle ) / ( rcVertex[2]->pos.y - rcVertex[1]->pos.y );      \
                                                                                                \
        b.z     = rcVertex[1]->pos.z + ( rcVertex[2]->pos.z - rcVertex[1]->pos.z ) * factor;    \
        b.u     = rcVertex[1]->uvm.x + ( rcVertex[2]->uvm.x - rcVertex[1]->uvm.x ) * factor;    \
        b.v     = rcVertex[1]->uvm.y + ( rcVertex[2]->uvm.y - rcVertex[1]->uvm.y ) * factor;    \
    }                                                                                           \
}

#define mcr_rayComputePolygonSides_Depth_TexCoords_Color(a, b)                                  \
{                                                                                               \
    factor  = (RAYfloat)( y - yStart ) / ( rcVertex[2]->pos.y - rcVertex[0]->pos.y );           \
                                                                                                \
    a.z     = rcVertex[0]->pos.z + ( rcVertex[2]->pos.z - rcVertex[0]->pos.z ) * factor;        \
    a.u     = rcVertex[0]->uvm.x + ( rcVertex[2]->uvm.x - rcVertex[0]->uvm.x ) * factor;        \
    a.v     = rcVertex[0]->uvm.y + ( rcVertex[2]->uvm.y - rcVertex[0]->uvm.y ) * factor;        \
    rayInterpolateColor(a.clr, rcVertex[0]->clr, rcVertex[2]->clr, factor);                     \
                                                                                                \
    if (y < yMiddle)                                                                            \
    {                                                                                           \
        factor  = (RAYfloat)( y - yStart ) / ( rcVertex[1]->pos.y - rcVertex[0]->pos.y );       \
                                                                                                \
        b.z     = rcVertex[0]->pos.z + ( rcVertex[1]->pos.z - rcVertex[0]->pos.z ) * factor;    \
        b.u     = rcVertex[0]->uvm.x + ( rcVertex[1]->uvm.x - rcVertex[0]->uvm.x ) * factor;    \
        b.v     = rcVertex[0]->uvm.y + ( rcVertex[1]->uvm.y - rcVertex[0]->uvm.y ) * factor;    \
        rayInterpolateColor(b.clr, rcVertex[0]->clr, rcVertex[1]->clr, factor);                 \
    }                                                                                           \
    else                                                                                        \
    {                                                                                           \
        factor  = (RAYfloat)( y - yMiddle ) / ( rcVertex[2]->pos.y - rcVertex[1]->pos.y );      \
                                                                                                \
        b.z     = rcVertex[1]->pos.z + ( rcVertex[2]->pos.z - rcVertex[1]->pos.z ) * factor;    \
        b.u     = rcVertex[1]->uvm.x + ( rcVertex[2]->uvm.x - rcVertex[1]->uvm.x ) * factor;    \
        b.v     = rcVertex[1]->uvm.y + ( rcVertex[2]->uvm.y - rcVertex[1]->uvm.y ) * factor;    \
        rayInterpolateColor(b.clr, rcVertex[1]->clr, rcVertex[2]->clr, factor);                 \
    }                                                                                           \
}

#define mcr_rayUpdateMainPart(mcrPolySideFunc)  \
    /* Compute the scnaline dimension */        \
    mcr_rayComputeScanlineDimension;            \
                                                \
    /* Repair the scanline dimension */         \
    if (xStart > xEnd)                          \
    {                                           \
        mcr_raySwapSingleInteger(xStart, xEnd); \
                                                \
        mcrPolySideFunc(lside, rside);          \
    }                                           \
    else                                        \
        mcrPolySideFunc(rside, lside);          \
                                                \
    /* Clamp the scanline dimension */          \
    mcr_rayClampScanlineDimension;

#define mcr_rayResizeTextureCoordinate(v)                                           \
    rcVertex[v]->uvm.x *= rcTexture[rcTextureLayer]->width   * rcVertex[v]->pos.z;  \
    rcVertex[v]->uvm.y *= rcTexture[rcTextureLayer]->height  * rcVertex[v]->pos.z;  \

#define mcr_rayResizeTextureCoordinates \
    mcr_rayResizeTextureCoordinate(0);  \
    mcr_rayResizeTextureCoordinate(1);  \
    mcr_rayResizeTextureCoordinate(2);  \
    mcr_rayResizeTextureCoordinate(3);

#define mcr_raySetColorIntensity(c, i) \
    c.r = c.r * i / 255; c.g = c.g * i / 255; c.b = c.b * i / 255;

#define mcr_rayLineBaseAlgorithm                                                \
    RAYint i, x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;     \
    RAYfloat z = rcVertex[v0]->pos.z;                                           \
                                                                                \
    RAYint x1 = (RAYint)rcVertex[v0]->pos.x, y1 = (RAYint)rcVertex[v0]->pos.y;  \
    RAYint x2 = (RAYint)rcVertex[v1]->pos.x, y2 = (RAYint)rcVertex[v1]->pos.y;  \
                                                                                \
    dx = x2 - x1;                                                               \
    dy = y2 - y1;                                                               \
                                                                                \
    incx = (dx > 0) ? 1 : (dx < 0) ? -1 : 0;                                    \
    incy = (dy > 0) ? 1 : (dy < 0) ? -1 : 0;                                    \
                                                                                \
    if (dx < 0) dx = -dx;                                                       \
    if (dy < 0) dy = -dy;                                                       \
                                                                                \
    if (dx > dy)                                                                \
    {                                                                           \
        pdx = incx; pdy = 0;                                                    \
        ddx = incx; ddy = incy;                                                 \
        es  = dy;   el  = dx;                                                   \
    }                                                                           \
    else                                                                        \
    {                                                                           \
        pdx = 0;    pdy = incy;                                                 \
        ddx = incx; ddy = incy;                                                 \
        es  = dx;   el  = dy;                                                   \
    }                                                                           \
                                                                                \
    if (el == 0)                                                                \
        return;                                                                 \
                                                                                \
    x   = x1;                                                                   \
    y   = y1;                                                                   \
    err = el/2;

#define mcr_rayLineInterpolate  \
    err -= es;                  \
    if (err < 0)                \
    {                           \
        err += el;              \
        x += ddx;               \
        y += ddy;               \
    }                           \
    else                        \
    {                           \
        x += pdx;               \
        y += pdy;               \
    }

#define mcr_rayLinePlot                     \
    rayPlotLocated(                         \
        x, y, z,                            \
        clr.r * rcVertex[v0]->clr.r / 255,  \
        clr.g * rcVertex[v0]->clr.g / 255,  \
        clr.b * rcVertex[v0]->clr.b / 255,  \
        clr.a * rcVertex[v0]->clr.a / 255   \
    );

#define mcr_rayClampRectangle                                                           \
    /* Check if the image is inside the screen */                                       \
    if (right < 0 || left >= rayScreenWidth || bottom < 0 || top >= rayScreenHeight)    \
        return;                                                                         \
                                                                                        \
    /* Clamp the rectangle */                                                           \
    rayClamp(left, 0, rayScreenWidth);                                                  \
    rayClamp(right, 0, rayScreenWidth);                                                 \
    rayClamp(top, 0, rayScreenHeight);                                                  \
    rayClamp(bottom, 0, rayScreenHeight);                                               \
                                                                                        \
    /* Get the start pixel buffer offset */                                             \
    i = top * rayScreenWidth + left;


/*
 * ========== Point functions ==========
 */

inline void rayRenderPoint_Color()
{
    
    // Plot the pixel
    rayPlotLocated(
        (RAYint)rcVertex[0]->pos.x, (RAYint)rcVertex[0]->pos.y, rcVertex[0]->pos.z,
        rcVertex[0]->clr.r, rcVertex[0]->clr.g, rcVertex[0]->clr.b, rcVertex[0]->clr.a
    );
    
}

inline void rayRenderPoint_Texture()
{
    
    // Temporary varibales
    SRayColor4i clr;
    SRayVector2f invmap;
    
    // Resize the texture coordinates
    rcVertex[0]->uvm.x *= rcTexture[rcTextureLayer]->width   * rcVertex[0]->pos.z;
    rcVertex[0]->uvm.y *= rcTexture[rcTextureLayer]->height  * rcVertex[0]->pos.z;
    
    invmap.x = rcVertex[0]->uvm.x / rcVertex[0]->pos.z;
    invmap.y = rcVertex[0]->uvm.y / rcVertex[0]->pos.z;
    
    // Get the texels color
    rayGetTexturePixelLinear(clr, invmap);
    
    // Plot the pixel
    rayPlotLocated(
        (RAYint)rcVertex[0]->pos.x, (RAYint)rcVertex[0]->pos.y, rcVertex[0]->pos.z,
        clr.r * rcVertex[0]->clr.r / 255,
        clr.g * rcVertex[0]->clr.g / 255,
        clr.b * rcVertex[0]->clr.b / 255,
        clr.a * rcVertex[0]->clr.a / 255
    );
    
}



/*
 * ========== Line functions ==========
 */

inline void rayRenderLine_Solid_ColorLinear()
{
    
    mcr_rayLineBaseAlgorithm;
    
    // Compute the step
    step.z = ( rcVertex[v1]->pos.z - rcVertex[v0]->pos.z ) / el;
    
    // Plot the first pixel
    rayPlotLocated(
        x, y, z,
        rcVertex[v0]->clr.r, rcVertex[v0]->clr.g, rcVertex[v0]->clr.b, rcVertex[v0]->clr.a
    );
    
    // Loop for all pixels
    for (t = 0; t < el; ++t)
    {
        mcr_rayLineInterpolate;
        
        z += step.z;
        
        // Plot the pixel
        rayPlotLocated(
            x, y, z,
            rcVertex[v0]->clr.r, rcVertex[v0]->clr.g, rcVertex[v0]->clr.b, rcVertex[v0]->clr.a
        );
    }
    
}

inline void rayRenderLine_Solid_ColorSmooth()
{
    
}

inline void rayRenderLine_Texture_ColorLinear()
{
    
    mcr_rayLineBaseAlgorithm;
    
    SRayColor4i clr;
    SRayVector2f map, invmap;
    
    // Compute the step
    step.z  = ( rcVertex[v1]->pos.z - rcVertex[v0]->pos.z ) / el;
    step.u  = ( rcVertex[v1]->uvm.x - rcVertex[v0]->uvm.x ) / el;
    step.v  = ( rcVertex[v1]->uvm.y - rcVertex[v0]->uvm.y ) / el;
    
    map.x   = rcVertex[v0]->uvm.x;
    map.y   = rcVertex[v0]->uvm.y;
    
    // Plot the first pixel
    mcr_rayLinePlot;
    
    // Loop for all pixels
    for (t = 0; t < el; ++t)
    {
        mcr_rayLineInterpolate;
        
        // Interpolate
        z       += step.z;
        map.x   += step.u;
        map.y   += step.v;
        
        invmap.x = map.x / z;
        invmap.y = map.y / z;
        
        // Get the texels color
        rayGetTexturePixelLinear(clr, invmap);
        
        // Plot the pixel
        mcr_rayLinePlot;
    }
    
}

inline void rayRenderLine_Texture_ColorSmooth()
{
    
}


/*
 * ========== Rectangle functions ==========
 */

inline void rayRenderRectangle_ColorLinear()
{
    // Temporary variables
    RAYint x, y, i;
    
    RAYint left = (RAYint)rcVertex[0]->pos.x, top = (RAYint)rcVertex[0]->pos.y;
    RAYint right = (RAYint)rcVertex[1]->pos.x, bottom = (RAYint)rcVertex[1]->pos.y;
    
    mcr_rayClampRectangle;
    
    if (rcVertex[0]->clr.a == 255)
    {
        // Loop for all pixels to draw
        for (y = top; y < bottom; ++y)
        {
            // Draw the scanline
            for (x = left; x < right; ++x, ++i)
                rayPlotPixel(i, rcVertex[0]->clr.r, rcVertex[0]->clr.g, rcVertex[0]->clr.b);
            
            // Push the indices to the next step
            i += rayScreenWidth - (right - left);
        }
    }
    else
    {
        // Loop for all pixels to draw
        for (y = top; y < bottom; ++y)
        {
            // Draw the scanline
            for (x = left; x < right; ++x, ++i)
                rayPlotPixel(i, rcVertex[0]->clr.r, rcVertex[0]->clr.g, rcVertex[0]->clr.b, rcVertex[0]->clr.a);
            
            // Push the indices to the next step
            i += rayScreenWidth - (right - left);
        }
    }
}

inline void rayRenderRectangle_ColorSmooth()
{
    
}

inline void rayRenderRectangle_TextureSize_PixelLinear()
{
    
}

inline void rayRenderRectangle_TextureSize_PixelSmooth()
{
    
}


/*
 * ========== Triangle functions ==========
 */

/*
 * ===== Colored triangle =====
 */

inline void rayRenderTriangle_Solid_ColorLinear()
{
    
    // Temporary variables
    mcr_rayStandardMembers;
    
    // Loop for all lines
    for (y = optyStart; y < optyEnd; ++y)
    {
        // Update the main part
        mcr_rayUpdateMainPart(mcr_rayComputePolygonSides_Depth);
        
        // Clamp the scanline dimension
        mcr_rayClampScanlineDimension;
        
        // Compute the step for linear interpolation
        step.z = ( rside.z - lside.z ) / ( xEnd - xStart );
        
        // Set the linear interpolation to the left side values
        i = y * rayScreenWidth + optxStart;
        z = lside.z + step.z * ( optxStart - xStart );
        
        // Loop for all pixel in the current scan line
        for (x = optxStart; x < optxEnd; ++x, ++i)
        {
            z += step.z;
            
            mcr_rayCheckPlotDepth;
            
            rayPlotIndexed(i, z, rcVertex[2]->clr.r, rcVertex[2]->clr.g, rcVertex[2]->clr.b, rcVertex[2]->clr.a);
        }
    }
    
}

inline void rayRenderTriangle_Solid_ColorSmooth()
{
    
    // Temporary variables
    mcr_rayStandardMembers;
    
    RAYint tmpsize;
    
    SRayColor4f clr, stepclr;
    
    // Loop for all lines
    for (y = optyStart; y < optyEnd; ++y)
    {
        // Update the main part
        mcr_rayUpdateMainPart(mcr_rayComputePolygonSides_Depth_Color);
        
        // Check scanline length
        if (xEnd - xStart == 0)
            continue;
        
        // Clamp the scanline dimension
        mcr_rayClampScanlineDimension;
        
        // Compute the step for linear interpolation
        tmpsize     = xEnd - xStart;
        step.z      = ( rside.z - lside.z ) / tmpsize;
        stepclr.r   = (RAYfloat)( rside.clr.r - lside.clr.r ) / tmpsize;
        stepclr.g   = (RAYfloat)( rside.clr.g - lside.clr.g ) / tmpsize;
        stepclr.b   = (RAYfloat)( rside.clr.b - lside.clr.b ) / tmpsize;
        stepclr.a   = (RAYfloat)( rside.clr.a - lside.clr.a ) / tmpsize;
        
        // Set the linear interpolation to the left side values
        tmpsize = optxStart - xStart;
        i       = y * rayScreenWidth + optxStart;
        z       = lside.z + step.z * tmpsize;
        clr.r   = lside.clr.r + stepclr.r * tmpsize;
        clr.g   = lside.clr.g + stepclr.g * tmpsize;
        clr.b   = lside.clr.b + stepclr.b * tmpsize;
        clr.a   = lside.clr.a + stepclr.a * tmpsize;
        
        // Loop for all pixel in the current scan line
        for (x = optxStart; x < optxEnd; ++x, ++i)
        {
            z       += step.z;
            clr.r   += stepclr.r;
            clr.g   += stepclr.g;
            clr.b   += stepclr.b;
            clr.a   += stepclr.a;
            
            mcr_rayCheckPlotDepth;
            
            rayPlotIndexed(i, z, (RAYcolor_t)clr.r, (RAYcolor_t)clr.g, (RAYcolor_t)clr.b, (RAYcolor_t)clr.a);
        }
    }
    
}


/*
 * ===== Textured triangle =====
 */

/*
 * Linear texture mapping
 */

inline void rayRenderTriangle_Texture_PixelLinear_ColorLinear()
{
    
    // Temporary variables
    mcr_rayStandardMembers;
    
    RAYint tmpsize;
    
    SRayColor4i clr;
    SRayVector2f map, invmap;
    
    // Resize the texture coordinates
    mcr_rayResizeTextureCoordinates;
    
    // Loop for all lines
    for (y = optyStart; y < optyEnd; ++y)
    {
        // Update the main part
        mcr_rayUpdateMainPart(mcr_rayComputePolygonSides_Depth_TexCoords);
        
        // Clamp the scanline dimension
        mcr_rayClampScanlineDimension;
        
        // Compute the step for linear interpolation
        tmpsize = xEnd - xStart;
        step.z  = ( rside.z - lside.z ) / tmpsize;
        step.u  = ( rside.u - lside.u ) / tmpsize;
        step.v  = ( rside.v - lside.v ) / tmpsize;
        
        // Set the linear interpolation to the left side values
        tmpsize = optxStart - xStart;
        i       = y * rayScreenWidth + optxStart;
        z       = lside.z + step.z * tmpsize;
        map.x   = lside.u + step.u * tmpsize;
        map.y   = lside.v + step.v * tmpsize;
        
        // Loop for all pixel in the current scan line
        for (x = optxStart; x < optxEnd; ++x, ++i)
        {
            z       += step.z;
            map.x   += step.u;
            map.y   += step.v;
            
            mcr_rayCheckPlotDepth;
            
            invmap.x = map.x / z;
            invmap.y = map.y / z;
            
            rayGetTexturePixelLinear(clr, invmap);
            
            rayPlotIndexed(
                i, z,
                clr.r * rcVertex[2]->clr.r / 255,
                clr.g * rcVertex[2]->clr.g / 255,
                clr.b * rcVertex[2]->clr.b / 255,
                clr.a * rcVertex[2]->clr.a / 255
            );
        }
    }
    
}

inline void rayRenderTriangle_Texture_PixelLinear_ColorSmooth()
{
    
    // Temporary variables
    mcr_rayStandardMembers;
    
    RAYint tmpsize;
    
    SRayColor4i mapclr;
    SRayColor4f clr, stepclr;
    SRayVector2f map, invmap;
    
    // Resize the texture coordinates
    mcr_rayResizeTextureCoordinates;
    
    // Loop for all lines
    for (y = optyStart; y < optyEnd; ++y)
    {
        // Update the main part
        mcr_rayUpdateMainPart(mcr_rayComputePolygonSides_Depth_TexCoords_Color);
        
        // Clamp the scanline dimension
        mcr_rayClampScanlineDimension;
        
        // Compute the step for linear interpolation
        tmpsize     = xEnd - xStart;
        step.z      = ( rside.z - lside.z ) / tmpsize;
        step.u      = ( rside.u - lside.u ) / tmpsize;
        step.v      = ( rside.v - lside.v ) / tmpsize;
        stepclr.r   = (RAYfloat)( rside.clr.r - lside.clr.r ) / tmpsize;
        stepclr.g   = (RAYfloat)( rside.clr.g - lside.clr.g ) / tmpsize;
        stepclr.b   = (RAYfloat)( rside.clr.b - lside.clr.b ) / tmpsize;
        stepclr.a   = (RAYfloat)( rside.clr.a - lside.clr.a ) / tmpsize;
        
        // Set the linear interpolation to the left side values
        tmpsize = optxStart - xStart;
        i       = y * rayScreenWidth + optxStart;
        z       = lside.z + step.z * tmpsize;
        map.x   = lside.u + step.u * tmpsize;
        map.y   = lside.v + step.v * tmpsize;
        clr.r   = lside.clr.r + stepclr.r * tmpsize;
        clr.g   = lside.clr.g + stepclr.g * tmpsize;
        clr.b   = lside.clr.b + stepclr.b * tmpsize;
        clr.a   = lside.clr.a + stepclr.a * tmpsize;
        
        // Loop for all pixel in the current scan line
        for (x = optxStart; x < optxEnd; ++x, ++i)
        {
            z       += step.z;
            map.x   += step.u;
            map.y   += step.v;
            clr.r   += stepclr.r;
            clr.g   += stepclr.g;
            clr.b   += stepclr.b;
            clr.a   += stepclr.a;
            
            mcr_rayCheckPlotDepth;
            
            invmap.x = map.x / z;
            invmap.y = map.y / z;
            
            rayGetTexturePixelLinear(mapclr, invmap);
            
            rayPlotIndexed(
                i, z,
                mapclr.r * (RAYcolor_t)clr.r / 255,
                mapclr.g * (RAYcolor_t)clr.g / 255,
                mapclr.b * (RAYcolor_t)clr.b / 255,
                mapclr.a * (RAYcolor_t)clr.a / 255
            );
        }
    }
    
}

inline void rayRenderTriangle_Texture_PixelSmooth_ColorLinear()
{
    
    // Temporary variables
    mcr_rayStandardMembers;
    
    RAYint tmpsize;
    
    SRayColor4i clr;
    SRayVector2f map, invmap;
    
    // Resize the texture coordinates
    mcr_rayResizeTextureCoordinates;
    
    // Loop for all lines
    for (y = optyStart; y < optyEnd; ++y)
    {
        // Update the main part
        mcr_rayUpdateMainPart(mcr_rayComputePolygonSides_Depth_TexCoords);
        
        // Clamp the scanline dimension
        mcr_rayClampScanlineDimension;
        
        // Compute the step for linear interpolation
        tmpsize = xEnd - xStart;
        step.z  = ( rside.z - lside.z ) / tmpsize;
        step.u  = ( rside.u - lside.u ) / tmpsize;
        step.v  = ( rside.v - lside.v ) / tmpsize;
        
        // Set the linear interpolation to the left side values
        tmpsize = optxStart - xStart;
        i       = y * rayScreenWidth + optxStart;
        z       = lside.z + step.z * tmpsize;
        map.x   = lside.u + step.u * tmpsize;
        map.y   = lside.v + step.v * tmpsize;
        
        // Loop for all pixel in the current scan line
        for (x = optxStart; x < optxEnd; ++x, ++i)
        {
            z       += step.z;
            map.x   += step.u;
            map.y   += step.v;
            
            mcr_rayCheckPlotDepth;
            
            invmap.x = map.x / z;
            invmap.y = map.y / z;
            
            rayGetTexturePixelSmooth(clr, invmap);
            
            rayPlotIndexed(
                i, z,
                clr.r * rcVertex[2]->clr.r / 255,
                clr.g * rcVertex[2]->clr.g / 255,
                clr.b * rcVertex[2]->clr.b / 255,
                clr.a * rcVertex[2]->clr.a / 255
            );
        }
    }
    
}

inline void rayRenderTriangle_Texture_PixelSmooth_ColorSmooth()
{
    
    // Temporary variables
    mcr_rayStandardMembers;
    
    RAYint tmpsize;
    
    SRayColor4i mapclr;
    SRayColor4f clr, stepclr;
    SRayVector2f map, invmap;
    
    // Resize the texture coordinates
    mcr_rayResizeTextureCoordinates;
    
    // Loop for all lines
    for (y = optyStart; y < optyEnd; ++y)
    {
        // Update the main part
        mcr_rayUpdateMainPart(mcr_rayComputePolygonSides_Depth_TexCoords_Color);
        
        // Clamp the scanline dimension
        mcr_rayClampScanlineDimension;
        
        // Compute the step for linear interpolation
        tmpsize     = xEnd - xStart;
        step.z      = ( rside.z - lside.z ) / tmpsize;
        step.u      = ( rside.u - lside.u ) / tmpsize;
        step.v      = ( rside.v - lside.v ) / tmpsize;
        stepclr.r   = (RAYfloat)( rside.clr.r - lside.clr.r ) / tmpsize;
        stepclr.g   = (RAYfloat)( rside.clr.g - lside.clr.g ) / tmpsize;
        stepclr.b   = (RAYfloat)( rside.clr.b - lside.clr.b ) / tmpsize;
        stepclr.a   = (RAYfloat)( rside.clr.a - lside.clr.a ) / tmpsize;
        
        // Set the linear interpolation to the left side values
        tmpsize = optxStart - xStart;
        i       = y * rayScreenWidth + optxStart;
        z       = lside.z + step.z * tmpsize;
        map.x   = lside.u + step.u * tmpsize;
        map.y   = lside.v + step.v * tmpsize;
        clr.r   = lside.clr.r + stepclr.r * tmpsize;
        clr.g   = lside.clr.g + stepclr.g * tmpsize;
        clr.b   = lside.clr.b + stepclr.b * tmpsize;
        clr.a   = lside.clr.a + stepclr.a * tmpsize;
        
        // Loop for all pixel in the current scan line
        for (x = optxStart; x < optxEnd; ++x, ++i)
        {
            z       += step.z;
            map.x   += step.u;
            map.y   += step.v;
            clr.r   += stepclr.r;
            clr.g   += stepclr.g;
            clr.b   += stepclr.b;
            clr.a   += stepclr.a;
            
            mcr_rayCheckPlotDepth;
            
            invmap.x = map.x / z;
            invmap.y = map.y / z;
            
            rayGetTexturePixelSmooth(mapclr, invmap);
            
            rayPlotIndexed(
                i, z,
                mapclr.r * (RAYcolor_t)clr.r / 255,
                mapclr.g * (RAYcolor_t)clr.g / 255,
                mapclr.b * (RAYcolor_t)clr.b / 255,
                mapclr.a * (RAYcolor_t)clr.a / 255
            );
        }
    }
    
}



/*
 * ========== Internal static functions ==========
 */


static void rayUpdateVertices()
{
    // Check if the normal must be normalized
    if (rayRenderStatus[RAY_LIGHTING] || rayRenderStatus[RAY_TEXGEN])
    {
        // Multiply the normals by the normal matrix
        rayMultMatrix(rayRenderMatrix[RAY_NORMAL_MATRIX], rcVertex[0]->nml);
        rayMultMatrix(rayRenderMatrix[RAY_NORMAL_MATRIX], rcVertex[1]->nml);
        rayMultMatrix(rayRenderMatrix[RAY_NORMAL_MATRIX], rcVertex[2]->nml);
        
        // Normalize the normals
        if (rayRenderStatus[RAY_NORMALIZE])
        {
            rayNormalizeVec3(rcVertex[0]->nml);
            rayNormalizeVec3(rcVertex[1]->nml);
            rayNormalizeVec3(rcVertex[2]->nml);
        }
    }
    
    // Process the mapping gneration
    if (rayRenderStatus[RAY_TEXGEN])
    {
        for (RAYint i = 0; i < RAY_TEXTURE_LAYER_SIZE; ++i)
        {
            rayUpdateMappingGen(rayMappingGenType[i].genu, rcVertex[0]->pos.x, rcVertex[0]->nml.x, rcVertex[0]->map[i].x);
            rayUpdateMappingGen(rayMappingGenType[i].genv, rcVertex[0]->pos.y, rcVertex[0]->nml.y, rcVertex[0]->map[i].y);
            
            rayUpdateMappingGen(rayMappingGenType[i].genu, rcVertex[1]->pos.x, rcVertex[1]->nml.x, rcVertex[1]->map[i].x);
            rayUpdateMappingGen(rayMappingGenType[i].genv, rcVertex[1]->pos.y, rcVertex[1]->nml.y, rcVertex[1]->map[i].y);
            
            rayUpdateMappingGen(rayMappingGenType[i].genu, rcVertex[2]->pos.x, rcVertex[2]->nml.x, rcVertex[2]->map[i].x);
            rayUpdateMappingGen(rayMappingGenType[i].genv, rcVertex[2]->pos.y, rcVertex[2]->nml.y, rcVertex[2]->map[i].y);
        }
    }
    
    // Check if lighting is used
    if (rayRenderStatus[RAY_LIGHTING])
    {
        if (rayRenderStatus[RAY_COLOR_SMOOTH])
        {
            mcr_raySetColorIntensity(
                rcVertex[0]->clr, (RAYint)( ( 0.2f + fabs(rcVertex[0]->nml.z) * 0.8f ) * 255 )
            );
            mcr_raySetColorIntensity(
                rcVertex[1]->clr, (RAYint)( ( 0.2f + fabs(rcVertex[1]->nml.z) * 0.8f ) * 255 )
            );
            mcr_raySetColorIntensity(
                rcVertex[2]->clr, (RAYint)( ( 0.2f + fabs(rcVertex[2]->nml.z) * 0.8f ) * 255 )
            );
        }
        else
        {
            RAYint ColorIntensity = (RAYint)( ( 0.2f + fabs(rcVertex[0]->nml.z) * 0.8f ) * 255 );
            mcr_raySetColorIntensity(rcVertex[0]->clr, ColorIntensity);
            mcr_raySetColorIntensity(rcVertex[1]->clr, ColorIntensity);
            mcr_raySetColorIntensity(rcVertex[2]->clr, ColorIntensity);
        }
    }
}


/*
 * ========== Static functions ==========
 */

RAYAPI void rayDrawPoint()
{
    
    /*
     * Render state correction
     */
    
    // Check if the normal must be normalized
    if (rayRenderStatus[RAY_LIGHTING] || rayRenderStatus[RAY_TEXGEN])
    {
        // Multiply the normals by the normal matrix
        rayMultMatrix(rayRenderMatrix[RAY_NORMAL_MATRIX], rcVertex[0]->nml);
        
        // Normalize the normals
        if (rayRenderStatus[RAY_NORMALIZE])
            rayNormalizeVec3(rcVertex[0]->nml);
    }
    
    // Process the mapping gneration
    if (rayRenderStatus[RAY_TEXGEN])
    {
        for (RAYint i = 0; i < RAY_TEXTURE_LAYER_SIZE; ++i)
        {
            rayUpdateMappingGen(rayMappingGenType[i].genu, rcVertex[0]->pos.x, rcVertex[0]->nml.x, rcVertex[0]->map[i].x);
            rayUpdateMappingGen(rayMappingGenType[i].genv, rcVertex[0]->pos.y, rcVertex[0]->nml.y, rcVertex[0]->map[i].y);
        }
    }
    
    // Check if lighting is used
    if (rayRenderStatus[RAY_LIGHTING])
    {
        RAYint ColorIntensity = (RAYint)( ( 0.5f + fabs(rcVertex[0]->nml.z) / 2 ) * 255 );
        
        mcr_raySetColorIntensity(rcVertex[0]->clr, ColorIntensity);
    }
    
    /*
     * First calculations for rasterization
     */
    
    // Back settings
    rcPLIndex               = 0;
    rcTextureLayer          = 0;
    rcTextureMipMapLevel    = 0;
    
    // Multiply the vertices with the modelview matrix
    rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[0]->pos);
    
    // Check if the z coordinate is to small
    if (rcVertex[0]->pos.z <= 0)
        return;
    
    // Project to the window coordinates
    rayProject3Dto2D(rcVertex[0]->pos);
    
    // Invert the z-axis
    rcVertex[0]->pos.z = 1.0 / rcVertex[0]->pos.z;
    
    /*
     * Loop for all layer
     */
    
    PFNRAYBLENDPROC cfgBlendProc = rayBlendProc;
    
    do
    {
        if (rcTexture[rcTextureLayer])
            rayRenderPoint_Texture();
        else if (rcTextureLayer == 0)
            rayRenderPoint_Color();
        
        rayBlendProc = rayBlendProc_Light;
    }
    while (++rcTextureLayer < rcMaxTextureLayer);
    
    rayBlendProc = cfgBlendProc;
    
}


RAYAPI void rayDrawLine()
{
    
    // Multiply the vertices with the modelview matrix
    rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[0]->pos);
    rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[1]->pos);
    
    // Render state correction
    rayUpdateVertices();
    
    /*
     * First calculations for rasterization
     */
    
    // Back settings
    rcPLIndex               = 0;
    rcTextureLayer          = 0;
    rcTextureMipMapLevel    = 0;
    
    // Check if the z coordinate is to small
    if (rcVertex[0]->pos.z <= 0 || rcVertex[1]->pos.z <= 0)
        return;
    
    // Project to the window coordinates
    rayProject3Dto2D(rcVertex[0]->pos);
    rayProject3Dto2D(rcVertex[1]->pos);
    
    // Check if the line is inside the screen
    if (mcr_rayPointInside(rcVertex[0]->pos) || mcr_rayPointInside(rcVertex[1]->pos))
        return;
    
    // Invert the z-axis
    rcVertex[0]->pos.z = 1.0 / rcVertex[0]->pos.z;
    rcVertex[1]->pos.z = 1.0 / rcVertex[1]->pos.z;
    
    /*
     * Loop for all layer
     */
    
    PFNRAYBLENDPROC cfgBlendProc = rayBlendProc;
    
    do
    {
        if (rcTexture[rcTextureLayer])
        {
            mcr_rayResizeTextureCoordinates;
            
            if (rayRenderStatus[RAY_COLOR_SMOOTH])
                rayRenderLine_Texture_ColorSmooth();
            else
                rayRenderLine_Texture_ColorLinear();
        }
        else if (rcTextureLayer == 0)
        {
            if (rayRenderStatus[RAY_COLOR_SMOOTH])
                rayRenderLine_Solid_ColorSmooth();
            else
                rayRenderLine_Solid_ColorLinear();
        }
        
        rayBlendProc = rayBlendProc_Light;
    }
    while (++rcTextureLayer < rcMaxTextureLayer);
    
    rayBlendProc = cfgBlendProc;
    
}


RAYAPI void rayDrawLineStrip()
{
    
    // Multiply the vertices with the modelview matrix
    rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[0]->pos);
    rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[1]->pos);
    rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[2]->pos);
    
    // Render state correction
    rayUpdateVertices();
    
    /*
     * First calculations for rasterization
     */
    
    // Back settings
    rcPLIndex       = 0;
    rcTextureLayer  = 0;
    
    // Check if the z coordinate is to small
    if (rcVertex[0]->pos.z <= 0 || rcVertex[1]->pos.z <= 0 || rcVertex[2]->pos.z <= 0)
        return;
    
    // Project to the window coordinates
    rayProject3Dto2D(rcVertex[0]->pos);
    rayProject3Dto2D(rcVertex[1]->pos);
    rayProject3Dto2D(rcVertex[2]->pos);
    
    // Check if the triangle is visible
    if (!rayTriangleFaceVisible() || !rayTriangleVisible())
        return;
    
    // Check if the line is inside the screen
    if (mcr_rayPointInside(rcVertex[0]->pos) || mcr_rayPointInside(rcVertex[1]->pos) || mcr_rayPointInside(rcVertex[2]->pos))
        return;
    
    // Invert the z-axis
    rcVertex[0]->pos.z = 1.0 / rcVertex[0]->pos.z;
    rcVertex[1]->pos.z = 1.0 / rcVertex[1]->pos.z;
    rcVertex[2]->pos.z = 1.0 / rcVertex[2]->pos.z;
    
    // Determine if smooth color is needed
    RAYint SmoothColor = !rayCheckColorEquality(rcVertex[0]->clr, rcVertex[1]->clr, rcVertex[2]->clr);
    
    /*
     * Loop for all layer
     */
    
    #define RENDER_LINE_PROC(p) \
    {                           \
        v0 = 0, v1 = 1, p();    \
        v0 = 1, v1 = 2, p();    \
        v0 = 2, v1 = 0, p();    \
    }
    
    PFNRAYBLENDPROC cfgBlendProc = rayBlendProc;
    
    do
    {
        if (rcTexture[rcTextureLayer])
        {
            rayComputeMipMapLevel();
            
            mcr_rayResizeTextureCoordinates;
            
            if (rayRenderStatus[RAY_COLOR_SMOOTH] && SmoothColor)
                RENDER_LINE_PROC(rayRenderLine_Texture_ColorSmooth)
            else
                RENDER_LINE_PROC(rayRenderLine_Texture_ColorLinear)
        }
        else if (rcTextureLayer == 0)
        {
            if (rayRenderStatus[RAY_COLOR_SMOOTH] && SmoothColor)
                RENDER_LINE_PROC(rayRenderLine_Solid_ColorSmooth)
            else
                RENDER_LINE_PROC(rayRenderLine_Solid_ColorLinear)
        }
        
        rayBlendProc = rayBlendProc_Light;
    }
    while (++rcTextureLayer < rcMaxTextureLayer);
    
    v0 = 0, v1 = 1;
    
    rayBlendProc = cfgBlendProc;
    
    #undef RENDER_LINE_PROC
    
}


RAYAPI void rayDrawRectangle()
{
    rcPLIndex = 0;
    rcTextureMipMapLevel = 0;
    
    if (rcTexture[0])
    {
        if (rayRenderStatus[RAY_TEXTURE_SMOOTH])
            rayRenderRectangle_TextureSize_PixelLinear();
        else
            rayRenderRectangle_TextureSize_PixelSmooth();
    }
    else
    {
        if (rayRenderStatus[RAY_COLOR_SMOOTH])
            rayRenderRectangle_ColorLinear();
        else
            rayRenderRectangle_ColorSmooth();
    }
}


RAYAPI void rayDrawTriangle()
{
    
    /*
     * First calculations for rasterization
     */
    
    // Back settings
    rcPLIndex       = 0;
    rcTextureLayer  = 0;
    
    // Multiply the vertices with the modelview matrix
    if (rcUpdateVerticesMatrix == RAY_TRUE)
    {
        rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[0]->pos);
        rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[1]->pos);
        rayMultMatrix(rayRenderMatrix[RAY_MODELVIEW_MATRIX], rcVertex[2]->pos);
        
        // Render state correction
        rayUpdateVertices();
    }
    
    // Check if the depth of all vertices
    if ( ( rcVertex[0]->pos.z < rayPlaneNear && rcVertex[1]->pos.z < rayPlaneNear && rcVertex[2]->pos.z < rayPlaneNear ) ||
         ( rcVertex[0]->pos.z > rayPlaneFar  && rcVertex[1]->pos.z > rayPlaneFar  && rcVertex[2]->pos.z > rayPlaneFar  ) )
    {
        return;
    }
    
    // Get the count of vertics which are outside the viewport
    // (in dependet of te near-z-plane)
    RAYint vi[2];
    
    RAYint outvrts = 0;
    
    if (rcVertex[0]->pos.z < rayPlaneNear) vi[outvrts++] = 0;
    if (rcVertex[1]->pos.z < rayPlaneNear) vi[outvrts++] = 1;
    if (rcVertex[2]->pos.z < rayPlaneNear) vi[outvrts++] = 2;
    
    // Check the cases of the possible outside vertices
    if (outvrts == 1)
    {
        RAYint vi2[2];
        
        rayGetFreeVisibleVertexIndex(vi[0], vi2);
        
        memcpy(&rcVertex[3]->pos.x, &rcVertex[vi[0]]->pos.x, sizeof(SRayVertex));
        
        // Clamp the outside vertices to the plane
        rayGetVertexClippingPlane(rcVertex[vi[0]], rcVertex[vi2[0]]);
        rayGetVertexClippingPlane(rcVertex[3    ], rcVertex[vi2[1]]);
        
        // Get the third vertex index for the second triangle
        RAYint vi3 = vi[0] - 1;
        if (vi3 < 0) vi3 = 2;
        
        //if (rcVertex[3]->pos.x < rcVertex[vi[0]]->pos.x)
        //    raySwapVertex(rcVertex[3], rcVertex[vi[0]]);
        
        // Save the next three vertices
        memcpy(&rcMemVertex[0]->pos.x, &rcVertex[vi3  ]->pos.x, sizeof(SRayVertex));
        memcpy(&rcMemVertex[1]->pos.x, &rcVertex[3    ]->pos.x, sizeof(SRayVertex));
        memcpy(&rcMemVertex[2]->pos.x, &rcVertex[vi[0]]->pos.x, sizeof(SRayVertex));
        
        //outvrts = 0;
    }
    else if (outvrts == 2)
    {
        RAYint vi2 = rayGetFreeVisibleVertexIndex(vi[0], vi[1]);
        
        // Clamp the outside vertices to the plane
        rayGetVertexClippingPlane(rcVertex[vi[0]], rcVertex[vi2]);
        rayGetVertexClippingPlane(rcVertex[vi[1]], rcVertex[vi2]);
    }
    
    // Project to the window coordinates
    rayProject3Dto2D(rcVertex[0]->pos);
    rayProject3Dto2D(rcVertex[1]->pos);
    rayProject3Dto2D(rcVertex[2]->pos);
    
    // Check if the triangle is visible
    if (!rayTriangleFaceVisible() || !rayTriangleVisible())
        return;
    
    // Sort the vertices in dependet of the y axis
    if (rcVertex[0]->pos.y > rcVertex[1]->pos.y)
        raySwapVertex(rcVertex[0], rcVertex[1]);
    if (rcVertex[0]->pos.y > rcVertex[2]->pos.y)
        raySwapVertex(rcVertex[0], rcVertex[2]);
    if (rcVertex[1]->pos.y > rcVertex[2]->pos.y)
        raySwapVertex(rcVertex[1], rcVertex[2]);
    
    // Clamp the triangle area
    rayGetTriangleArea(optyStart, optyEnd);
    
    // Set the vertices position
    yStart  = (RAYint)rcVertex[0]->pos.y;
    yMiddle = (RAYint)rcVertex[1]->pos.y;
    yEnd    = (RAYint)rcVertex[2]->pos.y;
    
    // Compute the dimensions
    yMiddleStart    = yMiddle - yStart;
    yEndMiddle      = yEnd - yMiddle;
    yEndStart       = yEnd - yStart;
    
    // Invert the z-axis
    rcVertex[0]->pos.z = 1.0 / rcVertex[0]->pos.z;
    rcVertex[1]->pos.z = 1.0 / rcVertex[1]->pos.z;
    rcVertex[2]->pos.z = 1.0 / rcVertex[2]->pos.z;
    
    // Determine if smooth color is needed
    RAYint SmoothColor = !rayCheckColorEquality(rcVertex[0]->clr, rcVertex[1]->clr, rcVertex[2]->clr);
    
    /*
     * Loop for all layer
     */
    
    PFNRAYBLENDPROC cfgBlendProc = rayBlendProc;
    
    do
    {
        if (rcTexture[rcTextureLayer])
        {
            rayComputeMipMapLevel();
            
            if (rayRenderStatus[RAY_TEXTURE_SMOOTH])
            {
                if (rayRenderStatus[RAY_COLOR_SMOOTH] && SmoothColor)
                    rayRenderTriangle_Texture_PixelSmooth_ColorSmooth();
                else
                    rayRenderTriangle_Texture_PixelSmooth_ColorLinear();
            }
            else
            {
                if (rayRenderStatus[RAY_COLOR_SMOOTH] && SmoothColor)
                    rayRenderTriangle_Texture_PixelLinear_ColorSmooth();
                else
                    rayRenderTriangle_Texture_PixelLinear_ColorLinear();
            }
        }
        else if (rcTextureLayer == 0)
        {
            if (rayRenderStatus[RAY_COLOR_SMOOTH] && SmoothColor)
                rayRenderTriangle_Solid_ColorSmooth();
            else
                rayRenderTriangle_Solid_ColorLinear();
        }
        
        rayBlendProc = rayBlendProc_Light;
    }
    while (++rcTextureLayer < rcMaxTextureLayer);
    
    rayBlendProc = cfgBlendProc;
    
    if (outvrts == 1)
    {
        raySwapVertex(rcVertex[0], rcMemVertex[0]);
        raySwapVertex(rcVertex[1], rcMemVertex[1]);
        raySwapVertex(rcVertex[2], rcMemVertex[2]);
        
        rcUpdateVerticesMatrix  = RAY_FALSE;
        rayDrawTriangle();
        rcUpdateVerticesMatrix  = RAY_TRUE;
    }
    
}


/*
 * ========== Fast drawing functions ==========
 */

RAYAPI void rayDrawRectangleSelected(RAYenum Type)
{
    switch (Type)
    {
        case RAY_RECT_LINEAR:
            rayRenderRectangle_ColorLinear(); break;
        case RAY_RECT_SMOOTH:
            rayRenderRectangle_ColorSmooth(); break;
        case RAY_RECT_TEXTURELINEAR:
            rayRenderRectangle_TextureSize_PixelLinear(); break;
        case RAY_RECT_TEXTURESMOOTH:
            rayRenderRectangle_TextureSize_PixelSmooth(); break;
    }
}

RAYAPI void rayDrawImage2DFast(RAYuint TexID, RAYint x, RAYint y)
{
    SRayTexture2D* tex = reinterpret_cast<SRayTexture2D*>(TexID);
    
    if (!tex || !tex->image)
        return;
    
    // Temporary variables
    register RAYint ix, iy, i = 0, j = 0;
    
    RAYint left = x, top = y, right = x + tex->width, bottom = y + tex->height;
    
    mcr_rayClampRectangle;
    
    // Check the outside values
    if (y < 0)
    {
        if (x < 0)
            j = ( -y * tex->width - x ) << 2;
        else
            j = ( -y * tex->width ) << 2;
    }
    else if (x < 0)
        j = -x << 2;
    
    // Loop for all pixels to draw
    for (iy = top; iy < bottom; ++iy)
    {
        // Draw the scanline
        for (ix = left; ix < right; ++ix, ++i, j += 4)
            rayPlotPixel(i, tex->image[0][j], tex->image[0][j+1], tex->image[0][j+2]);
        
        // Push the indices to the next step
        i += rayScreenWidth - (right - left);
        j += ( tex->width - ( right - left ) ) << 2;
    }
}







// ================================================================================
