/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for scene loading SPS-scene file format
 *
 */

#include "spSceneLoaderSPS.hpp"


namespace sp
{
namespace scene
{


SceneLoaderSPS::SceneLoaderSPS() : CurEntity_(0)
{
}
SceneLoaderSPS::~SceneLoaderSPS()
{
}

Entity* SceneLoaderSPS::loadScene(SceneManager* SceneMngr, io::stringc &Filename, io::stringc &TexturePath, s32 Tesselation)
{
    Filename_       = Filename;
    TexturePath_    = TexturePath;
    
    hEntity_        = new Entity();
    
    hFile_ = hFileLoader_->openFile(Filename_, io::FILE_READ_BINARY);
    
    if (hFile_)
        loadSceneData();
    
    hFileLoader_->closeFile(hFile_);
    
    return hEntity_;
}


/*
 * ========== Private: ==========
 */


// !!! U N F I N I S H E D !!!

void SceneLoaderSPS::loadSceneData()
{
    u8 ObjType = 0;
    
    while (!hFileLoader_->isEOF(hFile_))
    {
        ObjType = hFileLoader_->readValue<u8>(hFile_);
        
        switch (ObjType)
        {
            case SCENEOBJ_PRIMITIVE:
                readSceneObjPrimitive();
                break;
            case SCENEOBJ_MODEL:
                readSceneObjModel();
                break;
        }
        
        readSceneObjLocation();
    }
}

void SceneLoaderSPS::readSceneObjPrimitive()
{
    
    
}

void SceneLoaderSPS::readSceneObjModel()
{
    
    
}

void SceneLoaderSPS::readSceneObjLocation()
{
    dim::vector3df Position, Rotation, Scale;
    
    // Read location (coordination, rotation, scale)
    
    Position.X = hFileLoader_->readValue<f32>(hFile_);
    Position.Y = hFileLoader_->readValue<f32>(hFile_);
    Position.Z = hFileLoader_->readValue<f32>(hFile_);
    
    Rotation.X = hFileLoader_->readValue<f32>(hFile_);
    Rotation.Y = hFileLoader_->readValue<f32>(hFile_);
    Rotation.Z = hFileLoader_->readValue<f32>(hFile_);
    
    Scale.X = hFileLoader_->readValue<f32>(hFile_);
    Scale.Y = hFileLoader_->readValue<f32>(hFile_);
    Scale.Z = hFileLoader_->readValue<f32>(hFile_);
    
    // Work with read values
    
    if (CurEntity_)
    {
        u32 pVertices[3];
        std::list<video::Texture*> tTextureList = CurEntity_->getTextureList();
        
        CurEntity_->doMeshTransform(Scale);
        CurEntity_->doMeshTurn(Rotation);
        CurEntity_->doMeshTranslate(Position);
        
        hEntity_->createSurface();
        
        for (u32 i = 0; i < CurEntity_->getMeshVerticesCount(); ++i)
        {
            hEntity_->addVertex(CurEntity_->getVertexCoord(i),
                                CurEntity_->getVertexTexCoord(i),
                                CurEntity_->getVertexColor(i));
        }
        for (u32 i = 0; i < CurEntity_->getMeshTriangleCount(); ++i)
        {
            CurEntity_->getTriangleVertices(i, pVertices);
            hEntity_->addTriangle(pVertices[0], pVertices[1], pVertices[2]);
        }
        for (std::list<video::Texture*>::iterator it = tTextureList.begin(); it != tTextureList.end(); ++it)
        {
            hEntity_->addTexture(*it);
        }
        
        CurEntity_ = 0;
    }
    
}





} // /namespace scene

} // /namespace sp








// ================================================================================
