/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * cameras
 *
 */

#include "spSceneManagerCamera.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace scene
{


Camera::Camera() : Node(NODE_CAMERA)
{
    /* General settings */
    Viewport_       = dim::rect2di(0, 0, __Screen_Width, __Screen_Height);
    NearRange_      = CAMERADEF_RANGE_NEAR;
    FarRange_       = CAMERADEF_RANGE_FAR;
    ZoomAngle_      = DEF_PERSPECTIVE_FOV;
    isOrtho_        = false;
    isMirror_       = false;
    
    MirrorMatrix_.matrixReset();
    
    /* Initialize the perspective matrix */
    setPerspective(Viewport_, NearRange_, FarRange_, ZoomAngle_);
}
Camera::~Camera()
{
}

void Camera::updatePerspective()
{
    /* Check which projection matrix is used */
    if (isOrtho_)
    {
        PerspectiveMatrix_.setOrthoLH(
            static_cast<f32>(Viewport_.Left) / ZoomAngle_,
            static_cast<f32>(Viewport_.Left + Viewport_.Right) / ZoomAngle_,
            static_cast<f32>(Viewport_.Top) / ZoomAngle_,
            static_cast<f32>(Viewport_.Top + Viewport_.Bottom) / ZoomAngle_,
            NearRange_, FarRange_
        );
    }
    else
    {
        #if 0
        PerspectiveMatrix_.setPerspectiveLH(
            ZoomAngle_, (f32)Viewport_.Right / (f32)Viewport_.Bottom, NearRange_, FarRange_
        );
        #else
        /* Temporary computaion */
        const f32 AspectRatio = (f32)Viewport_.Right / (f32)Viewport_.Bottom;
        
        /* Check which projection matrix the renderer is using */
        if (__spVideoDriver->getProjectionMatrixType() == dim::MATRIX_LEFTHANDED)
            PerspectiveMatrix_.setPerspectiveLH(ZoomAngle_, AspectRatio, NearRange_, FarRange_);
        else
            PerspectiveMatrix_.setPerspectiveRH(ZoomAngle_, AspectRatio, NearRange_, FarRange_);
        #endif
    }
    
    /* Set the viewport */
    __spVideoDriver->setViewport(Viewport_.Left, Viewport_.Top, Viewport_.Right, Viewport_.Bottom);
    __spVideoDriver->setClippingRange(NearRange_, FarRange_);
    
    /* Update the render matrices */
    spProjectionMatrix = PerspectiveMatrix_;
    spViewMatrix.matrixReset();
    spWorldMatrix.matrixReset();
}

void Camera::setPerspective(dim::rect2di Viewport, f32 NearRange, f32 FarRange, f32 ZoomAngle)
{
    /* General settings */
    Viewport_   = Viewport;
    NearRange_  = NearRange;
    FarRange_   = FarRange;
    ZoomAngle_  = ZoomAngle;
    
    /* Update the perspective */
    updatePerspective();
}

void Camera::getPerspective(dim::rect2di &Viewport, f32 &NearRange, f32 &FarRange, f32 &ZoomAngle)
{
    Viewport    = Viewport_;
    NearRange   = NearRange_;
    FarRange    = FarRange_;
    ZoomAngle   = ZoomAngle_;
}

// !!! INCOMPLETE (for other FOVs as 74.0) !!!

dim::point2di Camera::getProjection(dim::vector3df Position)
{
    /* Generate the line coordinates in dependent to the frustum culling */
    Position = getGlobalLocationInverse() * Position;
    
    if (Position.Z <= 0)
        return dim::point2di(-10000, -10000);
    
    dim::point2df ScreenCoord;
    
    /* Compute the 2d coordinates */
    if (isOrtho_)
    {
        ScreenCoord.set(
              Position.X * ZoomAngle_ + Viewport_.Right /2 + Viewport_.Left,
            - Position.Y * ZoomAngle_ + Viewport_.Bottom/2 + Viewport_.Top
        );
    }
    else
    {
        const f32 aspect = (f32)Viewport_.Right / (f32)Viewport_.Bottom;
        
        ScreenCoord.set(
              Position.X / Position.Z * f32(Viewport_.Right/2) * math::STDASPECT / aspect + Viewport_.Right /2 + Viewport_.Left,
            - Position.Y / Position.Z * f32(Viewport_.Right/2) * math::STDASPECT / aspect + Viewport_.Bottom/2 + Viewport_.Top
        );
    }
    
    return dim::point2di((s32)ScreenCoord.X, (s32)ScreenCoord.Y);
}

// !!! INCOMPLETE (for other FOVs as 74.0 and does not work as a child camera) !!!

dim::line3df Camera::getPickingLine(dim::point2di Position, f32 Length)
{
    if (Length < 0.0f)
        Length = FarRange_;
    
    /* Temporary variables */
    dim::line3df Line;
    dim::point2df Coord((f32)Position.X - Viewport_.Left, (f32)Position.Y - Viewport_.Top);
    
    dim::matrix4f Mat(getGlobalLocation());
    
    if (isOrtho_)
    {
        dim::point2df Origin(
            Coord.X - f32(Viewport_.Right /2), Coord.Y - f32(Viewport_.Bottom/2)
        );
        
        Origin /= ZoomAngle_;
        
        Line.Corner1 = Mat * dim::vector3df(Origin.X, -Origin.Y, 0.0f);
        Line.Corner2 = Mat * dim::vector3df(Origin.X, -Origin.Y, Length);
    }
    else
    {
        Coord.make3DFrustum(Viewport_.Right, Viewport_.Bottom);
        
        Line.Corner1 = Mat.getPosition();
        Line.Corner2 = dim::vector3df(Coord.X, Coord.Y, 1.0f).normalize() * Length;
        Line.Corner2 = Mat * Line.Corner2;
    }
    
    /* Return the result */
    return Line;
}

void Camera::lookAt(dim::vector3df Position, bool isGlobal)
{
    /* Settings */
    dim::vector3df Pos(getPosition(isGlobal));
    dim::vector3df Rot(getRotation(isGlobal));
    
    /* Calculate rotation */
    Rot.X = -ASIN( (Position.Y - Pos.Y) / math::getDistance(Pos, Position) );
    Rot.Y = -ASIN( (Position.X - Pos.X) / math::getDistance(Pos.X, Pos.Z, Position.X, Position.Z) ) + 180.0f;
    Rot.Z = 0.0f;
    
    if (Pos.Z < Position.Z)
        Rot.Y = 180.0f - Rot.Y;
    
    /* Final rotation */
    setRotation(Rot, isGlobal);
}

void Camera::updateViewmode()
{
    /* Viewport */
    __spVideoDriver->setViewport(Viewport_.Left, Viewport_.Top, Viewport_.Right, Viewport_.Bottom);
    
    /* Projection matrix */
    __spVideoDriver->setProjectionMatrix(PerspectiveMatrix_);
}

void Camera::updateModelviewMatrix()
{
    /* Work with an identified matrix */
    spWorldMatrix.matrixReset();
    
    /* Compute the inverse location */
    updateGlobalLocationInverse();
    
    /* Apply the possible mirror matrix */
    if (isMirror_)
        spWorldMatrix *= MirrorMatrix_;
    
    /* Use the right matrix */
    __spVideoDriver->setViewMatrix(spWorldMatrix);
    spWorldMatrix.matrixReset();
}


} // /namespace scene

} // /namespace sp







// ================================================================================
