/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for lights
 *
 */

#ifndef __SP_SCENEMANAGER_LIGHT_H__
#define __SP_SCENEMANAGER_LIGHT_H__


#include "spStandard.hpp"
#include "spSceneManagerNode.hpp"
#include "spDimensionMatrix4.hpp"


namespace sp
{
namespace scene
{


/*
 * Macros
 */

static const f32 LIGHTDEF_RANGE_NEAR    = 20.0f;
static const f32 LIGHTDEF_RANGE_FAR     = 50.0f;

static const s32 MAX_COUNT_OF_LIGHTS    = 0x0D31;

extern bool __spLightIDList[MAX_COUNT_OF_LIGHTS];


/*
 * Light class
 */

class SP_EXPORT Light : public Node
{
    
    public:
        
        Light();
        Light(ELightTypes Type);
        ~Light();
        
        /* Lighting type (diffuse, spot) */
        
        inline void setLightingType(ELightTypes Type)
        {
            LightingType_ = Type;
        }
        inline ELightTypes getLightingType()
        {
            return LightingType_;
        }
        
        /* Lighting color */
        
        void setLightingColor(video::color Color, video::color AmbientColor = 255, video::color SpecularColor = 0);
        void getLightingColor(video::color &Color, video::color &AmbientColor, video::color &SpecularColor);
        
        /* Spot range */
        
        void setSpotRange(f32 Near, f32 Far);
        void getSpotRange(f32 &Near, f32 &Far);
        
        /* Volumetric */
        
        void setVolumetric(bool isVolumetric);
        inline bool getVolumetric() const
        {
            return isVolumetric_;
        }
        
        void setVolumetricRadius(f32 Radius);
        f32 getVolumetricRadius() const;
        
        void setVolumetricRange(f32 Constant, f32 Linear, f32 Quadratic);
        void getVolumetricRange(f32 &Constant, f32 &Linear, f32 &Quadratic) const;
        
        /* Other operation functions */
        
        inline void setDirection(const dim::matrix4f &Matrix)
        {
            Direction_ = Matrix * dim::vector3df(0, 0, 1);
        }
        inline dim::vector3df getDirection() const
        {
            return Direction_;
        }
        
        void setVisible(bool isVisible);
        
        /* Update the light information */
        
        void update();
        
    private:
        
        /* === Members ===*/
        
        // Identity information
        u32 LightID_;
        ELightTypes LightingType_;
        
        // Spot information
        dim::vector3df Direction_;
        f32 NearSpot_, FarSpot_;
        
        // Volumetrical information
        bool isVolumetric_;
        f32 AttenuationConstant_;
        f32 AttenuationLinear_;
        f32 AttenuationQuadratic_;
        
        // Color data
        video::color AmbientColor_, SpecularColor_;
        
};


} // /namespace scene

} // /namespace sp


#endif







// ================================================================================
