/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for standard models
 *
 */

#ifndef __SP_SCENEMANAGER_MODELCREATOR_H__
#define __SP_SCENEMANAGER_MODELCREATOR_H__


#include "spStandard.hpp"
#include "spSceneManagerEntity.hpp"
#include "spMath.hpp"


namespace sp
{
namespace scene
{


enum EPrimitiveModelTypes
{
    ENTITY_CUBE,
    ENTITY_CONE,
    ENTITY_CYLINDER,
    ENTITY_SPHERE,
    ENTITY_TORUS,
    ENTITY_SPIRAL,
    ENTITY_PIPE,
    ENTITY_PLANE,
    ENTITY_DISK,
    ENTITY_CUBOCTAHEDRON,
    ENTITY_TETRAHEDRON,
    ENTITY_OCTAHEDRON,
    ENTITY_DODECAHEDRON,
    ENTITY_ICOSAHEDRON,
    ENTITY_TEAPOT,
};

enum ESuperShapeModeTypes
{
    SUPERSHAPE_SMALL_CRYSTAL1 = 0,
    SUPERSHAPE_SMALL_CRYSTAL2,
    
    SUPERSHAPE_SMALL_STAR1,
    SUPERSHAPE_SMALL_STAR2,
    SUPERSHAPE_SMALL_STAR3,
    SUPERSHAPE_SMALL_STAR4,
    
    SUPERSHAPE_BIG_STAR1,
    
    SUPERSHAPE_URCHIN1,
    
    SUPERSHAPE_CUSHION1,
    
    SUPERSHAPE_RANDOM
};


class SP_EXPORT ModelCreator
{
    
    public:
        
        ModelCreator();
        ~ModelCreator();
        
        Entity* createModel(u32 Type, s32 Detail, f32 Radius1, f32 Radius2, f32 DegreeLength, f32 Size, bool isCap);
        Entity* createSuperShape(f32 ValueList[12], s32 Detail);
        
        void getSuperShapeModel(u32 Type, f32 ValueList[12]);
        
    private:
        
        /* Functions */
        
        void createCube(f32 Radius);
        void createCone(s32 Detail, f32 Radius, bool isCap);
        void createCylinder(s32 Detail, f32 Radius, bool isCap);
        void createSphere(s32 Detail, f32 Radius);
        void createTorus(s32 Detail, f32 Radius1, f32 Radius2);
        void createPipe(s32 Detail, f32 Radius1, f32 Radius2, bool isCap);
        void createSpiral(s32 Detail, f32 Radius1, f32 Radius2, f32 DegreeLength, f32 Size, bool isCap);
        void createPlane(s32 Detail);
        void createDisk(s32 Detail, f32 Radius1, f32 Radius2, bool isCap);
        void createCuboctahedron(); // (cube which has 8 tetrahedron corners cut)
        void createTetrahedron();   // 4 faces (4 triangles)
        void createOctahedron();    // 8 faces (2 square Pyramides)
        void createDodecahedron();  // 12 faces (5 corners for each face)
        void createIcosahedron();   // 20 faces (3 rectangles inner)
        void createTeapot();
        
        void createSuperShapeSurface(f32 ValueList[12], s32 Detail);
        f32 computeSuperShapeNextFrame(f32 m, f32 n1, f32 n2, f32 n3, f32 a, f32 b, f32 phi);
        
        void addVertex(f32 x, f32 y, f32 z, f32 u = 0.0f, f32 v = 0.0f, f32 w = 0.0f);
        void addFace(u32 v0, u32 v1, u32 v2);
        void addFace(u32 v0, u32 v1, u32 v2, u32 v3);
        void addFace(u32 v0, u32 v1, u32 v2, u32 v3, u32 v4);
        
        /* Members */
        
        Entity* hEntity_;
        
};


} // /namespace scene

} // /namespace sp


#endif






// ================================================================================
