/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for special effects actor (BumpMapping, FurEffect, CelShading, BlurEffect)
 *
 */

#ifndef __SP_SCENEMANAGER_SPECIALEFFECTSACTOR_H__
#define __SP_SCENEMANAGER_SPECIALEFFECTSACTOR_H__


#include "spStandard.hpp"
#include "spSceneManagerEntity.hpp"
#include "spSceneManagerCamera.hpp"
#include "spSceneManagerLight.hpp"
#include "spVideoDriverTexture.hpp"
#include "spInputOutputStandard.hpp"
#include "spDimension.hpp"
#include "spMath.hpp"

#include <list>


namespace sp
{
namespace scene
{
namespace effect
{


/* === Bump mapping === */

#define __SP_BM_MAX_EMBOSS 0.008f


/* === Fur effect === */


/* === Cel shading === */

#define __SP_CS_STDLINESIZE 5

struct SP_EXPORT CelShadow
{
    public:
        CelShadow();
        CelShadow(u32 ID);
        ~CelShadow();
        
        inline u32 getID()
        {
            return ID_;
        }
        
    private:
        u32 ID_;
};


/* === Blur effect === */

#define __SP_BE_STDLEVELSCOUNT  25
#define __SP_BE_STDLEVELSDIST   0.02f
#define __SP_BE_STDTEXSIZE      128



class SP_EXPORT SpecialEffectsActor
{
    
    public:
        
        SpecialEffectsActor();
        virtual ~SpecialEffectsActor();
        
        virtual void registerCamera(Camera* hCamera);
        virtual void unregisterCamera(Camera* hCamera);
        
        /* === Bump mapping === */
        
        
        /* === Fur effect === */
        
        virtual void createFurObject(
            Entity* hEntity, video::Texture* FurTexture,
            s32 LayerCount = 25, f32 HairLength = 0.2f, s32 HairCloseness = 2
        );
        
        /* === Cel shading === */
        
        virtual CelShadow* createCelShadow(
            s32 LevelsCount = 8, video::color ShadowColor = video::color(255),
            video::color DarknessColor = video::color(0)
        );
        virtual CelShadow* createCelShadow(s32 LevelsCount, video::color* LevelsColor);
        
        virtual void deleteCelShadow(CelShadow* &hCelShadow);
        
        virtual inline void setCelShadowLineSize(s32 Size)
        {
            CSLineWidth_ = Size;
        }
        virtual inline void setCelShadowBorderColor(video::color Color)
        {
            CSBorderColor_ = Color;
        }
        
        /* === Render functions === */
        
        /*virtual void renderBumpMapping(
            Entity* hEntity, video::Texture* BumpTexture, f32 BumpHeight, Light* hLight
        );*/
        
        virtual void renderCelShading(
            Entity* hEntity, CelShadow* hCelShadow, Light* hLight
        );
        
        virtual void renderBlurEffect(
            Entity* hEntity, s32 LevelsCount = __SP_BE_STDLEVELSCOUNT,
            f32 LevelsDist = __SP_BE_STDLEVELSDIST, u8 ColorAlpha = 255
        );
        
    protected:
        
        /* === Bump mapping === */
        
        virtual void setUpBumps(f32* VertexCoord, f32* Normal, f32* LightPosition, f32* TexCoordVectorS, f32* TexCoordVectorT, f32 BumpHeight);
        
        /* === Blur effect === */
        
        virtual void setBlurTextureSize(s32 TextureSize);
        
        /* Members */
        
        std::list<Camera*> RegisteredCameras_;
        
        s32 CSLineWidth_;
        video::color CSBorderColor_;
        video::Texture* BEBlurTexture_;
        
};


} // /namespace effect

} // /namespace scene

} // /namespace sp


#endif







// ================================================================================
