/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for sprites
 *
 */

#ifndef __SP_SCENEMANAGER_SPRITE_H__
#define __SP_SCENEMANAGER_SPRITE_H__


#include "spStandard.hpp"
#include "spSceneManagerNode.hpp"
#include "spVideoDriverTexture.hpp"
#include "spVideoDriverMaterial.hpp"


namespace sp
{
namespace scene
{


/* Pre-declarations */

class SceneManager;


class SP_EXPORT Sprite : public Node
{
    
    public:
        
        Sprite();
        Sprite(video::Texture* hTexture);
        ~Sprite();
        
        /* Base position */
        
        inline void setBasePosition(dim::point2df Position)
        {
            BasePosition_ = Position;
        }
        inline dim::point2df getBasePosition() const
        {
            return BasePosition_;
        }
        
        /* Material (lighting / no lighting) */
        
        inline void setMaterial(video::SMeshMaterial Material)
        {
            Material_ = Material;
        }
        inline video::SMeshMaterial getMaterial() const
        {
            return Material_;
        }
        inline video::SMeshMaterial &getThisMaterial()
        {
            return Material_;
        }
        
        inline void setLighting(bool isLighting)
        {
            Material_.isLighting = isLighting;
        }
        inline bool getLighting() const
        {
            return Material_.isLighting;
        }
        
        /* Orientation */
        
        inline void setOrder(s8 Order) // (Only 'normal' & 'uncut')
        {
            Order_ = Order;
            Material_.isZBuffer = (Order == 0);
        }
        inline s8 getOrder() const
        {
            return Order_;
        }
        
        /* Blending mode */
        
        inline void setBlendingMode(u8 BlendingMode)
        {
            setBlending(BlendingMode, Material_.BlendSource, Material_.BlendTarget);
        }
        void setBlendingMode(s32 BlendSource, s32 BlendTarget);
        
        /* Texture */
        
        inline void setTexture(video::Texture* hTexture)
        {
            hTexture_ = hTexture;
        }
        inline video::Texture* getTexture() const
        {
            return hTexture_;
        }
        
        /* Rendering */
        
        void render();
        
    private:
        
        /* Friends */
        
        friend bool sortSprites(Sprite* &obj1, Sprite* &obj2);
        friend class SceneManager;
        
        /* Private functions */
        
        void init();
        
        /* === Members === */
        
        video::Texture* hTexture_;
        dim::point2df BasePosition_;
        s8 Order_;
        
        /* final transformation */
        dim::vector3df Location_;
        
        /* Material information */
        video::SMeshMaterial Material_;
        
};


} // /namespace scene

} // /namespace sp


#endif







// ================================================================================
