/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Terrains
 *
 */

#include "spSceneManagerTerrain.hpp"


namespace sp
{
namespace scene
{


Terrain::Terrain() : Node(NODE_TERRAIN), hTexture_(0), hTextureHighMap_(0)
{
    ChangeDist_         = dim::vector3df(__SP_TERRAIN_STD_CHANGEDIST).getLengthSq();
    memset(hEntityPatches_, 0, sizeof(hEntityPatches_));
}
Terrain::Terrain(video::Texture* hTextureHighMap, video::Texture* hTexture, f32 ChangeDist) : Node(NODE_TERRAIN)
{
    ChangeDist_         = dim::vector3df(ChangeDist).getLengthSq();
    hTextureHighMap_    = hTextureHighMap;
    hTexture_           = hTexture;
    
    if (hTextureHighMap_)
        createHighMap();
    else
        io::printMessage("Error: could not generate high map for terrain!");
    
    if (hTexture)
    {
        for (s32 patch = 0, x, z; patch < __SP_TERRAIN_MAXDETAIL_PATCHES; ++patch)
            for (x = 0; x < __SP_TERRAIN_MAXDETAIL_X; ++x)
                for (z = 0; z < __SP_TERRAIN_MAXDETAIL_Z; ++z)
                    hEntityPatches_[x][z][patch]->addTexture(hTexture);
    }
}
Terrain::~Terrain()
{
    if (hEntityPatches_)
        delete [] hEntityPatches_;
}

// Rendering

void Terrain::render(dim::vector3df CamPosition)
{
    
    /* Matrix transformation */
    updateModelviewMatrix();
    
    // Rendering
    
    f32 tDistanceSQ;
    for (s32 x = 0, z, patch; x < __SP_TERRAIN_MAXDETAIL_X; ++x)
    {
        for (z = 0; z < __SP_TERRAIN_MAXDETAIL_Z; ++z)
        {
            // Check distance
            tDistanceSQ = dim::vector3df( CamPosition -
                          dim::vector3df( Position_+
                          dim::vector3df(((f32)x+0.5f)*Scale_.X/__SP_TERRAIN_MAXDETAIL_X, 0, -((f32)z+0.5f)*Scale_.Z/__SP_TERRAIN_MAXDETAIL_Z)
                        ) ).getLengthSq();
            // Patch selection
            patch = (s32)(tDistanceSQ/ChangeDist_);
            if (patch >= __SP_TERRAIN_MAXDETAIL_PATCHES)
                patch = __SP_TERRAIN_MAXDETAIL_PATCHES-1;
            // Render
            hEntityPatches_[x][z][__SP_TERRAIN_MAXDETAIL_PATCHES - 1 - patch]->render();
        }
    }
    
}

// Create

// ! incomplete !

void Terrain::createHighMap()
{
    
    s32 tDetail, tDetailX, tDetailZ, mapx, mapy;
    s32 Width = hTextureHighMap_->getOriginalSize().Width;
    s32 Height = hTextureHighMap_->getOriginalSize().Height;
    f32 _x, _y, _z;
    
    tDetail = 10;
    
    for (s32 patch = 0, x, z; patch < __SP_TERRAIN_MAXDETAIL_PATCHES; ++patch)
    {
        
        tDetail = tDetail*2;
        tDetailX = tDetail/__SP_TERRAIN_MAXDETAIL_X;
        tDetailZ = tDetail/__SP_TERRAIN_MAXDETAIL_Z;
        
        for (z = 0; z < __SP_TERRAIN_MAXDETAIL_Z; ++z)
        {
            for (x = 0; x < __SP_TERRAIN_MAXDETAIL_X; ++x)
            {
                
                // Create entity
                hEntityPatches_[x][z][patch] = new Entity();
                hEntityPatches_[x][z][patch]->createSurface();
                
                // Geo mipmapping generating
                for (mapy = tDetail*z/__SP_TERRAIN_MAXDETAIL_Z; mapy <= tDetail*(z+1)/__SP_TERRAIN_MAXDETAIL_Z; ++mapy)
                {
                    for (mapx = tDetail*x/__SP_TERRAIN_MAXDETAIL_X; mapx <= tDetail*(x+1)/__SP_TERRAIN_MAXDETAIL_X; ++mapx)
                    {
                        _x = (f32)(mapx) / tDetail;
                        _y = hTextureHighMap_->getPixelColor(dim::point2di(mapx*Width/tDetail, mapy*Height/tDetail)).getBrightness<f32>() / 255;
                        _z = (f32)(mapy) / tDetail;
                        hEntityPatches_[x][z][patch]->addVertex(dim::vector3df(_x, _y, -_z), dim::point2df(_x, _z));
                    }
                }
                for (mapy = 0; mapy < tDetailZ; ++mapy)
                {
                    for (mapx = 0; mapx < tDetailX; ++mapx)
                    {
                        hEntityPatches_[x][z][patch]->addTriangle(mapy*(tDetailZ+1)+mapx, mapy*(tDetailZ+1)+mapx+1, (mapy+1)*(tDetailZ+1)+mapx);
                        hEntityPatches_[x][z][patch]->addTriangle(mapy*(tDetailZ+1)+mapx+1, (mapy+1)*(tDetailZ+1)+mapx+1, (mapy+1)*(tDetailZ+1)+mapx);
                    }
                }
                
                // Update modeling
                hEntityPatches_[x][z][patch]->updateNormals();
                
            } // /for z
        } // /for x
        
    } // next patch
    
}

void Terrain::updateModelviewMatrix()
{
    /* Compute the normal location */
    updateGlobalLocation();
}


// Something else functions


void Terrain::setWireframe(s32 Type)
{
    for (s32 patch = 0, x, z; patch < __SP_TERRAIN_MAXDETAIL_PATCHES; ++patch)
        for (x = 0; x < __SP_TERRAIN_MAXDETAIL_X; ++x)
            for (z = 0; z < __SP_TERRAIN_MAXDETAIL_Z; ++z)
                hEntityPatches_[x][z][patch]->setWireframe(Type);
}





} // /namespace scene

} // /namespace sp







// ================================================================================
