/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Script extensions (SoftPixel loading script 1.1)
 *
 */

#include "spScriptExtension.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace script
{


ScriptLoader::ScriptLoader() : hFileLoader_(0), hFile_(0)
{
}
ScriptLoader::ScriptLoader(u32 TexMax, u32 SoundMax, u32 ModelMax) : hFileLoader_(0), hFile_(0)
{
    /* General settings */
    strLine_    = trimLine_     = stdPath_      = "";
    texIndex_   = soundIndex_   = modelIndex_   = 0;
    soundCount_ = 3;
    TexMax_     = TexMax;
    SoundMax_   = SoundMax;
    ModelMax_   = ModelMax;
    
    /* Allocate all memories */
    TextureList_    = new video::Texture*[TexMax];
    SoundList_      = new sound::Sound*[SoundMax];
    EntityList_     = new scene::Entity*[ModelMax];
    
    /* Initialize the memories */
    memset(TextureList_, 0, sizeof(video::Texture*)*TexMax);
    memset(SoundList_, 0, sizeof(sound::Sound*)*SoundMax);
    memset(EntityList_, 0, sizeof(scene::Entity*)*ModelMax);
}
ScriptLoader::~ScriptLoader()
{
}

bool ScriptLoader::loadScriptFile(io::stringc Filename, scene::SceneManager* sceneMngr)
{
    SoundLoader_    = sound::__ssDevice;
    SceneMngr_      = sceneMngr;
    
    hFileLoader_ = new io::FileLoader();
    
    if (!hFileLoader_->doesFileExist(Filename))
    {
        io::printError(io::stringc("could not found file: \"")+Filename+io::stringc("\""));
        return false;
    }
    else
        hFile_ = hFileLoader_->openFile(Filename, io::FILE_READ_ASCII);
    
    readScript();
    
    hFileLoader_->closeFile(hFile_);
    
    spDeletePointer(hFileLoader_);
    
    return true;
}

void ScriptLoader::clearLists()
{
    /* Delete the texture information */
    for (s32 i = 0; i < TexMax_; ++i)
    {
        if (TextureList_[i] && __spVideoDriver->isTexture(TextureList_[i]))
            __spVideoDriver->deleteTexture(TextureList_[i]);
    }
    spDeleteArray(TextureList_);
    
    /* Delete the sound information */
    for (s32 i = 0; i < SoundMax_; ++i)
    {
        if (SoundLoader_->isSound(SoundList_[i]))
            SoundLoader_->closeSound(SoundList_[i]);
    }
    spDeleteArray(SoundList_);
    
    /* Delete the model information */
    for (s32 i = 0; i < ModelMax_; ++i)
    {
        if (EntityList_[i])
            SceneMngr_->deleteNode(EntityList_[i]);
    }
    spDeleteArray(EntityList_);
}


// ========== Private: ==========

/*

Script prencip:
    keywords must not be upper 'PRINT' also 'Print' also 'print'
    they must not stand on the left side

// Comment (the same comments like in 'ANSI C' & 'C++')
PRINT "Loading now" // Prints a normal message 'io::printMessage(...)'
ERROR "Error loading!" // Prints an error message 'io::printError(...)'
WARNING "Warning info!" // Prints a warning message 'io::printWarning(...)'
LOADTEX 0 "gfx/tex1.bmp"
LOADTEX 1 "gfx/tex2.jpg" LINEAR
LOADTEX "gfx/tex2.jpg" SMOOTH // autom. index = 2, SMOOTH is the default value
TEXINDEX 10
STDPATH "gfx/"
LOADTEX "tex3.tga" // autom. index = 10, full path = gfx/tex3.tga and SMOOTH
STDPATH "sfx/"
SOUNDCOUNT 3 // Sounds count
LOADSOUND 0 "sound1.wav" LOW // index = 0, LOW is the quality (LOW, MEDUIM, HIGH) default: ignored
LOADSOUND "sound2.wav" // index = 1, normal quality, full path = sfx/sound2.wav
STDPATH ""
SOUNDINDEX 10
MODELINDEX 10
LOADMODEL "models/mod1.3ds" SHOW // index = 10
LOADMODEL 0 "models/mod2.ms3d" HIDE // index = 0 (force value), default: HIDE
LOADSOUND Nr1 "Filename.wav" [VOL = 0.3, PAN = -0.25, SPEED = 0.0] HIGH
LOADMODEL Nr2 "Filename.3ds" [X = 0.5, Y = 0.0, Z = -15] SHOW

*/

void ScriptLoader::readScript()
{
    
    io::stringc strLine;
    s32 commentIndex = 0;
    strLine_ = trimLine_ = stdPath_ = "";
    texIndex_ = soundIndex_ = modelIndex_ = 0;
    
    while (!hFileLoader_->isEOF(hFile_))
    {
        
        /* Read the full line */
        
        strLine_ = hFileLoader_->readString(hFile_);
        trimLine_ = strLine_.upper().trim();
        
        /* Check for comments */
        
        if (strLine_.trim().left(2) == "//")
            continue;
        if (strLine_.trim().left(2) == "/*")
            ++commentIndex;
        else if (commentIndex > 0 && strLine_.trim().left(2) == "*/")
            --commentIndex;
        if (commentIndex > 0)
            continue;
        
        /* Read the script parts */
        
        if (readPartMessages())
            continue;
        if (readPartIndices())
            continue;
        if (readPartPathes())
            continue;
        
        /* Read the current attribute list */
        readAttributeList();
        
        /* Read & load the current object */
        readPartLoading();
        
        /* Clear the attrbiute list */
        AttribtueList_.clear();
        
    }
    
}

bool ScriptLoader::readPartMessages()
{
    s32 len1 = strLine_.find("\"", 0)+1;
    s32 len2 = strLine_.find("\"", len1);
    io::stringc tStr;
    
    if (len1 != -1 && len2 != -1)
        tStr = strLine_.section(len1, len2);
    
    if (trimLine_.left(5) == "PRINT")
        io::printMessage(tStr);
    else if (trimLine_.left(5) == "ERROR")
        io::printError(tStr);
    else if (trimLine_.left(7) == "WARNING")
        io::printWarning(tStr);
    else
        return false;
    
    return true;
}

bool ScriptLoader::readPartIndices()
{
    if (trimLine_.left(8) == "TEXINDEX")
    {
        texIndex_ = strLine_.section( strLine_.find(" ", strLine_.find("TEXINDEX", 0)+8),
                                      strLine_.size() ).trim().val<s32>();
    }
    else if (trimLine_.left(10) == "SOUNDINDEX")
    {
        soundIndex_ = strLine_.section( strLine_.find(" ", strLine_.find("SOUNDINDEX", 0)+10),
                                        strLine_.size() ).trim().val<s32>();
    }
    else if (trimLine_.left(10) == "MODELINDEX")
    {
        modelIndex_ = strLine_.section( strLine_.find(" ", strLine_.find("MODELINDEX", 0)+10),
                                        strLine_.size() ).trim().val<s32>();
    }
    else if (trimLine_.left(10) == "SOUNDCOUNT")
    {
        soundCount_ = strLine_.section( strLine_.find(" ", strLine_.find("SOUNDCOUNT", 0)+10),
                                        strLine_.size() ).trim().val<s32>();
    }
    else
        return false;
    
    return true;
}

bool ScriptLoader::readPartPathes()
{
    if (trimLine_.left(7) == "STDPATH")
    {
        s32 len1 = strLine_.find("\"", 0);
        s32 len2 = strLine_.find("\"", len1+1);
        if (len1 != -1 && len2 != -1)
            stdPath_ = strLine_.section(len1+1, len2);
    } 
    else
        return false;
    
    return true;
}

bool ScriptLoader::readPartLoading()
{
    
    /* Temporary variables */
    s32 len1 = strLine_.find("\"", 0);
    s32 len2 = strLine_.find("\"", len1+1);
    
    io::stringc Filename;
    
    if (len1 != -1 && len2 != -1)
        Filename = strLine_.section(len1+1, len2);
    
    if (trimLine_.left(7) == "LOADTEX")
    {
        
        u8 RenderMode = video::TEXTURE_SMOOTH;
        
        /* Read the redner mode */
        if (trimLine_.right(6) == "LINEAR")
            RenderMode = video::TEXTURE_LINEAR;
        else if (trimLine_.right(6) == "SMOOTH")
            RenderMode = video::TEXTURE_SMOOTH;
        
        /* Load the texture */
        if (trimLine_.mid(7, 1) == "\"")
        {
            TextureList_[texIndex_++] = __spVideoDriver->loadTexture(stdPath_+Filename);//, RenderMode);
        }
        else
        {
            len1 = 8;
            len2 = strLine_.find("\"", 0)-1;
            texIndex_ = strLine_.section(len1, len2).trim().val<u32>();
            TextureList_[texIndex_] = __spVideoDriver->loadTexture(stdPath_+Filename);//, RenderMode);
        }
        
    }
    else if (trimLine_.left(9) == "LOADSOUND")
    {
        
        /* Temporary variables */
        u8 Quality = sound::SOUNDQUALITY_IGNORE;
        
        sound::Sound* CurSound;
        
        /* Read the quality */
        if (trimLine_.right(3) == "LOW")
            Quality = sound::SOUNDQUALITY_LOW;
        else if (trimLine_.right(6) == "MEDIUM")
            Quality = sound::SOUNDQUALITY_MEDIUM;
        else if (trimLine_.right(4) == "HIGH")
            Quality = sound::SOUNDQUALITY_HIGH;
        
        /* Load the sound */
        if (trimLine_.mid(9, 1) == "\"")
        {
            CurSound = SoundList_[soundIndex_++] = SoundLoader_->loadSound(stdPath_+Filename, Quality, soundCount_);
        }
        else
        {
            len1 = 10;
            len2 = strLine_.find("\"", 0)-1;
            soundIndex_ = strLine_.section(len1, len2).trim().val<u32>();
            CurSound = SoundList_[soundIndex_] = SoundLoader_->loadSound(stdPath_+Filename, Quality);
        }
        
        f32 TmpVol = 2.0f;
        
        /* Check the attributes */
        for (std::vector<SAttribute>::iterator it = AttribtueList_.begin(); it != AttribtueList_.end(); ++it)
        {
            if (it->Name == "VOL")
            {
                TmpVol = it->Value;
                SoundLoader_->setSoundVolume(CurSound, it->Value);
            }
            else if (it->Name == "PAN")
                SoundLoader_->setSoundPan(CurSound, it->Value, TmpVol);
            else if (it->Name == "SPEED")
                SoundLoader_->setSoundSpeed(CurSound, it->Value);
            else if (it->Name == "SEEK")
                SoundLoader_->setSoundSeek(CurSound, it->Value, true);
            else if (it->Name == "DO::PLAY")
                SoundLoader_->playSound(CurSound);
            else if (it->Name == "DO::PLAYLOOP")
                SoundLoader_->playSound(CurSound, true);
        }
        
    }
    else if (trimLine_.left(9) == "LOADMODEL" && SceneMngr_)
    {
        
        /* Temporary variables */
        bool isShow = false;
        
        /* Read the visiblity */
        if (trimLine_.right(4) == "HIDE")
            isShow = false;
        else if (trimLine_.right(4) == "SHOW")
            isShow = true;
        
        if (!hFileLoader_->doesFileExist(stdPath_+Filename))
            io::printError(io::stringc("could not found file: \"")+stdPath_+Filename+io::stringc("\""));
        
        /* Load the model */
        if (trimLine_.mid(9, 1) == "\"")
        {
            //EntityList_[modelIndex_] = SceneMngr_->loadModel(stdPath_+Filename);
            //EntityList_[modelIndex_]->setVisible(isShow);
            ++modelIndex_;
        }
        else
        {
            len1 = 10;
            len2 = strLine_.find("\"", 0)-1;
            modelIndex_ = strLine_.section(len1, len2).trim().val<u32>();
            //EntityList_[modelIndex_] = SceneMngr_->loadModel(stdPath_+Filename);
            //EntityList_[modelIndex_]->setVisible(isShow);
        }
        
    }
    else
        return false;
    
    return true;
    
}

bool ScriptLoader::readAttributeList()
{
    /* Temporary variables */
    io::stringc Atbts, Section;
    
    s32 pos1, pos2;
    
    SAttribute Item;
    
    /* Get the attributes line */
    if ( ( pos1 = strLine_.find("[", 1) ) == -1)
        return false;
    
    if ( ( pos2 = strLine_.find("]", 1) ) == -1)
        return false;
    
    Atbts = strLine_.section(pos1+1, pos2).upper();
    
    /* Reset the positions */
    pos1 = pos2 = 0;
    
    /* Loop while search for attributes */
    while (pos1 < Atbts.size())
    {
        
        /* Get the next section */
        if ( ( pos2 = Atbts.find(",", pos1) ) == -1 )
            pos2 = Atbts.size();
        
        /* Get the current section */
        Section = Atbts.section(pos1, pos2).trim();
        
        /* Check for doing operations */
        if (Section.left(4) == "DO::")
        {
            /* Set the item data */
            Item.Name   = Section;
            Item.Value  = 0;
            
            /* Add the item to the list */
            AttribtueList_.push_back(Item);
        }
        else
        {
            /* Get the spliting symbol */
            s32 tmp = Section.find("=", 1);
            
            /* Get the name */
            Item.Name = Section.left(tmp);
            
            /* Get the value */
            Item.Value = Section.right(Section.size() - tmp - 1).val<f32>();
            
            /* Add the item to the list */
            AttribtueList_.push_back(Item);
        }
        
        /* Update the first position */
        pos1 = pos2 + 1;
        
    }
    
    /* Exit the function */
    return true;
}





} // /namespace video

} // /namespace sp






// ================================================================================
