/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for all sound & music operations (sound, music, video-to-texture)
 *
 */

#ifndef __SP_SENSITIVESOUND_H__
#define __SP_SENSITIVESOUND_H__


#include "spStandard.hpp"
#include "spMemoryManagement.hpp"
#include "spSensitiveSoundSound.hpp"
#include "spSensitiveSoundRecord.hpp"
#include "spInputOutput.hpp"
#include "spMath.hpp"

#include <list>
#include <stdio.h>


namespace sp
{
namespace sound
{


static const s32 DEF_SOUND_ALL  = 0; // can be used instead of a "Sound" pointer


class SP_EXPORT SensitiveSoundDevice
{
    
    public:
        
        SensitiveSoundDevice(f32 NearHearing, f32 FarHearing, f32 NearVolume, f32 FarVolume);
        ~SensitiveSoundDevice();
        
        /* === Sounds === */
        
        // Loading & removing
        Sound* loadSound(io::stringc Filename, u8 Quality = SOUNDQUALITY_IGNORE, u8 Count = DEF_SOUND_LAYERCOUNT);
        void closeSound(Sound* hSound);
        
        // Playing
        void playSound(Sound* hSound, bool isLooped = false, bool isVolumetric = false);
        
        // Stoping
        void stopSound(Sound* hSound);
        
        // Pausing
        void pauseSound(Sound* hSound, bool isPaused = true);
        
        bool isSound(Sound* hSound);
        
        // Volume
        void setSoundVolume(Sound* hSound, f32 Volume, bool isOnlyCurrent = false); // 0.0f to +1.0f
        f32 getSoundVolume(Sound* hSound);
        
        // Seek (status)
        void setSoundSeek(Sound* hSound, f32 SeekPosition, bool isOnlyCurrent = false); // 0.0f to +1.0f
        f32 getSoundSeek(Sound* hSound);
        
        // Speed (frequency)
        void setSoundSpeed(Sound* hSound, f32 Speed = 0.0, bool isOnlyCurrent = false); // -1.0f to +1.0f
        f32 getSoundSpeed(Sound* hSound);
        
        // Pan (balance)
        void setSoundPan(Sound* hSound, f32 Balance = 0.0, f32 Volume = 1.0f, bool isOnlyCurrent = false); // -1.0f to +1.0f
        f32 getSoundPan(Sound* hSound);
        
        // State
        bool isSoundFinished(Sound* hSound);
        f32 getSoundLength(Sound* hSound);
        
        /* === Recordings === */
        
        Record* createRecord();
        void closeRecord(Record* hRecord);
        
        /* === Sensitive sound system === */
        
        void deleteDevice();
        
        void changeAttributes(f32 NearHearing, f32 FarHearing, f32 NearVolume, f32 FarVolume);
        
        void updateSounds3D(dim::vector3df CamPos, f32 CamYaw);
        void updateSounds();
        
    private:
        
        /* Functions */
        
        #ifdef WIN32
        std::string SndMciMsg(io::stringc Message);
        #endif
        #ifdef LINUX
        
        #endif
        
        /* Members */
        
        std::list<Sound*> SoundList_;
        
        f32 NearHearing_, FarHearing_;
        f32 NearVolume_, FarVolume_;
        f32 VolumeMiddle_, HearingMiddle_;
        
};


extern SensitiveSoundDevice* __ssDevice;

/* Creates the main SensitiveSound Device (for 2D and 3D sounds) */
SP_EXPORT SensitiveSoundDevice* createListener(
    f32 NearHearing = 0.0, f32 FarHearing = 100.0, f32 NearVolume = 1.0, f32 FarVolume = 0.0
);


} // /namespace sound;

} // /namespace sp;


#endif








// ================================================================================
