/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for sounds
 *
 */

#ifndef __SP_SENSITIVESOUND_SOUND_H__
#define __SP_SENSITIVESOUND_SOUND_H__


#include "spStandard.hpp"
#include "spDimensionVector3D.hpp"
#include "spInputOutputString.hpp"


namespace sp
{
namespace sound
{


static const s32 DEF_SOUND_LAYERCOUNT   = 3;


enum ESoundQualityTypes
{
    SOUNDQUALITY_IGNORE = 0,
    SOUNDQUALITY_LOW,
    SOUNDQUALITY_MEDIUM,
    SOUNDQUALITY_HIGH
};


class SensitiveSoundDevice;


class SP_EXPORT Sound
{
    
    public:
        
        Sound();
        Sound(io::stringc Filename, u8 Quality, u8 Count);
        ~Sound();
        
        /* Functions */
        
        void addIDCount(); // (Not very usful for users)
        
        void useSubSound(u32 Nr);
        
        /* Inline functions */
        
        inline u32 getID() const
        {
            return ID_;
        }
        inline u32 getCurID() const
        {
            return ID_ + CurID_;
        }
        inline u8 getCount() const
        {
            return Count_;
        }
        inline u8 getQuality() const
        {
            return Quality_;
        }
        inline io::stringc getFilename() const
        {
            return Filename_;
        }
        
        inline bool isPaused() const
        {
            return isPaused_;
        }
        inline bool isLooped() const
        {
            return isLooped_;
        }
        inline bool isVolumetric() const
        {
            return isVolumetric_;
        }
        
        inline dim::vector3df getPosition() const
        {
            return Position_;
        }
        inline void setPosition(dim::vector3df Position)
        {
            Position_ = Position;
        }
        
    private:
        
        friend class SensitiveSoundDevice;
        
        /* Functions */
        
        void init();
        
        /* Members */
        
        u32 ID_, CurID_;
        u8 Quality_, Count_;
        io::stringc Filename_;
        
        dim::vector3df Position_;
        bool isPaused_, isLooped_, isVolumetric_;
        
};


} // /namespace sound

} // /namespace sp


#endif






// ================================================================================
