/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * SoftwareRenderer Textures
 *
 */

#include "spSoftwareTexture.hpp"
#include "spTextureManager.hpp"
#include "spMainDevice.hpp"
#include "spSoftwareDriver.hpp"

#include "spRaySoftwareRenderer.h"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace video
{


SoftwareTexture::SoftwareTexture() : Texture()
{
}
SoftwareTexture::SoftwareTexture(
    u32 ID, io::stringc Filename, dim::size2di Size, u8* ImageData,
    s32 RenderMode, s32 FilterMode, s32 Format, s32 Type, bool isEdgeSmooth, bool isMipMapping)
    : Texture(ID, Filename, Size, ImageData, RenderMode, FilterMode, Format, Type, isEdgeSmooth, isMipMapping)
{
    /* Initialization */
    updateFormat();
    
    if (OriginalImageData_)
    {
        /* Update the image data */
        makeIdentityImageData();
        
        /* Recreate the image data */
        recreateImageData();
    }
}
SoftwareTexture::~SoftwareTexture()
{
    clear();
}

void SoftwareTexture::changeAttributes(s32 RenderMode, s32 FilterMode, bool isEdgeSmooth)
{
    /* Set the new render mode */
    RenderMode_ = RenderMode;
    
    /* Set the new filter moder */
    FilterMode_ = FilterMode;
}

void SoftwareTexture::setColorIntensity(f32 Red, f32 Green, f32 Blue)
{
    
}

void SoftwareTexture::flipColors()
{
    /* Check if the image is not empty */
    if (!ImageData_ || hasAnimation())
        return;
    
    /* Determine the image data size */
    const s32 ImageDataSize = Size_.Width * Size_.Height * InternalFormat_;
    
    /* Flip the image data */
    for (s32 i = 0; i < ImageDataSize; ++i)
        ImageData_[i] = 255 - ImageData_[i];
}

void SoftwareTexture::setMipMapFilter(s32 MinFilter, s32 MagFilter, bool isMipMapping)
{
}

void SoftwareTexture::updateImageData(EUpdateDataFunc UpdateDataFunc)
{
    /* Check if the image is not empty */
    if (ImageData_)
    {
        if (UpdateDataFunc == GET_IMAGEDATA)
            receiveImageData();
        else if (UpdateDataFunc == SET_IMAGEDATA)
            recreateImageData();
    }
}

void SoftwareTexture::bind(s32 Level)
{
    rayUseTextureLayer(Level);
    rayBindTexture(ID_);
}

void SoftwareTexture::unbind(s32 Level)
{
    rayUseTextureLayer(Level);
    rayUnbindTexture();
}

/*
 * ========== Private: ==========
 */


void SoftwareTexture::updateFormat()
{
    /* Use the default 1 component */
    InternalFormat_ = 1;
    
    /* Determine the SoftPixel type -> and set the renderer format */
    switch (Type_)
    {
        case TEXTURE_GRAYALPHA:
            InternalFormat_ = 2; break;
        case TEXTURE_RGB:
            InternalFormat_ = 3; break;
        case TEXTURE_RGBA:
            InternalFormat_ = 4; break;
    }
}

void SoftwareTexture::receiveImageData()
{
    /* Receive the image data */
    rayGetImage2D(ID_, 0, InternalFormat_, RAY_UNSIGNED_BYTE, ImageData_);
}

void SoftwareTexture::recreateImageData()
{
    #if 1 // !no other type supported yet!
    DimensionType_  = TEXTURE_2D;
    Depth_          = 1;
    #endif
    
    /* Update the correct format */
    updateFormat();
    
    /* Check if mipmap levels are in use */
    if (isMipMapping_)
    {
        rayBindTexture(ID_);
        
        /* Recreate the image data */
        static_cast<SoftwareDriver*>(__spVideoDriver)->build2DMipmaps(
            InternalFormat_, RAY_UNSIGNED_BYTE, Size_.Width, Size_.Height, ImageData_
        );
    }
    else
    {
        /* Recreate the image data */
        rayTexImage2D(ID_, 0, Size_.Width, Size_.Height, InternalFormat_, RAY_UNSIGNED_BYTE, ImageData_);
    }
}

void SoftwareTexture::clear()
{
    rayDeleteTexture(ID_);
}


} // /namespace video

} // /namespace sp







// ================================================================================
