/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Texture loading BMP-fileformat file
 *
 */

#include "spTextureLoaderBMP.hpp"


namespace sp
{
namespace video
{


TextureLoaderBMP::TextureLoaderBMP(io::stringc &Filename) : TextureLoaderRAW(Filename)
{
}
TextureLoaderBMP::~TextureLoaderBMP()
{
}

textureRAWInfo* TextureLoaderBMP::loadImageData()
{
    /* Check if the file has opened correct */
    if (isFileError())
        return 0;
    
    /* Allocate new texture RAW data & header buffer */
    textureRAWInfo* texture = new textureRAWInfo();
    
    headerBMPInfo HeaderInfo;
    
    /* === Header === */
    
    // Read the header
    fread(&HeaderInfo, 1, sizeof(headerBMPInfo), hFile_);
    
    // Check the identity
    if (HeaderInfo.ID != 0x4D42) // "BM"
    {
        io::printError("bitmap file has incorrect identity");
        return 0;
    }
    
    // General settings
    texture->Width  = HeaderInfo.Width;
    texture->Height = HeaderInfo.Height;
    
    /* === Color mask === */
    
    if (HeaderInfo.Compression == __SP_BMP_BI_BITFIELDS)
    {
        s32 Bitmasks[3];
        fread(Bitmasks, sizeof(s32), 3, hFile_);
        // !!! Compression is not supported yet !!!
    }
    
    /* === Color pallete === */
    
    u32* Pallete = 0;
    
    // Check the pallete
    if (HeaderInfo.Colors == 0)
    {
        if (HeaderInfo.bpp == 1 || HeaderInfo.bpp == 4 || HeaderInfo.bpp == 8)
        {
            Pallete = new u32[math::Pow2Pow(HeaderInfo.bpp)];
            fread(Pallete, sizeof(u32), math::Pow2Pow(HeaderInfo.bpp), hFile_);
        }
    }
    else
    {
        Pallete = new u32[HeaderInfo.Colors];
        fread(Pallete, sizeof(u32), HeaderInfo.Colors, hFile_);
    }
    
    /* === Color data === */
    
    HeaderInfo.BitmapDataSize = HeaderInfo.Width * HeaderInfo.Height * HeaderInfo.bpp / 8;
    
    // Get the bits per pixel & Allocate the image data memory
    switch (HeaderInfo.bpp)
    {
        case 8:
        {
            // Allocate memory for the image data
            texture->ImageData = new u8[HeaderInfo.Width * HeaderInfo.Height * 3];
            u8* pImageData = new u8[HeaderInfo.BitmapDataSize];
            
            // Read the body
            fread(pImageData, sizeof(u8), HeaderInfo.BitmapDataSize, hFile_);
            
            // Convert from 8 bits to 24 bits
            for (u32 i = 0; i < HeaderInfo.BitmapDataSize; ++i)
            {
                texture->ImageData[i*3+0] = video::getBlue  (Pallete[pImageData[i]]);
                texture->ImageData[i*3+1] = video::getGreen (Pallete[pImageData[i]]);
                texture->ImageData[i*3+2] = video::getRed   (Pallete[pImageData[i]]);
            }
            
            // Delete the temporary memory
            spDeletePointer(pImageData);
        }
        break;
        
        case 24:
        {
            // Allocate memory for the image data
            texture->ImageData = new u8[HeaderInfo.BitmapDataSize];
            
            // Read the body
            fread(texture->ImageData, sizeof(u8), HeaderInfo.BitmapDataSize, hFile_);
        }
        break;
        
        default:
        {
            io::printError(io::stringc("BMP file has unsupported bpp value (")+io::stringc(HeaderInfo.bpp)+io::stringc(")"));
        }
        return 0;
    }
    
    // Delete the pallete
    spDeletePointer(Pallete);
    
    // Flip colors
    TextureManager::flipImageColors(texture->Width, texture->Height, texture->ImageData);
    
    // Flip Y axis
    TextureManager::flipImageDataY(texture->Width, texture->Height, texture->ImageData);
    
    return texture;
}


} // /namespace video

} // /namespace sp






// ================================================================================
