/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Texture loading JPG-fileformat file
 *
 */

#include "spTextureLoaderJPG.hpp"

#ifdef __SP_EXPORT_TO_LIBRARY__
#   include "jpeglib/jpeglib.h"
#else
#   include <jpeglib.h> // "jpeg.lib"
#endif

namespace sp
{
namespace video
{


TextureLoaderJPG::TextureLoaderJPG(io::stringc &Filename) : TextureLoaderRAW(Filename)
{
}
TextureLoaderJPG::~TextureLoaderJPG()
{
}

textureRAWInfo* TextureLoaderJPG::loadImageData()
{
    /* Check if the file is not corrupt */
    if (checkForError())
        return 0;
    
    /* Allocate new texture RAW data & header buffer */
    textureRAWInfo* texture = new textureRAWInfo();
    
    jpeg_decompress_struct cInfo;
    
    jpeg_error_mgr jerr;
	
    cInfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&cInfo);
    jpeg_stdio_src(&cInfo, hFile_);
	
	/* Decompress the JPG image data */
    decompressJPG(&cInfo, texture);
    
    jpeg_destroy_decompress(&cInfo);
	
	return texture;
}

/*
 * ======= Private: =======
 */

bool TextureLoaderJPG::checkForError()
{
    /* Check if the file has opened correct */
    if (isFileError())
        return true;
    
    s32 JFIF = 0;
    
    fseek(hFile_, 6, SEEK_SET);
    fread(&JFIF, 1, sizeof(s32), hFile_);
    fseek(hFile_, 0, SEEK_SET);
    
    /* Check if the file is corrupt */
    if (!(JFIF == 0x4A464946 || JFIF == 0x4649464A))
    {
        /* Print a detailed error message */
        io::printError(io::stringc("JPEG file is corrupt and has incorrect JFIF header ( 0x")+io::getHex(JFIF)+io::stringc(" )"));
        
        /* An error has been detected, the file is corrupt */
        return true;
    }
    
    /* No error has been detected */
    return false;
}

void TextureLoaderJPG::decompressJPG(jpeg_decompress_struct* cInfo, textureRAWInfo* TextureInfo)
{
    /* Start the decompression */
    jpeg_read_header(cInfo, TRUE);
    jpeg_start_decompress(cInfo);
    
    /* Settings */
    s32 RowSpan             = cInfo->image_width * cInfo->num_components;
    TextureInfo->Width      = cInfo->image_width;
    TextureInfo->Height     = cInfo->image_height;
    TextureInfo->ImageData  = new u8[RowSpan * TextureInfo->Height];
    
    /* Loop the array y-axis */
    u8** rowPtr = new u8*[TextureInfo->Height];
    for (u32 i = 0; i < TextureInfo->Height; ++i)
        rowPtr[i] = &(TextureInfo->ImageData[i*RowSpan]);
    
    u32 rowsRead = 0;
    while (cInfo->output_scanline < cInfo->output_height)
        rowsRead += jpeg_read_scanlines(cInfo, &rowPtr[rowsRead], cInfo->output_height - rowsRead);
    
    /* Delete the array pointer */
    spDeleteArray(rowPtr);
    
    /* Finish the decompression */
    jpeg_finish_decompress(cInfo);
}


} // /namespace video

} // /namespace sp






// ================================================================================
