/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for texture loading PCX-fileformat
 *
 */

#ifndef __SP_TEXTURELOADER_PCX_H__
#define __SP_TEXTURELOADER_PCX_H__


#include "spStandard.hpp"
#include "spTextureManager.hpp"


namespace sp
{
namespace video
{


class SP_EXPORT TextureLoaderPCX : public TextureLoaderRAW
{
    
    public:
        
        TextureLoaderPCX(io::stringc &Filename);
        ~TextureLoaderPCX();
        
        textureRAWInfo* loadImageData();
        
    private:
        
        /* Structures */
        
        typedef struct SHeaderPCXInfo
        {
            u8 manufacturer, version;
            u8 encoding, bpp;
            
            u16 xmin, ymin;
            u16 xmax, ymax;
            u16 horzRes, vertRes;
            
            u8 palette[48];
            u8 reserved, numColorPlanes;
            
            u16 bytesPerScanLine, paletteType;
            u16 horzSize, vertSize;
            
            u8 padding[54];
        }
        headerPCXInfo;
        
        /* Functions */
        
        void readPCX1bit(headerPCXInfo* pHeader, textureRAWInfo* texture);
        void readPCX4bit(headerPCXInfo* pHeader, textureRAWInfo* texture);
        void readPCX8bit(headerPCXInfo* pHeader, textureRAWInfo* texture);
        void readPCX24bit(headerPCXInfo* pHeader, textureRAWInfo* texture);
        
};


} // /namespace video

} // /namespace sp


#endif






// ================================================================================
