/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for texture loading TGA-fileformat
 *
 */

#ifndef __SP_TEXTURELOADER_TGA_H__
#define __SP_TEXTURELOADER_TGA_H__


#include "spStandard.hpp"
#include "spTextureManager.hpp"


namespace sp
{
namespace video
{


class SP_EXPORT TextureLoaderTGA : public TextureLoaderRAW
{
    
    public:
        
        TextureLoaderTGA(io::stringc &Filename);
        ~TextureLoaderTGA();
        
        textureRAWInfo* loadImageData();
        
    private:
        
        /* Enumerations */
        
        enum ETGAImageTypes
        {
            TGA_IMGDATA_NONE                    = 0,
            
            TGA_IMGDATA_INDEXED                 = 1,
            TGA_IMGDATA_RGB                     = 2,
            TGA_IMGDATA_MONOCHROM               = 3,
            
            TGA_IMGDATA_INDEXED_COMPRESSED      = 9,
            TGA_IMGDATA_RGB_COMPRESSED          = 10,
            TGA_IMGDATA_MONOCHROM_COMPRESSED    = 11,
        };
        
        /* Structures */
        
        #if defined(_MSC_VER)
        #   pragma pack(push, packing)
        #   pragma pack(1)
        #   define __SP_PACK_STRUCT
        #elif defined(__GNUC__)
        #   define __SP_PACK_STRUCT __attribute__((packed))
        #else
        #   define __SP_PACK_STRUCT
        #endif
        
        typedef struct SHeaderTGAInfo
        {
            u8 IDSize;
            u8 ColorMapType;
            u8 ImageType; // (ETGAImageTypes)
            
            u16 ColorMapStart;
            u16 ColorMapSize;
            u8 ColorMapBits;
            
            u16 OriginX;
            u16 OriginY;
            u16 ImageWidth;
            u16 ImageHeight;
            
            u8 bpp;
            u8 ImageDescriptor;
        } __SP_PACK_STRUCT headerTGAInfo;
        
        #ifdef _MSC_VER
        #	pragma pack(pop, packing)
        #endif
        
        #undef __SP_PACK_STRUCT
        
        /* Functions */
        
        bool loadUncompressedTGA(textureRAWInfo* texture);
        bool loadCompressedTGA(textureRAWInfo* texture);
        
        /* Members */
        
        headerTGAInfo MainHeader_;
        
};


} // /namespace video

} // /namespace sp


#endif






// ================================================================================
