/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for texture loading WAD-fileformat
 *
 */

#ifndef __SP_TEXTURELOADER_WAD_H__
#define __SP_TEXTURELOADER_WAD_H__


#include "spStandard.hpp"
#include "spVideoDriverTexture.hpp"

#include <vector>


namespace sp
{
namespace video
{


class SP_EXPORT TextureLoaderWAD
{
    
    public:
        
        TextureLoaderWAD();
        ~TextureLoaderWAD();
        
        std::vector<Texture*> loadTextureList(io::stringc Filename);
        
    private:
        
        /* === Structures === */
        
        struct SHeaderWAD
        {
            c8 Magic[4];
            u32 CountTextures;
            u32 DirOffset;
        };
        
        struct STextureWAD
        {
            u32 Offset;
            u32 CompressedSize;
            u32 UncompressedSize;
            s8 Type;
            s8 CompressionType;
            s16 Padding;
            c8 Name[16];
        };
        
        struct SWallTextureBSP
        {
            c8 Name[16];
            u32 Width, Height;
            u32 Offset1;
            u32 Offset2;
            u32 Offset4;
            u32 Offset8;
        };
        
        /* === Functions === */
        
        void init();
        void clear();
        
        void reset();
        
        bool readHeader();
        void readTextureInfo();
        void readTextures();
        
        /* === Members === */
        
        io::FileLoader* FileLoader_;
        io::File* File_;
        
        SHeaderWAD Header_;
        
        std::vector<STextureWAD> TextureInfoList_;
        std::vector<Texture*> TextureList_;
        
};


} // /namespace video

} // /namespace sp


#endif






// ================================================================================
