/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Texture managing file
 *
 */

#include "spTextureManager.hpp"


namespace sp
{
namespace video
{


/*
 * ========== STextureRAWInfo structure ==========
 */

STextureRAWInfo::STextureRAWInfo()
    : ImageData(0), Format(TEXTURE_RGB), InternalFormat(3), bpp(24)
{
}


/*
 * ========== TextureManager class ==========
 */

TextureManager::TextureManager()
{
}
TextureManager::~TextureManager()
{
}

void TextureManager::flipImageColors(s32 Width, s32 Height, u8* ImageData, s32 InternalFormat)
{
    /* Loop the array */
    for (s32 y = 0, x, w = Width*InternalFormat; y < Height; ++y)
        for (x = 0; x < w; x += InternalFormat)
            math::Swap(ImageData[ y*w + x ], ImageData[ y*w + x + 2 ]);
}

void TextureManager::flipImageDataX(s32 Width, s32 Height, u8* ImageData, s32 InternalFormat)
{
    /* Temporary variables */
    s32 HalfWidth = Width*InternalFormat/2;
    
    /* Loop the array */
    for (s32 y = 0, x, w = Width*InternalFormat; y < Height; ++y)
    {
        for (x = 0; x < HalfWidth; x += 3)
        {
            math::Swap(ImageData[ y*w + ( w - x - 1 ) - 0 ], ImageData[ y*w + x + 2 ]);
            math::Swap(ImageData[ y*w + ( w - x - 1 ) - 1 ], ImageData[ y*w + x + 1 ]);
            math::Swap(ImageData[ y*w + ( w - x - 1 ) - 2 ], ImageData[ y*w + x + 0 ]);
        }
    }
}

void TextureManager::flipImageDataY(s32 Width, s32 Height, u8* ImageData, s32 InternalFormat)
{
    /* Temporary variables */
    s32 HalfHeight = Height/2;
    
    /* Loop the array */
    for (s32 y = 0, x, w = Width*InternalFormat; y < HalfHeight; ++y)
        for (x = 0; x < w; ++x)
            math::Swap(ImageData[ (Height - y - 1)*w + x ], ImageData[ y*w + x ]);
}

void TextureManager::scaleImageData(u8* &ImageData, s32 Width, s32 Height, s32 NewWidth, s32 NewHeight, s32 InternalFormat)
{
    
    /* Check if the memory is not empty */
    if ( !ImageData || ( Width == NewWidth && Height == NewHeight ) )
        return;
    
    /* Temporary variables */
    register s32 x, y, i, j, k;
    
    /* Allocate new memory */
    u8* NewImageData = new u8[ NewWidth * NewHeight * InternalFormat ];
    
    /* Loop for the new image size */
    for (y = 0; y < NewHeight; ++y)
    {
        for (x = 0; x < NewWidth; ++x)
        {
            for (i = 0; i < InternalFormat; ++i)
            {
                /* Get the image data indices */
                j = y * NewWidth + x;
                k = ( y * Height / NewHeight ) * Width + ( x * Width / NewWidth );
                
                /* Fill the new image data */
                NewImageData[ j * InternalFormat + i ] = ImageData[ k * InternalFormat + i ];
            }
        }
    }
    
    /* Delete the old memory */
    delete [] ImageData;
    
    /* Use the new memory */
    ImageData = NewImageData;
    
}

void TextureManager::scaleHalfImageData(u8* &ImageData, s32 Width, s32 Height, s32 InternalFormat)
{
    
    /* Check if the memory is not empty */
    if (!ImageData)
        return;
    
    /* Temporary variables */
    register s32 x, y, i, x2, y2;
    
    /* New image data size */
    s32 NewWidth    = Width;
    s32 NewHeight   = Height;
    
    /* Change the new size */
    if (NewWidth > 1)
        NewWidth >>= 1;
    if (NewHeight > 1)
        NewHeight >>= 1;
    
    /* Save the old image data */
    u8* OldImageData = ImageData;
    
    /* Allocate new image data */
    ImageData = new u8[ NewWidth * NewHeight * InternalFormat ];
    
    if (Width > 1 && Height > 1)
    {
        
        /* Loop for the half image size */
        for (y = y2 = 0; y < NewHeight; ++y, y2 = y << 1)
        {
            for (x = x2 = 0; x < NewWidth; ++x, x2 = x << 1)
            {
                for (i = 0; i < InternalFormat; ++i)
                {
                    /* Fill the current new data elements */
                    ImageData[ ( y * NewWidth + x ) * InternalFormat + i ] = (
                        OldImageData[ ( (y2    ) * Width + (x2    ) ) * InternalFormat + i ] +
                        OldImageData[ ( (y2    ) * Width + (x2 + 1) ) * InternalFormat + i ] +
                        OldImageData[ ( (y2 + 1) * Width + (x2    ) ) * InternalFormat + i ] +
                        OldImageData[ ( (y2 + 1) * Width + (x2 + 1) ) * InternalFormat + i ]
                    ) / 4;
                } // next color component
            } // next pixel
        } // next line
        
    }
    else if (Width == 1 || Height == 1)
    {
        
        s32 size = math::Max(NewWidth, NewHeight);
        
        /* Loop for the half image size */
        for (x = 0, x2 = 0; x < size; ++x, x2 = x << 1)
        {
            for (i = 0; i < InternalFormat; ++i)
            {
                /* Fill the current new data elements */
                ImageData[ x * InternalFormat + i ] = (
                    OldImageData[ ( x2     ) * InternalFormat + i ] +
                    OldImageData[ ( x2 + 1 ) * InternalFormat + i ]
                ) / 2;
            } // next color component
        } // next pixel
        
    }
    
    /* Delete the old image data */
    delete [] OldImageData;
    
}

/* ======= Conversion macors ======= */

#define __SP_CONVERT_1_TO_2(src, dest, i, j)            \
    dest[j+0] = src[i+0];                               \
    dest[j+1] = 255;
#define __SP_CONVERT_1_TO_3(src, dest, i, j)            \
    dest[j+0] = src[i+0];                               \
    dest[j+1] = src[i+0];                               \
    dest[j+2] = src[i+0];
#define __SP_CONVERT_1_TO_4(src, dest, i, j)            \
    dest[j+0] = src[i+0];                               \
    dest[j+1] = src[i+0];                               \
    dest[j+2] = src[i+0];                               \
    dest[j+3] = 255;

#define __SP_CONVERT_2_TO_1(src, dest, i, j)            \
    dest[j+0] = src[i+0];
#define __SP_CONVERT_2_TO_3(src, dest, i, j)            \
    dest[j+0] = src[i+0];                               \
    dest[j+1] = src[i+0];                               \
    dest[j+2] = src[i+0];
#define __SP_CONVERT_2_TO_4(src, dest, i, j)            \
    dest[j+0] = src[i+0];                               \
    dest[j+1] = src[i+0];                               \
    dest[j+2] = src[i+0];                               \
    dest[j+3] = src[i+1];

#define __SP_CONVERT_3_TO_1(src, dest, i, j)            \
    dest[j+0] = ( src[i+0] + src[i+1] + src[i+2] ) / 3;
#define __SP_CONVERT_3_TO_2(src, dest, i, j)            \
    dest[j+0] = ( src[i+0] + src[i+1] + src[i+2] ) / 3; \
    dest[j+1] = 255;
#define __SP_CONVERT_3_TO_4(src, dest, i, j)            \
    dest[j+0] = src[i+0];                               \
    dest[j+1] = src[i+1];                               \
    dest[j+2] = src[i+2];                               \
    dest[j+3] = 255;

#define __SP_CONVERT_4_TO_1(src, dest, i, j)            \
    dest[j+0] = ( src[i+0] + src[i+1] + src[i+2] ) / 3;
#define __SP_CONVERT_4_TO_2(src, dest, i, j)            \
    dest[j+0] = ( src[i+0] + src[i+1] + src[i+2] ) / 3; \
    dest[j+1] = src[i+3];
#define __SP_CONVERT_4_TO_3(src, dest, i, j)            \
    dest[j+0] = src[i+0];                               \
    dest[j+1] = src[i+1];                               \
    dest[j+2] = src[i+2];


void TextureManager::convertImageData(u8* &ImageData, s32 Width, s32 Height, s32 Type, s32 NewType)
{
    
    /* Check if the memory is not empty */
    if (!ImageData)
        return;
    
    /* Get the internal formats */
    s32 OldInternalFormat = getInternalFormat(Type);
    s32 NewInternalFormat = getInternalFormat(NewType);
    
    /* Check if the internal format has not changed */
    if (OldInternalFormat == NewInternalFormat)
        return;
    
    /* Temporary variables */
    register s32 x, y, i = 0, j = 0;
    
    /* Allocate new memory */
    u8* NewImageData = new u8[ Width * Height * NewInternalFormat ];
    
    /* Loop for the image size */
    for (y = 0; y < Height; ++y)
    {
        for (x = 0; x < Width; ++x, i += OldInternalFormat, j += NewInternalFormat)
        {
            
            /* Select the source's internal format */
            switch (OldInternalFormat)
            {
                case 1:
                    switch (NewInternalFormat)
                    {
                        case 2: __SP_CONVERT_1_TO_2(ImageData, NewImageData, i, j); break;
                        case 3: __SP_CONVERT_1_TO_3(ImageData, NewImageData, i, j); break;
                        case 4: __SP_CONVERT_1_TO_4(ImageData, NewImageData, i, j); break;
                    }
                    break;
                    
                case 2:
                    switch (NewInternalFormat)
                    {
                        case 1: __SP_CONVERT_2_TO_1(ImageData, NewImageData, i, j); break;
                        case 3: __SP_CONVERT_2_TO_3(ImageData, NewImageData, i, j); break;
                        case 4: __SP_CONVERT_2_TO_4(ImageData, NewImageData, i, j); break;
                    }
                    break;
                    
                case 3:
                    switch (NewInternalFormat)
                    {
                        case 1: __SP_CONVERT_3_TO_1(ImageData, NewImageData, i, j); break;
                        case 2: __SP_CONVERT_3_TO_2(ImageData, NewImageData, i, j); break;
                        case 4: __SP_CONVERT_3_TO_4(ImageData, NewImageData, i, j); break;
                    }
                    break;
                    
                case 4:
                    switch (NewInternalFormat)
                    {
                        case 1: __SP_CONVERT_4_TO_1(ImageData, NewImageData, i, j); break;
                        case 2: __SP_CONVERT_4_TO_2(ImageData, NewImageData, i, j); break;
                        case 3: __SP_CONVERT_4_TO_3(ImageData, NewImageData, i, j); break;
                    }
                    break;
            } // /switch
            
        } // next pixel
    } // next line
    
    /* Delete the old memory */
    delete [] ImageData;
    
    /* Use the new memory */
    ImageData = NewImageData;
    
}

void TextureManager::blurImageData(u8* &ImageData, s32 Width, s32 Height, s32 InternalFormat)
{
    
    /* Check if the iamge data is not empty */
    if (!ImageData)
        return;
    
    /* Temporary variables */
    register s32 y, x, i;
    
    /* Allocate new memory */
    u8* NewImageData = new u8[ Width * Height * InternalFormat ];
    
    /* Loop the image data (main body) */
    for (y = 0; y < Height - 1; ++y)
    {
        for (x = 0; x < Width - 1; ++x)
        {
            for (i = 0; i < InternalFormat; ++i)
            {
                /* Fill the new image data */
                NewImageData[ ( y * Width + x ) * InternalFormat + i ] = (
                    ImageData[ ( y * Width + x       ) * InternalFormat + i ] + 
                    ImageData[ ( y * Width + x+1     ) * InternalFormat + i ] + 
                    ImageData[ ( (y+1) * Width + x+1 ) * InternalFormat + i ] + 
                    ImageData[ ( (y+1) * Width + x   ) * InternalFormat + i ]
                ) / 4;
            }
        } // next pixel
    } // next line
    
    /* Loop the image data (edge - x axis) */
    for (x = 0; x < Width - 1; ++x)
    {
        for (i = 0; i < InternalFormat; ++i)
        {
            /* Fill the new image data */
            NewImageData[ ( (Height-1) * Width + x ) * InternalFormat + i ] = (
                ImageData[ ( (Height-1) * Width + x   ) * InternalFormat + i ] + 
                ImageData[ ( (Height-1) * Width + x+1 ) * InternalFormat + i ] + 
                ImageData[ ( x+1                      ) * InternalFormat + i ] + 
                ImageData[ ( x                        ) * InternalFormat + i ]
            ) / 4;
        }
    } // next pixel
    
    /* Loop the image data (edge - y axis) */
    for (y = 0; y < Height - 1; ++y)
    {
        for (i = 0; i < InternalFormat; ++i)
        {
            /* Fill the new image data */
            NewImageData[ ( y * Width + (Width-1) ) * InternalFormat + i ] = (
                ImageData[ ( y * Width + (Width-1)     ) * InternalFormat + i ] + 
                ImageData[ ( y * Width                 ) * InternalFormat + i ] + 
                ImageData[ ( (y+1) * Width + (Width-1) ) * InternalFormat + i ] + 
                ImageData[ ( (y+1) * Width             ) * InternalFormat + i ]
            ) / 4;
        }
    } // next pixel
    
    /* Loop the image data (edge right bottom) */
    for (i = 0; i < InternalFormat; ++i)
    {
        /* Fill the new image data */
        NewImageData[ ( (Height-1) * Width + (Width-1) ) * InternalFormat + i ] = (
            ImageData[ ( (Height-1) * Width + (Width-1) ) * InternalFormat + i ] + 
            ImageData[ ( (Height-1) * Width             ) * InternalFormat + i ] + 
            ImageData[                                                       i ] + 
            ImageData[ ( Width-1                        ) * InternalFormat + i ]
        ) / 4;
    }
    
    /* Delete the old memory */
    delete [] ImageData;
    
    /* Use the new memory */
    ImageData = NewImageData;
    
}

void TextureManager::changeImageData(u8* &ImageData, s32 Width, s32 Height, s32 Type, s32 NewWidth, s32 NewHeight, s32 NewType)
{
    /* Scale the iamge */
    scaleImageData(ImageData, Width, Height, NewWidth, NewHeight, getInternalFormat(Type));
    
    /* Convert the image */
    convertImageData(ImageData, Width, Height, Type, NewType);
}

s32 TextureManager::getInternalFormat(s32 Type)
{
    s32 InternalFormat;
    
    switch (Type)
    {
        case TEXTURE_GRAYALPHA:
            InternalFormat = 2; break;
        case TEXTURE_RGB:
            InternalFormat = 3; break;
        case TEXTURE_RGBA:
            InternalFormat = 4; break;
        default:
            InternalFormat = 1; break;
    };
    
    return InternalFormat;
}

bool TextureManager::checkImageSize(dim::size2di &InputSize)
{
    dim::size2di OutputSize;
    
    /* Get the correct size */
    OutputSize.Width    = math::Round2(InputSize.Width);
    OutputSize.Height   = math::Round2(InputSize.Height);
    
    /* Check if the size has changed */
    if (OutputSize != InputSize)
    {
        InputSize = OutputSize;
        return true;
    }
    
    return false;
}

s32 TextureManager::getMipmapLevelsCount(s32 Width, s32 Height)
{
    return math::Round( log(math::Max(Width, Height)) / log(2.0) );
}


/*
 * ========== TextureLoaderRAW class ==========
 */

TextureLoaderRAW::TextureLoaderRAW(io::stringc &Filename)
    : hFile_(0), isFileError_(true), Filename_(Filename)
{
    /* Open the file */
    if (hFile_ = fopen(Filename.c_str(), "rb"))
        isFileError_ = false;
    else
        io::printError(io::stringc("could not read file: \"")+Filename+io::stringc("\""));
}
TextureLoaderRAW::~TextureLoaderRAW()
{
    if (hFile_)
        fclose(hFile_);
}


/*
 * ========== TextureSaverRAW class ==========
 */

TextureSaverRAW::TextureSaverRAW(io::stringc &Filename)
    : hFile_(0), isFileError_(true), Filename_(Filename)
{
    /* Open the file */
    if (hFile_ = fopen(Filename.c_str(), "wb"))
        isFileError_ = false;
    else
        io::printError(io::stringc("could not create file: \"")+Filename+io::stringc("\""));
}
TextureSaverRAW::~TextureSaverRAW()
{
    if (hFile_)
        fclose(hFile_);
}


} // /namespace video

} // /namespace sp






// ================================================================================
