/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Texture saving BMP-fileformat file
 *
 */

#include "spTextureSaverBMP.hpp"


namespace sp
{
namespace video
{


TextureSaverBMP::TextureSaverBMP(io::stringc &Filename) : TextureSaverRAW(Filename)
{
}
TextureSaverBMP::~TextureSaverBMP()
{
}

bool TextureSaverBMP::saveImageData(textureRAWInfo* texture)
{
    /* Check if the file has opened correct */
    if (isFileError())
        return false;
    
    /* Get the image data */
    const u8* ImageData = texture->ImageData;
    
    /* Get the internal format */
    s32 InternalFormat = texture->InternalFormat;
    
    /* Check if the internal format is big anough */
    if (InternalFormat < 3)
        return false;
    
    /* Header settings */
    headerBMPInfo HeaderInfo;
    HeaderInfo.ID               = 0x4D42; // ("BM")
    HeaderInfo.Reserved         = 0;
    HeaderInfo.BitmapDataOffset = sizeof(HeaderInfo);
    HeaderInfo.BitmapHeaderSize = 0x28;
    HeaderInfo.Width            = texture->Width;
    HeaderInfo.Height           = texture->Height;
    HeaderInfo.Planes           = 1;
    HeaderInfo.bpp              = 24;
    HeaderInfo.Compression      = 0;
    HeaderInfo.PixelPerMeterX   = 0;
    HeaderInfo.PixelPerMeterY   = 0;
    HeaderInfo.Colors           = 0;
    HeaderInfo.ImportantColors  = 0;
    HeaderInfo.BitmapDataSize   = HeaderInfo.Width * HeaderInfo.Height * HeaderInfo.bpp / 8;
    HeaderInfo.FileSize         = HeaderInfo.BitmapDataOffset + HeaderInfo.BitmapDataSize;
    
    /* Write header into the image file */
    fwrite(&HeaderInfo, sizeof(HeaderInfo), 1, hFile_);
    
    /* Loop for the image data */
    for (s32 y = HeaderInfo.Height - 1, x; y >= 0; --y)
    {
        for (x = 0; x < HeaderInfo.Width; ++x)
        {
            /* Write the image data */
            fwrite(&(ImageData[ (y*HeaderInfo.Width+x)*InternalFormat + 2 ]), sizeof(u8), 1, hFile_);
            fwrite(&(ImageData[ (y*HeaderInfo.Width+x)*InternalFormat + 1 ]), sizeof(u8), 1, hFile_);
            fwrite(&(ImageData[ (y*HeaderInfo.Width+x)*InternalFormat + 0 ]), sizeof(u8), 1, hFile_);
        }
    }
    
    /* Loading successful */
    return true;
}


} // /namespace video

} // /namespace sp






// ================================================================================
