/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * user control
 *
 */

#include "spUserControl.hpp"
#include "spMainDevice.hpp"
#include "spInternalDeclerations.hpp"

#if defined(WIN32)
#   include <windows.h>
#   include <mmsystem.h>
#endif


namespace sp
{

extern SoftPixelDevice* __spDevice;


UserControl::UserControl() : isCursorSpeedBlocked_(false)
{
    LastCursorPos_ = getCursorPosition();
}
UserControl::~UserControl()
{
}

#if defined(WIN32)

void UserControl::setCursorPosition(dim::point2di Position)
{
    RECT rc;
    GetWindowRect(__spDevice->getWindowHandle(), &rc);
    SetCursorPos(Position.X+rc.left, Position.Y+rc.top);
    LastCursorPos_ = dim::point2di(Position.X, Position.Y);
}

dim::point2di UserControl::getCursorPosition()
{
    POINT Position;
    GetCursorPos(&Position);
    RECT rc;
    GetWindowRect(__spDevice->getWindowHandle(), &rc);
    return dim::point2di(Position.x-rc.left, Position.y-rc.top);
}

dim::point2di UserControl::getCursorSpeed()
{
    if (!isCursorSpeedBlocked_)
    {
        dim::point2di tPos(getCursorPosition());
        CursorSpeed_ = tPos - LastCursorPos_;
        LastCursorPos_ = tPos;
        isCursorSpeedBlocked_ = true;
    }
    return CursorSpeed_;
}

void UserControl::hideCursor(bool isHide)
{
    ShowCursor(!isHide);
}

s16 UserControl::getMouseWheel()
{
    return __MouseWheel;
}

/* Key/ mouse pressed */

bool UserControl::isKeyPressed(s16 KeyCode)
{
    return __isKey[KeyCode];
}
bool UserControl::isMouseButtonPressed(s16 ButtonType)
{
    return __isMouseKey[ButtonType];
}

/* Key/ mouse released */

bool UserControl::isKeyReleased(s16 KeyCode)
{
    return __wasKey[KeyCode];
}
bool UserControl::isMouseButtonReleased(s16 ButtonType)
{
    return __wasMouseKey[ButtonType];
}

/* Key/ mouse hit */

bool UserControl::isKeyHit(s16 KeyCode)
{
    return __hitKey[KeyCode];
}
bool UserControl::isMouseButtonHit(s16 ButtonType)
{
    return __hitMouseKey[ButtonType];
}

/* Key/ mouse simulate pressed */

void UserControl::doKeyPressed(s16 KeyCode)
{
    keybd_event((BYTE)KeyCode, (BYTE)KeyCode, 0, GetMessageExtraInfo());
}
void UserControl::doMouseButtonPressed(s16 ButtonType)
{
    switch (ButtonType)
    {
        case MOUSE_LEFT:
            ButtonType = MOUSEEVENTF_LEFTDOWN; break;
        case MOUSE_RIGHT:
            ButtonType = MOUSEEVENTF_RIGHTDOWN; break;
        case MOUSE_MIDDLE:
            ButtonType = MOUSEEVENTF_MIDDLEDOWN; break;
    }
    dim::point2di Pos(getCursorPosition());
    mouse_event(ButtonType, Pos.X, Pos.Y, 0, GetMessageExtraInfo());
}

/* Key/ mouse simulate released */

void UserControl::doKeyReleased(s16 KeyCode)
{
    keybd_event((BYTE)KeyCode, (BYTE)KeyCode, KEYEVENTF_KEYUP, GetMessageExtraInfo());
}
void UserControl::doMouseButtonReleased(s16 ButtonType)
{
    switch (ButtonType)
    {
        case MOUSE_LEFT:
            ButtonType = MOUSEEVENTF_LEFTUP; break;
        case MOUSE_RIGHT:
            ButtonType = MOUSEEVENTF_RIGHTUP; break;
        case MOUSE_MIDDLE:
            ButtonType = MOUSEEVENTF_MIDDLEUP; break;
    }
    dim::point2di Pos(getCursorPosition());
    mouse_event(ButtonType, Pos.X, Pos.Y, 0, GetMessageExtraInfo());
}

bool UserControl::isKeyPressedEx(s16 KeyCode)
{
    return HIWORD(GetKeyState(KeyCode));
}

void UserControl::setMouseWheel(s16 Value)
{
    dim::point2di Pos(getCursorPosition());
    mouse_event(MOUSEEVENTF_WHEEL, Pos.X, Pos.Y, Value, GetMessageExtraInfo());
}

dim::vector3df UserControl::getJoystickPosition() // (-1.0f - +1.0f)
{
    JOYINFO joyInfo;
    
    if (joyGetPos(JOYSTICKID1, &joyInfo) == JOYERR_NOERROR)
    {
        f32 X = (f32)joyInfo.wXpos / __SP_WORDSIZE - 1.0f;
        f32 Y = (f32)joyInfo.wYpos / __SP_WORDSIZE - 1.0f;
        f32 Z = (f32)joyInfo.wZpos / __SP_WORDSIZE;
        return dim::vector3df(X, -Y, Z);
    }
    
    return dim::vector3df();
}

// ! incomplete !
bool UserControl::isJoystickButtonPressed(u8 ButtonType)
{
    JOYINFO joyInfo;
    --ButtonType;
    
    if (joyGetPos(JOYSTICKID1, &joyInfo) == JOYERR_NOERROR)
    {
        u32 Count = (u32)pow(2, ButtonType);
        for (u32 i = 0; i < Count; i += (u32)pow(2, i))
        {
            if (joyInfo.wButtons == Count)
                return true;
            joyInfo.wButtons -= i;
        }
    }
    
    return false;
}

#elif defined(LINUX)

#endif


} // /namespace sp







// ================================================================================
