/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Video operations (drawing, coloring, textureing)
 *
 */

#include "spVideoDriver.hpp"
#include "spSceneManagerCamera.hpp"
#include "spSceneManager.hpp"


namespace sp
{

extern scene::SceneManager* __spSceneManager;

namespace video
{


effect::TextureManipulator* __spTextureManip    = 0;
movie::MovieManager*        __spMovieManager    = 0;
gui::GUI*                   __spGUI             = 0;

VideoDriver::VideoDriver()
{
}
VideoDriver::VideoDriver(EDriverTypes DriverType)
{
    /* General settings */
    DriverType_         = DriverType;
    
    FogMode_            = FOG_NONE;
    
    TexGenRenderMode_   = TEXTURE_SMOOTH;
    TexGenFilterMode_   = TEXTURE_TRILINEAR;
    TexGenAnisotropy_   = 1;
    TexGenMipMapping_   = true;
    TexGenEdgeSmooth_   = false;
    
    MaxClippingPlanes_  = 0;
    
    isFrontFace_        = true;
    
    scene::spProjectionMatrix   .matrixReset();
    scene::spViewMatrix         .matrixReset();
    scene::spWorldMatrix        .matrixReset();
    scene::spColorMatrix        .matrixReset();
    
    for (s32 i = 0; i < MAX_COUNT_OF_TEXTURES; ++i)
        scene::spTextureMatrix[i].matrixReset();
    
    setFillColor(255);
}
VideoDriver::~VideoDriver()
{
    /* Delete all extendet lists */
    for (std::list<Shader*>::iterator it = ShaderList_.begin(); it != ShaderList_.end(); ++it)
        spDeletePointer(*it);
    
    /* Delete all pointers */
    spDeletePointer(__spTextureManip);
    spDeletePointer(__spMovieManager);
}

/*
 * ========== All sub classes functions ==========
 */

/* Texture effect management are running with the texture manipulator */
effect::TextureManipulator* VideoDriver::getTextureManipulator()
{
    return __spTextureManip ? __spTextureManip : __spTextureManip = new effect::TextureManipulator();
}

/* Font manager for video sequence rendering */
movie::MovieManager* VideoDriver::getMovieManager()
{
    return __spMovieManager ? __spMovieManager : __spMovieManager = new movie::MovieManager();
}

/* GUI (GraphicsUserInterface) for simple windows-using */
gui::GUI* VideoDriver::getGUI()
{
    return __spGUI ? __spGUI : __spGUI = new gui::GUI();
}

void VideoDriver::clearGUI()
{
    spDeletePointer(__spGUI);
}


/*
 * === Video driver identity ===
 */

dim::EMatrixProjectionTypes VideoDriver::getProjectionMatrixType() const
{
    return dim::MATRIX_LEFTHANDED;
}

void VideoDriver::printWarning() { }

/*
 * === User controll functions ===
 */

void VideoDriver::clearBuffers() { }
void VideoDriver::flipBuffers() { }

/*
 * === Setting-/ getting functions ===
 */

void VideoDriver::setShadeMode(EShadeModeTypes ShadeMode) { }
void VideoDriver::setRenderMode(EDriverRenderModes RenderMode) { }
void VideoDriver::setPolygonMode(EDriverRenderModes RenderFace, EDriverRenderModes RenderMode) { }
void VideoDriver::setFrontFace(bool isFrontFace)
{
    isFrontFace_ = isFrontFace;
}

void VideoDriver::setClearColor(color Color) { }
void VideoDriver::setColorMask(bool isRed, bool isGreen, bool isBlue, bool isAlpha) { }

void VideoDriver::setVsync(bool isVsync) { }

void VideoDriver::setAntiAlias(bool isAntiAlias) { }

/*
 * === Context functions (for initializing the engine) ===
 */

void VideoDriver::initAntiAlias(
    HWND Wnd, bool &isAntiAlias, s32 &MultiSamplePixelFormat, s32 &MultiSampleFormat, s32 ColorDepth
) { }

/*
 * ========== Rendering 3D scenes ==========
 */

void VideoDriver::setMaterialStatesTriangleList(SMeshMaterial &Material, dim::vector3df &Rotation) { }

void VideoDriver::setMaterialStatesSprite(SMeshMaterial &Material) { }

void VideoDriver::drawTriangleList(
    u32 &BufferID, u32* IndicesList, u32 &IndicesCount, std::vector<SSurfaceTexture> &TextureList
) { }

void VideoDriver::drawTriangleList(u32 &BufferID, u32* IndicesList, u32 &IndicesCount) { }

void VideoDriver::drawSprite(
    Texture* &hTexture, f32 &RotationZ, video::color &Color, dim::vector3df &Scale, dim::point2df &BasePosition
) { }

void VideoDriver::updateLight(
    u32 &LightID, scene::ELightTypes &LightingType, bool &isVolumetric,
    dim::vector3df Direction, f32 &NearSpot, f32 &FarSpot,
    f32 &AttenuationConstant, f32 &AttenuationLinear, f32 &AttenuationQuadratic
) { }

void VideoDriver::enableVertexBuffer() { }
void VideoDriver::disableVertexBuffer() { }

void VideoDriver::enableVertexArray() { }
void VideoDriver::disableVertexArray() { }

void VideoDriver::createVertexBuffer(u32 &BufferID) { }
void VideoDriver::deleteVertexBuffer(u32 &BufferID) { }
void VideoDriver::updateVertexBuffer(u32 &BufferID, SMeshVertex3D* pVerticesList, s32 VerticesCount) { }

void VideoDriver::disableTriangleListStates() { }
void VideoDriver::disable3DRenderStates() { }
void VideoDriver::disableTexturing() { }

void VideoDriver::setDefaultAlphaBlending() { }

void VideoDriver::enableBlending() { }
void VideoDriver::disableBlending() { }

void VideoDriver::updateWireframeMode(s32 &ModeFront, s32 &ModeBack) { }

void VideoDriver::addDynamicLightSource(
    u32 LightID, scene::ELightTypes Type,
    video::color &Diffuse, video::color &Ambient, video::color &Specular,
    f32 AttenuationConstant_, f32 AttenuationLinear_, f32 AttenuationQuadratic_)
{ }
void VideoDriver::setLightStatus(u32 LightID, bool isEnable) { }
void VideoDriver::setLightColor(u32 LightID, video::color &Diffuse, video::color &Ambient, video::color &Specular) { }

void VideoDriver::setFogMode(s32 FogMode) { }
void VideoDriver::setFogColor(video::color Color) { }
void VideoDriver::setFogRange(f32 Range, f32 NearFog, f32 FarFog, s32 FogType) { }

void VideoDriver::clearStencilBuffer() { }

void VideoDriver::drawStencilShadowVolume(
    const dim::vector3df* pTriangleList, s32 Count, bool ZFailMethod, bool VolumetricShadow
) { }
void VideoDriver::drawStencilShadow(video::color Color) { }

void VideoDriver::setClipPlane(u32 Index, const dim::plane3df &Plane, bool Enable) { }



/*
 * ======= Shader effects =======
 */

Shader* VideoDriver::createShader(EStandardShaderTypes Type)
{
    Shader* NewShader = new Shader();
    
    ShaderList_.push_back(NewShader);
    
    return NewShader;
}

Shader* VideoDriver::loadShader(io::stringc FilenameVertex, io::stringc FilenameFragment, bool UseHighLevel)
{
    Shader* NewShader = new Shader();
    
    ShaderList_.push_back(NewShader);
    
    return NewShader;
}
Shader* VideoDriver::loadShader(
    io::stringc FilenameVertex, io::stringc FilenamePixel,
    io::stringc EntryPointVertex, io::stringc EntryPointPixel,
    EHLSLVertexShaderTypes CompilerTargetVertex,
    EHLSLPixelShaderTypes CompilerTargetPixel,
    bool UseHighLevel)
{
    return loadShader(FilenameVertex, FilenamePixel);
}

void VideoDriver::unbindShader() { }

void VideoDriver::deleteShader(Shader* &hShader)
{
    for (std::list<Shader*>::iterator it = ShaderList_.begin(); it != ShaderList_.end();)
    {
        if (*it == hShader)
            it = ShaderList_.erase(it);
        else
            ++it;
    }
    
    spDeletePointer(hShader);
}

/*
 * ======= Stack operation functions =======
 */

void VideoDriver::pushViewport() { }
void VideoDriver::popViewport() { }

void VideoDriver::pushMatrix()
{
    StackMatrix_ = scene::spWorldMatrix;
}

void VideoDriver::popMatrix()
{
    scene::spWorldMatrix = StackMatrix_;
}

/*
 * === Texturing ===
 */

void VideoDriver::bindTexture(u32 ID) { }
void VideoDriver::unbindTexture() { }

void VideoDriver::setTextureSubImage(u32 ID, s32 X, s32 Y, s32 Width, s32 Height, const u8* ImageData) { }

/*
 * ========== Drawing 2D objects ==========
 */

/* === General setting function === */

void VideoDriver::beginDrawing2D(bool isSettingViewport) { }
void VideoDriver::endDrawing2D() { }

void VideoDriver::beginDrawing3D(bool isLighting, bool isFog) { }
void VideoDriver::endDrawing3D() { }

void VideoDriver::beginAlphaBlending(u8 Type) { }
void VideoDriver::endAlphaBlending() { }

void VideoDriver::beginMasking(u8 Type) { }
void VideoDriver::endMasking() { }

void VideoDriver::beginTextureDrawing(Texture* hTexture, s32 RenderMode, u32 TextureDataType) { }
void VideoDriver::endTextureDrawing(bool isUpdateImageData) { }

void VideoDriver::beginDrawingRect(dim::rect2di Dimension) { }
void VideoDriver::beginDrawingRect(dim::point2di Position, dim::size2di Size) { }
void VideoDriver::endDrawingRect() { }

void VideoDriver::setDrawingArea(dim::rect2di Dimension) { }
void VideoDriver::setDrawingArea(dim::point2di Position, dim::size2di Size) { }

void VideoDriver::beginDrawingShader(Shader* hShader) { }
void VideoDriver::endDrawingShader() { }

void VideoDriver::bindTexture(Texture* hTexture, s32 Level) { }
void VideoDriver::unbindTexture(s32 Level) { }

void VideoDriver::setLineSize(s32 Size) { }
void VideoDriver::setPointSize(s32 Size) { }

void VideoDriver::setSolidMode(bool isSolidMode) { }

/* === Texture drawing === */

void VideoDriver::draw2DImage(Texture* hTexture, dim::point2di Position, color Color, bool isTransFilterUsing) { }

void VideoDriver::draw2DImage(
    Texture* hTexture, dim::rect2di Position, dim::rect2df Clipping, color Color, bool isTransFilterUsing
) { }

void VideoDriver::draw2DImage(
    Texture* hTexture, dim::point2di Position, f32 Rotation, f32 Radius, color Color, bool isTransFilterUsing
) { }

void VideoDriver::draw2DImage(
    Texture* hTexture, dim::rect2di Position, dim::rect2df Clipping, bool isTransFilterUsing,
    f32 Rotation, dim::point2df RotationPoint,
    color lefttopColor, color righttopColor, color rightbottomColor, color leftbottomColor
) { }

void VideoDriver::draw2DImage(
    Texture* hTexture,
    dim::point2di lefttopPosition, dim::point2di righttopPosition, dim::point2di rightbottomPosition, dim::point2di leftbottomPosition,
    dim::point2df lefttopClipping, dim::point2df righttopClipping, dim::point2df rightbottomClipping, dim::point2df leftbottomClipping,
    bool isTransFilterUsing,
    color lefttopColor, color righttopColor, color rightbottomColor, color leftbottomColor
) { }

/* === Primitive drawing === */

color VideoDriver::getPixel(dim::point2di Position) { }

void VideoDriver::draw2DPixel(dim::point2di Position, color Color) { }

void VideoDriver::draw2DLine(dim::point2di Position1, dim::point2di Position2, color Color) { }
void VideoDriver::draw2DLine(dim::point2di Position1, dim::point2di Position2, color Color1, color Color2) { }
void VideoDriver::draw2DLine(dim::point2di Position1, dim::point2di Position2, color Color, s32 DotLength) { }

void VideoDriver::draw2DRectangle(dim::rect2di Position, color Color, EDrawingModeTypes Type) { }
void VideoDriver::draw2DRectangle(
    dim::rect2di Position, color lefttopColor, color righttopColor,
    color rightbottomColor, color leftbottomColor, EDrawingModeTypes Type
) { }

void VideoDriver::draw2DRoundRectangle(dim::rect2di Position, dim::size2di CornerRadius, color Color) { }

void VideoDriver::draw2DCircle(dim::point2di Position, s32 Radius, color Color) { }

void VideoDriver::draw2DEllipse(dim::point2di Position, dim::size2di Radius, color Color) { }
void VideoDriver::draw2DEllipse(
    dim::point2di Position, dim::size2di Radius, f32 RotationDegree, color Color, bool isSolid
) { }

void VideoDriver::draw2DPolygon(SPrimitiveVertex* VerticesList, u32 AryCount, EDrawingModeTypes Type) { }
void VideoDriver::draw2DPolygonImage(
    Texture* hTexture, SPrimitiveVertex* VerticesList, u32 AryCount, EDrawingModeTypes Type)
{ }

/* === Extra drawing functions === */

void VideoDriver::draw2DArc(
    dim::point2di Position, dim::size2di Radius, s32 RadialDegree1, s32 RadialDegree2, f32 RotationDegree, color Color
) { }
void VideoDriver::draw2DBezier(
    dim::point2di Position1, dim::point2di Position2, dim::point2di Radial1, dim::point2di Radial2, color Color, f32 Detail
) { }
void VideoDriver::draw2DSpline(math::Spline2D &Spline, s32 Detail, dim::point2di Pos, color Color) { }
void VideoDriver::draw2DFloodFill(dim::point2di Position, color FillColor, color BorderColor) { }

/* === 3D drawing functions === */

void VideoDriver::draw3DPixel(dim::vector3df Position, color Color, scene::Camera* hCamera) { }
void VideoDriver::draw3DLine(
    dim::vector3df Position1, dim::vector3df Position2, color Color, scene::Camera* hCamera
) { }
void VideoDriver::draw3DLine(
    dim::vector3df Position1, dim::vector3df Position2, color Color1, color Color2, scene::Camera* hCamera
) { }
void VideoDriver::draw3DEllipse(
    dim::vector3df Position, dim::vector3df Rotation, dim::size2df Radius, color Color, scene::Camera* hCamera
) { }
void VideoDriver::draw3DTriangle(
    Texture* hTexture, dim::triangle3df Triangle, dim::vector3df Position, dim::vector3df Rotation,
    dim::vector3df Scale, color Color, scene::Camera* hCamera
) { }
void VideoDriver::draw3DBox(
    dim::vector3df Corner1, dim::vector3df Corner2, dim::vector3df Position, dim::vector3df Rotation,
    dim::vector3df Scale, color Color, scene::Camera* hCamera
) { }

void VideoDriver::draw3DBone(scene::Entity* Obj, dim::matrix4f &BoneTransformation, scene::Camera* hCamera) { }


/*
 * ========== Texture loading & creating ==========
 */

void VideoDriver::setTextureGenFlags(
    ETextureRenderModeTypes RenderMode, ETextureFilterTypes FilterMode, bool isMipMapping, bool isEdgeSmooth)
{
    TexGenRenderMode_ = RenderMode;
    TexGenFilterMode_ = FilterMode;
    TexGenMipMapping_ = isMipMapping;
    TexGenEdgeSmooth_ = isEdgeSmooth;
}

void VideoDriver::setTextureGenFlags(ETextureGenFlags Flag, s32 Value)
{
    switch (Flag)
    {
        case TEXGEN_RENDERMODE:
            TexGenRenderMode_ = (ETextureRenderModeTypes)Value; break;
        case TEXGEN_FILTER:
            TexGenFilterMode_ = (ETextureFilterTypes)Value; break;
        case TEXGEN_MIPMAPS:
            TexGenMipMapping_ = (Value != 0); break;
        case TEXGEN_CLAMPEDGES:
            TexGenEdgeSmooth_ = (Value != 0); break;
        case TEXGEN_ANISOTROPY:
            TexGenAnisotropy_ = Value; break;
    }
}

void VideoDriver::deleteTexture(Texture* &hTexture)
{
    if (hTexture)
    {
        /* Remove the texture from the texture list */
        for (std::vector<Texture*>::iterator it = TextureList_.begin(); it != TextureList_.end(); ++it)
        {
            if (*it == hTexture)
            {
                TextureList_.erase(it);
                break;
            }
        }
        
        /* Delete the pointer */
        spDeletePointer(hTexture);
    }
}

std::vector<Texture*> VideoDriver::loadTextureList(io::stringc Filename)
{
    return TextureLoaderWAD().loadTextureList(Filename);
}

/*
 * Saves the specified texture allways as a bitmap image
 */

bool VideoDriver::saveTexture(Texture* hTexture, io::stringc Filename)
{
    
    /* Information message */
    io::printMessage(io::stringc("Save texture: \"")+Filename+io::stringc("\""));
    
    if (hTexture)
    {
        
        /* Allocate some memory & temporary variables */
        
        textureRAWInfo* texture     = 0;
        TextureSaverRAW* texraw    = 0;
        
        io::stringc FileFormat4     = Filename.right(4).upper();
        io::stringc FileFormat5     = Filename.right(5).upper();
        
        /* === Select format === */
        
        if (FileFormat4 == ".BMP")
            texraw = new TextureSaverBMP(Filename);
        else
        {
            /* Error message */
            io::printError(io::stringc("texture file format: \"")+Filename.getFileExtension().upper()+io::stringc("\" is not supported"));
            
            /* Exit the function */
            return false;
        }
        
        /* Check if the texture raw file loader does exist */
        if (!texraw)
            return false;
        
        /* Allocate a new texture raw data structure */
        texture = new textureRAWInfo();
        
        /* Check if the creation of the texture raw structure was success */
        if (!texture)
            return false;
        
        /* === Fill the texture raw structure === */
        
        texture->Width          = hTexture->getSize().Width;
        texture->Height         = hTexture->getSize().Height;
        texture->InternalFormat = hTexture->getFormat();
        texture->ImageData      = hTexture->getImageData();
        
        /* Save the image */
        texraw->saveImageData(texture);
        
        /* Delete the raw texture loader */
        spDeletePointer(texraw);
        
        /* Delete the RAW texture */
        spDeletePointer(texture);
        
    }
    else
    {
        io::printWarning("empty texture pointer can not be used for saving images");
        return false;
    }
    
    /* Exit the function */
    return true;
    
}

void VideoDriver::createScreenShot(Texture* hTexture, dim::point2di Position) { }


/* === Cube map auto generation === */

void VideoDriver::updateCubeMapDirectionTexture(ECubeMapDirections Direction, Texture* CubeMapTexture) { }

void VideoDriver::updateCubeMapDirection(
    scene::Camera* Cam, dim::matrix4f CamDir, dim::vector3df Location, ECubeMapDirections Direction, Texture* CubeMapTexture)
{
    dim::matrix4f Mat;
    Mat.matrixReset();
    Mat.matrixTranslate(Location);
    Mat *= CamDir;
    
    switch (Direction)
    {
        case CUBEMAP_POSITIVE_X:
            Mat.matrixRotateY(-90);
            break;
        case CUBEMAP_NEGATIVE_X:
            Mat.matrixRotateY(90);
            break;
        case CUBEMAP_POSITIVE_Y:
            Mat.matrixRotateX(-90);
            break;
        case CUBEMAP_NEGATIVE_Y:
            Mat.matrixRotateX(90);
            break;
        case CUBEMAP_POSITIVE_Z:
            break;
        case CUBEMAP_NEGATIVE_Z:
            Mat.matrixRotateY(180);
            break;
    }
    
    Mat.matrixRotateZ(180);
    
    Cam->setMatrix(Mat);
    
    __spSceneManager->sortDepthEntityList(Cam);
    __spSceneManager->renderScene(Cam);
    
    updateCubeMapDirectionTexture(Direction, CubeMapTexture);
}

void VideoDriver::updateCubeMap(Texture* hTexture, dim::vector3df GlobalLocation)
{
    /* Check if the cube map can be updated */
    if (!hTexture || hTexture->getDimensionType() != TEXTURE_CUBEMAP || !__spSceneManager)
        return;
    
    /* Temporary variables */
    struct
    {
        dim::rect2di Viewport;
        f32 NearRange, FarRange, FieldOfView;
        scene::Camera* Camera;
    }
    Stack;
    
    /* Stack the last camera settings */
    Stack.Camera = __spSceneManager->getActiveCamera();
    
    if (!Stack.Camera)
        return;
    
    scene::Camera* Cam  = new scene::Camera();
    __spSceneManager->setActiveCamera(Cam);
    
    Stack.Camera->getPerspective(Stack.Viewport, Stack.NearRange, Stack.FarRange, Stack.FieldOfView);
    
    Cam->setAutoMatrix(false);
    Cam->setPerspective(
        dim::rect2di(0, 0, hTexture->getSize().Width, hTexture->getSize().Height / 6),
        Stack.NearRange, Stack.FarRange, 90
    );
    
    /* Transform the camera direction matrix */
    dim::matrix4f CamDir(Stack.Camera->getGlobalLocation());
    CamDir.setPosition(0);
    CamDir.matrixRotateX(180);
    CamDir.matrixRotateZ(180);
    
    /* Locate camera */
    updateCubeMapDirection(Cam, CamDir, GlobalLocation, CUBEMAP_POSITIVE_X, hTexture);
    updateCubeMapDirection(Cam, CamDir, GlobalLocation, CUBEMAP_NEGATIVE_X, hTexture);
    updateCubeMapDirection(Cam, CamDir, GlobalLocation, CUBEMAP_POSITIVE_Y, hTexture);
    updateCubeMapDirection(Cam, CamDir, GlobalLocation, CUBEMAP_NEGATIVE_Y, hTexture);
    updateCubeMapDirection(Cam, CamDir, GlobalLocation, CUBEMAP_POSITIVE_Z, hTexture);
    updateCubeMapDirection(Cam, CamDir, GlobalLocation, CUBEMAP_NEGATIVE_Z, hTexture);
    
    /* Back settings */
    __spSceneManager->setActiveCamera(Stack.Camera);
    
    spDeletePointer(Cam);
}


/* Normal map generation */

void VideoDriver::makeNormalMap(Texture* HeightMap, f32 Amplitude)
{
    /* Check if the texture it not empty */
    if (!HeightMap)
        return;
    
    /* Make sure the texture has at least 3 color components */
    HeightMap->setType(TEXTURE_RGB);
    
    /* Temporary variables */
    dim::vector3df p1, p2, p3;
    dim::vector3df Normal;
    
    s32 Width   = HeightMap->getSize().Width;
    s32 Height  = HeightMap->getSize().Height;
    
    const s32 ImageDataSize = Width * Height * 3;
    
    /* Copy the image data temporary */
    u8* ImageData = new u8[ImageDataSize];
    memcpy(ImageData, HeightMap->getImageData(), ImageDataSize);
    
    /* Loop for each texel */
    for (s32 y = 0, x; y < Height; ++y)
    {
        for (x = 0; x < Width; ++x)
        {
            /* Get the heights */
            p1.set(x, y, Amplitude * (f32)ImageData[(y * Width + x)*3] / 255);
            
            if (x < Width - 1)
                p2.set(x + 1, y, Amplitude * (f32)ImageData[(y * Width + x + 1)*3] / 255);
            else
                p2.set(x + 1, y, Amplitude * (f32)ImageData[(y * Width)*3] / 255);
            
            if (y < Height - 1)
                p3.set(x, y + 1, Amplitude * (f32)ImageData[((y + 1) * Width + x)*3] / 255);
            else
                p3.set(x, y + 1, Amplitude * (f32)ImageData[x*3] / 255);
            
            /* Compute the normal */
            Normal = math::getNormalVectorSQ(p1, p2, p3);
            Normal.normalize();
            
            Normal *= 0.5f;
            Normal += 0.5f;
            Normal *= 255.0f;
            
            /* Set the new texel */
            HeightMap->setPixelColor(
                dim::point2di(x, y), video::color((u8)Normal.X, (u8)Normal.Y, (u8)Normal.Z)
            );
        }
    }
    
    delete [] ImageData;
    
    /* Update the image data */
    HeightMap->updateImageData(SET_IMAGEDATA);
}


/*
 * ======= Texture list: reloading & clearing =======
 */

void VideoDriver::setFillColor(video::color Color)
{
    StdFillColor_[0] = Color.Red;
    StdFillColor_[1] = Color.Green;
    StdFillColor_[2] = Color.Blue;
    StdFillColor_[3] = Color.Alpha;
}

void VideoDriver::clearTextureList()
{
    for (std::vector<Texture*>::iterator it = TextureList_.begin(); it != TextureList_.end();)
    {
        if (isTexture(*it))
        {
            spDeletePointer(*it);
            it = TextureList_.erase(it);
        }
        else
            ++it;
    }
}

void VideoDriver::reloadTextureList()
{
    for (std::vector<Texture*>::iterator it = TextureList_.begin(); it != TextureList_.end(); ++it)
    {
        if (isTexture(*it))
            (*it)->updateImageData(video::SET_IMAGEDATA);
    }
}

/*
 * ========== Font loading & drawing ==========
 */

void VideoDriver::deleteFont(Font* hFont) { }

void VideoDriver::lockTexture(Texture* hTexture) { }
void VideoDriver::unlockTexture() { }

void VideoDriver::draw2DText(Font* hFont, dim::point2di Position, io::stringc Text, color Color) { }
void VideoDriver::draw3DText(
    Font* hFont, io::stringc Text, dim::vector3df Position, dim::vector3df Rotation, color Color, scene::Camera* hCamera) { }

/*
 * === Matrix controll ===
 */

void VideoDriver::matrixWorldViewReset()
{
    scene::spViewMatrix.matrixReset();
    scene::spWorldMatrix.matrixReset();
}

void VideoDriver::setProjectionMatrix(const dim::matrix4f &Matrix)
{
    scene::spProjectionMatrix = Matrix;
}
void VideoDriver::setViewMatrix(const dim::matrix4f &Matrix)
{
    scene::spViewMatrix = Matrix;
}
void VideoDriver::setWorldMatrix(const dim::matrix4f &Matrix)
{
    scene::spWorldMatrix = Matrix;
}
void VideoDriver::setTextureMatrix(const dim::matrix4f &Matrix, u8 TextureLayer)
{
    scene::spTextureMatrix[TextureLayer] = Matrix;
}
void VideoDriver::setColorMatrix(const dim::matrix4f &Matrix)
{
    scene::spColorMatrix = Matrix;
}

dim::matrix4f VideoDriver::getProjectionMatrix() const
{
    return scene::spProjectionMatrix;
}
dim::matrix4f VideoDriver::getViewMatrix() const
{
    return scene::spViewMatrix;
}
dim::matrix4f VideoDriver::getWorldMatrix() const
{
    return scene::spWorldMatrix;
}
dim::matrix4f VideoDriver::getTextureMatrix(u8 TextureLayer) const
{
    return scene::spTextureMatrix[TextureLayer];
}
dim::matrix4f VideoDriver::getColorMatrix() const
{
    return scene::spColorMatrix;
}


/*
 * === Other renderer option functions ===
 */

void VideoDriver::setViewport(s32 X, s32 Y, s32 Width, s32 Height) { }

void VideoDriver::setClippingRange(f32 Near, f32 Far)
{
    RangeNear_  = Near;
    RangeFar_   = Far;
}


} // /namespace video

} // /namespace sp






// ================================================================================
