/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * color header
 *
 */

#ifndef __SP_VIDEODRIVER_COLOR_H__
#define __SP_VIDEODRIVER_COLOR_H__


#include "spStandard.hpp"


namespace sp
{
namespace video
{


inline u32 getRed(u32 Color)
{
    return (Color >> 16) & 0xFF;
}
inline u32 getGreen(u32 Color)
{
    return (Color >> 8) & 0xFF;
}
inline u32 getBlue(u32 Color)
{
    return Color & 0xFF;
}
inline u32 getAlpha(u32 Color)
{
    return Color >> 24;
}
inline u32 getColor(u32 Red, u32 Green, u32 Blue, u32 Alpha)
{
    return ((Alpha & 0xFF) << 24) | ((Red & 0xFF) << 16) | ((Green & 0xFF) << 8) | (Blue & 0xFF);
}


class color
{
    
    public:
        
        color() : Red(0), Green(0), Blue(0), Alpha(0)
        {
        }
        color(u8 Brightness)
        {
            Red = Green = Blue = Brightness; Alpha = 255;
        }
        color(u8 R, u8 G, u8 B)
        {
            Red = R; Green = G; Blue = B; Alpha = 255;
        }
        color(u8 R, u8 G, u8 B, u8 A)
        {
            Red = R; Green = G; Blue = B; Alpha = A;
        }
        color(const color &other)
        {
            Red = other.Red; Green = other.Green; Blue = other.Blue; Alpha = other.Alpha;
        }
        ~color()
        {
        }
        
        /* === Operators - comparisions === */
        
        bool operator == (const color &other)
        {
            return Red == other.Red && Green == other.Green && Blue == other.Blue && Alpha == other.Alpha;
        }
        bool operator != (const color &other)
        {
            return Red != other.Red || Green != other.Green || Blue != other.Blue || Alpha != other.Alpha;
        }
        
        /* === Operators - addition, subtraction, division, multiplication === */
        
        color operator + (const color &other) const
        {
            return color(Red + other.Red, Green + other.Green, Blue + other.Blue, Alpha + other.Alpha);
        }
        color& operator += (const color &other)
        {
            Red += other.Red; Green += other.Green; Blue += other.Blue; Alpha += other.Alpha; return *this;
        }
        
        color operator - (const color &other) const
        {
            return color(Red - other.Red, Green - other.Green, Blue - other.Blue, Alpha - other.Alpha);
        }
        color& operator -= (const color &other)
        {
            Red -= other.Red; Green -= other.Green; Blue -= other.Blue; Alpha -= other.Alpha; return *this;
        }
        
        color operator / (const color &other) const
        {
            return color(Red/other.Red, Green/other.Green, Blue/other.Blue, Alpha/other.Alpha);
        }
        color& operator /= (const color &other)
        {
            Red /= other.Red; Green /= other.Green; Blue /= other.Blue; Alpha /= other.Alpha; return *this;
        }
        
        color operator / (u8 Factor) const
        {
            return color(Red/Factor, Green/Factor, Blue/Factor, Alpha);
        }
        color& operator /= (u8 Factor)
        {
            Red /= Factor; Green /= Factor; Blue /= Factor; return *this;
        }
        
        color operator * (const color &other) const
        {
            return color(Red*other.Red, Green*other.Green, Blue*other.Blue, Alpha*other.Alpha);
        }
        color& operator *= (const color &other)
        {
            Red *= other.Red; Green *= other.Green; Blue *= other.Blue; Alpha *= other.Alpha; return *this;
        }
        
        color operator * (u8 Factor) const
        {
            return color(Red*Factor, Green*Factor, Blue*Factor, Alpha*Factor);
        }
        color& operator *= (u8 Factor)
        {
            Red *= Factor; Green *= Factor; Blue *= Factor; Alpha *= Factor; return *this;
        }
        
        color operator * (f32 Factor) const
        {
            return color(u8(f32(Red)*Factor), u8(f32(Green)*Factor), u8(f32(Blue)*Factor), u8(f32(Alpha)*Factor));
        }
        color& operator *= (f32 Factor)
        {
            Red = u8(f32(Red)*Factor); Green = u8(f32(Green)*Factor); Blue = u8(f32(Blue)*Factor); Alpha = u8(f32(Alpha)*Factor);
            return *this;
        }
        
        /* === Additional operators === */
        
        inline const u8 operator [] (s32 i) const
        {
            switch (i)
            {
                case 0: return Red;
                case 1: return Green;
                case 2: return Blue;
                case 3: return Alpha;
            }
            return 0;
        }
        
        inline u8& operator [] (s32 i)
        {
            return *(&Red + i);
        }
        
        /* === Extra functions === */
        
        void flipColors(bool isRedFliped = true, bool isGreenFliped = true, bool isBlueFliped = true, bool isAlphaFliped = false)
        {
            Red     = isRedFliped   ? 255 - Red   : Red;
            Green   = isGreenFliped ? 255 - Green : Green;
            Blue    = isBlueFliped  ? 255 - Blue  : Blue;
            Alpha   = isAlphaFliped ? 255 - Alpha : Alpha;
        }
        
        void getFloatArray(f32* ColorArray)
        {
            ColorArray[0] = (f32)Red    / 255;
            ColorArray[1] = (f32)Green  / 255;
            ColorArray[2] = (f32)Blue   / 255;
            ColorArray[3] = (f32)Alpha  / 255;
        }
        
        void setIntensity(const color &other)
        {
            Red     = u8( ((s32)Red     * (s32)other.Red    ) / 255 );
            Green   = u8( ((s32)Green   * (s32)other.Green  ) / 255 );
            Blue    = u8( ((s32)Blue    * (s32)other.Blue   ) / 255 );
            Alpha   = u8( ((s32)Alpha   * (s32)other.Alpha  ) / 255 );
        }
        
        color getIntensity(const color &other)
        {
            color clr(*this);
            clr.setIntensity(other);
            return clr;
        }
        
        void setIntensity(f32 Intensity)
        {
            Red     = u8( ((f32)Red     * Intensity) );
            Green   = u8( ((f32)Green   * Intensity) );
            Blue    = u8( ((f32)Blue    * Intensity) );
            Alpha   = u8( ((f32)Alpha   * Intensity) );
        }
        
        color getIntensity(f32 Intensity)
        {
            color clr(*this);
            clr.setIntensity(Intensity);
            return clr;
        }
        
        void set(u8 R, u8 G, u8 B, u8 A)
        {
            Red = R; Green = G; Blue = B; Alpha = A;
        }
        
        u32 getSingle() const
        {
            return getColor(Red, Green, Blue, Alpha);
        }
        
        template <typename T> inline T getBrightness()
        {
            return static_cast<T>( static_cast<s32>(Red + Green + Blue)/3 );
        }
        
        /* Members */
        
        u8 Red, Green, Blue, Alpha;
        
};


} // /namespace video

} // /namespace sp


#endif







// ================================================================================
