/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Fonts
 *
 */

#include "spVideoDriverFont.hpp"
#include "spMainDevice.hpp"

#if defined(WIN32)
#   include <windows.h>
#endif


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace video
{


#if defined(WIN32)

Font::Font() : Texture_(0), DCFontID_(0)
{
    FontName_       = "";
    Size_           = dim::size2di();
    is3DSpaceUsing_ = false;
}
Font::Font(u32 ID, io::stringc FontName, dim::size2di Size, Texture* hTexture, s32 DCFontID, bool is3DSpaceUsing)
    : Texture_(hTexture), DCFontID_(DCFontID)
{
    ID_             = ID;
    FontName_       = FontName;
    Size_           = Size;
    is3DSpaceUsing_ = is3DSpaceUsing;
}
Font::~Font()
{
    if (DCFontID_)
        DeleteObject((HFONT)DCFontID_);
}

dim::size2di Font::getStringSize(io::stringc Text) const
{
    dim::size2di Size(Size_);
    
    HFONT hDCFont = (HFONT)DCFontID_;
    
    /* Check if the font was created as an device-context-font */
    if (hDCFont)
    {
        
        /* Get the last font & set the new font */
        HFONT hOldFont = (HFONT)SelectObject(__spVideoDriver->hDC_, hDCFont);
        
        /* Loop for the text size */
        for (register s32 i = 0, c = Text.size(), tmp; i < c; ++i)
        {
            /* Get the current char with */
            GetCharWidth(__spVideoDriver->getDeviceContext(), (s32)Text[i], (s32)Text[i], &tmp);
            
            /* Resize the output dimension */
            Size.Width += tmp;
        }
        
        /* Back setting to the old font */
        SelectObject(__spVideoDriver->getDeviceContext(), hOldFont);
        
    }
    
    return Size;
}

#elif defined(LINUX)

#endif


} // /namespace video

} // /namespace sp







// ================================================================================
