/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Graphics user interface (GUI)
 *
 */

#include "spVideoDriverGUI.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace video
{
namespace gui
{


// Sorting functions

bool sortGUIWindows(GUIWindow* &obj1, GUIWindow* &obj2)
{
    return obj1->getZAxis() > obj2->getZAxis();
}


GUI::GUI() : LastFocusedWindow_(0)
{
    /* Print header */
    io::printMessage("Graphics User Interface (GUI) - v.1.0");
    io::printMessage("Copyright (c) 2008 - Lukas Hermanns");
    io::printMessage("");
    
    /* Allocate managers */
    __spGUIfont = __spVideoDriver->loadFont("arial", 16, FONT_BOLD);
}
GUI::~GUI()
{
    // Elements
    for (std::list<GUIWindow*>::iterator it = WindowList_.begin(); it != WindowList_.end(); ++it)
        spDeletePointer(*it);
    
    if (__spGUIfont)
        __spVideoDriver->deleteFont(__spGUIfont);
}

// Creations/ deletings

GUIWindow* GUI::createWindow(io::stringc Title, dim::point2di Position, dim::size2di Size, s32 Param)
{
    static dim::point2di StdPos;
    
    if (Position == __SP_GUIWNDSTD_POS)
        Position = StdPos;
    if (Size == __SP_GUIWNDSTD_SIZE)
        Size = dim::size2di(__Screen_Width/2, __Screen_Height/2);
    
    GUIWindow* tWindow = __spGUIFocusedWindow = new GUIWindow(Title, Position, Size, Param);
    WindowList_.push_back(tWindow);
    updateGUIEvent();
    
    StdPos += dim::point2di((__Screen_Width/2)/__SP_GUIWNDSTD_POSADD_COUNT,
                            (__Screen_Height/2)/__SP_GUIWNDSTD_POSADD_COUNT);
    if (StdPos.X >= __Screen_Width/2 || StdPos.Y >= __Screen_Height/2)
        StdPos = dim::point2di();
    
    return tWindow;
}

bool GUI::deleteWindow(GUIWindow* hWindow)
{
    for (std::list<GUIWindow*>::iterator it = WindowList_.begin(); it != WindowList_.end(); ++it)
    {
        if (*it == hWindow)
        {
            WindowList_.erase(it);
            delete hWindow;
            return true;
        }
    }
    return false;
}

// Updating/ rendering

s32 GUI::updateGUIEvent()
{
    s32 tResult, Result = 0;
    
    __spGUIcurpos = __spGUIcontrol->getCursorPosition();
    
    std::list<GUIWindow*>::iterator it = WindowList_.end(), begin = WindowList_.begin(); --begin;
    for (--it; it != begin; --it)
    {
        tResult = (*it)->update();
        if (tResult)
            Result = tResult;
        if (dim::rect2di((*it)->getPosition().X, (*it)->getPosition().Y,
                          (*it)->getPosition().X+(*it)->getSize().Width, (*it)->getPosition().Y+(*it)->getSize().Height)
            .isPointCollided(__spGUIcurpos))
        {
            __spGUIForegroundWindow = *it;
            break;
        }
    }
    
    if (LastFocusedWindow_ != __spGUIFocusedWindow)
    {
        LastFocusedWindow_ = __spGUIFocusedWindow;
        s32 ZAxis = __spGUIFocusedWindow->getZAxis();
        for (std::list<GUIWindow*>::iterator it = WindowList_.begin(); it != WindowList_.end(); ++it)
            (*it)->updateZAxis(ZAxis);
        WindowList_.sort(sortGUIWindows);
    }
    
    return Result;
}

void GUI::renderGUI()
{
    
    __spVideoDriver->beginDrawing2D();
    {
        for (std::list<GUIWindow*>::iterator it = WindowList_.begin(); it != WindowList_.end(); ++it)
            (*it)->render();
    }
    __spVideoDriver->endDrawingRect();
    
}

void GUI::setRasterMode(bool isRasterUsing)
{
    __spGUIraster = isRasterUsing;
}

GUIWindow* GUI::getFocusedWindow()
{
    return __spGUIFocusedWindow;
}
GUIObject* GUI::getFocusedObject()
{
    return __spGUIFocusedObject;
}



} // /namespace gui

} // /namespace video

} // /namespace sp







// ================================================================================
