/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * GUI gadgets (buttons, text, lists ...)
 *
 */

#include "spVideoDriverGUIObjects.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace video
{
namespace gui
{


// Internal variables

Font* __spGUIfont               = 0;
UserControl* __spGUIcontrol     = 0;
dim::point2di __spGUIcurpos;
bool __spGUIraster = true;

GUIWindow* __spGUIForegroundWindow = 0;
GUIWindow* __spGUIFocusedWindow = 0;
GUIObject* __spGUIFocusedObject = 0;


// Sorting functions

bool sortGUIButtons(GUIButton* &obj1, GUIButton* &obj2)
{
    return obj1->getZAxis() < obj2->getZAxis();
}
bool sortGUIEditboxes(GUIEditbox* &obj1, GUIEditbox* &obj2)
{
    return obj1->getZAxis() < obj2->getZAxis();
}
bool sortGUILists(GUIList* &obj1, GUIList* &obj2)
{
    return obj1->getZAxis() < obj2->getZAxis();
}


/*
 * ========== GUI Objects ==========
 */

GUIObject::GUIObject() : hParent_(0)
{
    initGUIObject();
}
GUIObject::GUIObject(io::stringc Title, dim::point2di Position, dim::size2di Size, s32 Param)
{
    Title_      = Title;
    Position_   = OrigPosition_ = Position;
    Size_       = Size;
    Event_      = 0;
    Param_      = Param;
    ZAxis_      = 1000;
    isEnable_   = true;
    hParent_    = 0;
}
GUIObject::~GUIObject()
{
}

void GUIObject::initGUIObject()
{
    Title_      = "";
    Position_   = OrigPosition_ = dim::point2di();
    Size_       = dim::size2di();
    Event_      = 0;
    Param_      = 0;
    ZAxis_      = 1000;
    isEnable_   = true;
}


/*
 * ========== GUI Windows ==========
 */

GUIWindow::GUIWindow() : GUIObject()
{
    init();
}
GUIWindow::GUIWindow(io::stringc Title, dim::point2di Position, dim::size2di Size, s32 Param) :
    GUIObject(Title, Position, Size, Param)
{
    init();
    setParameter(Param);
    updateResize();
}
GUIWindow::~GUIWindow()
{
    clear();
}

GUIButton* GUIWindow::createButton(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param)
{
    GUIButton* tButton = new GUIButton(Title, Position, Size, Type, Param);
    ButtonList_.push_back(tButton);
    update();
    return tButton;
}
GUIEditbox* GUIWindow::createEditbox(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param)
{
    GUIEditbox* tEditbox = new GUIEditbox(Title, Position, Size, Type, Param);
    EditboxList_.push_back(tEditbox);
    update();
    return tEditbox;
}
GUIList* GUIWindow::createList(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param)
{
    GUIList* tList = new GUIList(Title, Position, Size, Type, Param);
    ListItemList_.push_back(tList);
    update();
    return tList;
}

bool GUIWindow::deleteButton(GUIButton* hButton)
{
    for (std::list<GUIButton*>::iterator it = ButtonList_.begin(); it != ButtonList_.end(); ++it)
    {
        if (*it == hButton)
        {
            ButtonList_.erase(it);
            spDeletePointer(hButton);
            return true;
        }
    }
    return false;
}
bool GUIWindow::deleteEditbox(GUIEditbox* hEditlist)
{
    for (std::list<GUIEditbox*>::iterator it = EditboxList_.begin(); it != EditboxList_.end(); ++it)
    {
        if (*it == hEditlist)
        {
            EditboxList_.erase(it);
            spDeletePointer(hEditlist);
            return true;
        }
    }
    return false;
}
bool GUIWindow::deleteList(GUIList* hList)
{
    for (std::list<GUIList*>::iterator it = ListItemList_.begin(); it != ListItemList_.end(); ++it)
    {
        if (*it == hList)
        {
            ListItemList_.erase(it);
            spDeletePointer(hList);
            return true;
        }
    }
    return false;
}

void GUIWindow::createMenu()
{
    isMenu_ = true;
}
void GUIWindow::deleteMenu()
{
    isMenu_ = false;
}
GUIMenu* GUIWindow::addMenuItem(s32 Type, io::stringc Title, GUIMenu* hParent)
{
    GUIMenu* tMenu = new GUIMenu(Type, Title, hParent);
    MenuList_.push_back(tMenu);
    update();
    return tMenu;
}
bool GUIWindow::removeMenuItem(GUIMenu* hMenuItem)
{
    for (std::list<GUIMenu*>::iterator it = MenuList_.begin(); it != MenuList_.end(); ++it)
    {
        if (*it == hMenuItem)
        {
            MenuList_.erase(it);
            spDeletePointer(hMenuItem);
            return true;
        }
    }
    return false;
}

s32 GUIWindow::update()
{
    s32 tResult, Result = 0;
    bool isWinRect = false;
    
    updateWindowEvents();
    Result = Event_;
    Event_ = 0;
    
    dim::point2di tPosition = Position_+dim::point2di(1, __SP_GUIWND_TITLE_HEIGHT);
    
    if (dim::rect2di(Position_.X, Position_.Y, Position_.X+Size_.Width, Position_.Y+Size_.Height).isPointCollided(__spGUIcurpos))
        isWinRect = true;
    
    for (std::list<GUIList*>::iterator it = ListItemList_.begin(); it != ListItemList_.end(); ++it)
    {
        tResult = (*it)->update(tPosition, isWinRect);
        if (tResult)
            Result = tResult;
    }
    
    for (std::list<GUIEditbox*>::iterator it = EditboxList_.begin(); it != EditboxList_.end(); ++it)
    {
        tResult = (*it)->update(tPosition, isWinRect);
        if (tResult)
            Result = tResult;
    }
    
    for (std::list<GUIButton*>::iterator it = ButtonList_.begin(); it != ButtonList_.end(); ++it)
    {
        tResult = (*it)->update(tPosition, isWinRect);
        if (tResult)
            Result = tResult;
    }
    
    for (std::list<GUIMenu*>::iterator it = MenuList_.begin(); it != MenuList_.end(); ++it)
    {
        tResult = (*it)->update(tPosition, isWinRect);
        if (tResult)
            Result = tResult;
    }
    
    if (Result)
    {
        ButtonList_.sort(sortGUIButtons);
        EditboxList_.sort(sortGUIEditboxes);
        ListItemList_.sort(sortGUILists);
    }
    
    return Result;
}

void GUIWindow::render()
{
    
    renderEmptyWindow();
    
    __spVideoDriver->beginDrawingRect(
        dim::point2di(Position_.X, Position_.Y+__SP_GUIWND_TITLE_HEIGHT+1),
        dim::size2di(Size_.Width-2, Size_.Height-__SP_GUIWND_TITLE_HEIGHT-2-__SP_GUIWND_STATUS_HEIGHT*(s32)isStatusBar_)
    );
    
    for (std::list<GUIList*>::iterator it = ListItemList_.begin(); it != ListItemList_.end(); ++it)
        (*it)->render();
    
    for (std::list<GUIEditbox*>::iterator it = EditboxList_.begin(); it != EditboxList_.end(); ++it)
        (*it)->render();
    
    for (std::list<GUIButton*>::iterator it = ButtonList_.begin(); it != ButtonList_.end(); ++it)
        (*it)->render();
    
    for (std::list<GUIMenu*>::iterator it = MenuList_.begin(); it != MenuList_.end(); ++it)
        (*it)->render();
    
    if (isMenu_)
    {
        __spVideoDriver->draw2DLine(
            dim::point2di(Position_.X, Position_.Y+__SP_GUIWND_TITLE_HEIGHT+__SP_GUIMENU_HEIGHT),
            dim::point2di(Position_.X+Size_.Width, Position_.Y+__SP_GUIWND_TITLE_HEIGHT+__SP_GUIMENU_HEIGHT), color(0)
        );
    }
    
    __spVideoDriver->endDrawingRect();
    
}

void GUIWindow::init()
{
    WndEventMode_       = WNDEVENT_NONE;
    WndMovingPos_       = dim::point2di();
    
    isButtonX_          = isButtonMin_  = isButtonMax_  = isButtonSize_ = false;
    isStatusBar_        = isMenu_       = isScrollVert_ = isScrollHorz_ = false;
    
    TitleBarLength_     = 0;
    MaxTitleBarLength_  = 0;
    StatusBarType_      = STATUSBAR_LEFT;
    StatusBarPos_       = 3;
    ScrollVertState_    = ScrollHorzState_ = 0;
    ScrollVertMax_      = ScrollHorzMax_ = 0;
    ScrollVertLength_   = ScrollHorzLength_ = 0;
    StatusBarText_      = io::stringc("");
    
    bgColor_            = color(128);
    titleLfColor_       = color(0, 0, 200);
    titleRtColor_       = color(200, 200, 255);
    
    LastSize_           = dim::size2di(-1, -1);
    LastPosition_       = dim::point2di(-1, -1);
    MinSize_            = dim::size2di(100, __SP_GUIWND_TITLE_HEIGHT+1);
    MaxSize_            = dim::size2di(__Screen_Width, __Screen_Height);
    RenderRect_         = dim::rect2di();
}

void GUIWindow::clear()
{
    // Elements
    for (std::list<GUIList*>::iterator it = ListItemList_.begin(); it != ListItemList_.end(); ++it)
        spDeletePointer(*it);
    for (std::list<GUIEditbox*>::iterator it = EditboxList_.begin(); it != EditboxList_.end(); ++it)
        spDeletePointer(*it);
    for (std::list<GUIButton*>::iterator it = ButtonList_.begin(); it != ButtonList_.end(); ++it)
        spDeletePointer(*it);
    for (std::list<GUIMenu*>::iterator it = MenuList_.begin(); it != MenuList_.end(); ++it)
        spDeletePointer(*it);
    // Clear lists
    ListItemList_.clear();
    EditboxList_.clear();
    ButtonList_.clear();
    MenuList_.clear();
}

void GUIWindow::renderEmptyWindow()
{
    
    /* === Render body (background) === */
    
    color tColor = color(s32(bgColor_.Red+255)/2, s32(bgColor_.Green+255)/2, s32(bgColor_.Blue+255)/2, bgColor_.Alpha);
    
    __spVideoDriver->draw2DRectangle(
        dim::rect2di(
            Position_.X, Position_.Y+__SP_GUIWND_TITLE_HEIGHT,
            Position_.X+Size_.Width-2, Position_.Y+Size_.Height-1
        ),
        color(255, 255, 255, bgColor_.Alpha), tColor, bgColor_, tColor
    );
    __spVideoDriver->draw2DPixel(dim::point2di(Position_.X-1, Position_.Y+Size_.Height), color(0));
    __spVideoDriver->draw2DRectangle(
        dim::rect2di(
            Position_.X, Position_.Y+__SP_GUIWND_TITLE_HEIGHT,
            Position_.X+Size_.Width-1, Position_.Y+Size_.Height-1
        ),
        color(0), DRAW_FRAME
    );
    
    /* === Render head (title) === */
    
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X, Position_.Y+4),
        dim::point2di(Position_.X, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleLfColor_
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X+1, Position_.Y+2),
        dim::point2di(Position_.X+1, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleLfColor_
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X+2, Position_.Y+1),
        dim::point2di(Position_.X+2, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleLfColor_
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X+3, Position_.Y+1),
        dim::point2di(Position_.X+3, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleLfColor_
    );
    
    __spVideoDriver->draw2DRectangle(
        dim::rect2di(
            Position_.X+3, Position_.Y,
            Position_.X+Size_.Width-4, Position_.Y+__SP_GUIWND_TITLE_HEIGHT),
        titleLfColor_, titleRtColor_, titleRtColor_, titleLfColor_
    );
    
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X+Size_.Width-1, Position_.Y+4),
        dim::point2di(Position_.X+Size_.Width-1, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleRtColor_
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X+Size_.Width-2, Position_.Y+2),
        dim::point2di(Position_.X+Size_.Width-2, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleRtColor_
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X+Size_.Width-3, Position_.Y+1),
        dim::point2di(Position_.X+Size_.Width-3, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleRtColor_
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position_.X+Size_.Width-4, Position_.Y+1),
        dim::point2di(Position_.X+Size_.Width-4, Position_.Y+__SP_GUIWND_TITLE_HEIGHT), titleRtColor_
    );
    
    /* === Render title bar === */
    
    s32 tbPos = 0;
    if (Position_.Y < __SP_GUIWND_TITLE_HEIGHT/4-1 && Position_.Y > -__SP_GUIWND_TITLE_HEIGHT/4+1)
        tbPos = __SP_GUIWND_TITLE_HEIGHT/4-Position_.Y-1;
    
    __spVideoDriver->draw2DEllipse(
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX,
            Position_.Y+__SP_GUIWND_TITLE_HEIGHT/2-__SP_GUIWND_TITLE_POSY+tbPos-1),
        dim::size2di(__SP_GUIWND_TITLE_HEIGHT/2-1, __SP_GUIWND_TITLE_HEIGHT/2),
        0, __SP_GUIWND_TITLE_COLOR_LF, true
    );
    __spVideoDriver->draw2DEllipse(
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX+TitleBarLength_+1,
            Position_.Y+__SP_GUIWND_TITLE_HEIGHT/2-__SP_GUIWND_TITLE_POSY+tbPos-1),
        dim::size2di(__SP_GUIWND_TITLE_HEIGHT/2-1, __SP_GUIWND_TITLE_HEIGHT/2),
        0, __SP_GUIWND_TITLE_COLOR_RT, true
    );
    
    __spVideoDriver->draw2DCircle(
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX,
            Position_.Y+__SP_GUIWND_TITLE_HEIGHT/2-__SP_GUIWND_TITLE_POSY+tbPos),
        __SP_GUIWND_TITLE_HEIGHT/2, color(0)
    );
    __spVideoDriver->draw2DCircle(
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX+TitleBarLength_,
            Position_.Y+__SP_GUIWND_TITLE_HEIGHT/2-__SP_GUIWND_TITLE_POSY+tbPos),
        __SP_GUIWND_TITLE_HEIGHT/2, color(0)
    );
    
    __spVideoDriver->draw2DRectangle(
        dim::rect2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX-3,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos,
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX+TitleBarLength_,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos+__SP_GUIWND_TITLE_HEIGHT-1),
        __SP_GUIWND_TITLE_COLOR_LF, __SP_GUIWND_TITLE_COLOR_RT,
        __SP_GUIWND_TITLE_COLOR_RT, __SP_GUIWND_TITLE_COLOR_LF
    );
    
    __spVideoDriver->draw2DLine(
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos-1),
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX+TitleBarLength_,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos-1),
        color(0)
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos+__SP_GUIWND_TITLE_HEIGHT-1),
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX+TitleBarLength_,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos+__SP_GUIWND_TITLE_HEIGHT-1),
        color(0)
    );
    
    io::stringc winTitle;
    
    if (MaxTitleBarLength_ == TitleBarLength_)
    {
        s32 c = Title_.size() - 1;
        while (__spGUIfont->getStringSize(Title_.left(c) + "...").Width > TitleBarLength_)
            --c;
        winTitle = Title_.left(c) + "...";
    }
    else
        winTitle = Title_;
    
    __spVideoDriver->draw2DText(
        __spGUIfont,
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX+1,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos-2+1),
        winTitle, color(0)
    );
    __spVideoDriver->draw2DText(
        __spGUIfont,
        dim::point2di(
            Position_.X+__SP_GUIWND_TITLE_HEIGHT/2+__SP_GUIWND_TITLE_POSX,
            Position_.Y-__SP_GUIWND_TITLE_POSY+tbPos-2),
        winTitle
    );
    
    /* === Render system menu buttons (X, Min, Max) === */
    
    s32 BtnPosX = Position_.X+Size_.Width-__SP_GUIWND_BTN_SIZE-__SP_GUIWND_BTN_SIZE/2;
    s32 BtnPosY = Position_.Y+__SP_GUIWND_TITLE_HEIGHT/2-__SP_GUIWND_BTN_SIZE/2;
    s32 BtnPos = 0;
    
    if (isButtonX_)
    {
        if (drawButton(dim::point2di(BtnPosX, BtnPosY)))
        {
            if (__spGUIcontrol->isMouseButtonPressed(MOUSE_LEFT))
                BtnPos = 1;
            if (__spGUIcontrol->isMouseButtonReleased(MOUSE_LEFT))
                Event_ = WINDOW_CLOSE;
        }
        
        __spVideoDriver->draw2DLine(
            dim::point2di(BtnPosX+BtnPos+1, BtnPosY+BtnPos+1),
            dim::point2di(BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-1, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-1),
            color(0)
        );
        __spVideoDriver->draw2DLine(
            dim::point2di(BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-1, BtnPosY+BtnPos+1),
            dim::point2di(BtnPosX+BtnPos+1, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-1),
            color(0)
        );
        
        __spVideoDriver->draw2DLine(
            dim::point2di(BtnPosX+BtnPos+2, BtnPosY+BtnPos+1),
            dim::point2di(BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-1, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-2),
            color(0)
        );
        __spVideoDriver->draw2DLine(
            dim::point2di(BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-1, BtnPosY+BtnPos+2),
            dim::point2di(BtnPosX+BtnPos+2, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-1),
            color(0)
        );
        
        __spVideoDriver->draw2DLine(
            dim::point2di(BtnPosX+BtnPos+1, BtnPosY+BtnPos+2),
            dim::point2di(BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-2, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-1),
            color(0)
        );
        __spVideoDriver->draw2DLine(
            dim::point2di(BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-2, BtnPosY+BtnPos+1),
            dim::point2di(BtnPosX+BtnPos+1, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-2),
            color(0)
        );
        
        BtnPosX -= __SP_GUIWND_BTN_SIZE+__SP_GUIWND_BTN_DIST;
    }
    
    if (isButtonMax_)
    {
        BtnPos = 0;
        if (drawButton(dim::point2di(BtnPosX, BtnPosY)))
        {
            if (__spGUIcontrol->isMouseButtonPressed(MOUSE_LEFT))
                BtnPos = 1;
            if (__spGUIcontrol->isMouseButtonReleased(MOUSE_LEFT))
            {
                if (LastSize_ == dim::size2di(-1, -1))
                {
                    LastPosition_ = Position_;
                    Position_ = dim::point2di(0, 0);
                    LastSize_ = Size_;
                    Size_ = dim::size2di(__Screen_Width, __Screen_Height);
                    Event_ = WINDOW_MAXIMIZE;
                }
                else
                {
                    Position_ = LastPosition_;
                    LastPosition_ = dim::point2di(-1, -1);
                    Size_ = LastSize_;
                    LastSize_ = dim::size2di(-1, -1);
                    Event_ = WINDOW_NORMALIZE;
                }
                OrigPosition_ = Position_;
                updateResize();
            }
        }
        
        __spVideoDriver->draw2DRectangle(
            dim::rect2di(
                BtnPosX+BtnPos+2, BtnPosY+BtnPos+1,
                BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-1, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-2),
            color(0), DRAW_FRAME
        );
        __spVideoDriver->draw2DLine(
            dim::point2di(
                BtnPosX+BtnPos+2, BtnPosY+BtnPos+2),
                dim::point2di(BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-1, BtnPosY+BtnPos+2),
            color(0)
        );
        
        BtnPosX -= __SP_GUIWND_BTN_SIZE+__SP_GUIWND_BTN_DIST;
    }
    
    if (isButtonMin_)
    {
        BtnPos = 0;
        if (drawButton(dim::point2di(BtnPosX, BtnPosY)))
        {
            if (__spGUIcontrol->isMouseButtonPressed(MOUSE_LEFT))
                BtnPos = 1;
            if (__spGUIcontrol->isMouseButtonReleased(MOUSE_LEFT))
            {
                if (LastSize_ == dim::size2di(-1, -1))
                {
                    LastPosition_ = Position_;
                    LastSize_ = Size_;
                    Size_ = dim::size2di(Size_.Width, 0);
                    Event_ = WINDOW_MINIMIZE;
                }
                else
                {
                    Position_ = LastPosition_;
                    LastPosition_ = dim::point2di(-1, -1);
                    Size_ = LastSize_;
                    LastSize_ = dim::size2di(-1, -1);
                    Event_ = WINDOW_NORMALIZE;
                }
                OrigPosition_ = Position_;
                updateResize();
            }
        }
        
        __spVideoDriver->draw2DRectangle(
            dim::rect2di(
                BtnPosX+BtnPos+1, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-3,
                BtnPosX+BtnPos+__SP_GUIWND_BTN_SIZE-3, BtnPosY+BtnPos+__SP_GUIWND_BTN_SIZE-1),
            color(0)
        );
    }
    
    /* === Render status bar === */
    
    if (isStatusBar_)
    {
        
        __spVideoDriver->draw2DRectangle(
            dim::rect2di(
                Position_.X, Position_.Y+Size_.Height-__SP_GUIWND_STATUS_HEIGHT-1,
                Position_.X+Size_.Width-2, Position_.Y+Size_.Height-1),
            color(255), color(255), bgColor_, bgColor_
        );
        __spVideoDriver->draw2DText(
            __spGUIfont, dim::point2di(Position_.X+StatusBarPos_, Position_.Y+Size_.Height-__SP_GUIWND_STATUS_HEIGHT-__SP_GUIWND_STATUS_HEIGHT/2+2),
            StatusBarText_, color(64, 64, 255)
        );
        
        // Render size button
        
        if (isButtonSize_)
        {
            dim::point2di ResizePos(Position_.X+Size_.Width-1, Position_.Y+Size_.Height-1);
            __spVideoDriver->draw2DLine(ResizePos-dim::point2di(3, 0), ResizePos-dim::point2di(0, 3), color(0));
            __spVideoDriver->draw2DLine(ResizePos-dim::point2di(6, 0), ResizePos-dim::point2di(0, 6), color(0));
            __spVideoDriver->draw2DLine(ResizePos-dim::point2di(9, 0), ResizePos-dim::point2di(0, 9), color(0));
            __spVideoDriver->draw2DLine(ResizePos-dim::point2di(12, 0), ResizePos-dim::point2di(0, 12), color(0));
        }
        
    }
    
    
}

bool GUIWindow::drawButton(dim::point2di Position)
{
    bool Result = false;
    color BtnColor(188);
    
    if (dim::rect2di(Position.X, Position.Y, Position.X+__SP_GUIWND_BTN_SIZE, Position.Y+__SP_GUIWND_BTN_SIZE).isPointCollided(__spGUIcurpos))
    {
        BtnColor = color(222);
        Result = true;
        if (__spGUIcontrol->isMouseButtonPressed(MOUSE_LEFT))
            Position += dim::point2di(1);
    }
    
    __spVideoDriver->draw2DRectangle(
        dim::rect2di(Position.X, Position.Y, Position.X+__SP_GUIWND_BTN_SIZE, Position.Y+__SP_GUIWND_BTN_SIZE),
        BtnColor
    );
    
    __spVideoDriver->draw2DLine(
        dim::point2di(Position.X, Position.Y-1),
        dim::point2di(Position.X+__SP_GUIWND_BTN_SIZE, Position.Y-1), color(0)
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position.X, Position.Y+__SP_GUIWND_BTN_SIZE),
        dim::point2di(Position.X+__SP_GUIWND_BTN_SIZE, Position.Y+__SP_GUIWND_BTN_SIZE), color(0)
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position.X, Position.Y),
        dim::point2di(Position.X, Position.Y+__SP_GUIWND_BTN_SIZE), color(0)
    );
    __spVideoDriver->draw2DLine(
        dim::point2di(Position.X+1+__SP_GUIWND_BTN_SIZE, Position.Y),
        dim::point2di(Position.X+1+__SP_GUIWND_BTN_SIZE, Position.Y+__SP_GUIWND_BTN_SIZE), color(0)
    );
    
    return Result;
}

void GUIWindow::updateResize()
{
    
    // Update all min. & max. values
    
    if (Size_.Width < MinSize_.Width)
        Size_.Width = MinSize_.Width;
    if (Size_.Height < MinSize_.Height)
        Size_.Height = MinSize_.Height;
    
    if (Size_.Width > MaxSize_.Width)
        Size_.Width = MaxSize_.Width;
    if (Size_.Height > MaxSize_.Height)
        Size_.Height = MaxSize_.Height;
    
    TitleBarLength_ = __spGUIfont->getStringSize(Title_).Width; // !!! ( font->strLength(title) )
    
    MaxTitleBarLength_ = Size_.Width - __SP_GUIWND_TITLE_POSX*2 - __SP_GUIWND_BTN_SIZE*2 -
        (__SP_GUIWND_BTN_SIZE+__SP_GUIWND_BTN_DIST) * ((s32)isButtonX_ + (s32)isButtonMin_ + (s32)isButtonMax_);
    if (TitleBarLength_ > MaxTitleBarLength_)
        TitleBarLength_ = MaxTitleBarLength_;
    
    switch (StatusBarType_)
    {
        case STATUSBAR_LEFT:
            StatusBarPos_ = 3;
            break;
        case STATUSBAR_RIGHT:
            StatusBarPos_ = Size_.Width-3;//-font->strlen(statusbartext)
            break;
        case STATUSBAR_CENTER:
            StatusBarPos_ = Size_.Width/2;//-font->strlen(statusbartext)/2
            break;
    }
    
    
    
}

void GUIWindow::updateWindowEvents()
{
    
    if (WndEventMode_ && !__spGUIcontrol->isMouseButtonPressed(MOUSE_LEFT))
        WndEventMode_ = WNDEVENT_NONE;
    
    switch (WndEventMode_)
    {
        case WNDEVENT_MOVING:
            Position_ = WndMovingPos_ + __spGUIcurpos;
            // Check raster
            if (__spGUIraster)
            {
                if (Position_.X > -__SP_GUIWND_RASTER_SIZE && Position_.X < __SP_GUIWND_RASTER_SIZE)
                    Position_.X = 0;
                else if (Position_.X > __Screen_Width-Size_.Width-__SP_GUIWND_RASTER_SIZE &&
                         Position_.X < __Screen_Width-Size_.Width+__SP_GUIWND_RASTER_SIZE)
                {
                    Position_.X = __Screen_Width-Size_.Width;
                }
                if (Position_.Y > -__SP_GUIWND_RASTER_SIZE && Position_.Y < __SP_GUIWND_RASTER_SIZE)
                    Position_.Y = 0;
                else if (Position_.Y > __Screen_Height-Size_.Height-__SP_GUIWND_RASTER_SIZE &&
                         Position_.Y < __Screen_Height-Size_.Height+__SP_GUIWND_RASTER_SIZE)
                {
                    Position_.Y = __Screen_Height-Size_.Height;
                }
            }
            OrigPosition_ = Position_;
            break;
            
        case WNDEVENT_RESIZE:
        {
            dim::point2di tPoint(WndMovingPos_ - Position_ + __spGUIcurpos);
            Size_ = dim::size2di(tPoint.X, tPoint.Y);
            // Check raster
            if (__spGUIraster)
            {
                if (Size_.Width > __Screen_Width-Position_.X-__SP_GUIWND_RASTER_SIZE &&
                    Size_.Width < __Screen_Width-Position_.X+__SP_GUIWND_RASTER_SIZE)
                {
                    Size_.Width = __Screen_Width-Position_.X;
                }
                if (Size_.Height > __Screen_Height-Position_.Y-__SP_GUIWND_RASTER_SIZE &&
                    Size_.Height < __Screen_Height-Position_.Y+__SP_GUIWND_RASTER_SIZE)
                {
                    Size_.Height = __Screen_Height-Position_.Y;
                }
            }
            // Update resize
            updateResize();
        }
        break;
            
        default:
            if (__spGUIcontrol->isMouseButtonHit(MOUSE_LEFT))
            {
                // Moving
                if (dim::rect2di(Position_.X, Position_.Y,
                                 Position_.X+Size_.Width - (__SP_GUIWND_BTN_SIZE+__SP_GUIWND_BTN_DIST) *
                                 ((s32)isButtonX_ + (s32)isButtonMin_ + (s32)isButtonMax_),
                                 Position_.Y+__SP_GUIWND_TITLE_HEIGHT)
                    .isPointCollided(__spGUIcurpos))
                {
                    __spGUIFocusedWindow = this;
                    WndEventMode_ = WNDEVENT_MOVING;
                    WndMovingPos_ = Position_ - __spGUIcurpos;
                }
                // Resize
                if (isButtonSize_ && dim::rect2di(Position_.X+Size_.Width-__SP_GUIWND_RESIZE_SIZE, Position_.Y+Size_.Height-__SP_GUIWND_RESIZE_SIZE,
                    Position_.X+Size_.Width, Position_.Y+Size_.Height).isPointCollided(__spGUIcurpos))
                {
                    __spGUIFocusedWindow = this;
                    WndEventMode_ = WNDEVENT_RESIZE;
                    WndMovingPos_ = Position_ + dim::point2di(Size_.Width, Size_.Height) - __spGUIcurpos;
                }
                // Focus
                if (dim::rect2di(Position_.X, Position_.Y, Position_.X+Size_.Width, Position_.Y+Size_.Height).isPointCollided(__spGUIcurpos))
                    __spGUIFocusedWindow = this;
            }
            break;
    }
    
}

void GUIWindow::updateZAxis(s32 FocusedWindowZ)
{
    if (ZAxis_ < FocusedWindowZ)
        ++ZAxis_;
    else if (ZAxis_ == FocusedWindowZ)
        ZAxis_ = 0;
}

void GUIWindow::setStatusbarType(u8 Type)
{
    StatusBarType_ = Type;
    updateResize();
}

void GUIWindow::setParameter(s32 Param)
{
    // General settings
    isButtonX_      = (Param & WINDOW_BUTTON_X);
    isButtonMin_    = (Param & WINDOW_BUTTON_MIN);
    isButtonMax_    = (Param & WINDOW_BUTTON_MAX);
    isButtonSize_   = (Param & WINDOW_BUTTON_SIZE);
    isStatusBar_    = (Param & WINDOW_STATUSBAR);
    isScrollVert_   = (Param & WINDOW_SCROLL_VERT);
    isScrollHorz_   = (Param & WINDOW_SCROLL_HORZ);
    // Rest settings
    if (Param & WINDOW_CENTERED)
    {
        Position_ = dim::point2di(__Screen_Width/2-Size_.Width/2, __Screen_Height/2-Size_.Height/2);
    }
    else if (Param & WINDOW_MINIMIZED)
    {
        LastPosition_ = Position_;
        LastSize_ = Size_;
        Size_ = dim::size2di(Size_.Width, 0);
    }
    else if (Param & WINDOW_MAXIMIZED)
    {
        LastPosition_ = Position_;
        Position_ = dim::point2di(0, 0);
        LastSize_ = Size_;
        Size_ = dim::size2di(__Screen_Width, __Screen_Height);
    }
    // Dependent settings
    OrigPosition_   = Position_;
    MinSize_.Height = isStatusBar_ ? __SP_GUIWND_TITLE_HEIGHT+__SP_GUIWND_STATUS_HEIGHT+2 : __SP_GUIWND_TITLE_HEIGHT+1;
}



/*
 * ========== GUI Buttons (Normal, text, checkbox, option, frame) ==========
 */

GUIButton::GUIButton() : GUIObject()
{
    init();
}
GUIButton::GUIButton(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param) :
    GUIObject(Title, Position, Size, Param)
{
    init();
    Type_ = Type;
    if (Type_ == BUTTON_IMAGE)
        bgColor_ = color(255);
}
GUIButton::~GUIButton()
{
}

s32 GUIButton::update(dim::point2di WndPosition, bool isWinRect)
{
    
    Position_ = OrigPosition_ + WndPosition;
    
    switch (Type_)
    {
        case BUTTON_NORMAL:
            if (isWinRect && dim::rect2di(Position_.X, Position_.Y, Position_.X+Size_.Width, Position_.Y+Size_.Height).isPointCollided(__spGUIcurpos))
            {
                HighLightColor_ = __SP_GUIBTN_HIGHLIGHT_COLOR_OVER;
                if (__spGUIcontrol->isMouseButtonPressed(MOUSE_LEFT))
                {
                    HighLightColor_ = __SP_GUIBTN_HIGHLIGHT_COLOR_CLICKED;
                    __spGUIFocusedObject = this;
                }
                if (__spGUIcontrol->isMouseButtonReleased(MOUSE_LEFT))
                {
                    return BUTTON_CLICKED;
                    __spGUIFocusedObject = this;
                }
            }
            else
                HighLightColor_ = __SP_GUIBTN_HIGHLIGHT_COLOR_NORMAL;
            break;
            
        case BUTTON_TEXT:
            
            break;
            
        case BUTTON_CHECKBOX:
            
            break;
            
        case BUTTON_OPTION:
            
            break;
            
        case BUTTON_FRAME:
            
            break;
            
        case BUTTON_IMAGE:
            if (isWinRect && dim::rect2di(Position_.X, Position_.Y,
                Position_.X+Size_.Width, Position_.Y+Size_.Height).isPointCollided(__spGUIcurpos) &&
                __spGUIcontrol->isMouseButtonReleased(MOUSE_LEFT))
            {
                __spGUIFocusedObject = this;
                return BUTTON_CLICKED;
            }
            break;
    }
    
    return 0;
}

void GUIButton::render()
{
    
    switch (Type_)
    {
        case BUTTON_NORMAL:
        {
            
            s32 tPosX, tPos;
            drawButton(Position_, Size_);
            
            tPos = (HighLightColor_ == __SP_GUIBTN_HIGHLIGHT_COLOR_CLICKED) ? 1 : 0;
            
            if (Param_ & TEXT_RIGHT)
                tPosX = Size_.Width-5 - __spGUIfont->getStringSize(Title_).Width;
            else if (Param_ & TEXT_CENTER)
                tPosX = Size_.Width/2 - __spGUIfont->getStringSize(Title_).Width/2;
            else
                tPosX = 5;
            
            __spVideoDriver->draw2DText(
                __spGUIfont, dim::point2di(Position_.X+tPosX+tPos, Position_.Y+tPos+Size_.Height/2-11), Title_, color(0)
            );
            
        }
        break;
            
        case BUTTON_TEXT:
            
            break;
            
        case BUTTON_CHECKBOX:
            
            break;
            
        case BUTTON_OPTION:
            
            break;
            
        case BUTTON_FRAME:
            
            break;
            
        case BUTTON_IMAGE:
            if (hTexture_)
            {
                __spVideoDriver->draw2DImage(
                    hTexture_, dim::rect2di(Position_.X, Position_.Y, Size_.Width, Size_.Height),
                    TextureClip_, bgColor_
                );
            }
            break;
    }
    
    HighLightColor_ = __SP_GUIBTN_HIGHLIGHT_COLOR_NORMAL;
    
}

void GUIButton::init()
{
    Type_           = BUTTON_NORMAL;
    isToggled_      = false;
    bgColor_        = color(128);
    HighLightColor_ = __SP_GUIBTN_HIGHLIGHT_COLOR_NORMAL;
    hTexture_       = 0;
    TextureClip_    = dim::rect2df(0, 0, 1, 1);
}

bool GUIButton::drawButton(dim::point2di Position, dim::size2di Size)
{
    __spVideoDriver->draw2DRectangle(
        dim::rect2di(Position.X+1, Position.Y+1, Position.X+Size.Width-1, Position.Y+Size.Height-1),
        HighLightColor_
    );
    __spVideoDriver->draw2DRectangle(
        dim::rect2di(Position.X+3, Position.Y+3, Position.X+Size.Width-3, Position.Y+Size.Height-3),
        color(255), color(255), bgColor_, bgColor_
    );
    __spVideoDriver->draw2DLine(dim::point2di(Position.X+1, Position.Y+1), dim::point2di(Position.X+1, Position.Y+Size.Height-1), color(0));
    __spVideoDriver->draw2DLine(dim::point2di(Position.X+Size.Width, Position.Y+1), dim::point2di(Position.X+Size.Width, Position.Y+Size.Height-1), color(0));
    __spVideoDriver->draw2DLine(dim::point2di(Position.X+1, Position.Y), dim::point2di(Position.X+Size.Width-1, Position.Y+1), color(0));
    __spVideoDriver->draw2DLine(dim::point2di(Position.X+1, Position.Y+Size.Height-1), dim::point2di(Position.X+Size.Width-1, Position.Y+Size.Height-1), color(0));
}

bool GUIButton::drawCheckbox(dim::point2di Position, dim::size2di Size)
{
    
}

bool GUIButton::drawOption(dim::point2di Position, dim::size2di Size)
{
    
}


/*
 * ========== GUI Editbox (string, editor) ==========
 */

GUIEditbox::GUIEditbox() : GUIObject()
{
}
GUIEditbox::GUIEditbox(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param) :
    GUIObject(Title, Position, Size, Param)
{
    Type_ = Type;
}
GUIEditbox::~GUIEditbox()
{
}

s32 GUIEditbox::update(dim::point2di WndPosition, bool isWinRect)
{
    
}

void GUIEditbox::render()
{
    
}


/*
 * ========== GUI List (normal, iconed) ==========
 */

GUIList::GUIList() : GUIObject()
{
}
GUIList::GUIList(io::stringc Title, dim::point2di Position, dim::size2di Size, u8 Type, s32 Param) :
    GUIObject(Title, Position, Size, Param)
{
    Type_ = Type;
    Scroll_X_Thumb_ = Scroll_X_Max_ = Scroll_Y_Thumb_ = Scroll_Y_Max_ = 0;
}
GUIList::~GUIList()
{
}

s32 GUIList::update(dim::point2di WndPosition, bool isWinRect)
{
    
}

void GUIList::render()
{
    
}

void GUIList::thumbInfoUpdate()
{
    // Recalc the 'Scroll_X_Thumb_' ... variables
}


/*
 * ========== GUI Menu ==========
 */

bool sortMenuCategories(const GUIMenu* &obj1, const GUIMenu* &obj2)
{
    return obj1->getCategorySub() > obj2->getCategorySub() || 
           ( obj1->getCategorySub() == obj2->getCategorySub() && obj1->getCategoryChapter() > obj2->getCategoryChapter() );
}

//s32 glbMenuIndexX = 0, glbMenuIndexY = 0;
//std::vector<s32> glbMenuX, glbMenuY;

GUIMenu::GUIMenu()
{
    init();
}
GUIMenu::GUIMenu(s32 Type, io::stringc Title, GUIMenu* hParent)
{
    init();
    Type_       = Type;
    Title_      = Title;
    hParent_    = hParent;
}
GUIMenu::~GUIMenu()
{
}

s32 GUIMenu::update(dim::point2di WndPosition, bool isWinRect)
{
    
}

void GUIMenu::render()
{
    
}

void GUIMenu::init()
{
    isVisible_      = false;
    Type_           = MENU_CHAPTER;
    Title_          = io::stringc("");
    Position_       = dim::point2di();
    Size_           = dim::size2di();
    hParent_        = 0;
    CategorySub_    = CategoryChapter_ = 0;
}

// ! untested !

void GUIMenu::calcLocation(std::list<GUIMenu*> &WndMenuList)
{
    
    s32 MaxWidth = 0;
    s32 CurCategoryChapter = 0, CurCategorySub = 0;
    
    // Sorting width
    
    for (std::list<GUIMenu*>::iterator itMen = WndMenuList.begin(); itMen != WndMenuList.end(); ++itMen)
    {
        
        (*itMen)->Size_.set(__spGUIfont->getStringSize((*itMen)->Title_).Width+__SP_GUIMENU_BORDERWIDTH*2, __SP_GUIMENU_HEIGHT);
        
        for (std::list<GUIMenu*>::iterator it = WndMenuList.begin(); it != WndMenuList.end(); ++it)
        {
            if (*it == (*itMen)->hParent_)
            {
                switch ((*it)->getType())
                {
                    case MENU_CHAPTER:
                        ++(*itMen)->CategoryChapter_; break;
                    case MENU_SUBITEM:
                        ++(*itMen)->CategorySub_; break;
                }
            }
        }
        
    }
    
    //WndMenuList.sort(sortMenuCategories); // ???
    
    for (std::list<GUIMenu*>::iterator itMen = WndMenuList.begin(), it, start = WndMenuList.begin(); itMen != WndMenuList.end(); ++itMen)
    {
        
        if ((*itMen)->CategorySub_ != CurCategorySub && (*itMen)->CategoryChapter_ != CurCategoryChapter)
        {
            for (it = start; it != itMen; ++it)
                (*it)->Size_.Width = MaxWidth;
            start = itMen;
            MaxWidth = 0;
        }
        if ((*itMen)->Size_.Width > MaxWidth)
            MaxWidth = (*itMen)->Size_.Width;
        CurCategorySub = (*itMen)->CategorySub_;
        CurCategoryChapter = (*itMen)->CategoryChapter_;
        
    }
    
    // Parent position
    
    for (std::list<GUIMenu*>::iterator itMen = WndMenuList.begin(); itMen != WndMenuList.end(); ++itMen)
    {
        
        for (std::list<GUIMenu*>::iterator it = WndMenuList.begin(); it != WndMenuList.end(); ++it)
        {
            
            if (*it == (*itMen)->hParent_)
            {
                switch ((*it)->getType())
                {
                    case MENU_CHAPTER:
                        (*itMen)->Position_ += dim::point2di(0, (*it)->getSize().Height); break;
                    case MENU_SUBITEM:
                        (*itMen)->Position_ += dim::point2di((*it)->getSize().Width, 0); break;
                }
            }
            
        }
        
        (*itMen)->OrigPosition_ = (*itMen)->Position_;
        
    }
    
    // Final position
    
    for (std::list<GUIMenu*>::iterator it = WndMenuList.begin(); it != WndMenuList.end(); ++it)
    {
        
        
    }
    
}




} // /namespace gui

} // /namespace video

} // /namespace sp







// ================================================================================
