/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Movie sequence operations
 *
 */

#include "spVideoDriverMovieManager.hpp"
#include "spMovieSequencePacket.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace video
{
namespace movie
{


#if defined(WIN32)

void flipDataBufferASM(unsigned char* &buf, int len);


MovieManager::MovieManager()
{
}
MovieManager::~MovieManager()
{
}

Movie* MovieManager::loadMovie(io::stringc Filename, s32 Resolution)
{
    io::printMessage(io::stringc("Load movie: \"")+Filename+io::stringc("\""));
    
    /* General settings */
    SMovieSequencePacket* MovieData = new SMovieSequencePacket;
    
    MovieData->next         = MovieData->frame       = 0;
    MovieData->state        = MOVIESTATE_STOPED;
    MovieData->time         = MovieData->last_time   = 0;
    MovieData->data         = 0;
    MovieData->Resolution   = Resolution;
    
    MovieData->hDC          = CreateCompatibleDC(0);
    MovieData->hdd          = DrawDibOpen();
    
    AVIFileInit();
    
    /* Open AVI-video stream */
    if (AVIStreamOpenFromFile(&MovieData->pavi, Filename.c_str(), streamtypeVIDEO, 0, OF_READ, 0))
    {
        io::printError(io::stringc("could not open video stream from file: \"")+io::stringc(Filename)+io::stringc("\""));
        return 0;
    }
    
    /* Opens AVI-audio stream */
    /*if (AVIStreamOpenFromFile(&MovieData->paudio, Filename.c_str(), streamtypeAUDIO, 0, OF_READ, 0))
    {
        io::printMessage(io::stringc("could not open audio stream from file: \"")+io::stringc(Filename)+io::stringc("\""));
        return false;
    }*/
    
    if (AVIStreamInfo(MovieData->pavi, &MovieData->psi, sizeof(MovieData->psi)))
    {
        io::printError("video stream information process failed");
        return 0;
    }
    
    MovieData->Size.Width   = MovieData->psi.rcFrame.right - MovieData->psi.rcFrame.left;
    MovieData->Size.Height  = MovieData->psi.rcFrame.bottom - MovieData->psi.rcFrame.top;
    
    MovieData->lastframe    = AVIStreamLength(MovieData->pavi);
    
    if (MovieData->lastframe == -1)
    {
        io::printError("video stream length is invalid");
        return 0;
    }
    
    MovieData->mpf = AVIStreamSampleToTime(MovieData->pavi, MovieData->lastframe) / MovieData->lastframe;
    
    if (MovieData->mpf == -1)
    {
        io::printError("video stream sample is invalid");
        return 0;
    }
    
    MovieData->bmih.biSize          = sizeof(BITMAPINFOHEADER);
    MovieData->bmih.biPlanes        = 1;
    MovieData->bmih.biBitCount      = 24;
    MovieData->bmih.biWidth         = MovieData->Resolution;
    MovieData->bmih.biHeight        = MovieData->Resolution;
    MovieData->bmih.biCompression   = BI_RGB;
    
    MovieData->hBitmap = CreateDIBSection(
        MovieData->hDC, (BITMAPINFO*)(&MovieData->bmih), DIB_RGB_COLORS, (void**)(&MovieData->data), 0, 0
    );
    SelectObject(MovieData->hDC, MovieData->hBitmap);
    
    MovieData->pgf = AVIStreamGetFrameOpen(MovieData->pavi, 0);
    if (MovieData->pgf == 0)
    {
        io::printError("could not open first video stream");
        return 0;
    }
    
    Movie* NewMovie = new Movie(Filename, (s32)MovieData);
    
    //MovieList.push_back ...
    
    return NewMovie;
}

void MovieManager::closeMovie(Movie* &hMovie)
{
    SMovieSequencePacket* MovieData = (SMovieSequencePacket*)hMovie->getSequenceData();
    {
        DeleteObject(MovieData->hBitmap);
        DrawDibClose(MovieData->hdd);
        AVIStreamGetFrameClose(MovieData->pgf);
        AVIStreamRelease(MovieData->pavi);
        AVIFileExit();
    }
    spDeletePointer(MovieData);
    spDeletePointer(hMovie);
}

void MovieManager::playMovie(Movie* hMovie, bool isLoop)
{
    SMovieSequencePacket* MovieData = (SMovieSequencePacket*)hMovie->getSequenceData();
    {
        MovieData->state    = MOVIESTATE_PLAYED;
        MovieData->time     = MovieData->last_time = clock();
        MovieData->isLoop   = isLoop;
        //AVIStreamStart(MovieData->paudio);
    }
    hMovie->getSequenceData() = (s32)MovieData;
}
void MovieManager::stopMovie(Movie* hMovie)
{
    ((SMovieSequencePacket*)hMovie->getSequenceData())->state = MOVIESTATE_STOPED;
}
void MovieManager::pauseMovie(Movie* hMovie, bool isPaused)
{
    SMovieSequencePacket* MovieData = (SMovieSequencePacket*)hMovie->getSequenceData();
    {
        if (isPaused && MovieData->state == MOVIESTATE_PLAYED)
        {
            MovieData->state        = MOVIESTATE_PAUSED;
        }
        else if (!isPaused && MovieData->state == MOVIESTATE_PAUSED)
        {
            MovieData->state        = MOVIESTATE_PLAYED;
            MovieData->last_time    = clock();
        }
    }
    hMovie->getSequenceData() = (s32)MovieData;
}
bool MovieManager::isMovie(Movie* hMovie)
{
    return hMovie != 0;
}

bool MovieManager::isMovieFinished(Movie* hMovie)
{
    SMovieSequencePacket* MovieData = (SMovieSequencePacket*)hMovie->getSequenceData();
    return MovieData->frame >= MovieData->lastframe;
}

u32 MovieManager::getMovieLength(Movie* hMovie)
{
    return ((SMovieSequencePacket*)hMovie->getSequenceData())->lastframe;
}

void MovieManager::renderMovieTexture(Movie* hMovie, Texture* hTexture)
{
    SMovieSequencePacket* MovieData = (SMovieSequencePacket*)hMovie->getSequenceData();
    
    if (MovieData->state == MOVIESTATE_PLAYED)
    {
        MovieData->time         = clock();
        MovieData->next         += (MovieData->time - MovieData->last_time);
        MovieData->last_time    = clock();
        MovieData->frame        = MovieData->next / MovieData->mpf;
        
        if (MovieData->frame >= MovieData->lastframe)
        {
            MovieData->next = MovieData->frame = 0;
            if (!MovieData->isLoop)
                MovieData->state = MOVIESTATE_STOPED;
        }
        
        LPBITMAPINFOHEADER lpbi;
        lpbi = (LPBITMAPINFOHEADER)AVIStreamGetFrame(MovieData->pgf, MovieData->frame);
        MovieData->pdata = (s8*)lpbi + lpbi->biSize + lpbi->biClrUsed * sizeof(RGBQUAD);
        
        DrawDibDraw(
            MovieData->hdd, MovieData->hDC, 0, 0, MovieData->Resolution, MovieData->Resolution,
            lpbi, MovieData->pdata, 0, 0, MovieData->Size.Width, MovieData->Size.Height, 0
        );
        
        //flipDataBuffer(MovieData->data, MovieData->Resolution * MovieData->Resolution * 3);
        
        /* Copy to texture */
        __spVideoDriver->setTextureSubImage(
            hTexture->getID(), 0, 0, MovieData->Resolution, MovieData->Resolution, MovieData->data
        );
        
        hMovie->getSequenceData() = (s32)MovieData;
    }
}

void MovieManager::flipDataBuffer(void* buffer, u32 Len)
{
    register u8* b = (u8*)buffer;
    register s32 tmp, i = 0;
    
    while (i < Len)
    {
        tmp     = b[i];
        b[i]    = b[i+2];
        b[i+2]  = tmp;
        i += 3;
    }
}

#elif defined(LINUX)

#endif


} // /namespace movie

} // /namespace video

} // /namespace sp







// ================================================================================
