/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Header for shader
 *
 */

#ifndef __SP_VIDEODRIVER_SHADER_H__
#define __SP_VIDEODRIVER_SHADER_H__


#include "spStandard.hpp"
#include "spInputOutput.hpp"
#include "spDimension.hpp"
#include "spVideoDriverColor.hpp"

#include <vector>


namespace sp
{
namespace video
{


enum EStandardShaderTypes
{
    SHADER_MONOCHROME,
    SHADER_NEGATIVE,
    SHADER_GREY,
    SHADER_BGR,
};

enum EConstantTypes
{
    CONSTANT_INT,
    CONSTANT_FLOAT,
    CONSTANT_VECTOR2,
    CONSTANT_VECTOR3,
    CONSTANT_VECTOR4,
    CONSTANT_MATRIX2,
    CONSTANT_MATRIX3,
    CONSTANT_MATRIX4,
};

enum EShaderTypes
{
    SHADER_VERTEX,
    SHADER_PIXEL,
};

enum EHLSLVertexShaderTypes
{
    HLSL_VERTEX_1_1 = 0,
    HLSL_VERTEX_2_0,
    HLSL_VERTEX_2_a,
    HLSL_VERTEX_3_0,
    
    HLSL_VERTEX_COUNT
};

enum EHLSLPixelShaderTypes
{
    HLSL_PIXEL_1_1 = 0,
    HLSL_PIXEL_1_2,
    HLSL_PIXEL_1_3,
    HLSL_PIXEL_1_4,
    HLSL_PIXEL_2_0,
    HLSL_PIXEL_2_a,
    HLSL_PIXEL_2_b,
    HLSL_PIXEL_3_0,
    
    HLSL_PIXEL_COUNT
};


class Shader;

typedef void (*PFNSHADERCALLBACKPROC)(Shader* Shd);


class SP_EXPORT Shader
{
    
    public:
        
        Shader();
        virtual ~Shader();
        
        /* Binds & unbinds the shader */
        
        virtual void bind();
        virtual void unbind();
        
        /* Set the constants (by number) */
        
        virtual void setConstant(s32 Number, EConstantTypes Type, const f32 Value);
        virtual void setConstant(s32 Number, EConstantTypes Type, dim::vector3df Position);
        virtual void setConstant(s32 Number, EConstantTypes Type, video::color Color);
        virtual void setConstant(s32 Number, EConstantTypes Type, dim::matrix4f &Matrix);
        virtual void setConstant(s32 Number, EConstantTypes Type, const f32* pFloats, int Count);
        
        /* Set the constants (by name) */
        
        virtual bool setConstant(io::stringc Name, const f32 Value);
        virtual bool setConstant(io::stringc Name, dim::vector3df Position);
        virtual bool setConstant(io::stringc Name, video::color Color);
        virtual bool setConstant(io::stringc Name, dim::matrix4f &Matrix);
        virtual bool setConstant(io::stringc Name, const f32* pFloats, int Count);
        
        /* Set the constants for assembly shaders */
        
        virtual bool setConstant(const f32* pFloats, s32 StartRegister, s32 ConstAmount);
        
        /* Inline functions */
        
        inline void useShader(EShaderTypes Type)
        {
            CurrentShader_ = Type;
        }
        
        inline void setShaderCallback(PFNSHADERCALLBACKPROC CallbackProc)
        {
            pShaderCallbackProc_ = CallbackProc;
        }
        
        inline bool isHighLevelLanguage() const
        {
            return HighLevelLanguage_;
        }
        
    protected:
        
        /* Members */
        
        EShaderTypes CurrentShader_;
        
        PFNSHADERCALLBACKPROC pShaderCallbackProc_;
        
        bool HighLevelLanguage_;
        
};


} // /namespace scene

} // /namespace sp


#endif







// ================================================================================
