/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Texture manipulator
 *
 */

#include "spVideoDriverTextureManipulator.hpp"
#include "spMainDevice.hpp"


namespace sp
{

extern video::VideoDriver* __spVideoDriver;

namespace video
{
namespace effect
{


TextureManipulator::TextureManipulator()
{
}
TextureManipulator::~TextureManipulator()
{
}

void TextureManipulator::renderMosaic(Texture* &hTexture, s32 PixelSize, dim::rect2di Rect)
{
    checkCorrectDimensions(hTexture, Rect);
    
    // Loop the image data
    for (register s32 y = Rect.Top, x; y < Rect.Bottom; ++y)
    {
        for (x = Rect.Left; x < Rect.Right; ++x)
        {
            hTexture->setPixelColor(
                dim::point2di(x, y),
                hTexture->getPixelColor(dim::point2di( ( x / PixelSize ) * PixelSize, ( y / PixelSize ) * PixelSize ))
            );
        }
    }
    
    hTexture->updateImageData(SET_IMAGEDATA);
}

void TextureManipulator::renderBlur(Texture* &hTexture, s32 PixelSize, dim::rect2di Rect)
{
    checkCorrectDimensions(hTexture, Rect);
    
    Texture* NewTexture = __spVideoDriver->copyTexture(hTexture);
    
    register s32 x, y, sx, sy, w, h, c;
    register video::color Color;
    register s32 r, g, b;
    
    PixelSize /= 2;
    
    // Loop the image data
    for (y = Rect.Top; y < Rect.Bottom; ++y)
    {
        for (x = Rect.Left; x < Rect.Right; ++x)
        {
            // Settings
            r = g = b = 0;
            sy = y - PixelSize >= 0 ? -PixelSize : -y;
            w = x + PixelSize < Rect.Right ? PixelSize : Rect.Right - x;
            h = y + PixelSize < Rect.Bottom ? PixelSize : Rect.Bottom - y;
            
            // Loop the image data (around the current pixel by 'PixelSize')
            for (c = 0; sy <= h; ++sy)
            {
                sx = x - PixelSize >= 0 ? -PixelSize : -x;
                
                for (; sx <= w; ++sx, ++c)
                {
                    Color = NewTexture->getPixelColor(dim::point2di(x + sx, y + sy));
                    r += Color.Red;
                    g += Color.Green;
                    b += Color.Blue;
                }
            }
            
            // Change the pixel color
            hTexture->setPixelColor(dim::point2di(x, y), color(r/c, g/c, b/c));
        }
    }
    
    hTexture->updateImageData(SET_IMAGEDATA);
    //hTexture->updateImageData(GET_IMAGEDATA);
    
    __spVideoDriver->deleteTexture(NewTexture);
}

void TextureManipulator::checkCorrectDimensions(Texture* hTexture, dim::rect2di &Rect)
{
    if (Rect == __SP_TEXMANIP_RECTIGNORE)
        Rect = dim::rect2di(0, 0, hTexture->getSize().Width, hTexture->getSize().Height);
    
    Rect.repair();
    
    if (hTexture->getSize().Width < Rect.Right)
        Rect.Right = hTexture->getSize().Width;
    if (hTexture->getSize().Height < Rect.Bottom)
        Rect.Bottom = hTexture->getSize().Height;
}


} // /namespace effect

} // /namespace video

} // /namespace sp







// ================================================================================
