/*
 * **********************************
 *                                   *
 * HLC Games, Spiele und Software     *
 *                                     *
 * (c) Copyright 2003 - Lukas Hermanns  *
 *                                       *
 * ****************************************
 *  ___    ___    ___          _____
 * /\  \  /\  \  /\  \        /  ___\
 * \ \  \_\_\  \ \ \  \      /\  \__/
 *  \ \   ___   \ \ \  \     \ \  \
 *   \ \  \__/\  \ \ \  \____ \ \  \___
 *    \ \__\ \ \__\ \ \______\ \ \_____\
 *     \/__/  \/__/  \/______/  \/_____/
 *
 * ****************************************
 *
 * ********************** HLC SoftPixelEngine - (3.1.2008) ************************
 *
 * (c) Copyright 2008 - Lukas Hermanns
 *
 * Wrapper (for PureBasic, BlitzBasic, VisualBasic)
 *
 */

#include "SoftPixelEngine.hpp"


#undef SP_EXPORT
#define SP_EXPORT       extern "C" __declspec(dllexport)

#define LP              int // Long-Pointer
#define STATIC(c, p)    (static_cast<c>(p))
#define CASTTO(c, p)    (reinterpret_cast<c>(p))
#define CAST(p)         CASTTO(LP, p)

#define NODE(p)         CASTTO(sp::scene::Node*, p)
#define ENTITY(p)       CASTTO(sp::scene::Entity*, p)
#define LIGHT(p)        CASTTO(sp::scene::Light*, p)
#define SPRITE(p)       CASTTO(sp::scene::Sprite*, p)
#define CAMERA(p)       CASTTO(sp::scene::Camera*, p)
#define ANIM(p)         CASTTO(sp::scene::anim::Animation*, p)
#define SHADER(p)       CASTTO(sp::video::Shader*, p)
#define FONT(p)         CASTTO(sp::video::Font*, p)
#define TEXTURE(p)      CASTTO(sp::video::Texture*, p)
#define SOUND(p)        CASTTO(sp::sound::Sound*, p)
#define RECORD(p)       CASTTO(sp::sound::Record*, p)
#define FILE(p)         CASTTO(sp::io::File*, p)


// ======================= Main variables =======================

sp::SoftPixelDevice*                    __spDevice              = 0;
sp::sound::SensitiveSoundDevice*        __spListener            = 0;
sp::UserControl*                        __spUserControl         = 0;

sp::video::VideoDriver*                 __spVideoDriver         = 0;
sp::video::gui::GUI*                    __spGUI                 = 0;
sp::video::movie::MovieManager*         __spMovieManager        = 0;
sp::video::effect::TextureManipulator*  __spTextureManip        = 0;

sp::scene::SceneManager*                __spSceneManager        = 0;
sp::scene::CollisionDetector*           __spCollisionDetector   = 0;
sp::scene::anim::Animator*              __spAnimator            = 0;
sp::scene::StencilManager*              __spStencil             = 0;

sp::io::FileLoader*                     __spFileSystem          = 0;
sp::io::OSInformator*                   __spOSInformator        = 0;

sp::script::ScriptLoader*               __spScriptLoader        = 0;


// ======================= All functions =======================


/* ========== namespace sp ========== */


SP_EXPORT LP spCreateDevice(int DriverType, int DimensionWidth, int DimensionHeight, int ColorDepth, char* Title, bool isFullScreen, bool isResizeAble, bool isVsync, bool isAntiAlias, int MultiSample)
{
    return CAST(__spDevice = sp::createDevice(
        STATIC(sp::video::EDriverTypes, DriverType), sp::dim::size2di(DimensionWidth, DimensionHeight), ColorDepth, Title, isFullScreen, isResizeAble, isVsync, isAntiAlias, MultiSample)
    );
}
SP_EXPORT LP spCreateDeviceWnd(int hWndParent, int DimensionWidth, int DimensionHeight, int ColorDepth, char* Title, bool isVsync, bool isAntiAlias, int MultiSample)
{
    return CAST(__spDevice = sp::createDeviceWnd(
        STATIC(sp::video::EDriverTypes, DriverType), CASTTO(HWND, hWndParent), sp::dim::size2di(DimensionWidth, DimensionHeight), ColorDepth, Title, isVsync, isAntiAlias, MultiSample)
    );
}

SP_EXPORT LP spGetVideoDriver()
{
    return CAST(__spVideoDriver = __spDevice->getVideoDriver());
}
SP_EXPORT LP spGetSceneManager()
{
    return CAST(__spSceneManager = __spDevice->getSceneManager());
}
SP_EXPORT LP spGetCollisionDetector()
{
    return CAST(__spCollisionDetector = __spDevice->getCollisionDetector());
}
SP_EXPORT LP spGetUserControl()
{
    return CAST(__spUserControl = __spDevice->getUserControl());
}

SP_EXPORT void spDeleteDevice()
{
    sp::deleteDevice();
}
SP_EXPORT bool spUpdateEvent()
{
    return __spDevice->updateEvent();
}

SP_EXPORT int spGetScreenDimensionWidth()
{
    return __spDevice->getScreenDimension().Width;
}
SP_EXPORT int spGetScreenDimensionHeight()
{
    return __spDevice->getScreenDimension().Height;
}
SP_EXPORT LP spGetWindowHandle()
{
    return (LP)__spDevice->getWindowHandle();
}
SP_EXPORT void spChangeGFXMode(LP hWnd, int DimensionWidth, int DimensionHeight, int ColorDepth, char* Title, bool isFullScreen, bool isResizeAble)
{
    __spDevice->changeGFXMode((HWND)hWnd, sp::dim::size2di(DimensionWidth, DimensionHeight), ColorDepth, Title, isFullScreen, isResizeAble);
}
SP_EXPORT void spSetRenderMode(int RenderMode)
{
    __spVideoDriver->setRenderMode(STATIC(sp::video::EDriverRenderModes, RenderMode));
}
SP_EXPORT void spSetFrameRate(float FrameRate)
{
    __spDevice->setFrameRate(FrameRate);
}
SP_EXPORT void spSetResizeMode(bool isResizeUpdate)
{
    __spDevice->setResizeMode(isResizeUpdate);
}
SP_EXPORT void spSetVsync(bool isVsync)
{
    __spVideoDriver->setVsync(isVsync);
}
SP_EXPORT float spGetFPS()
{
    return __spDevice->getFPS();
}
SP_EXPORT int spGetMilliseconds()
{
    return (int)__spDevice->getMilliseconds();
}
SP_EXPORT int spGetTime(int Type)
{
    return (int)__spDevice->getTime((unsigned char)Type);
}
SP_EXPORT void spBeep(int Milliseconds, int Frequency)
{
    __spDevice->beep((unsigned)Milliseconds, (unsigned)Frequency);
}


// User control

SP_EXPORT void spSetCursorPosition(int PositionX, int PositionY)
{
    __spUserControl->setCursorPosition(sp::dim::point2di(PositionX, PositionY));
}
SP_EXPORT int spGetCursorPositionX()
{
    return __spUserControl->getCursorPosition().X;
}
SP_EXPORT int spGetCursorPositionY()
{
    return __spUserControl->getCursorPosition().Y;
}
SP_EXPORT int spGetCursorSpeedX()
{
    return __spUserControl->getCursorSpeed().X;
}
SP_EXPORT int spGetCursorSpeedY()
{
    return __spUserControl->getCursorSpeed().Y;
}
SP_EXPORT void spHideCursor(bool isHide)
{
    __spUserControl->hideCursor(isHide);
}
SP_EXPORT int spGetMouseWheel()
{
    return __spUserControl->getMouseWheel();
}
SP_EXPORT void spSetMouseWheel(int Value)
{
    __spUserControl->setMouseWheel(Value);
}
SP_EXPORT bool spIsKeyPressed(int KeyCode)
{
    return __spUserControl->isKeyPressed(KeyCode);
}
SP_EXPORT bool spIsMouseButtonPressed(int ButtonType)
{
    return __spUserControl->isMouseButtonPressed(ButtonType);
}
SP_EXPORT bool spIsKeyReleased(int KeyCode)
{
    return __spUserControl->isKeyReleased(KeyCode);
}
SP_EXPORT bool spIsMouseButtonReleased(int ButtonType)
{
    return __spUserControl->isMouseButtonReleased(ButtonType);
}
SP_EXPORT bool spIsKeyHit(int KeyCode)
{
    return __spUserControl->isKeyHit(KeyCode);
}
SP_EXPORT bool spIsMouseButtonHit(int ButtonType)
{
    return __spUserControl->isMouseButtonHit(ButtonType);
}


/* ========== namespace io ========== */


SP_EXPORT void spOpenConsole(char* Title)
{
    sp::io::openConsole(Title);
}
SP_EXPORT void spCloseConsole()
{
    sp::io::closeConsole();
}
SP_EXPORT void spClearConsole()
{
    sp::io::clearConsole();
}
SP_EXPORT void spPrintMessage(char* Message)
{
    sp::io::printMessage(Message);
}
SP_EXPORT void spPrintWarning(char* Message)
{
    sp::io::printWarning(Message);
}
SP_EXPORT void spPrintError(char* Message)
{
    sp::io::printError(Message);
}
SP_EXPORT void spGetMessage(char* Message)
{
    sp::io::getMessage(Message);
}
SP_EXPORT void spGetWarning(char* Message)
{
    sp::io::getWarning(Message);
}
SP_EXPORT void spGetError(char* Message)
{
    sp::io::getError(Message);
}
SP_EXPORT void spSetDebugLog(bool isDebugLog, char* Filename)
{
    sp::io::setDebugLog(isDebugLog, Filename);
}

// File system




/* ========== namespace dim ========== */




/* ========== namespace math ========== */




/* ========== namespace video ========== */

/*
SP_EXPORT int spRGB(int Red, int Green, int Blue)
{
    return (signed)( ((255 & 0xFF) << 24) | ((Red & 0xFF) << 16) | ((Green & 0xFF) << 8) | (Blue & 0xFF) );
}
SP_EXPORT int spRGBA(int Red, int Green, int Blue, int Alpha)
{
    return (signed)( ((Alpha & 0xFF) << 24) | ((Red & 0xFF) << 16) | ((Green & 0xFF) << 8) | (Blue & 0xFF));
}
SP_EXPORT int spRed(int Color)
{
    return (signed)( ((unsigned)(Color) >> 16) & 0xFF );
}
SP_EXPORT int spGreen(int Color)
{
    return (signed)( ((unsigned)(Color) >> 8) & 0xFF );
}
SP_EXPORT int spBlue(int Color)
{
    return (signed)( (unsigned)(Color) & 0xFF );
}
SP_EXPORT int spAlpha(int Color)
{
    return (signed)( (unsigned)(Color) >> 24 );
}
*/

SP_EXPORT void spFlipBuffers()
{
    __spVideoDriver->flipBuffers();
}
SP_EXPORT void spClearBuffers()
{
    __spVideoDriver->clearBuffers();
}
SP_EXPORT void spSetClearColor(int ColorRed, int ColorGreen, int ColorBlue)
{
    __spVideoDriver->setClearColor(sp::video::color(ColorRed, ColorGreen, ColorBlue, 255));
}

SP_EXPORT LP spGetMovieManager()
{
    return CAST(__spMovieManager = __spVideoDriver->getMovieManager());
}

// Textures

SP_EXPORT void spSetTextureGenFlags(int RenderMode, int TextureFilter, bool isMipMapping, bool isEdgeSmooth)
{
    __spVideoDriver->setTextureGenFlags(
        (sp::video::ETextureRenderModeTypes)RenderMode,
        (sp::video::ETextureFilterTypes)TextureFilter,
        isMipMapping,
        isEdgeSmooth
    );
}
SP_EXPORT LP spLoadTexture(char* Filename)
{
    return CAST(__spVideoDriver->loadTexture(Filename));
}
SP_EXPORT void spDeleteTexture(LP Texture)
{
    sp::video::Texture* vidTex = TEXTURE(Texture);
    __spVideoDriver->deleteTexture(vidTex);
}
SP_EXPORT bool spIsTexture(LP Texture)
{
    return __spVideoDriver->isTexture(TEXTURE(Texture));
}
SP_EXPORT LP spCreateTexture(int DimensionWidth, int DimensionHeight, int TextureDataType, char* Filename)
{
    return CAST(__spVideoDriver->createTexture(sp::dim::size2di(DimensionWidth, DimensionHeight), TextureDataType, Filename));
}
SP_EXPORT void spSetTextureTransFilter1(LP Texture, int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    TEXTURE(Texture)->setTransFilter(sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha));
}
SP_EXPORT void spSetTextureTransFilter2(LP Texture, int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha, int Variation)
{
    TEXTURE(Texture)->setTransFilter(sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha), Variation);
}
SP_EXPORT void spSetTextureTransFilter3(LP Texture, int PositionX, int PositionY, int Alpha, int Variation)
{
    TEXTURE(Texture)->setTransFilter(sp::dim::point2di(PositionX, PositionY), Alpha, Variation);
}
SP_EXPORT void spSetTextureTransFilterAlpha(LP Texture, int Mode)
{
    TEXTURE(Texture)->setTransFilterAlpha(Mode);
}
SP_EXPORT void spSetTextureSize(LP Texture, int SizeWidth, int SizeHeight)
{
    TEXTURE(Texture)->setSize(sp::dim::size2di(SizeWidth, SizeHeight));
}
SP_EXPORT void spMixTexture(LP Texture, LP MixedTexture, int Type)
{
    TEXTURE(Texture)->mixTexture(TEXTURE(MixedTexture), STATIC(sp::video::ETextureMixTypes, Type));
}
SP_EXPORT void spChangeTextureAttributes(LP Texture, int RenderMode, bool isEdgeSmooth)
{
    TEXTURE(Texture)->changeAttributes(RenderMode, isEdgeSmooth);
}
SP_EXPORT void spSetTextureColorIntensity(LP Texture, int ColorRed, int ColorGreen, int ColorBlue)
{
    TEXTURE(Texture)->setColorIntensity(sp::video::color(ColorRed, ColorGreen, ColorBlue));
}


// Drawing Manager

SP_EXPORT void spBeginDrawing2D()
{
    __spVideoDriver->beginDrawing2D();
}
SP_EXPORT void spEndDrawing2D()
{
    __spVideoDriver->endDrawing2D();
}
SP_EXPORT void spBeginDrawing3D()
{
    __spVideoDriver->beginDrawing3D();
}
SP_EXPORT void spEndDrawing3D()
{
    __spVideoDriver->endDrawing3D();
}
SP_EXPORT void spBeginAlphaBlending(int Type)
{
    __spVideoDriver->beginAlphaBlending((sp::u8)Type);
}
SP_EXPORT void spEndAlphaBlending()
{
    __spVideoDriver->endAlphaBlending();
}
SP_EXPORT void spBeginTextureDrawing(LP Texture)
{
    __spVideoDriver->beginTextureDrawing(TEXTURE(Texture));
}
SP_EXPORT void spEndTextureDrawing()
{
    __spVideoDriver->endTextureDrawing();
}
SP_EXPORT void spSetLineSize(int Size)
{
    __spVideoDriver->setLineSize(Size);
}
SP_EXPORT void spSetPointSize(int Size)
{
    __spVideoDriver->setPointSize(Size);
}
SP_EXPORT void spDraw2DImage1(
    LP Texture,
    int X, int Y,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha,
    bool isTransFilterUsing)
{
    __spVideoDriver->draw2DImage(
        TEXTURE(Texture),
        sp::dim::point2di(X, Y),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha),
        isTransFilterUsing
    );
}
SP_EXPORT void spDraw2DImage2(
    LP Texture,
    int PositionLeft, int PositionTop, int PositionRight, int PositionBottom,
    float ClippingLeft, float ClippingTop, float ClippingRight, float ClippingBottom,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha,
    bool isTransFilterUsing)
{
    __spVideoDriver->draw2DImage(
        TEXTURE(Texture),
        sp::dim::rect2di(PositionLeft, PositionTop, PositionRight, PositionBottom),
        sp::dim::rect2df(ClippingLeft, ClippingTop, ClippingRight, ClippingBottom),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha),
        isTransFilterUsing
    );
}
SP_EXPORT void spDraw2DImage3(
    LP Texture,
    int X, int Y,
    float Rotation, float Radius,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha,
    bool isTransFilterUsing)
{
    __spVideoDriver->draw2DImage(
        TEXTURE(Texture),
        sp::dim::point2di(X, Y),
        Rotation,
        Radius,
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha),
        isTransFilterUsing
    );
}
/*SP_EXPORT void spDraw2DImage4()
{
    __spVideoDriver->draw2DImage();
}
SP_EXPORT void spDraw2DImage5()
{
    __spVideoDriver->draw2DImage();
}*/

SP_EXPORT int spGetPixelRed(int X, int Y)
{
    return __spVideoDriver->getPixel(sp::dim::point2di(X, Y)).Red;
}
SP_EXPORT int spGetPixelGreen(int X, int Y)
{
    return __spVideoDriver->getPixel(sp::dim::point2di(X, Y)).Green;
}
SP_EXPORT int spGetPixelBlue(int X, int Y)
{
    return __spVideoDriver->getPixel(sp::dim::point2di(X, Y)).Blue;
}
SP_EXPORT void spDraw2DPixel(int X, int Y, int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    __spVideoDriver->draw2DPixel(sp::dim::point2di(X, Y), sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha));
}
SP_EXPORT void spDraw2DLine1(
    int X1, int Y1, int X2, int Y2,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    __spVideoDriver->draw2DLine(
        sp::dim::point2di(X1, Y1), sp::dim::point2di(X2, Y2),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha)
    );
}
SP_EXPORT void spDraw2DLine2(
    int X1, int Y1, int X2, int Y2,
    int Color1Red, int Color1Green, int Color1Blue, int Color1Alpha,
    int Color2Red, int Color2Green, int Color2Blue, int Color2Alpha)
{
    __spVideoDriver->draw2DLine(
        sp::dim::point2di(X1, Y1), sp::dim::point2di(X2, Y2),
        sp::video::color(Color1Red, Color1Green, Color1Blue, Color1Alpha),
        sp::video::color(Color2Red, Color2Green, Color2Blue, Color2Alpha)
    );
}
SP_EXPORT void spDraw2DLine3(
    int X1, int Y1, int X2, int Y2,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha,
    int DotLength)
{
    __spVideoDriver->draw2DLine(
        sp::dim::point2di(X1, Y1), sp::dim::point2di(X2, Y2),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha),
        DotLength
    );
}
SP_EXPORT void spDraw2DRectangle1(
    int PositionLeft, int PositionTop, int PositionRight, int PositionBottom,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha,
    int Type)
{
    __spVideoDriver->draw2DRectangle(
        sp::dim::rect2di(PositionLeft, PositionTop, PositionRight, PositionBottom),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha),
        STATIC(sp::video::EDrawingModeTypes, Type)
    );
}
SP_EXPORT void spDraw2DRectangle2(
    int PositionLeft, int PositionTop, int PositionRight, int PositionBottom,
    int lefttopColorRed, int lefttopColorGreen, int lefttopColorBlue, int lefttopColorAlpha,
    int righttopColorRed, int righttopColorGreen, int righttopColorBlue, int righttopColorAlpha,
    int rightbottomColorRed, int rightbottomColorGreen, int rightbottomColorBlue, int rightbottomColorAlpha,
    int leftbottomColorRed, int leftbottomColorGreen, int leftbottomColorBlue, int leftbottomColorAlpha,
    int Type)
{
    __spVideoDriver->draw2DRectangle(
        sp::dim::rect2di(PositionLeft, PositionTop, PositionRight, PositionBottom),
        sp::video::color(lefttopColorRed, lefttopColorGreen, lefttopColorBlue, lefttopColorAlpha),
        sp::video::color(righttopColorRed, righttopColorGreen, righttopColorBlue, righttopColorAlpha),
        sp::video::color(rightbottomColorRed, rightbottomColorGreen, rightbottomColorBlue, rightbottomColorAlpha),
        sp::video::color(leftbottomColorRed, leftbottomColorGreen, leftbottomColorBlue, leftbottomColorAlpha),
        STATIC(sp::video::EDrawingModeTypes, Type)
    );
}
/*SP_EXPORT void sp()
{
    __spVideoDriver->();
}
SP_EXPORT void sp()
{
    __spVideoDriver->();
}
SP_EXPORT void sp()
{
    __spVideoDriver->();
}
SP_EXPORT void sp()
{
    __spVideoDriver->();
}*/




// Font

SP_EXPORT LP spLoadFont(
    char* FontName, int FontSize,
    int TypeFlag, int RotationAngle,
    bool isSymbolUsing, bool is3DSpaceUsing,
    float FontDepth, float DetailDeviation,
    int Space3DMode)
{
    return CAST(__spVideoDriver->loadFont(
        FontName,
        FontSize,
        TypeFlag,
        RotationAngle,
        isSymbolUsing,
        is3DSpaceUsing,
        FontDepth,
        DetailDeviation,
        STATIC(sp::video::EDrawingModeTypes, Space3DMode)
    ));
}
SP_EXPORT LP spLoadFontSize(
    char* FontName,
    int FontSizeWidth, int FontSizeHeight,
    int TypeFlag, int RotationAngle,
    bool isSymbolUsing, bool is3DSpaceUsing,
    float FontDepth, float DetailDeviation,
    int Space3DMode)
{
    return CAST(__spVideoDriver->loadFont(
        FontName,
        sp::dim::size2di(FontSizeWidth, FontSizeHeight),
        TypeFlag,
        RotationAngle,
        isSymbolUsing,
        is3DSpaceUsing,
        FontDepth,
        DetailDeviation,
        STATIC(sp::video::EDrawingModeTypes, Space3DMode)
    ));
}
SP_EXPORT void spDeleteFont(LP Font)
{
    __spVideoDriver->deleteFont(FONT(Font));
}
SP_EXPORT void spDraw2DText(
    LP Font,
    int PositionX, int PositionY,
    char* Text,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    __spVideoDriver->draw2DText(
        FONT(Font),
        sp::dim::point2di(PositionX, PositionY),
        Text,
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha)
    );
}
SP_EXPORT void spDraw3DText(
    LP Font,
    char* Text,
    float PositionX, float PositionY, float PositionZ,
    float RotationX, float RotationY, float RotationZ,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha,
    LP Camera)
{
    __spVideoDriver->draw3DText(
        FONT(Font),
        Text,
        sp::dim::vector3df(PositionX, PositionY, PositionZ),
        sp::dim::vector3df(RotationX, RotationY, RotationZ),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha),
        CAMERA(Camera)
    );
}





/* ========== namespace video::gui ========== */


SP_EXPORT LP spGetGUI()
{
    return CAST(__spGUI = __spVideoDriver->getGUI());
}




/* ========== namespace video::effect ========== */




/* ========== namespace video::movie ========== */




/* ========== namespace scene ========== */



SP_EXPORT LP spGetStencilManager()
{
    return CAST(__spStencil = __spSceneManager->getStencilManager());
}
SP_EXPORT LP spGetAnimator()
{
    return CAST(__spAnimator = __spSceneManager->getAnimator());
}

SP_EXPORT void spRenderScene()
{
    __spSceneManager->renderScene();
}

SP_EXPORT LP spCreateShader(int Type)
{
    return CAST(__spVideoDriver->createShader(STATIC(sp::video::EStandardShaderTypes, Type)));
}
SP_EXPORT LP spLoadShader(char* FilenameVertex, char* FilenameFragment)
{
    return CAST(__spVideoDriver->loadShader(FilenameVertex, FilenameFragment));
}
SP_EXPORT void spDeleteShader(LP Shader)
{
    sp::video::Shader* hShader = SHADER(Shader);
    __spVideoDriver->deleteShader(hShader);
}

SP_EXPORT LP spCreatePivot()
{
    return CAST(__spSceneManager->createPivot());
}
SP_EXPORT LP spCreateModel(int Type, int Detail, float Radius1, float Radius2, bool isCap)
{
    return CAST(__spSceneManager->createModel((unsigned)Type, Detail, Radius1, Radius2, isCap));
}
SP_EXPORT LP spCreateModelEx(int Type, int Detail, float Radius1, float Radius2, float DegreeLength, float Size, bool isCap)
{
    return CAST(__spSceneManager->createModel((unsigned)Type, Detail, Radius1, Radius2, DegreeLength, Size, isCap));
}
SP_EXPORT LP spCreateSuperShape(
    float v1, float v2, float v3, float v4, float v5, float v6,
    float v7, float v8, float v9, float v10, float v11, float v12, int Detail)
{
    float ValueList[12] = { v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12 };
    return CAST(__spSceneManager->createSuperShape(ValueList, Detail));
}
SP_EXPORT LP spCreateSkyBox(LP Texture1, LP Texture2, LP Texture3, LP Texture4, LP Texture5, LP Texture6, float Radius, int Type)
{
    sp::video::Texture* TextureList[6];
    TextureList[0] = (sp::video::Texture*)Texture1; TextureList[1] = (sp::video::Texture*)Texture2;
    TextureList[2] = (sp::video::Texture*)Texture3; TextureList[3] = (sp::video::Texture*)Texture4;
    TextureList[4] = (sp::video::Texture*)Texture5; TextureList[5] = (sp::video::Texture*)Texture6;
    return CAST(__spSceneManager->createSkyBox(TextureList, Radius, (unsigned)Type));
}
SP_EXPORT LP spCreateHighMap(int TextureHighMap, int Detail)
{
    return CAST(__spSceneManager->createHeightField((sp::video::Texture*)TextureHighMap, Detail));
}
SP_EXPORT LP spCreateCamera()
{
    return CAST(__spSceneManager->createCamera());
}
SP_EXPORT LP spCreateLight(int Type)
{
    return CAST(__spSceneManager->createLight(STATIC(sp::scene::ELightTypes, Type)));
}
SP_EXPORT LP spCreateSprite(LP Texture)
{
    return CAST(__spSceneManager->createSprite(TEXTURE(Texture)));
}
SP_EXPORT LP spCreateTerrain(LP TextureHighMap, LP Texture, float ChangeDist)
{
    return CAST(__spSceneManager->createTerrain(TEXTURE(TextureHighMap), TEXTURE(Texture), ChangeDist));
}
SP_EXPORT LP spLoadModel(char* Filename, char* TexturePath, int FileType)
{
    return CAST(__spSceneManager->loadModel(Filename, TexturePath, (sp::scene::EModelTypes)FileType));
}
SP_EXPORT LP spLoadScene(char* Filename, char* TexturePath, int FileType, int Tesselation)
{
    return CAST(__spSceneManager->loadScene(Filename, TexturePath, (sp::scene::ESceneTypes)FileType, Tesselation));
}

SP_EXPORT void spSetWireframe(int Mode)
{
    __spSceneManager->setWireframe(Mode);
}
SP_EXPORT void spSetLighting(bool isLighting)
{
    __spSceneManager->setLighting(isLighting);
}
SP_EXPORT void spSetStencilEffects(bool isStencilEffects)
{
    __spSceneManager->setStencilEffects(isStencilEffects);
}
SP_EXPORT void spSetFogMode(int FogMode)
{
    __spVideoDriver->setFogMode(FogMode);
}
SP_EXPORT void spSetFogColor(int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    __spVideoDriver->setFogColor(sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha));
}
SP_EXPORT void spSetFogRange(float Range, float NearFog, float FarFog, int FogType)
{
    __spVideoDriver->setFogRange(Range, NearFog, FarFog, FogType);
}
SP_EXPORT bool spDeleteNode(LP Node)
{
    return __spSceneManager->deleteNode(NODE(Node));
}

// Low-level functions



// Nodes

SP_EXPORT void spSetNodePosition(LP Node, float X, float Y, float Z, bool isGlobal)
{
    NODE(Node)->setPosition(sp::dim::vector3df(X, Y, Z), isGlobal);
}
SP_EXPORT void spSetNodeRotation(LP Node, float X, float Y, float Z, bool isGlobal)
{
    NODE(Node)->setRotation(sp::dim::vector3df(X, Y, Z), isGlobal);
}
SP_EXPORT void spSetNodeScale(LP Node, float X, float Y, float Z, bool isGlobal)
{
    NODE(Node)->setScale(sp::dim::vector3df(X, Y, Z), isGlobal);
}

SP_EXPORT float spGetNodePositionX(LP Node, bool isGlobal)
{
    return NODE(Node)->getPosition(isGlobal).X;
}
SP_EXPORT float spGetNodePositionY(LP Node, bool isGlobal)
{
    return NODE(Node)->getPosition(isGlobal).Y;
}
SP_EXPORT float spGetNodePositionZ(LP Node, bool isGlobal)
{
    return NODE(Node)->getPosition(isGlobal).Z;
}

SP_EXPORT void spDoNodeMove(LP Node, float X, float Y, float Z)
{
    NODE(Node)->doMove(sp::dim::vector3df(X, Y, Z));
}
SP_EXPORT void spDoNodeTranslate(LP Node, float X, float Y, float Z)
{
    NODE(Node)->doTranslate(sp::dim::vector3df(X, Y, Z));
}
SP_EXPORT void spDoNodeTransform(LP Node, float X, float Y, float Z)
{
    NODE(Node)->doTransform(sp::dim::vector3df(X, Y, Z));
}
SP_EXPORT void spDoNodeTurn(LP Node, float X, float Y, float Z)
{
    NODE(Node)->doTurn(sp::dim::vector3df(X, Y, Z));
}

SP_EXPORT void spSetNodeVisible(LP Node, bool isVisible)
{
    NODE(Node)->setVisible(isVisible);
}
SP_EXPORT bool spGetNodeVisible(LP Node)
{
    return NODE(Node)->getVisible();
}
SP_EXPORT void spSetNodeColor(LP Node, int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    NODE(Node)->setColor(sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha));
}
SP_EXPORT void spNodeLookAt(LP Node, float X, float Y, float Z)
{
    NODE(Node)->lookAt(sp::dim::vector3df(X, Y, Z));
}
SP_EXPORT void spSetNodeParent(LP Node, LP Parent)
{
    NODE(Node)->setParent(NODE(Parent));
}
SP_EXPORT LP spGetNodeParent(LP Node)
{
    return CAST(NODE(Node)->getParent());
}


// Entities

SP_EXPORT void spAddEntityTexture(LP Entity, LP Texture, int Level)
{
    ENTITY(Entity)->addTexture(TEXTURE(Texture), Level);
}
SP_EXPORT void spRemoveEntityTexture(LP Entity, int Level)
{
    ENTITY(Entity)->removeTexture(Level);
}
SP_EXPORT void spDoEntityTextureTranslate(LP Entity, int Level, float DirectionX, float DirectionY)
{
    ENTITY(Entity)->doTextureTranslate(Level, sp::dim::point2df(DirectionX, DirectionY));
}
SP_EXPORT void spDoEntityTextureTransform(LP Entity, int Level, float SizeWidth, float SizeHeight)
{
    ENTITY(Entity)->doTextureTransform(Level, sp::dim::size2df(SizeWidth, SizeHeight));
}
SP_EXPORT LP spGetEntityTexture(LP Entity, int Level)
{
    return CAST(ENTITY(Entity)->getTexture(Level));
}
SP_EXPORT void spUpdateModeling(LP Entity, bool isUpdateNormals)
{
    ENTITY(Entity)->updateModeling(isUpdateNormals);
}
SP_EXPORT void spUpdateNormals(LP Entity)
{
    ENTITY(Entity)->updateNormals();
}
SP_EXPORT void spDoMeshTranslate(LP Entity, float DirectionX, float DirectionY, float DirectionZ)
{
    ENTITY(Entity)->doMeshTranslate(sp::dim::vector3df(DirectionX, DirectionY, DirectionZ));
}
SP_EXPORT void spDoMeshTransform(LP Entity, float SizeX, float SizeY, float SizeZ)
{
    ENTITY(Entity)->doMeshTransform(sp::dim::vector3df(SizeX, SizeY, SizeZ));
}
SP_EXPORT void spDoMeshTurn(LP Entity, float RotationX, float RotationY, float RotationZ)
{
    ENTITY(Entity)->doMeshTurn(sp::dim::vector3df(RotationX, RotationY, RotationZ));
}
SP_EXPORT void spFitMesh(LP Entity, float PositionX, float PositionY, float PositionZ, float SizeWidth, float SizeHeight, float SizeDepth)
{
    ENTITY(Entity)->fitMesh(sp::dim::vector3df(PositionX, PositionY, PositionZ), sp::dim::vector3df(SizeWidth, SizeHeight, SizeDepth));
}
SP_EXPORT void spFlipMesh(LP Entity, bool isSelfModeling)
{
    ENTITY(Entity)->flipMesh(isSelfModeling);
}
SP_EXPORT void spSetEntityWireframe(LP Entity, int Mode)
{
    ENTITY(Entity)->setWireframe((unsigned char)Mode);
}
SP_EXPORT void spSetEntityColor(LP Entity, int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha, int Surface)
{
    ENTITY(Entity)->setColor(sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha), Surface);
}

// Entity - vertices, triangles & surfaces

SP_EXPORT int spCreateSurface(LP Entity)
{
    return ENTITY(Entity)->createSurface();
}
SP_EXPORT void spSelectSurface(LP Entity, int Surface)
{
    ENTITY(Entity)->selectSurface((unsigned)Surface);
}
SP_EXPORT int spGetMeshSurfaceCount(LP Entity)
{
    return (signed)ENTITY(Entity)->getMeshSurfaceCount();
}
SP_EXPORT int spGetMeshVerticesCount(LP Entity, int Surface)
{
    return ENTITY(Entity)->getMeshVerticesCount((unsigned)Surface);
}
SP_EXPORT int spGetMeshTriangleCount(LP Entity, int Surface)
{
    return ENTITY(Entity)->getMeshTriangleCount((unsigned)Surface);
}
SP_EXPORT float spGetVertexCoordX(LP Entity, int VertexIndex)
{
    return ENTITY(Entity)->getVertexCoord((unsigned)VertexIndex).X;
}
SP_EXPORT float spGetVertexCoordY(LP Entity, int VertexIndex)
{
    return ENTITY(Entity)->getVertexCoord((unsigned)VertexIndex).X;
}
SP_EXPORT float spGetVertexCoordZ(LP Entity, int VertexIndex)
{
    return ENTITY(Entity)->getVertexCoord((unsigned)VertexIndex).Z;
}
SP_EXPORT void spSetVertexCoord(LP Entity, int VertexIndex, float X, float Y, float Z)
{
    ENTITY(Entity)->setVertexCoord((unsigned)VertexIndex, sp::dim::vector3df(X, Y, Z));
}
SP_EXPORT float spGetVertexTexCoordX(LP Entity, int VertexIndex, int Level)
{
    return ENTITY(Entity)->getVertexTexCoord((unsigned)VertexIndex, (unsigned char)Level).X;
}
SP_EXPORT float spGetVertexTexCoordY(LP Entity, int VertexIndex, int Level)
{
    return ENTITY(Entity)->getVertexTexCoord((unsigned)VertexIndex, (unsigned char)Level).Y;
}
SP_EXPORT void spSetVertexTexCoord(LP Entity, int VertexIndex, float U, float V, int TexLevel)
{
    ENTITY(Entity)->setVertexTexCoord((unsigned)VertexIndex, sp::dim::point2df(U, V), (unsigned char)TexLevel);
}
SP_EXPORT int spGetVertexColorRed(LP Entity, int VertexIndex)
{
    return (int)ENTITY(Entity)->getVertexColor((unsigned)VertexIndex).Red;
}
SP_EXPORT int spGetVertexColorGreen(LP Entity, int VertexIndex)
{
    return (int)ENTITY(Entity)->getVertexColor((unsigned)VertexIndex).Green;
}
SP_EXPORT int spGetVertexColorBlue(LP Entity, int VertexIndex)
{
    return (int)ENTITY(Entity)->getVertexColor((unsigned)VertexIndex).Blue;
}
SP_EXPORT int spGetVertexColorAlpha(LP Entity, int VertexIndex)
{
    return (int)ENTITY(Entity)->getVertexColor((unsigned)VertexIndex).Alpha;
}
SP_EXPORT void spSetVertexColor(LP Entity, int VertexIndex, int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    ENTITY(Entity)->setVertexColor((unsigned)VertexIndex, sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha));
}
SP_EXPORT float spGetVertexNormalX(LP Entity, int VertexIndex)
{
    return ENTITY(Entity)->getVertexNormal((unsigned)VertexIndex).X;
}
SP_EXPORT float spGetVertexNormalY(LP Entity, int VertexIndex)
{
    return ENTITY(Entity)->getVertexNormal((unsigned)VertexIndex).Y;
}
SP_EXPORT float spGetVertexNormalZ(LP Entity, int VertexIndex)
{
    return ENTITY(Entity)->getVertexNormal((unsigned)VertexIndex).Z;
}
SP_EXPORT void spSetVertexNormal(LP Entity, int VertexIndex, float X, float Y, float Z)
{
    ENTITY(Entity)->setVertexNormal((unsigned)VertexIndex, sp::dim::vector3df(X, Y, Z));
}
SP_EXPORT void spAddEntityVertex(
    LP Entity,
    float X, float Y, float Z, float U, float V,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    ENTITY(Entity)->addVertex(
        sp::dim::vector3df(X, Y, Z),
        sp::dim::point2df(U, V),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha)
    );
}
SP_EXPORT void spAddEntityVertexEx(
    LP Entity,
    float X, float Y, float Z, float NX, float NY, float NZ, float U, float V,
    float Fog,
    int ColorRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    ENTITY(Entity)->addVertex(
        sp::dim::vector3df(X, Y, Z),
        sp::dim::vector3df(NX, NY, NZ),
        sp::dim::point2df(U, V),
        sp::video::color(ColorRed, ColorGreen, ColorBlue, ColorAlpha),
        Fog
    );
}
SP_EXPORT void spRemoveEntityVertex(LP Entity, int VertexIndex)
{
    ENTITY(Entity)->removeVertex((unsigned)VertexIndex);
}
SP_EXPORT int spAddEntityTriangle(LP Entity, int Vertex1, int Vertex2, int Vertex3)
{
    return (signed)ENTITY(Entity)->addTriangle((unsigned)Vertex1, (unsigned)Vertex2, (unsigned)Vertex3);
}
SP_EXPORT void spRemoveEntityTriangle(LP Entity, int TriangleIndex)
{
    ENTITY(Entity)->removeTriangle((unsigned)TriangleIndex);
}
SP_EXPORT void spSetEntityVertexCounter(LP Entity, int VertexCounter)
{
    ENTITY(Entity)->setVertexCounter((unsigned)VertexCounter);
}
SP_EXPORT void spAddEntityVertexCounter(LP Entity, int VertexCounter)
{
    ENTITY(Entity)->addVertexCounter((unsigned)VertexCounter);
}

// Entity - collision

SP_EXPORT bool spEntityDetectCollision(LP Entity, int CollisionID)
{
    return ENTITY(Entity)->detectCollision(CollisionID);
}
SP_EXPORT int spGetEntityCollisionCount(LP Entity)
{
    return (int)ENTITY(Entity)->getCollisionCount();
}
SP_EXPORT float spGetEntityCollisionPointX(LP Entity)
{
    return ENTITY(Entity)->getCollisionPoint().X;
}
SP_EXPORT float spGetEntityCollisionPointY(LP Entity)
{
    return ENTITY(Entity)->getCollisionPoint().Y;
}
SP_EXPORT float spGetEntityCollisionPointZ(LP Entity)
{
    return ENTITY(Entity)->getCollisionPoint().Z;
}
SP_EXPORT float spGetEntityCollisionNormalX(LP Entity)
{
    return ENTITY(Entity)->getCollisionNormal().X;
}
SP_EXPORT float spGetEntityCollisionNormalY(LP Entity)
{
    return ENTITY(Entity)->getCollisionNormal().Y;
}
SP_EXPORT float spGetEntityCollisionNormalZ(LP Entity)
{
    return ENTITY(Entity)->getCollisionNormal().Z;
}
SP_EXPORT LP spGetEntityCollisionObject(LP Entity)
{
    return CAST(ENTITY(Entity)->getCollisionObject());
}
SP_EXPORT void spSetEntityCollisionUpdate(LP Entity, bool isCollUpdate)
{
    ENTITY(Entity)->setCollisionUpdate(isCollUpdate);
}
SP_EXPORT bool spGetEntityCollisionUpdate(LP Entity)
{
    return ENTITY(Entity)->getCollisionUpdate();
}

// Entity - material & co.

SP_EXPORT void spSetEntityShader(LP Entity, LP Shader)
{
    ENTITY(Entity)->setShader(SHADER(Shader));
}
SP_EXPORT void spSetEntityShininess(LP Entity, float Intensity)
{
    ENTITY(Entity)->setShininess(Intensity);
}
SP_EXPORT float spGetEntityShininess(LP Entity)
{
    return ENTITY(Entity)->getShininess();
}
SP_EXPORT void spSetEntityLighting(LP Entity, bool isLighting)
{
    ENTITY(Entity)->setLighting(isLighting);
}
SP_EXPORT bool spGetEntityLighting(LP Entity)
{
    return ENTITY(Entity)->getLighting();
}
SP_EXPORT void spSetEntityOrder(LP Entity, int Order)
{
    ENTITY(Entity)->setOrder((char)Order);
}
SP_EXPORT int spGetEntityOrder(LP Entity)
{
    return (int)ENTITY(Entity)->getOrder();
}
SP_EXPORT void spSetEntityNormalsType(LP Entity, int NormalsType)
{
    ENTITY(Entity)->setNormalsType((unsigned char)NormalsType);
}
SP_EXPORT int spGetEntityNormalsType(LP Entity)
{
    return (int)ENTITY(Entity)->getNormalsType();
}
SP_EXPORT void spSetEntityBlendingMode(LP Entity, int BlendingMode)
{
    ENTITY(Entity)->setBlendingMode((unsigned char)BlendingMode);
}
SP_EXPORT void spSetEntityBlendingModeEx(LP Entity, int Blend1, int Blend2)
{
    ENTITY(Entity)->setBlendingMode(Blend1, Blend2);
}
SP_EXPORT void spSetEntitySphereMapping(LP Entity, int Type, int TexLevel)
{
    ENTITY(Entity)->setSphereMapping(STATIC(sp::video::ESphereMappingTypes, Type), (unsigned char)TexLevel);
}
SP_EXPORT LP spCopyEntity(LP Entity)
{
    CAST(ENTITY(Entity)->copy());
}
SP_EXPORT void spRenderEntity(LP Entity)
{
    ENTITY(Entity)->render();
}

// Camera

SP_EXPORT void spSetCameraPerspective(
    LP Camera,
    int ViewportLeft, int ViewportRight, int ViewportTop, int ViewportBottom,
    float NearClipping, float FarClipping, float ZoomAngle)
{
    CAMERA(Camera)->setPerspective(
        sp::dim::rect2di(ViewportLeft, ViewportRight, ViewportTop, ViewportBottom),
        NearClipping, FarClipping, ZoomAngle
    );
}
SP_EXPORT void spCameraLookAt(LP Camera, float X, float Y, float Z)
{
    CAMERA(Camera)->lookAt(sp::dim::vector3df(X, Y, Z));
}

// Sprite

SP_EXPORT void spSetSpriteBasePosition(LP Sprite, float X, float Y)
{
    SPRITE(Sprite)->setBasePosition(sp::dim::point2df(X, Y));
}
SP_EXPORT float spGetSpriteBasePositionX(LP Sprite)
{
    return SPRITE(Sprite)->getBasePosition().X;
}
SP_EXPORT float spGetSpriteBasePositionY(LP Sprite)
{
    return SPRITE(Sprite)->getBasePosition().Y;
}
SP_EXPORT void spSetSpriteLighting(LP Sprite, bool isLighting)
{
    SPRITE(Sprite)->setLighting(isLighting);
}
SP_EXPORT bool spGetSpriteLighting(LP Sprite)
{
    return SPRITE(Sprite)->getLighting();
}
SP_EXPORT void spSetSpriteOrder(LP Sprite, int Order)
{
    SPRITE(Sprite)->setOrder((char)Order);
}
SP_EXPORT int spGetSpriteOrder(LP Sprite)
{
    return (int)(SPRITE(Sprite)->getOrder());
}
SP_EXPORT void spSetSpriteBlendingMode(LP Sprite, int BlendingMode)
{
    SPRITE(Sprite)->setBlendingMode((unsigned char)BlendingMode);
}
SP_EXPORT void spSetSpriteBlendingModeEx(LP Sprite, int Blend1, int Blend2)
{
    SPRITE(Sprite)->setBlendingMode(Blend1, Blend2);
}
SP_EXPORT void spSetSpriteTexture(LP Sprite, LP Texture)
{
    SPRITE(Sprite)->setTexture(TEXTURE(Texture));
}
SP_EXPORT void spRenderSprite(LP Sprite)
{
    SPRITE(Sprite)->render();
}

// Shader

SP_EXPORT void spShaderBegin(LP Shader)
{
    SHADER(Shader)->bind();
}
SP_EXPORT void spShaderEnd()
{
    __spVideoDriver->unbindShader();
}
SP_EXPORT bool spSetShaderConstant1(LP Shader, char* Name, float Value)
{
    SHADER(Shader)->setConstant(Name, Value);
}
SP_EXPORT bool spSetShaderConstant2(LP Shader, char* Name, float X, float Y, float Z)
{
    sp::dim::vector3df Position(X, Y, Z);
    SHADER(Shader)->setConstant(Name, Position);
}


/* ========== namespace scene::anim ========== */


SP_EXPORT void spUpdateAnimations()
{
    __spAnimator->updateAnimations();
}
SP_EXPORT void spUpdateAnimation(LP Animation)
{
    ANIM(Animation)->updateAnimation();
}
SP_EXPORT LP spGetAnimation(LP Node)
{
    return CAST(__spAnimator->getAnimation(NODE(Node)));
}

SP_EXPORT void spDoAnimate(LP Animation, int AnimType, int AnimTime, int Start, int End)
{
    ANIM(Animation)->doAnimate((char)AnimType, AnimTime, (unsigned)Start, (unsigned)End);
}
SP_EXPORT void spDoAnimateNode(LP Animation, int AnimType, int AnimTime, int Start, int End)
{
    ANIM(Animation)->doAnimateNode((char)AnimType, AnimTime, (unsigned)Start, (unsigned)End);
}
SP_EXPORT void spDoAnimateMesh(LP Animation, int AnimType, int AnimTime, int Start, int End)
{
    ANIM(Animation)->doAnimateMesh((char)AnimType, AnimTime, (unsigned)Start, (unsigned)End);
}
SP_EXPORT void spDoAnimateSkeleton(LP Animation, int AnimType, int Start, int End)
{
    ANIM(Animation)->doAnimateSkeleton((char)AnimType, (unsigned)Start, (unsigned)End);
}

SP_EXPORT bool spIsAnimate(LP Animation)
{
    return ANIM(Animation)->isAnimate();
}
SP_EXPORT bool spIsAnimateNode(LP Animation)
{
    return ANIM(Animation)->isAnimateNode();
}
SP_EXPORT bool spIsAnimateMesh(LP Animation)
{
    return ANIM(Animation)->isAnimateMesh();
}
SP_EXPORT bool spIsAnimateSkeleton(LP Animation)
{
    return ANIM(Animation)->isAnimateSkeleton();
}

SP_EXPORT int spGetAnimLengthNode(LP Animation)
{
    return (signed)ANIM(Animation)->getAnimLengthNode();
}
SP_EXPORT int spGetAnimLengthMesh(LP Animation)
{
    return (signed)ANIM(Animation)->getAnimLengthMesh();
}

SP_EXPORT void spSetAnimSeekNode(LP Animation, float AnimSeek, int AnimSeekSeq)
{
    ANIM(Animation)->setAnimSeekNode(AnimSeek, (unsigned)AnimSeekSeq);
}
SP_EXPORT void spSetAnimSeekMesh(LP Animation, float AnimSeek, int AnimSeekSeq)
{
    ANIM(Animation)->setAnimSeekMesh(AnimSeek, (unsigned)AnimSeekSeq);
}

SP_EXPORT void spPauseAnimation(LP Animation, bool isPaused)
{
    ANIM(Animation)->pauseAnim(isPaused);
}
SP_EXPORT void spPauseAnimationNode(LP Animation, bool isPaused)
{
    ANIM(Animation)->pauseNode(isPaused);
}
SP_EXPORT void spPauseAnimationMesh(LP Animation, bool isPaused)
{
    ANIM(Animation)->pauseMesh(isPaused);
}
SP_EXPORT void spPauseAnimationSkeleton(LP Animation, bool isPaused)
{
    ANIM(Animation)->pauseSkeleton(isPaused);
}

SP_EXPORT void spStopAnimation(LP Animation)
{
    ANIM(Animation)->stopAnim();
}
SP_EXPORT void spStopAnimationNode(LP Animation)
{
    ANIM(Animation)->stopNode();
}
SP_EXPORT void spStopAnimationMesh(LP Animation)
{
    ANIM(Animation)->stopMesh();
}
SP_EXPORT void spStopAnimationSkeleton(LP Animation)
{
    ANIM(Animation)->stopSkeleton();
}

SP_EXPORT void spSetAnimationNode(LP Animation, LP Node)
{
    ANIM(Animation)->setNode(NODE(Node));
}
SP_EXPORT LP spGetAnimationNode(LP Animation)
{
    return CAST(ANIM(Animation)->getNode());
}
SP_EXPORT void spSetAnimationMesh(LP Animation, LP Mesh)
{
    ANIM(Animation)->setMesh(ENTITY(Mesh));
}
SP_EXPORT LP spGetAnimationMesh(LP Animation)
{
    return CAST(ANIM(Animation)->getMesh());
}

SP_EXPORT void spRenderAnimationSkeleton(LP Animation, LP Camera)
{
    ANIM(Animation)->renderSkeleton(CAMERA(Camera));
}


/* ========== namespace scene::stencil ========== */


SP_EXPORT void spSetVolumetricShadow(bool isVolumetricShadow)
{
    //__spStencil->setVolumetricShadow(isVolumetricShadow);
}
SP_EXPORT void spSetShadowColor(int ColroRed, int ColorGreen, int ColorBlue, int ColorAlpha)
{
    //__spStencil->setShadowColor(sp::video::color(ColroRed, ColorGreen, ColorBlue, ColorAlpha));
}
SP_EXPORT void spSetShadowLength(float Length)
{
    //__spStencil->setShadowLength(Length);
}
SP_EXPORT void spAddEntityCastCloud(LP Entity)
{
    __spStencil->addEntityCastCloud(ENTITY(Entity));
}
SP_EXPORT void spRemoveEntityCastCloud(LP Entity)
{
    __spStencil->removeEntityCastCloud(ENTITY(Entity));
}
SP_EXPORT void spAddLightShadowSource(LP Light)
{
    __spStencil->addLightSource(new sp::scene::SShadowLightSource(NODE(Light)));
}
SP_EXPORT void spRemoveLightShadowSource(LP Light)
{
    //__spStencil->removeLightSource(LIGHT(Light));
}
SP_EXPORT void spUpdateStencilShadow(LP Entity)
{
    __spStencil->updateStencilShadow(ENTITY(Entity));
}


/* ========== namespace scene::effect ========== */




/* ========== namespace script ========== */




/* ========== namespace sound ========== */


SP_EXPORT LP spCreateListener(float NearHearing, float FarHearing, float NearVolume, float FarVolume)
{
    return CAST(__spListener = sp::sound::createListener(NearHearing, FarHearing, NearVolume, FarVolume));
}
SP_EXPORT void spDeleteListener()
{
    __spListener->deleteDevice();
}

SP_EXPORT LP spLoadSound(char* Filename, int Quality, int Count)
{
    return CAST(__spListener->loadSound(Filename, Quality, Count));
}
SP_EXPORT void spPlaySound(LP Sound, bool isLooped, bool isVectorUsing)
{
    __spListener->playSound(SOUND(Sound), isLooped, isVectorUsing);
}
SP_EXPORT void spStopSound(LP Sound)
{
    __spListener->stopSound(SOUND(Sound));
}
SP_EXPORT void spPauseSound(LP Sound, bool isPaused)
{
    __spListener->pauseSound(SOUND(Sound), isPaused);
}
SP_EXPORT void spUpdateSounds3D(float PositionX, float PositionY, float PositionZ, float CamYaw)
{
    __spListener->updateSounds3D(sp::dim::vector3df(PositionX, PositionY, PositionZ), CamYaw);
}
SP_EXPORT void spUpdateSounds()
{
    __spListener->updateSounds();
}

SP_EXPORT bool spIsSound(LP Sound)
{
    return __spListener->isSound(SOUND(Sound));
}
SP_EXPORT void spSetSoundVolume(LP Sound, float Volume)
{
    __spListener->setSoundVolume(SOUND(Sound), Volume);
}
SP_EXPORT float spGetSoundVolume(LP Sound)
{
    return __spListener->getSoundVolume(SOUND(Sound));
}
SP_EXPORT void spSetSoundSeek(LP Sound, float Seek)
{
    __spListener->setSoundSeek(SOUND(Sound), Seek);
}
SP_EXPORT float spGetSoundSeek(LP Sound)
{
    return __spListener->getSoundSeek(SOUND(Sound));
}
SP_EXPORT void spSetSoundSpeed(LP Sound, float Speed)
{
    __spListener->setSoundSpeed(SOUND(Sound), Speed);
}
SP_EXPORT float spGetSoundSpeed(LP Sound)
{
    return __spListener->getSoundSpeed(SOUND(Sound));
}
SP_EXPORT void spSetSoundPan(LP Sound, float Balance, float Volume)
{
    __spListener->setSoundPan(SOUND(Sound), Balance, Volume);
}
SP_EXPORT float spGetSoundPan(LP Sound)
{
    return __spListener->getSoundPan(SOUND(Sound));
}
SP_EXPORT void spSetSoundPosition(LP Sound, float PositionX, float PositionY, float PositionZ)
{
    SOUND(Sound)->setPosition(sp::dim::vector3df(PositionX, PositionY, PositionZ));
}
SP_EXPORT float spGetSoundPositionX(LP Sound)
{
    return SOUND(Sound)->getPosition().X;
}
SP_EXPORT float spGetSoundPositionY(LP Sound)
{
    return SOUND(Sound)->getPosition().Y;
}
SP_EXPORT float spGetSoundPositionZ(LP Sound)
{
    return SOUND(Sound)->getPosition().Z;
}

// Records

SP_EXPORT LP spCreateRecord()
{
    return CAST(__spListener->createRecord());
}
SP_EXPORT void spCloseRecord(LP Record)
{
    __spListener->closeRecord(RECORD(Record));
}
SP_EXPORT void spBeginRecording(LP Record)
{
    RECORD(Record)->beginRecording();
}
SP_EXPORT void spEndRecording(LP Record)
{
    RECORD(Record)->endRecording();
}
SP_EXPORT void spSaveRecord(LP Record, char* Filename)
{
    RECORD(Record)->saveRecord(Filename);
}
SP_EXPORT bool spIsRecording(LP Record)
{
    return RECORD(Record)->isRecording();
}







/* ========== namespace network ========== */






/*

SP_EXPORT void spFunc()
{
    __spPointer->spFunc();
}

*/





// ================================================================================
